@@ -1,4 +1,4 @@
-;;;; binary.io -- serializing binary data
+;;; binary.io -- serializing binary data
 
 ;;; Created:    <2003-01-20 23:25:06 foof>
 ;;; Time-stamp: <2003-01-29 20:53:45 foof>
@@ -12,13 +12,11 @@
 ;; employments.
 ;;          -- from "Gulliver's Travels" by Jonathan Swift
 
-;; [SK]: 
-;;
+;; This module was originally written by Alex Shinn, in pure Scheme.
+;; Shiro Kawai rewrote primitive routines in C for the better performance,
+;; renamed them for shorter names, and added uvector access routines.
 
 (define-module binary.io
-  (use gauche.uvector)
-  (use srfi-1)  ;; list library
-  (use srfi-13) ;; string library
   (export read-uint read-u8 read-u16 read-u32 read-u64
           read-sint read-s8 read-s16 read-s32 read-s64
           read-ber-integer read-f16 read-f32 read-f64
@@ -35,7 +33,7 @@
           put-u16be! put-u16le! put-u32be! put-u32le! put-u64be! put-u64le!
           put-s16be! put-s16le! put-s32be! put-s32le! put-s64be! put-s64le!
           put-f16be! put-f16le! put-f32be! put-f32le! put-f64be! put-f64le!
-          
+
           ;; old names
           read-binary-uint
           read-binary-uint8 read-binary-uint16
@@ -60,8 +58,9 @@
 
 (dynamic-load "binary--io")
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; config
+;;;
+;;; config
+;;;
 
 (define-constant *bit-size* 2)   ;; hey, you never know :)
 (define-constant *byte-size* 8)
@@ -69,23 +68,23 @@
 (define-constant *byte-mask* (- *byte-magnitude* 1))
 (define-constant *byte-right-shift* (* -1 *byte-size*))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; basic reading
+;;;
+;;; basic reading
+;;;
 
 ;; mind-numblingly slow, consider a uvector approach but it doesn't
 ;; handle endianess
 (define (read-uint size :optional (port (current-input-port))
-                   (endian (default-endian)))
+                                  (endian (default-endian)))
   (case size
     [(1) (read-u8 port endian)]
     [(2) (read-u16 port endian)]
     [(4) (read-u32 port endian)]
     [(8) (read-u64 port endian)]
     [else
-     (let loop ((ls '())
-                (cnt 0))
+     (let loop ([ls '()] [cnt 0])
        (if (= cnt size)
-         (fold (lambda (a b) (+ a (* b *byte-magnitude*)))
+         (fold (^[a b] (+ a (* b *byte-magnitude*)))
                0
                (if (eq? endian 'big-endian) (reverse ls) ls))
          (let1 byte (read-byte port)
@@ -98,7 +97,7 @@
   (logand (lognot int) (- (expt *bit-size* (* *byte-size* bytes)) 1)))
 
 (define (uint->sint int bytes)
-  (let ((highbit (- (* *byte-size* bytes) 1)))
+  (let1 highbit (- (* *byte-size* bytes) 1)
     (if (logbit? highbit int)
       (* -1 (+ 1 (lognot-small int bytes)))
       int)))
@@ -108,9 +107,8 @@
     (+ 1 (lognot-small (abs int) bytes))
     int))
 
-(define (read-sint size :optional
-                   (port (current-input-port))
-                   (endian (default-endian)))
+(define (read-sint size :optional (port (current-input-port))
+                                  (endian (default-endian)))
   (case size
     [(1) (read-s8 port endian)]
     [(2) (read-s16 port endian)]
@@ -118,21 +116,21 @@
     [(8) (read-s64 port endian)]
     [else (uint->sint (read-uint size port endian) size)]))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; basic writing
+;;;
+;;; basic writing
+;;;
 
-(define (write-uint size int :optional
-                    (port (current-output-port))
-                    (endian (default-endian)))
+(define (write-uint size int :optional (port (current-output-port))
+                                       (endian (default-endian)))
   (case size
     [(1) (write-u8 int port endian)]
     [(2) (write-u16 int port endian)]
     [(4) (write-u32 int port endian)]
     [(8) (write-u64 int port endian)]
     [else
-     (let ((ls '()))
+     (let ([ls '()])
        ;; build a list of bytes
-       (dotimes (i size)
+       (dotimes [i size]
          (push! ls (logand int *byte-mask*))
          (set! int (ash int *byte-right-shift*)))
        ;; reverse if big-endian
@@ -141,9 +139,8 @@
        ;; write the list
        (for-each (cut write-byte <> port) ls))]))
 
-(define (write-sint size int :optional
-                    (port (current-output-port))
-                    (endian (default-endian)))
+(define (write-sint size int :optional (port (current-output-port))
+                                       (endian (default-endian)))
   (case size
     [(1) (write-s8 int port endian)]
     [(2) (write-s16 int port endian)]
@@ -151,9 +148,9 @@
     [(8) (write-s64 int port endian)]
     [else (write-uint size (sint->uint int size) port endian)]))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; compatibility
-;;
+;;;
+;;; compatibility
+;;;
 
 ;; These are used in binary.pack for the (unofficial) features to
 ;; read/write integers in "native" width of C system on the platform.
@@ -196,8 +193,9 @@
 (define write-binary-float  write-f32)
 (define write-binary-double write-f64)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; bignum encodings -- Basic Encoding Rules (BER) from X.209
+;;;
+;;; bignum encodings -- Basic Encoding Rules (BER) from X.209
+;;;
 
 ;; A BER compressed integer is an unsigned integer in base 128, most
 ;; significant digit first, where the high bit is set on all but the
@@ -207,25 +205,22 @@
 
 (define (read-ber-integer :optional (port (current-input-port)))
   (let1 first (read-byte port)
-    (if (eof-object? first)
-      first ;; stop on eof
-      (if (< first 128)
-        first
-        (let loop ((res (ash (logand first #b01111111) 7))
-                   (byte (read-u8 port)))
-          (if (< byte 128)
-            (+ res byte) ;; final byte
-            (loop (ash (+ res (logand byte #b01111111)) 7)
-                  (read-u8 port))))))))
+    (cond [(eof-object? first) first] ;; stop on eof
+          [(< first 128) first]
+          [else (let loop ([res (ash (logand first #b01111111) 7)]
+                           [byte (read-u8 port)])
+                  (if (< byte 128)
+                    (+ res byte) ;; final byte
+                    (loop (ash (+ res (logand byte #b01111111)) 7)
+                          (read-u8 port))))])))
 
 (define (write-ber-integer number :optional (port (current-output-port)))
-  (let ((final (logand number #b01111111))
-        (start (ash number -7)))
+  (let ([final (logand number #b01111111)]
+        [start (ash number -7)])
     (unless (zero? start)
-      (let loop ((n start))
+      (let loop ([n start])
         (cond [(< n 128) (write-u8 (logior n #b10000000))]
-              [else
-               (loop (ash n -7)) ;; write high bytes first
-               (write-u8 (logior (logand n #b01111111) #b10000000))])))
+              [else (loop (ash n -7)) ;; write high bytes first
+                    (write-u8 (logior (logand n #b01111111) #b10000000))])))
     (write-u8 final)))
 
