@@ -21,13 +21,13 @@ typedef struct MD5Context {
 	unsigned char in[64];
 } MD5_CTX;
 
-extern void   MD5Init(MD5_CTX *context);
-extern void   MD5Update(MD5_CTX *context, unsigned char const *buf,
+extern void   MD5_Init(MD5_CTX *context);
+extern void   MD5_Update(MD5_CTX *context, unsigned char const *buf,
 	       unsigned len);
-extern void   MD5Final(unsigned char digest[MD5_HASHBYTES], MD5_CTX *context);
-extern void   MD5Transform(u_int32_t buf[4], u_int32_t const in[16]);
-extern char * MD5End(MD5_CTX *, char *);
-extern char * MD5File(const char *, char *);
-extern char * MD5Data (const unsigned char *, unsigned int, char *);
+extern void   MD5_Final(unsigned char digest[MD5_HASHBYTES], MD5_CTX *context);
+extern void   MD5_Transform(u_int32_t buf[4], u_int32_t const in[16]);
+extern char * MD5_End(MD5_CTX *, char *);
+extern char * MD5_File(const char *, char *);
+extern char * MD5_Data (const unsigned char *, unsigned int, char *);
 
 #endif /* !_MD5_H_ */
