@@ -2,7 +2,7 @@
 ;;; sha - SHA-1/SHA-224/SHA-256/SHA-384/SHA-512 message-digest
 ;;;
 ;;;   Copyright (c) 2002-2003 Kimura Fuyuki, All rights reserved.
-;;;   Copyright (c) 2008-2010  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2008-2012  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -52,20 +52,17 @@
 (define-constant *sha-unit-len* 4096)
 
 (define (gen-digest init update end)
-  (lambda ()
-    (let ((ctx (make <sha-context>))
-          (buf  (make-u8vector *sha-unit-len*)))
-      (init ctx)
-      (port-for-each
-       (lambda (x) (update ctx x))
-       (lambda ()
-         (let1 count (read-block! buf)
-           (if (eof-object? count)
-             count
-             (if (< count *sha-unit-len*)
-               (uvector-alias <u8vector> buf 0 count)
-               buf)))))
-      (end ctx))))
+  (^[] (let ([ctx (make <sha-context>)]
+             [buf (make-u8vector *sha-unit-len*)])
+         (init ctx)
+         (generator-for-each
+          (^x (update ctx x))
+          (^[] (let1 count (read-block! buf)
+                 (cond [(eof-object? count) count]
+                       [(< count *sha-unit-len*)
+                        (uvector-alias <u8vector> buf 0 count)]
+                       [else buf]))))
+         (end ctx))))
 
 (define sha1-digest   (gen-digest %sha1-init   %sha1-update   %sha1-final))
 (define sha224-digest (gen-digest %sha224-init %sha224-update %sha224-final))
@@ -84,12 +81,12 @@
 ;;;
 
 (define-macro (define-framework n)
-  (let ((meta   (string->symbol #`"<sha,|n|-meta>"))
-        (cls    (string->symbol #`"<sha,|n|>"))
-        (init   (string->symbol #`"%sha,|n|-init"))
-        (update (string->symbol #`"%sha,|n|-update"))
-        (final  (string->symbol #`"%sha,|n|-final"))
-        (digest (string->symbol #`"sha,|n|-digest")))
+  (let ([meta   (string->symbol #`"<sha,|n|-meta>")]
+        [cls    (string->symbol #`"<sha,|n|>")]
+        [init   (string->symbol #`"%sha,|n|-init")]
+        [update (string->symbol #`"%sha,|n|-update")]
+        [final  (string->symbol #`"%sha,|n|-final")]
+        [digest (string->symbol #`"sha,|n|-digest")])
     `(begin
        (define-class ,meta (<message-digest-algorithm-meta>) ())
        (define-class ,cls (<message-digest-algorithm>)
@@ -125,7 +122,7 @@
 
  "#define LIBGAUCHE_EXT_BODY"
  "#include <gauche/extern.h>  /* fix SCM_EXTERN in SCM_CLASS_DECL */"
- 
+
  "typedef struct ScmShaContextRec {"
  " SCM_HEADER;"
  " SHA_CTX ctx;"
