@@ -20,17 +20,17 @@
   ((strings :initform '() :init-keyword :strings))
   :metaclass <string-seq-meta>)
 
-(define-method call-with-iterator ((seq <string-seq>) proc . opts)
-  (let* ((start (get-keyword :start opts #f))
-         (ss    (slot-ref seq 'strings))
-         (p     (if start (list-tail ss start) ss)))
-    (proc (lambda () (null? p))
-          (lambda () (pop! p)))))
+(define-method call-with-iterator ((seq <string-seq>) proc :key (start #f)
+                                   :allow-other-keys)
+  (let* ([ss (slot-ref seq 'strings)]
+         [p  (if start (list-tail ss start) ss)])
+    (proc (^[] (null? p))
+          (^[] (pop! p)))))
 
-(define-method call-with-builder ((seq <string-seq-meta>) proc . opts)
-  (let ((q (make-queue)))
-    (proc (lambda (item) (enqueue! q (x->string item)))
-          (lambda () (make <string-seq> :strings (dequeue-all! q))))))
+(define-method call-with-builder ((seq <string-seq-meta>) proc . _)
+  (let1 q (make-queue)
+    (proc (^[item] (enqueue! q (x->string item)))
+          (^[] (make <string-seq> :strings (dequeue-all! q))))))
 
 (define-method referencer ((seq <string-seq>))
   (lambda (o i . args)
@@ -71,45 +71,41 @@
        (fold list* '() "abc" (sseq 'a 'b 'c) '(a b c)))
 
 (test* "fold2" '(21 (6 5 4 3 2 1))
-       (receive r
-           (fold2 (lambda (n s m) (values (+ n s) (cons n m)))
-                  0 '() '(1 2 3 4 5 6))
-         r))
+       (values->list
+        (fold2 (^[n s m] (values (+ n s) (cons n m)))
+               0 '() '(1 2 3 4 5 6))))
 (test* "fold2 (n-ary)" '(195 (5 15 25 4 14 24 3 13 23 2 12 22 1 11 21))
-       (receive r
-           (fold2 (lambda (n0 n1 n2 s m)
-                    (values (+ n0 n1 n2 s)
-                            (list* n0 n1 n2 m)))
-                  0 '() '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))
-         r))
+       (values->list
+        (fold2 (^[n0 n1 n2 s m]
+                 (values (+ n0 n1 n2 s)
+                         (list* n0 n1 n2 m)))
+               0 '() '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))))
 
 (test* "fold3" '(21 720 (6 5 4 3 2 1))
-       (receive r
-           (fold3 (lambda (n s m l) (values (+ n s) (* n m) (cons n l)))
-                  0 1 '() '(1 2 3 4 5 6))
-         r))
+       (values->list
+        (fold3 (^[n s m l] (values (+ n s) (* n m) (cons n l)))
+               0 1 '() '(1 2 3 4 5 6))))
 (test* "fold3 (n-ary)" '(195 275701345920000
                          (5 15 25 4 14 24 3 13 23 2 12 22 1 11 21))
-       (receive r
-           (fold3 (lambda (n0 n1 n2 s m l)
-                    (values (+ n0 n1 n2 s)
-                            (* n0 n1 n2 m)
-                            (list* n0 n1 n2 l)))
-                  0 1 '()
-                  '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))
-         r))
+       (values->list
+        (fold3 (^[n0 n1 n2 s m l]
+                 (values (+ n0 n1 n2 s)
+                         (* n0 n1 n2 m)
+                         (list* n0 n1 n2 l)))
+               0 1 '()
+               '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))))
 
 (test* "map (list)" '(2 4 6 8 10)
-       (map (lambda (x) (* x 2)) '(1 2 3 4 5)))
+       (map (^x (* x 2)) '(1 2 3 4 5)))
 (test* "map (vector)" '(2 4 6 8 10)
-       (map (lambda (x) (* x 2)) '#(1 2 3 4 5)))
+       (map (^x (* x 2)) '#(1 2 3 4 5)))
 (test* "map (string)" '(2 4 6 8 10)
-       (map (lambda (x) (* (digit->integer x) 2)) "12345"))
+       (map (^x (* (digit->integer x) 2)) "12345"))
 (test* "map (hash-table)" '((c . "c") (b . "b") (a . "a"))
        (map identity (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c")))
        (cut lset= equal? <> <>))
 (test* "map (custom)" '(2 4 6 8 10)
-       (map (lambda (x) (* (string->number x) 2))
+       (map (^x (* (string->number x) 2))
             (sseq 1 2 3 4 5)))
 
 (test* "map (n-ary)" '(3 5 7 9 11)
@@ -119,25 +115,25 @@
 (test* "map (n-ary)" '(3 5 7 9 11)
        (map + '#(1 2 3 4 5) '#(2 3 4 5 6)))
 (test* "map (n-ary)" '("123" "234" "345" "456")
-       (map (lambda arg (apply string-append (map x->string arg)))
+       (map (^ arg (apply string-append (map x->string arg)))
             "12345" (sseq 2 3 4 5) '#(3 4 5 6 7 8)))
 
 (test* "map-to (list->vector)" '#(2 4 6 8)
-       (map-to <vector> (lambda (x) (* x 2)) '(1 2 3 4)))
+       (map-to <vector> (^x (* x 2)) '(1 2 3 4)))
 (test* "map-to (list->string)" "2468"
-       (map-to <string> (lambda (x) (integer->digit (* x 2)))
+       (map-to <string> (^x (integer->digit (* x 2)))
                '(1 2 3 4)))
 (test* "map-to (list->custom)" '("2" "4" "6" "8")
        (slot-ref
-        (map-to <string-seq> (lambda (x) (* x 2)) '(1 2 3 4))
+        (map-to <string-seq> (^x (* x 2)) '(1 2 3 4))
         'strings))
 (test* "map-to (vector->list" '(2 4 6 8)
-       (map-to <list> (lambda (x) (* x 2)) '#(1 2 3 4)))
+       (map-to <list> (^x (* x 2)) '#(1 2 3 4)))
 (test* "map-to (vector->string" "1234"
        (map-to <string> integer->digit '#(1 2 3 4)))
 (test* "map-to (vector->custom)" '("2" "4" "6" "8")
        (slot-ref
-        (map-to <string-seq> (lambda (x) (* x 2)) '#(1 2 3 4))
+        (map-to <string-seq> (^x (* x 2)) '#(1 2 3 4))
         'strings))
 
 (test* "map-to (nary)" '#(3 5 7 9 11)
@@ -148,45 +144,35 @@
        (map-to <vector> + '(1 2 3 4 5) '#(2 3 4 5)))
 
 (test* "map-accum" '((45 30 15) 5)
-       (receive r 
-           (map-accum (lambda (elt seed) (values (* elt seed) seed))
-                      5 '(9 6 3))
-         r))
+       (values->list
+        (map-accum (^[elt seed] (values (* elt seed) seed))
+                   5 '(9 6 3))))
 (test* "map-accum" '((10 28 88) 19)
-       (receive r
-           (map-accum (lambda (elt seed) (values (* elt seed) (+ elt seed)))
-                      5 '(2 4 8))
-         r))
+       (values->list
+        (map-accum (^[elt seed] (values (* elt seed) (+ elt seed)))
+                   5 '(2 4 8))))
 (test* "map-accum (nary)" '((10 11 16) 15)
-       (receive r
-           (map-accum (lambda (x y seed)
-                        (values (+ x y seed) (+ x seed)))
-                      1 '(2 4 8) '(7 4 1))
-         r))
-
+       (values->list
+        (map-accum (^[x y seed] (values (+ x y seed) (+ x seed)))
+                   1 '(2 4 8) '(7 4 1))))
 
 (test* "for-each (list)" '(5 4 3 2 1)
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x)) '(1 2 3 4 5))
-         p))
+       (rlet1 p '()
+         (for-each (^x (push! p x)) '(1 2 3 4 5))))
 (test* "for-each (vector)" '(5 4 3 2 1)
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x)) '#(1 2 3 4 5))
-         p))
+       (rlet1 p '()
+         (for-each (^x (push! p x)) '#(1 2 3 4 5))))
 (test* "for-each (string)" '(#\5 #\4 #\3 #\2 #\1)
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x)) "12345")
-         p))
+       (rlet1 p '()
+         (for-each (^x (push! p x)) "12345")))
 (test* "for-each (hash-table)" '((c . "c") (b . "b") (a . "a"))
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x))
-                   (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c")))
-         p)
+       (rlet1 p '()
+         (for-each (^x (push! p x))
+                   (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c"))))
        (cut lset= equal? <> <>))
 (test* "for-each (custom)" '("5" "4" "3" "2" "1")
-       (let ((p '()))
-         (for-each (lambda (x) (push! p x)) (sseq 1 2 3 4 5))
-         p))
+       (rlet1 p '()
+         (for-each (^x (push! p x)) (sseq 1 2 3 4 5))))
 
 (test-section "searching and selection")
 
@@ -196,13 +182,43 @@
        (find even? '#(3 1 7 5 4 8 7)))
 (test* "find (string)" #\a
        (find char-lower-case? "YAEUB4309aBrnar"))
-(test* "for-each (hash-table)" '(b . "b")
-       (find (lambda (x) (string=? "b" (cdr x)))
+(test* "find (hash-table)" '(b . "b")
+       (find (^x (string=? "b" (cdr x)))
              (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c"))))
 (test* "find (custom)" "zoo"
-       (find (lambda (s) (= (size-of s) 3))
+       (find (^s (= (size-of s) 3))
              (sseq 'najr 'ej 'zoo 'bunr)))
 
+(define (test-find-minmax msg ex-min ex-max coll . args)
+  (test* #`"find-min (,|msg|)" ex-min (apply find-min coll args))
+  (test* #`"find-max (,|msg|)" ex-max (apply find-max coll args))
+  (test* #`"find-min&max (,|msg|)" (list ex-min ex-max)
+         (values->list (apply find-min&max coll args))))
+
+
+(test-find-minmax "list 0" -1 9
+                  '(3 9 -1 6))
+(test-find-minmax "list 1" '(c . -1) '(b . 9)
+                  '((a . 3) (b . 9) (c . -1) (d . 6)) :key cdr)
+(test-find-minmax "list 2" '(c . -1) '(b . 9)
+                  '((c . -1) (a . 3) (b . 9) (d . 6)) :key cdr)
+(test-find-minmax "list 3" '(c . -1) '(c . -1)
+                  '((c . -1)) :key cdr)
+(test-find-minmax "list 4" #f #f
+                  '() :key cdr)
+(test-find-minmax "list 5" 'foo 'foo
+                  '() :key cdr :default 'foo)
+(test-find-minmax "list 6" '("a" . 3) '("d" . 6)
+                  '(("c" . -1) ("b" . 9) ("a" . 3) ("d" . 6))
+                  :key car :compare string<?)
+(test-find-minmax "vector" '(c . -1) '(b . 9)
+                  '#((a . 3) (b . 9) (c . -1) (d . 6)) :key cdr)
+(test-find-minmax "string" '#\a '#\Z
+                  "IBaenRuZgaeKG" :compare char-ci<?)
+(test-find-minmax "hash-table" '(c . -1) '(b . 9)
+                  (hash-table 'eq? '(a . 3) '(b . 9) '(c . -1) '(d . 6))
+                  :key cdr)
+
 (test* "filter (list)" '(2 4 6)
        (filter even? '(1 2 3 4 5 6 7)))
 (test* "filter (vector)" '(2 4 6)
@@ -210,11 +226,11 @@
 (test* "filter (string)" '(#\a #\r #\b)
        (filter char-lower-case? "UBaBrGLbO"))
 (test* "filter (hash-table)" '((b . "b"))
-       (filter (lambda (x) (string=? "b" (cdr x)))
+       (filter (^x (string=? "b" (cdr x)))
                (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c")))
        (cut lset= equal? <> <>))
 (test* "filter (custom)" '("zoo" "zn")
-       (filter (lambda (s) (string-prefix? "z" s))
+       (filter (^s (string-prefix? "z" s))
                (sseq 'urnb 'zoo 'nbak 'zn 'run)))
 
 (test* "filter-to (vector)" '#(2 4 6)
@@ -226,7 +242,7 @@
 (test* "filter-to (hash-table)" '((b . "b") (z . "b"))
        (hash-table-map
         (filter-to <hash-table>
-                   (lambda (x) (string=? "b" (cdr x)))
+                   (^x (string=? "b" (cdr x)))
                    (hash-table 'eq?
                                '(a . "a") '(b . "b")
                                '(c . "c") '(z . "b")))
@@ -240,11 +256,11 @@
 (test* "remove (string)" '(#\U #\B #\B #\G #\L #\O)
        (remove char-lower-case? "UBaBrGLbO"))
 (test* "remove (hash-table)" '((a . "a") (c . "c"))
-       (remove (lambda (x) (string=? "b" (cdr x)))
+       (remove (^x (string=? "b" (cdr x)))
                (hash-table 'eq? '(a . "a") '(b . "b") '(c . "c")))
        (cut lset= equal? <> <>))
 (test* "remove (custom)" '("urnb" "nbak" "run")
-       (remove (lambda (s) (string-prefix? "z" s))
+       (remove (^s (string-prefix? "z" s))
                (sseq 'urnb 'zoo 'nbak 'zn 'run)))
 
 (test* "remove-to (vector)" '#(1 3 5 7)
@@ -252,7 +268,7 @@
 (test* "remove-to (hash-table)" '((a . "a") (c . "c"))
        (hash-table-map
         (remove-to <hash-table>
-                   (lambda (x) (string=? "b" (cdr x)))
+                   (^x (string=? "b" (cdr x)))
                    (hash-table 'eq?
                                '(a . "a") '(b . "b")
                                '(c . "c") '(z . "b")))
@@ -264,22 +280,17 @@
         'strings))
 
 (test* "partition (list)" '((2 4 6) (1 3 5 7))
-       (receive r (partition even? '(1 2 3 4 5 6 7))
-         r))
+       (values->list (partition even? '(1 2 3 4 5 6 7))))
 (test* "partition (vector)" '((2 4 6) (1 3 5 7))
-       (receive r (partition even? '#(1 2 3 4 5 6 7))
-         r))
+       (values->list (partition even? '#(1 2 3 4 5 6 7))))
 (test* "partition (custom)" '(("2" "4" "6") ("1" "3" "5" "7"))
-       (receive r (partition (lambda (e) (even? (string->number e)))
-                             (sseq 1 2 3 4 5 6 7))
-         r))
+       (values->list (partition (^e (even? (string->number e)))
+                                (sseq 1 2 3 4 5 6 7))))
 
 (test* "partition-to (string)" '("ACE" "bdf")
-       (receive r (partition-to <string> char-upper-case? "AbCdEf")
-         r))
+       (values->list (partition-to <string> char-upper-case? "AbCdEf")))
 (test* "partition-to (vector)" '(#(2 4 6) #(1 3 5 7))
-       (receive r (partition-to <vector> even? '#(1 2 3 4 5 6 7))
-         r))
+       (values->list (partition-to <vector> even? '#(1 2 3 4 5 6 7))))
 
 (test-section "miscellaneous")
 
@@ -340,7 +351,7 @@
        (group-collection '(1 2 3 2 3 1 2 1 2 3 2 3)))
 (test* "group-collection w/test" '((1 3 3 1 1 3 3) (2 2 2 2 2))
        (group-collection '(1 2 3 2 3 1 2 1 2 3 2 3)
-                         :test (lambda (x y) (= (modulo x 2) (modulo y 2)))))
+                         :test (^[x y] (= (modulo x 2) (modulo y 2)))))
 (test* "group-collection w/key" '(((1 a) (1 c)) ((2 b) (2 q)) ((3 c) (3 d)))
        (group-collection '((1 a) (2 b) (3 c) (1 c) (3 d) (2 q))
                          :key car))
@@ -358,16 +369,16 @@
 (test* "ref (custom)" "3" (ref (sseq 1 2 3 4 5) 2))
 
 (test* "(setter ref) (list)" '(1 2 a 4 5)
-       (let ((x (list 1 2 3 4 5)))
+       (let1 x (list 1 2 3 4 5)
          (set! (ref x 2) 'a) x))
 (test* "(setter ref) (vector)" '#(1 2 a 4 5)
-       (let ((x (vector 1 2 3 4 5)))
+       (let1 x (vector 1 2 3 4 5)
          (set! (ref x 2) 'a) x))
 (test* "(setter ref) (string)" "12a45"
-       (let ((x (string-copy "12345")))
+       (let1 x (string-copy "12345")
          (set! (ref x 2) #\a) x))
 (test* "(setter ref) (custom)" '("1" "2" "a" "4" "5")
-       (let ((x (sseq 1 2 3 4 5)))
+       (let1 x (sseq 1 2 3 4 5)
          (set! (ref x 2) 'a) (slot-ref x 'strings)))
 
 (test* "subseq (list)" '(3 4 5)
@@ -396,41 +407,33 @@
        (slot-ref (subseq (sseq 1 2 3 4 5) 0 -1) 'strings))
 
 (test* "for-each-with-index (list)" '((2 . c) (1 . b) (0 . a))
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) '(a b c))
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) '(a b c))))
 (test* "for-each-with-index (vector)" '((2 . c) (1 . b) (0 . a))
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) '#(a b c))
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) '#(a b c))))
 (test* "for-each-with-index (string)" '((2 . #\c) (1 . #\b) (0 . #\a))
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) "abc")
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) "abc")))
 (test* "for-each-with-index (custom)" '((2 . "c") (1 . "b") (0 . "a"))
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e)))
-                              (sseq 'a 'b 'c))
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e)))
+                              (sseq 'a 'b 'c))))
 (test* "for-each-with-index (custom)" '((2 "c" c c) (1 "b" b b) (0 "a" a a))
-       (let1 r '()
-         (for-each-with-index (lambda x (push! r x))
+       (rlet1 r '()
+         (for-each-with-index (^ x (push! r x))
                               (sseq 'a 'b 'c)
                               '(a b c)
-                              '#(a b c))
-         r))
+                              '#(a b c))))
 (test* "for-each-with-index (boundary)" '()
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) '())
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) '())))
 (test* "for-each-with-index (boundary)" '()
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) (sseq))
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) (sseq))))
 (test* "for-each-with-index (boundary)" '()
-       (let1 r '()
-         (for-each-with-index (lambda (i e) (push! r (cons i e))) (sseq) '())
-         r))
+       (rlet1 r '()
+         (for-each-with-index (^[i e] (push! r (cons i e))) (sseq) '())))
 
 (test* "map-with-index (list)" '((0 . a) (1 . b) (2 . c))
        (map-with-index cons '(a b c)))
@@ -479,7 +482,7 @@
 (test* "fold-with-index (custom)" '((2 . "a") (1 . "b") (0 . "c"))
        (fold-with-index acons '() (sseq 'c 'b 'a)))
 (test* "fold-with-index (custom)" '((2 "a" a a) (1 "b" b b) (0 "c" c c))
-       (fold-with-index (lambda (i x y z r)
+       (fold-with-index (^[i x y z r]
                           (cons (list i x y z) r))
                         '()
                         (sseq 'c 'b 'a) '(c b a) '#(c b a)))
@@ -491,27 +494,27 @@
        (fold-with-index acons '() (sseq) '() '#()))
 
 (test* "find-with-index (list)" '(2 c)
-       (receive r (find-with-index (cut eq? 'c <>) '(a b c d e)) r))
+       (values->list (find-with-index (cut eq? 'c <>) '(a b c d e))))
 (test* "find-with-index (list)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'f <>) '(a b c d e)) r))
+       (values->list (find-with-index (cut eq? 'f <>) '(a b c d e))))
 (test* "find-with-index (vector)" '(2 c)
-       (receive r (find-with-index (cut eq? 'c <>) '#(a b c d e)) r))
+       (values->list (find-with-index (cut eq? 'c <>) '#(a b c d e))))
 (test* "find-with-index (vector)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'f <>) '#(a b c d e)) r))
+       (values->list (find-with-index (cut eq? 'f <>) '#(a b c d e))))
 (test* "find-with-index (string)" '(2 #\c)
-       (receive r (find-with-index (cut eqv? #\c <>) "abcde") r))
+       (values->list (find-with-index (cut eqv? #\c <>) "abcde")))
 (test* "find-with-index (string)" '(#f #f)
-       (receive r (find-with-index (cut eqv? #\f <>) "abcde") r))
+       (values->list (find-with-index (cut eqv? #\f <>) "abcde")))
 (test* "find-with-index (custom)" '(2 "c")
-       (receive r (find-with-index (cut equal? "c" <>) (sseq 'a 'b 'c 'd 'e)) r))
+       (values->list (find-with-index (cut equal? "c" <>) (sseq 'a 'b 'c 'd 'e))))
 (test* "find-with-index (custom)" '(#f #f)
-       (receive r (find-with-index (cut equal? "f" <>) (sseq 'a 'b 'c 'd 'e)) r))
+       (values->list (find-with-index (cut equal? "f" <>) (sseq 'a 'b 'c 'd 'e))))
 (test* "find-with-index (boundary)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'c <>) '()) r))
+       (values->list (find-with-index (cut eq? 'c <>) '())))
 (test* "find-with-index (boundary)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'c <>) '#()) r))
+       (values->list (find-with-index (cut eq? 'c <>) '#())))
 (test* "find-with-index (boundary)" '(#f #f)
-       (receive r (find-with-index (cut eq? 'c <>) (sseq)) r))
+       (values->list (find-with-index (cut eq? 'c <>) (sseq))))
 
 (test* "find-index (list)" 2
        (find-index (cut eq? 'c <>) '(a b c d e)))
@@ -522,6 +525,26 @@
 (test* "find-index (custom)" 2
        (find-index (cut equal? "c" <>) (sseq 'a 'b 'c 'd 'e)))
 
+(test* "fold-right (list)" '(a b c d e)
+       (fold-right cons '() '(a b c d e)))
+(test* "fold-right (vector)" '(a b c d e)
+       (fold-right cons '() '#(a b c d e)))
+(test* "fold-right (string)" '(#\a #\b #\c #\d #\e)
+       (fold-right cons '() "abcde"))
+(test* "fold-right (custom)" '("a" "b" "c" "d" "e")
+       (fold-right cons '() (sseq 'a 'b 'c 'd 'e)))
+(test* "fold-right (list+list)" '(a 0 b 1 c 2 d 3 e 4)
+       (fold-right cons* '() '(a b c d e) '(0 1 2 3 4)))
+(test* "fold-right (list+vector)" '(a 0 b 1 c 2 d 3 e 4)
+       (fold-right cons* '() '(a b c d e) '#(0 1 2 3 4)))
+(test* "fold-right (string+vector)" '(#\a 0 #\b 1 #\c 2 #\d 3 #\e 4)
+       (fold-right cons* '() "abcde" '#(0 1 2 3 4)))
+(test* "fold-right (different lengths)" '(#\a 0 x #\b 1 y #\c 2 z)
+       (fold-right cons* '() "abcd" '#(0 1 2 3 4) '(x y z)))
+(test* "fold-right (different lengths)" '(#\a 0 u #\b 1 v #\c 2 w #\d 3 x)
+       (fold-right cons* '() "abcdef" '#(0 1 2 3) '(u v w x y z)))
+(test* "fold-right (different lengths)" '(#\a 0 u #\b 1 v)
+       (fold-right cons* '() "ab" '#(0 1 2 3 4) '(u v w x y z)))
 
 (test* "group-sequence" '((1 1 1) (2) (3) (4 4) (2 2) (3) (1 1) (3))
        (group-sequence '(1 1 1 2 3 4 4 2 2 3 1 1 3)))
@@ -531,7 +554,54 @@
        (group-sequence '(1 1 1 2 3 4 4 2 2 3 1 1 3) :key (cut < <> 3)))
 (test* "group-sequence w/test" '((1 1 1) (2) (3) (4 4 2 2) (3 1 1 3))
        (group-sequence '(1 1 1 2 3 4 4 2 2 3 1 1 3)
-                       :test (lambda (x y)
-                               (= (modulo x 2) (modulo y 2)))))
+                       :test (^[x y] (= (modulo x 2) (modulo y 2)))))
+
+(define (permute-tester msg expected source order . fallback)
+  (define (unit type elt-coercer order-type)
+    (test* #`"permute,msg ,type by ,order-type"
+           (map-to type elt-coercer expected)
+           (apply permute
+                  (map-to type elt-coercer source)
+                  (coerce-to order-type order)
+                  (map elt-coercer fallback)))
+    (test* #`"permute!,msg ,type by ,order-type"
+           (if (= (size-of source) (size-of expected))
+             (map-to type elt-coercer expected)
+             *test-error*)
+           (let1 imp (map-to type elt-coercer source)
+             (apply permute!
+                    imp
+                    (coerce-to order-type order)
+                    (map elt-coercer fallback))
+             imp)))
+  (dolist (order-type `(,<list> ,<vector>))
+    (dolist (t `((,<list> ,values)
+                 (,<vector> ,values)
+                 (,<string> ,(^[elt] (string-ref (x->string elt) 0)))))
+      (unit (car t) (cadr t) order-type)))
+  )
+
+(permute-tester "" '(d a c b) '(a b c d) '(3 0 2 1))
+(permute-tester " (short)" '(d a) '(a b c d) '(3 0))
+(permute-tester " (long)"  '(d a z c b) '(a b c d) '(3 0 4 2 1) 'z)
+
+(define (shuffle-tester source)
+  (define (cmp a b)
+    (lset= eqv? (coerce-to <list> a) (coerce-to <list> b)))
+  (define (unit type)
+    (test* #`"shuffle ,type"
+           (coerce-to type source)
+           (shuffle (coerce-to type source))
+           cmp)
+    (test* #`"shuffle! ,type"
+           (coerce-to type source)
+           (let1 imp (coerce-to type source)
+             (shuffle! imp)
+             imp)
+           cmp))
+  (unit <list>) (unit <vector>) (unit <string>))
+
+(shuffle-tester '(#\a #\b #\c #\d #\e #\f #\g))
+(shuffle-tester '())
 
 (test-end)
