@@ -1,23 +1,23 @@
 ;;;
 ;;; mt-random - Mersenne Twister interface
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module math.mt-random
   (use gauche.uvector)
@@ -49,21 +49,21 @@
 (dynamic-load "math--mt-random")
 
 (define (%get-nword-random-int mt n)
-  (let loop ((i 0) (r (%mt-random-uint32 mt)))
+  (let loop ([i 0] [r (%mt-random-uint32 mt)])
     (if (= i n)
-        r
-        (loop (+ i 1)
-              (+ (ash r 32) (%mt-random-uint32 mt))))))
+      r
+      (loop (+ i 1)
+            (+ (ash r 32) (%mt-random-uint32 mt))))))
 
 (define (mt-random-integer mt n)
   (when (not (positive? n)) (error "invalid range" n))
   (if (<= n #x100000000)
-      (%mt-random-integer mt n)
-      (let* ((siz (ash (integer-length n) -5))
-             (q   (quotient (ash 1 (* 32 (+ siz 1))) n))
-             (qn  (* q n)))
-        (let loop ((r (%get-nword-random-int mt siz)))
-          (if (< r qn)
-              (quotient r q)
-              (loop (%get-nword-random-int mt siz)))))))
+    (%mt-random-integer mt n)
+    (let* ([siz (ash (integer-length n) -5)]
+           [q   (quotient (ash 1 (* 32 (+ siz 1))) n)]
+           [qn  (* q n)])
+      (let loop ([r (%get-nword-random-int mt siz)])
+        (if (< r qn)
+          (quotient r q)
+          (loop (%get-nword-random-int mt siz)))))))
 
