@@ -6,7 +6,6 @@
 
 (use gauche.test)
 (use gauche.uvector)
-(use gauche.experimental.app)
 (use srfi-1)
 (use srfi-13)
 (test-start "net")
@@ -19,14 +18,14 @@
 ;;-----------------------------------------------------------------
 (test-section "socket address")
 
-'(test* "sockaddr_un" #t
-        (let ((addr (make <sockaddr-un> :path "/tmp/xxx")))
+(test* "sockaddr_un" #t
+        (let1 addr (make <sockaddr-un> :path "/tmp/xxx")
           (and (eq? (sockaddr-family addr) 'unix)
                (equal? (sockaddr-name addr) "/tmp/xxx")
                #t)))
 
-'(test* "sockaddr_in" #t
-        (let ((addr (make <sockaddr-in> :host "127.0.0.1" :port 80)))
+(test* "sockaddr_in" #t
+        (let1 addr (make <sockaddr-in> :host "127.0.0.1" :port 80)
           (and (eq? (sockaddr-family addr) 'inet)
                (equal? (sockaddr-name addr) "127.0.0.1:80")
                (= (sockaddr-addr addr) #x7f000001)
@@ -34,39 +33,57 @@
                #t)))
 
 (test* "sockaddr_in" #t
-       (let ((addr (make <sockaddr-in> :host "localhost" :port 23)))
+       (let1 addr (make <sockaddr-in> :host "localhost" :port 23)
          (and (eq? (sockaddr-family addr) 'inet)
               (equal? (sockaddr-name addr) "127.0.0.1:23")
               #t)))
 
 (test* "sockaddr_in" #t
-       (let ((addr (make <sockaddr-in> :host #x7f000001 :port 23)))
+       (let1 addr (make <sockaddr-in> :host #x7f000001 :port 23)
          (and (eq? (sockaddr-family addr) 'inet)
               (equal? (sockaddr-name addr) "127.0.0.1:23")
               #t)))
 
 (test* "sockaddr_in" #t
-       (let ((addr (make <sockaddr-in> :host '#u8(127 0 0 1) :port 23)))
+       (let1 addr (make <sockaddr-in> :host '#u8(127 0 0 1) :port 23)
          (and (eq? (sockaddr-family addr) 'inet)
               (equal? (sockaddr-name addr) "127.0.0.1:23")
               #t)))
 
 (test* "sockaddr_in" #t
-       (let ((addr (make <sockaddr-in> :host :any :port 7777)))
+       (let1 addr (make <sockaddr-in> :host :any :port 7777)
          (and (eq? (sockaddr-family addr) 'inet)
               (equal? (sockaddr-name addr) "0.0.0.0:7777")
               #t)))
 
 (test* "sockaddr_in" #t
-       (let ((addr (make <sockaddr-in> :host :broadcast)))
+       (let1 addr (make <sockaddr-in> :host :broadcast)
          (and (eq? (sockaddr-family addr) 'inet)
               (equal? (sockaddr-name addr) "255.255.255.255:0")
               #t)))
 
+(let ([un1 (make <sockaddr-un> :path "/tmp/a")]
+      [un2 (make <sockaddr-un> :path "/tmp/aa")]
+      [un3 (make <sockaddr-un> :path "/tmp/a")]
+      [in1 (make <sockaddr-in> :host "127.0.0.1" :port 2030)]
+      [in2 (make <sockaddr-in> :host "127.0.0.2" :port 2030)]
+      [in3 (make <sockaddr-in> :host "127.0.0.1" :port 2031)]
+      [in4 (make <sockaddr-in> :host "127.0.0.1" :port 2030)])
+  (let-syntax ([t (syntax-rules ()
+                    [(_ exp a b)
+                     (test* (format "sockaddr equal? ~s" '(equal? a b))
+                            exp (equal? a b))])])
+    (t #f un1 un2)
+    (t #t un1 un3)
+    (t #f in1 in2)
+    (t #f in1 in3)
+    (t #t in1 in4)
+    (t #f un1 in1)))
+
 (cond-expand
- (gauche.net.ipv6
+ [gauche.net.ipv6
   (test* "sockaddr_in6" #t
-         (let ((addr (make <sockaddr-in6> :host "2001:200::8002:203:47ff:fea5:3085" :port 23)))
+         (let1 addr (make <sockaddr-in6> :host "2001:200::8002:203:47ff:fea5:3085" :port 23)
            (and (eq? (sockaddr-family addr) 'inet6)
                 (#/\[2001:200:0?:8002:203:47ff:fea5:3085\]:23/ (sockaddr-name addr))
                 (= (sockaddr-addr addr) #x2001020000008002020347fffea53085)
@@ -76,23 +93,34 @@
          (sockaddr-name (make <sockaddr-in6> :host 1)))
   (test* "sockaddr_in6" "[1:2:3:4:5:6:7:8]:0"
          (sockaddr-name (make <sockaddr-in6> :host '#u8(0 1 0 2 0 3 0 4 0 5 0 6 0 7 0 8))))
-  )
- (else #f))
+
+  (let ([4n1 (make <sockaddr-in6> :host 1 :port 80)]
+        [4n2 (make <sockaddr-in6> :host 1 :port 81)]
+        [4n3 (make <sockaddr-in6> :host "1:2::8" :port 80)]
+        [4n4 (make <sockaddr-in6> :host 1 :port 80)])
+    (let-syntax ([t (syntax-rules ()
+                      [(_ exp a b)
+                       (test* (format "sockaddr equal? ~s" '(equal? a b))
+                              exp (equal? a b))])])
+      (t #f 4n1 4n2)
+      (t #f 4n1 4n3)
+      (t #t 4n1 4n4)))]
+ [else])
 
 (let ()
   (define (addr-test desc input exp-val exp-vers)
-    (let ((uvresult (make-u8vector 16 0)))
+    (let1 uvresult (make-u8vector 16 0)
       (and exp-vers
-           (let loop ((val exp-val)
-                      (dig (if (= exp-vers AF_INET) 3 15)))
+           (let loop ([val exp-val]
+                      [dig (if (= exp-vers AF_INET) 3 15)])
              (when (>= dig 0)
                (u8vector-set! uvresult dig (logand val #xff))
                (loop (ash val -8) (- dig 1)))))
       (test* #`"inet-string->address  (,desc)" `(,exp-val ,exp-vers)
-             (receive r (inet-string->address input) r))
+             (values->list (inet-string->address input)))
       (test* #`"inet-string->address! (,desc)" `(,uvresult ,exp-vers)
-             (let* ((buf (make-u8vector 16 0))
-                    (ver (inet-string->address! input buf)))
+             (let* ([buf (make-u8vector 16 0)]
+                    [ver (inet-string->address! input buf)])
                (list buf ver)))))
 
   (addr-test "v4-1" "127.0.0.1" #x7f000001 AF_INET)
@@ -103,7 +131,7 @@
   (addr-test "v4-err3" "172.256.4.2" #f #f)
 
   (cond-expand
-   (gauche.net.ipv6
+   [gauche.net.ipv6
     (addr-test "v6-1" "1:2:3:4:fffc:fffd:fffe:ffff"
                #x0001000200030004fffcfffdfffeffff AF_INET6)
     (addr-test "v6-2" "::1"
@@ -112,31 +140,34 @@
                #x00000000000000000000000000010002 AF_INET6)
     (addr-test "v6-4" "::1:2:3:4:5:6:7"
                #x00000001000200030004000500060007 AF_INET6)
-    (addr-test "v6-err1" "::1:2:3:4:5:6:7:8" #f #f)
-    (addr-test "v6-err2" ":1:2:3:4:5:6:7:8" #f #f)
     (addr-test "v6-5" "ffe0::"
                #xffe00000000000000000000000000000 AF_INET6)
     (addr-test "v6-6" "ffe0:1::"
                #xffe00001000000000000000000000000 AF_INET6)
     (addr-test "v6-7" "ffe0::234:567"
                #xffe00000000000000000000002340567 AF_INET6)
-    (addr-test "v6-err3" "ffe0::234::567" #f #f)
-    (addr-test "v6-err4" "ffe0::234:567:" #f #f)
     (addr-test "v6-8" "::192.168.1.2"
                #x000000000000000000000000c0a80102 AF_INET6)
     (addr-test "v6-9" "1::2:0:0:192.168.1.2"
                #x000100000000000200000000c0a80102 AF_INET6)
-    )
-   (else #f))
+    (addr-test "v6-err1" ":1:2:3:4:5:6:7:8" #f #f)
+    (addr-test "v6-err2" "ffe0::234::567" #f #f)
+    ;; These two should be an error, but cygwin's inet_pton allows them.
+    (cond-expand
+     [(not gauche.os.cygwin)
+      (addr-test "v6-err3" "::1:2:3:4:5:6:7:8" #f #f)
+      (addr-test "v6-err4" "ffe0::234:567:" #f #f)]
+     [else])]
+   [else])
   )
 
 (let ()
   (define (addr-test desc input vers expected)
-    (let* ((size (cond ((= vers AF_INET) 4) ((= vers AF_INET6) 16)))
-           (uv (make-u8vector size 0)))
-      (do ((k (- size 1) (- k 1))
-           (val input (ash val -8)))
-          ((< k 0))
+    (let* ([size (cond [(= vers AF_INET) 4] [(= vers AF_INET6) 16])]
+           [uv (make-u8vector size 0)])
+      (do ([k (- size 1) (- k 1)]
+           [val input (ash val -8)])
+          [(< k 0)]
         (u8vector-set! uv k (logand val #xff)))
       (test* #`"inet-address->string (,desc,, int)" expected
              (inet-address->string input vers))
@@ -148,7 +179,7 @@
   (addr-test "v4-3" #xfffffc00 AF_INET "255.255.252.0")
 
   (cond-expand
-   (gauche.net.ipv6
+   [gauche.net.ipv6
     (addr-test "v6-1" #x0001000200030004fffcfffdfffeffff AF_INET6
                "1:2:3:4:fffc:fffd:fffe:ffff")
     (addr-test "v6-2" #x00000000000000000000000000000001 AF_INET6
@@ -171,8 +202,8 @@
                "ffe0:0:0:223::4")
     (addr-test "v6-11" #xffe00000000000000001000000000004 AF_INET6
                "ffe0::1:0:0:4")
-    )
-   (else #f))
+    ]
+   [else])
   )
 
 
@@ -180,7 +211,7 @@
 (test-section "netdb")
 
 (test* "gethostbyname" #t
-       (let ((host (sys-gethostbyname "localhost")))
+       (let1 host (sys-gethostbyname "localhost")
          (and host
               (or (equal? (slot-ref host 'name) "localhost")
                   (member "localhost" (slot-ref host 'aliases))
@@ -193,7 +224,7 @@
               #t)))
 
 (test* "gethostbyaddr" #t
-       (let ((host (sys-gethostbyaddr "127.0.0.1" AF_INET)))
+       (let1 host (sys-gethostbyaddr "127.0.0.1" AF_INET)
          (and host
               (or (equal? (slot-ref host 'name) "localhost")
                   (member "localhost" (slot-ref host 'aliases))
@@ -206,36 +237,30 @@
               #t)))
 
 (test* "getprotobyname" '(("icmp" 1) ("tcp" 6) ("udp" 17))
-       (let ((tcp (sys-getprotobyname "tcp"))
-             (udp (sys-getprotobyname "udp"))
-             (icmp (sys-getprotobyname "icmp")))
-         (map (lambda (proto)
-                (list (slot-ref proto 'name)
-                      (slot-ref proto 'proto)))
+       (let ([tcp (sys-getprotobyname "tcp")]
+             [udp (sys-getprotobyname "udp")]
+             [icmp (sys-getprotobyname "icmp")])
+         (map (^[proto] (list (slot-ref proto 'name)
+                              (slot-ref proto 'proto)))
               (list icmp tcp udp))))
 
 (test* "getprotobynumber" '(#t #t #t)
-       (map (lambda (proto name)
-              (or (member name (slot-ref proto 'aliases))
-                  (equal? name (slot-ref proto 'name))))
+       (map (^[proto name] (or (member name (slot-ref proto 'aliases))
+                               (equal? name (slot-ref proto 'name))))
             (map sys-getprotobynumber '(1 6 17))
             '("icmp" "tcp" "udp")))
 
 (test* "getservbyname" '(("telnet" 23 "tcp") ("ftp" 21 "tcp"))
-       (map (lambda (name proto)
-              (let ((x (sys-getservbyname name proto)))
-                (and x
-                     (map (lambda (s) (slot-ref x s))
-                          '(name port proto)))))
+       (map (^[name proto]
+              (let1 x (sys-getservbyname name proto)
+                (and x (map (^s (slot-ref x s)) '(name port proto)))))
             '("telnet" "ftp")
             '("tcp"    "tcp")))
 
 (test* "getservbyport" '(("telnet" 23 "tcp") ("ftp" 21 "tcp"))
-       (map (lambda (port proto)
-              (let ((x (sys-getservbyport port proto)))
-                (and x
-                     (map (lambda (s) (slot-ref x s))
-                          '(name port proto)))))
+       (map (^[port proto]
+              (let1 x (sys-getservbyport port proto)
+                (and x (map (^s (slot-ref x s)) '(name port proto)))))
             '(23       21)
             '("tcp"    "tcp")))
 
@@ -259,29 +284,36 @@
 
 (use gauche.process)
 
+;; sockargs is an expression that yields to a list of server sockets
 (define (run-simple-server sockargs)
   (with-output-to-file "testserv.o"
-    (lambda ()
+    (^[]
       (write '(use gauche.net))
+      (write '(use gauche.selector))
       (write '(use srfi-13))
-      (write '(define (simple-server socket)
+      (write '(define (simple-server sockets)
+                (define (handler fd flag)
+                  (let loop ([clnt (socket-accept
+                                    (find (^s (eq? (socket-fd s) fd)) sockets))])
+                    (let ([in  (socket-input-port clnt)]
+                          [out (socket-output-port clnt)])
+                      (let loop2 ([line (read-line in)])
+                        (cond [(eof-object? line)
+                               (socket-close clnt)]
+                              [(string=? line "END")
+                               (socket-close clnt)
+                               (for-each socket-close sockets)
+                               (sys-exit 33)]
+                              [else
+                               (display (string-upcase line) out)
+                               (newline out)
+                               (flush out)
+                               (loop2 (read-line in))])))))
                 (newline) (flush) ;; handshake
-                (let loop ((clnt (socket-accept socket)))
-                  (let ((in   (socket-input-port clnt))
-                        (out  (socket-output-port clnt)))
-                    (let loop2 ((line (read-line in)))
-                      (cond ((eof-object? line)
-                             (socket-close clnt)
-                             (loop (socket-accept socket)))
-                            ((string=? line "END")
-                             (socket-close clnt)
-                             (socket-close socket)
-                             (sys-exit 33))
-                            (else
-                             (display (string-upcase line) out)
-                             (newline out)
-                             (flush out)
-                             (loop2 (read-line in)))))))))
+                (let1 sel (make <selector>)
+                  (dolist [s sockets]
+                    (selector-add! sel (socket-fd s) handler '(r)))
+                  (do () [#f] (selector-select sel)))))
       (write `(define (main args)
                 (simple-server ,sockargs)
                 0)))
@@ -307,21 +339,21 @@
  [else
   (test* "unix server socket" #f
 	 (begin
-	   (run-simple-server '(make-server-socket 'unix "sock.o"))
+	   (run-simple-server '(list (make-server-socket 'unix "sock.o")))
 	   (let1 stat (sys-stat "sock.o")
-		 (not (memq (sys-stat->file-type stat) '(socket fifo))))))
+             (not (memq (sys-stat->file-type stat) '(socket fifo))))))
 
   (test* "unix client socket" '("ABC" "XYZ")
          (call-with-client-socket (make-client-socket 'unix "sock.o")
-           (lambda (in out)
+           (^[in out]
              (display "abc\n" out) (flush out)
-             (let ((abc (read-line in)))
+             (let1 abc (read-line in)
                (display "xyz\n" out) (flush out)
                (list abc (read-line in))))))
 
   (test* "unix client socket" #t
          (call-with-client-socket (make-client-socket 'unix "sock.o")
-           (lambda (in out)
+           (^[in out]
              (display (make-string *chunk-size* #\a) out)
              (newline out)
              (flush out)
@@ -330,7 +362,7 @@
   (test* "unix client socket" #t
          (call-with-client-socket
           (make-client-socket (make <sockaddr-un> :path "sock.o"))
-          (lambda (in out)
+          (^[in out]
             (display (make-string *chunk-size* #\a) out)
             (newline out)
             (flush out)
@@ -338,28 +370,28 @@
 
   (test* "unix client socket" 33
          (call-with-client-socket (make-client-socket 'unix "sock.o")
-           (lambda (in out)
+           (^[in out]
              (display "END\n" out) (flush out)
              (receive (pid code) (sys-wait)
-                      (sys-wait-exit-status code)))))
+               (sys-wait-exit-status code)))))
   ])
 
 (sys-unlink "sock.o")
 
 (test* "inet server socket" #t
-       (run-simple-server `(make-server-socket 'inet ,*inet-port* :reuse-addr? #t)))
+       (run-simple-server `(make-server-sockets #f ,*inet-port* :reuse-addr? #t)))
 
 (test* "inet client socket" '("ABC" "XYZ")
        (call-with-client-socket (make-client-socket 'inet "localhost" *inet-port*)
-         (lambda (in out)
+         (^[in out]
            (display "abc\n" out) (flush out)
-           (let ((abc (read-line in)))
+           (let1 abc (read-line in)
              (display "xyz\n" out) (flush out)
              (list abc (read-line in))))))
 
 (test* "inet client socket (host,port)" #t
        (call-with-client-socket (make-client-socket "localhost" *inet-port*)
-         (lambda (in out)
+         (^[in out]
            (display (make-string *chunk-size* #\a) out)
            (newline out)
            (flush out)
@@ -369,7 +401,7 @@
        (call-with-client-socket (make-client-socket
                                  (make <sockaddr-in>
                                    :host "localhost" :port *inet-port*))
-         (lambda (in out)
+         (^[in out]
            (display (make-string *chunk-size* #\a) out)
            (newline out)
            (flush out)
@@ -377,7 +409,7 @@
 
 (test* "inet client socket (termination)" 33
        (call-with-client-socket (make-client-socket 'inet "localhost" *inet-port*)
-         (lambda (in out)
+         (^[in out]
            (display "END\n" out) (flush out)
            (receive (pid code) (sys-wait)
              (sys-wait-exit-status code)))))
@@ -386,33 +418,33 @@
 (cond-expand
  [gauche.net.ipv6
   (test* "inet server socket (ipv6)" #t
-         (run-simple-server `(make-server-socket
-                              (make <sockaddr-in6>
-                                :host :any :port ,*inet-port*)
-                              :reuse-addr? #t)))
+         (run-simple-server `(list (make-server-socket
+                                    (make <sockaddr-in6>
+                                      :host :any :port ,*inet-port*)
+                                    :reuse-addr? #t))))
 
   ;; On IPv6 system, the loopback may have different name than "localhost".
   ;; We apply some heuristics here.
   (define (get-ipv6-sock)
-    (any (lambda (name)
-           (guard (e (else #f))
+    (any (^[name]
+           (guard (e [else #f])
              (make-client-socket
               (make <sockaddr-in6> :host name :port *inet-port*))))
          '("localhost" "ip6-localhost" "ipv6-localhost" "::1")))
-  
+
   (test* "inet client socket (ipv6)" #t
-         (and-let* ((sock (get-ipv6-sock)))
+         (and-let* ([sock (get-ipv6-sock)])
            (call-with-client-socket sock
-             (lambda (in out)
+             (^[in out]
                (display (make-string *chunk-size* #\a) out)
                (newline out)
                (flush out)
                (string=? (read-line in) (make-string *chunk-size* #\A))))))
 
   (test* "inet client socket (ipv6, termination)" 33
-         (and-let* ((sock (get-ipv6-sock)))
+         (and-let* ([sock (get-ipv6-sock)])
            (call-with-client-socket sock
-             (lambda (in out)
+             (^[in out]
                (display "END\n" out) (flush out)
                (receive (pid code) (sys-wait)
                  (sys-wait-exit-status code))))))
@@ -430,10 +462,10 @@
          (socket-output-port s)))
 
 (test* "getsockname/getpeername" #t
-       (let* ((addr (make <sockaddr-in> :host :loopback :port *inet-port*))
-              (serv (make-server-socket addr :reuse-addr? #t))
-              (clnt (make-client-socket addr)))
-         (begin0 (every (lambda (addr)
+       (let* ([addr (make <sockaddr-in> :host :loopback :port *inet-port*)]
+              [serv (make-server-socket addr :reuse-addr? #t)]
+              [clnt (make-client-socket addr)])
+         (begin0 (every (^[addr]
                           (and (= (sockaddr-addr addr) #x7f000001)
                                (= (sockaddr-port addr) *inet-port*)))
                         (list (socket-getsockname serv)
@@ -444,13 +476,13 @@
 (test* "udp server socket" #t
        (begin
          (with-output-to-file "testserv.o"
-           (lambda ()
+           (^[]
              (write '(use gauche.net))
              (write '(use srfi-13))
              (write
               `(define (main args)
-                 (let ((sock (make-socket PF_INET SOCK_DGRAM))
-                       (addr (make <sockaddr-in> :host :any :port ,*inet-port*)))
+                 (let ([sock (make-socket PF_INET SOCK_DGRAM)]
+                       [addr (make <sockaddr-in> :host :any :port ,*inet-port*)])
                    (socket-setsockopt sock SOL_SOCKET SO_REUSEADDR 1)
                    (socket-bind sock addr)
                    (newline) (flush) ;; handshake
@@ -464,8 +496,8 @@
            #t)))
 
 (test* "udp client socket" "ABC"
-       (let ((sock (make-socket PF_INET SOCK_DGRAM))
-             (addr (make <sockaddr-in> :host :loopback :port *inet-port*)))
+       (let ([sock (make-socket PF_INET SOCK_DGRAM)]
+             [addr (make <sockaddr-in> :host :loopback :port *inet-port*)])
          ;; NB: Cygwin weirdness... Somehow the first udp packet
          ;; is always dropped on CYGWIN_NT-5.1 1.5.25(0.156/4/2) on XP SP3,
          ;; so we send it twice.  Curiously such symptom does not appear
@@ -483,10 +515,10 @@
            (sys-wait))))
 
 (define (with-sr-udp proc)
-  (let ((s-sock (make-socket PF_INET SOCK_DGRAM))
-        (r-sock (make-socket PF_INET SOCK_DGRAM))
-        (s-addr (make <sockaddr-in> :host :loopback :port *inet-port*))
-        (r-addr (make <sockaddr-in> :host :any :port *inet-port*)))
+  (let ([s-sock (make-socket PF_INET SOCK_DGRAM)]
+        [r-sock (make-socket PF_INET SOCK_DGRAM)]
+        [s-addr (make <sockaddr-in> :host :loopback :port *inet-port*)]
+        [r-addr (make <sockaddr-in> :host :any :port *inet-port*)])
     (socket-setsockopt r-sock SOL_SOCKET SO_REUSEADDR 1)
     (socket-bind r-sock r-addr)
     (unwind-protect
@@ -495,7 +527,7 @@
              (socket-close s-sock)))))
 
 (with-sr-udp
- (lambda (s-sock s-addr r-sock r-addr)
+ (^[s-sock s-addr r-sock r-addr]
    (let ([from (make <sockaddr-in>)]
          [data (make-u8vector 1024 #x3c)]
          [buf  (make-u8vector 1024 #xa5)])
@@ -512,7 +544,7 @@
  [(and (not gauche.os.cygwin)
        (not gauche.os.windows))
   (with-sr-udp
-   (lambda (s-sock s-addr r-sock r-addr)
+   (^[s-sock s-addr r-sock r-addr]
      (let ([from   (make <sockaddr-in>)]
 	   [data   `#(,(make-string 255 #\a) ,(make-u8vector 255 48))]
 	   [sbuf   (make-u8vector 1024)]
@@ -521,16 +553,15 @@
        (define (xtest sbuf)
 	 (socket-sendmsg s-sock (socket-buildmsg r-addr data '() 0 sbuf))
 	 (receive (size f-addr) (socket-recvfrom! r-sock rbuf (list from))
-		  (list (eq? f-addr from)
-			(equal? (uvector-alias <u8vector> rbuf 0 size)
-				($ string->u8vector
-				   $ string-concatenate
-				   $ map (lambda (z)
-					   (if (string? z)
-					       z
-					       (u8vector->string z)))
-				   $ vector->list data)))))
-       
+           (list (eq? f-addr from)
+                 (equal? (uvector-alias <u8vector> rbuf 0 size)
+                         ($ string->u8vector
+                            $ string-concatenate
+                            $ map (^z (if (string? z)
+                                        z
+                                        (u8vector->string z)))
+                            $ vector->list data)))))
+
        (test* "udp sendmsg w/sendbuf" '(#t #t) (xtest sbuf))
        (test* "udp sendmsg w/o sendbuf" '(#t #t) (xtest #f)))))]
  [else #f])
