@@ -33,6 +33,7 @@
 
 (use gauche.test)
 (use srfi-1)
+(use gauche.generator)
 
 (test-start "parser.peg")
 
@@ -44,34 +45,33 @@
 ;;; helper macros
 ;;;
 (define (%compare test-type accessors)
-  (lambda (e o)
-    (and (test-type o)
-         (every equal? e (map (cut <> o) accessors)))))
+  (^[e o] (and (test-type o)
+               (every equal? e (map (cut <> o) accessors)))))
 
 (define-syntax test-succ
   (syntax-rules ()
-    ((_ label expect parse input)
+    [(_ label expect parse input)
      (test* #`",label (success)" expect
-            (peg-parse-string parse input)))))
+            (peg-parse-string parse input))]))
 
 (define-syntax test-fail
   (syntax-rules ()
-    ((_ label expect parse input)
+    [(_ label expect parse input)
      (test* #`",label (failure)" expect
-            (guard (e ((<parse-error> e)
-                       (list (ref e 'position) (ref e 'objects)))
-                      (else e))
+            (guard (e [(<parse-error> e)
+                       (list (ref e 'position) (ref e 'objects))]
+                      [else e])
               (peg-parse-string parse input)
-              (error "test-fail failed"))))))
+              (error "test-fail failed")))]))
 
 ;;;============================================================
 ;;; Ropes
 ;;;
 (receive (make-rope rope->string)
     (with-module parser.peg (values make-rope rope->string))
-  (let* ((rope1 (make-rope "abc"))
-         (rope2 (make-rope `("012" ,rope1 #\X)))
-         (rope3 (make-rope 'foo)))
+  (let* ([rope1 (make-rope "abc")]
+         [rope2 (make-rope `("012" ,rope1 #\X))]
+         [rope3 (make-rope 'foo)])
     (test* "rope->string" "abc"        (rope->string rope1))
     (test* "rope->string" "012abcX"    (rope->string rope2))
     (test* "rope->string" (test-error) (rope->string rope3))))
@@ -150,7 +150,7 @@
            ($satisfy char-numeric? "numeric")
            "a")
 
-(test-succ "eof" #t eof "")
+(test-succ "eof" (eof-object) eof "")
 (test-fail "eof" '(0 "end of input") eof "a")
 
 ;;;============================================================
@@ -328,7 +328,7 @@
            "a,b,")
 
 ;; $sep-end-by
-(let ((p ($sep-end-by ($seq ($one-of #[a-z]) ($one-of #[a-z])) ($string ","))))
+(let1 p ($sep-end-by ($seq ($one-of #[a-z]) ($one-of #[a-z])) ($string ","))
   (define (succ in exp) (test-succ "$sep-end-by" exp p in))
   (define (fail in exp) (test-fail "$sep-end-by" exp p in))
 
@@ -414,12 +414,12 @@
            "cbad")
 
 ;; $chain-left
-(let ((integer
-       ($do (v ($many digit 1))
-            ($return (string->number (apply string v)))))
-      (op
-       ($or ($do (($char #\*)) ($return *))
-            ($do (($char #\+)) ($return +)))))
+(let ([integer
+       ($do [v ($many digit 1)]
+            ($return (string->number (apply string v))))]
+      [op
+       ($or ($do [($char #\*)] ($return *))
+            ($do [($char #\+)] ($return +)))])
   (test-succ "$chain-left" 9
              ($chain-left integer op)
              "1+2*3")
@@ -431,12 +431,12 @@
              "abc"))
 
 ;; $chain-right
-(let ((integer
-       ($do (v ($many digit 1))
-            ($return (string->number (apply string v)))))
-      (op
-       ($or ($do (($char #\*)) ($return *))
-            ($do (($char #\+)) ($return +)))))
+(let ([integer
+       ($do [v ($many digit 1)]
+            ($return (string->number (apply string v))))]
+      [op
+       ($or ($do [($char #\*)] ($return *))
+            ($do [($char #\+)] ($return +)))])
   (test-succ "$chain-right" 7
              ($chain-right integer op)
              "1+2*3")
@@ -598,35 +598,35 @@
              records "\"a  \"\" \n\" , b  , c"))
 
 ;; Poor-man's XML parser
-(letrec ((open-tag ($->rope ($between ($char #\<) alphanum ($char #\>))))
-         (close-tag (lambda (tagname)
-                      ($seq ($string "</") ($string tagname) ($char #\>))))
-         (text ($->rope ($many ($none-of #[<]))))
-         (body ($do [t text]
+(letrec ([open-tag ($->rope ($between ($char #\<) alphanum ($char #\>)))]
+         [close-tag (^[tagname]
+                      ($seq ($string "</") ($string tagname) ($char #\>)))]
+         [text ($->rope ($many ($none-of #[<])))]
+         [body ($do [t text]
                     [r ($many ($do [e element]
                                    [t text]
                                    ($return (list e t))))]
-                    ($return `(,t ,@(apply append r)))))
-         (element ($do* [tagname ($try open-tag)]
-                        [body body]
-                        [ (close-tag (rope-finalize tagname)) ]
-                        ($return (cons tagname body)))))
+                    ($return `(,t ,@(apply append r))))]
+         [element ($do [tagname ($try open-tag)]
+                       [body body]
+                       [ (close-tag (rope-finalize tagname)) ]
+                       ($return (cons tagname body)))])
   (test-succ "tag element" '("a" "")
              element "<a></a>")
   (test-succ "tag element" '("a" "foo" ("b" "bar") "baz")
              element "<a>foo<b>bar</b>baz</a>"))
 
 ;; Calculator
-(letrec ((integer ($do (v ($many digit 1))
-                       ($return (string->number (apply string v)))))
-         (mulop ($or ($seq ($char #\*) ($return *))
-                     ($seq ($char #\/) ($return /))))
-         (addop ($or ($seq ($char #\+) ($return +))
-                     ($seq ($char #\-) ($return -))))
-         (factor ($lazy ($or ($between ($char #\() expr ($char #\)))
-                             integer)))
-         (term ($chain-left factor mulop))
-         (expr ($chain-left term addop)))
+(letrec ([integer ($do (v ($many digit 1))
+                       ($return (string->number (apply string v))))]
+         [mulop ($or ($seq ($char #\*) ($return *))
+                     ($seq ($char #\/) ($return /)))]
+         [addop ($or ($seq ($char #\+) ($return +))
+                     ($seq ($char #\-) ($return -)))]
+         [factor ($lazy ($or ($between ($char #\() expr ($char #\)))
+                             integer))]
+         [term ($chain-left factor mulop)]
+         [expr ($chain-left term addop)])
   (test-succ "calculator" 11 expr "1+2*3+4")
   (test-succ "calculator" 36 expr "2/2+5*(3+4)")
   (test-succ "calculator" -1 expr "1-2"))
@@ -731,6 +731,11 @@
    }
 ]"))
 
+(test* "Parsing sequence of json objects"
+       '((("a" . 1)("b" . 2)) (("c" . 3) ("d" . 4)))
+       (with-input-from-string "{\"a\":1, \"b\":2} {\"c\":3, \"d\":4} "
+         (^[] (generator->list (peg-parser->generator json-parser read-char)))))
+
 (let ()
   (define (test-writer name obj)
     (test* name obj
@@ -740,12 +745,14 @@
                '(("Image"
                   ("Width"  . 800)
                   ("Height" . 600)
-                  ("Title"  . "View from 15th Floor")
+                  ("Title"  . "View from 15th Floor \"magnificent\"")
                   ("Thumbnail"
                    ("Url"    . "http://www.example.com/image/481989943")
                    ("Height" . 125)
                    ("Width"  . "100"))
-                  ("IDs" . #(116 943 234 38793)))))
+                  ("Description" . "Foo\nbackslash \\and tab\t and \u00a1")
+                  ("IDs" . #(116 943 234 38793))
+                  ("Misc" . ()))))
 
   (test-writer "writing an array containing two objects"
                '#((("precision" . "zip")
@@ -768,7 +775,7 @@
 
 (let ()
   (define (t obj)
-    (test* #`"writer error ,obj" (test-error <json-construct-error>) 
+    (test* #`"writer error ,obj" (test-error <json-construct-error>)
            (construct-json-string obj)))
   (t "a")
   (t '#(1 2 x))
