@@ -1,12 +1,12 @@
 /*
  * spvec.c - Sparse vector
  *
- *   Copyright (c) 2009-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2009-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -162,12 +162,10 @@ ScmObj SparseVectorInc(SparseVector *sv, u_long index,
     }
 }
 
-#if SCM_DEBUG_HELPER
 void SparseVectorDump(SparseVector *sv)
 {
     CompactTrieDump(SCM_CUROUT, &sv->trie, sv->desc->dump, sv->desc);
 }
-#endif /*SCM_DEBUG_HELPER*/
 
 /*===================================================================
  * Individual vector types
@@ -236,7 +234,6 @@ static ScmObj g_iter(Leaf *leaf, int *index)
     return SCM_UNBOUND;
 }
 
-#if SCM_DEBUG_HELPER
 static void g_dump(ScmPort *out, Leaf *leaf, int indent, void *data)
 {
     int i;
@@ -247,9 +244,6 @@ static void g_dump(ScmPort *out, Leaf *leaf, int indent, void *data)
         }
     }
 }
-#else
-#define g_dump NULL
-#endif /*SCM_DEBUG_HELPER*/
 
 static SparseVectorDescriptor g_desc = {
     g_ref, g_set, g_allocate, g_delete, g_clear, g_copy, g_iter, g_dump, 1
@@ -360,7 +354,7 @@ U_REF(f64, MASK64, Scm_VMReturnFlonum)
         ULEAF(leaf)->elt[index&mask] = getter(val, SCM_CLAMP_ERROR, NULL); \
         U_SET_CHECK(mask);                                              \
     } while(0)
-    
+
 #define U_SET_CHECK(mask)                       \
     do {                                        \
         int z = U_HAS_ENTRY(leaf, index, mask); \
@@ -583,7 +577,7 @@ ScmObj MakeSparseVector(ScmClass *klass, u_long flags)
         Scm_TypeError("class", "subclass of <sparse-vector-base>",
                       SCM_OBJ(klass));
     }
-    
+
     SCM_SET_CLASS(v, klass);
     CompactTrieInit(&v->trie);
     v->numEntries = 0;
