@@ -3,7 +3,7 @@
 ;;
 
 ;; ssax test is derived from the original SSAX source.
-(load "./ssax-test.scm")
+(include "./ssax-test.scm")
 
 ;(load "./tree-trans-test.scm")
 ;(load "./to-html-test.scm")
@@ -43,6 +43,6 @@
     (t srl:sxml->html-noindent
        "<foo a=\"b\" c=\"d&quot;e&apos;f\"><g>h<i>j<k/></i></g></foo>")
     ))
-  
+
 (test-end)
 
