@@ -1,12 +1,12 @@
 /*
  * thread.c - Scheme thread API
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -75,50 +75,48 @@ ScmObj Scm_MakeThread(ScmProcedure *thunk, ScmObj name)
     return SCM_OBJ(vm);
 }
 
-/* Start a thread.  If the VM is in "NEW" state, create a new thread and
-   make it run.
-
-   With pthread, the real thread is started as "detached" mode; i.e. once
-   the thread exits, the resources allocated for the thread by the system
-   is collected, including the result of the thread.  During this
-   deconstruction phase, the handler thread_cleanup() runs and saves the
-   thread result to the ScmVM structure.  If nobody cares about the
-   result of the thread, ScmVM structure will eventually be GCed.
-   This is to prevent exitted thread's system resources from being
-   uncollected.
- */
-
-#ifdef GAUCHE_USE_PTHREADS
-static void thread_cleanup(void *data)
+/* Common routine for proper shutdown of a thread.  Usually this is called
+   by the thread owning VM, via pthread_cleanup mechanism.  However, in the
+   emergency shutdown, the canceller thread may call this
+   (see Scm_ThreadTerminate below).
+   Caller must hold vm->vmlock */
+static void thread_cleanup_inner(ScmVM *vm)
 {
-    ScmVM *vm = SCM_VM(data);
-    ScmObj exc;
-    
+    ScmObj e;
     /* Change this VM state to TERMINATED, and signals the change
        to the waiting threads. */
-    if (pthread_mutex_lock(&vm->vmlock) == EDEADLK) {
-        Scm_Panic("dead lock in vm_cleanup.");
-    }
     vm->state = SCM_VM_TERMINATED;
     if (vm->canceller) {
         /* This thread is cancelled. */
-        exc = Scm_MakeThreadException(SCM_CLASS_TERMINATED_THREAD_EXCEPTION, vm);
-        SCM_THREAD_EXCEPTION(exc)->data = SCM_OBJ(vm->canceller);
-        vm->resultException = exc;
+        e = Scm_MakeThreadException(SCM_CLASS_TERMINATED_THREAD_EXCEPTION, vm);
+        SCM_THREAD_EXCEPTION(e)->data = SCM_OBJ(vm->canceller);
+        vm->resultException = e;
     }
-    pthread_cond_broadcast(&vm->cond);
-    pthread_mutex_unlock(&vm->vmlock);
+    SCM_INTERNAL_COND_BROADCAST(vm->cond);
 }
 
-static void *thread_entry(void *data)
+/* Called by pthread_cleanup mechanism.   After this, Scm_VM() won't return
+   a valid VM pointer.  */
+static void thread_cleanup(void *data)
 {
     ScmVM *vm = SCM_VM(data);
-    pthread_cleanup_push(thread_cleanup, vm);
-    if (pthread_setspecific(Scm_VMKey(), vm) != 0) {
-        /* NB: at this point, theVM is not set and we can't use Scm_Error. */
+    SCM_INTERNAL_MUTEX_LOCK(vm->vmlock);
+    thread_cleanup_inner(vm);
+    SCM_INTERNAL_MUTEX_UNLOCK(vm->vmlock);
+    Scm_DetachVM(vm);
+}
+
+#if defined(GAUCHE_HAS_THREADS)
+static SCM_INTERNAL_THREAD_PROC_RETTYPE thread_entry(void *data)
+{
+    ScmVM *vm = SCM_VM(data);
+
+    if (!Scm_AttachVM(vm)) {
         vm->resultException =
-            Scm_MakeError(SCM_MAKE_STR("pthread_setspecific failed"));
+            Scm_MakeError(SCM_MAKE_STR("attaching VM to thread failed"));
+        thread_cleanup(vm);
     } else {
+        SCM_INTERNAL_THREAD_CLEANUP_PUSH(thread_cleanup, vm);
         SCM_UNWIND_PROTECT {
             vm->result = Scm_ApplyRec(SCM_OBJ(vm->thunk), SCM_NIL);
         } SCM_WHEN_ERROR {
@@ -139,9 +137,9 @@ static void *thread_entry(void *data)
                 break;
             }
         } SCM_END_PROTECT;
+        SCM_INTERNAL_THREAD_CLEANUP_POP();
     }
-    pthread_cleanup_pop(TRUE);
-    return NULL;
+    return SCM_INTERNAL_THREAD_PROC_RETVAL;
 }
 
 /* The default signal mask on the thread creation */
@@ -149,14 +147,23 @@ static struct threadRec {
     int dummy;                  /* required to place this in data area */
     sigset_t defaultSigmask;
 } threadrec = { 0 };
-#endif /* GAUCHE_USE_PTHREADS */
+#endif /* defined(GAUCHE_HAS_THREADS) */
 
+/* Start a thread.  If the VM is in "NEW" state, create a new thread and
+   make it run.
+
+   With pthread, the real thread is started as "detached" mode; i.e. once
+   the thread exits, the resources allocated for the thread by the system
+   is collected, including the result of the thread.  During this
+   deconstruction phase, the handler thread_cleanup() runs and saves the
+   thread result to the ScmVM structure.  If nobody cares about the
+   result of the thread, ScmVM structure will eventually be GCed.
+   This is to prevent exitted thread's system resources from being
+   uncollected.
+ */
 ScmObj Scm_ThreadStart(ScmVM *vm)
 {
-#ifdef GAUCHE_USE_PTHREADS
     int err_state = FALSE, err_create = FALSE;
-    pthread_attr_t thattr;
-    sigset_t omask;
 
     (void)SCM_INTERNAL_MUTEX_LOCK(vm->vmlock);
     if (vm->state != SCM_VM_NEW) {
@@ -164,42 +171,62 @@ ScmObj Scm_ThreadStart(ScmVM *vm)
     } else {
         SCM_ASSERT(vm->thunk);
         vm->state = SCM_VM_RUNNABLE;
-        pthread_attr_init(&thattr);
-        pthread_attr_setdetachstate(&thattr, PTHREAD_CREATE_DETACHED);
-        pthread_sigmask(SIG_SETMASK, &threadrec.defaultSigmask, &omask);
-        if (pthread_create(&vm->thread, &thattr, thread_entry, vm) != 0) {
-            vm->state = SCM_VM_NEW;
-            err_create = TRUE;
+#if defined(GAUCHE_USE_PTHREADS)
+        {
+            pthread_attr_t thattr;
+            sigset_t omask;
+            pthread_attr_init(&thattr);
+            pthread_attr_setdetachstate(&thattr, PTHREAD_CREATE_DETACHED);
+            pthread_sigmask(SIG_SETMASK, &threadrec.defaultSigmask, &omask);
+            if (pthread_create(&vm->thread, &thattr, thread_entry, vm) != 0) {
+                vm->state = SCM_VM_NEW;
+                err_create = TRUE;
+            }
+            pthread_sigmask(SIG_SETMASK, &omask, NULL);
+            pthread_attr_destroy(&thattr);
         }
-        pthread_sigmask(SIG_SETMASK, &omask, NULL);
-        pthread_attr_destroy(&thattr);
+#elif defined(GAUCHE_USE_WTHREADS)
+        {
+            HANDLE h = GC_CreateThread(NULL, /* security */
+                                       0,    /* default stack size */
+                                       thread_entry, /* start proc */
+                                       vm,           /* parameter */
+                                       0,            /* flags */
+                                       NULL);        /* thread id receiver */
+            if (h != NULL) {
+                vm->thread = h;
+            } else {
+                vm->state = SCM_VM_NEW;
+                err_create = TRUE;
+            }
+        }
+#else  /* no threads */
+        Scm_Error("Thread is not available.");
+#endif
     }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(vm->vmlock);
     if (err_state) Scm_Error("attempt to start an already-started thread: %S", vm);
     if (err_create) Scm_Error("couldn't start a new thread: %S", vm);
-#else  /*!GAUCHE_USE_PTHREADS*/
-    Scm_Error("not implemented!\n");
-#endif /*GAUCHE_USE_PTHREADS*/
     return SCM_OBJ(vm);
 }
 
 /* Thread join */
 ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
 {
-#ifdef GAUCHE_USE_PTHREADS
+#ifdef GAUCHE_HAS_THREADS
     struct timespec ts, *pts;
     ScmObj result = SCM_FALSE, resultx = SCM_FALSE;
     int intr = FALSE, tout = FALSE;
-    
+
     pts = Scm_GetTimeSpec(timeout, &ts);
     SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(target->vmlock);
     while (target->state != SCM_VM_TERMINATED) {
         if (pts) {
-            int tr = pthread_cond_timedwait(&(target->cond), &(target->vmlock), pts);
-            if (tr == ETIMEDOUT) { tout = TRUE; break; }
-            else if (tr == EINTR) { intr = TRUE; break; }
+            int tr = SCM_INTERNAL_COND_TIMEDWAIT(target->cond, target->vmlock, pts);
+            if (tr == SCM_INTERNAL_COND_TIMEDOUT) { tout = TRUE; break; }
+            else if (tr == SCM_INTERNAL_COND_INTR) { intr = TRUE; break; }
         } else {
-            pthread_cond_wait(&(target->cond), &(target->vmlock));
+            SCM_INTERNAL_COND_WAIT(target->cond, target->vmlock);
         }
     }
     if (!tout) {
@@ -219,10 +246,10 @@ ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
         result = Scm_Raise(resultx);
     }
     return result;
-#else  /*!GAUCHE_USE_PTHREADS*/
+#else  /*!GAUCHE_HAS_THREADS*/
     Scm_Error("not implemented!\n");
     return SCM_UNDEFINED;
-#endif /*!GAUCHE_USE_PTHREADS*/
+#endif /*!GAUCHE_HAS_THREADS*/
 }
 
 /* Attempt to stop other thread.   See process_queued_requests() in vm.c
@@ -238,9 +265,9 @@ ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
                                     * Wait on target->cond
                                       until target->state becomes STOPPED
    * If STOP flag is up,
-     set self->state to STOPPED 
+     set self->state to STOPPED
      and signal self->cond
-     then wait on self->cond until 
+     then wait on self->cond until
      target->state changes from STOPPED
                                     * Inspect target VM state
                                     * Set taget->state to RUNNABLE,
@@ -251,16 +278,17 @@ ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
 
 ScmObj Scm_ThreadStop(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
 {
-#ifdef GAUCHE_USE_PTHREADS
+#ifdef GAUCHE_HAS_THREADS
     struct timespec ts, *pts;
     ScmVM *vm = Scm_VM();
     ScmVM *taker = NULL;
-    int timedout = FALSE;
+    int timedout;
     int invalid_state = FALSE;
     pts = Scm_GetTimeSpec(timeout, &ts);
 
  retry:
-    pthread_mutex_lock(&target->vmlock);
+    timedout = FALSE;
+    SCM_INTERNAL_MUTEX_LOCK(target->vmlock);
     if (target->state != SCM_VM_RUNNABLE
         && target->state != SCM_VM_STOPPED) {
         invalid_state = TRUE;
@@ -272,24 +300,24 @@ ScmObj Scm_ThreadStop(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
         /* NB: if target->inspector == vm, it means we have requested to
            stop the thread before, and haven't called ThreadCont yet.
            In that case we don't need to set stopRequest again; either target
-           thread already cleared it and stopped/going to stop, or 
+           thread already cleared it and stopped/going to stop, or
            the target thread hasn't seen the flag and it's still ON. */
         if (target->inspector != vm) {
             target->inspector = vm;
-            target->stopRequest = TRUE;
+            target->stopRequest = SCM_VM_REQUEST_SUSPEND;
             target->attentionRequest = TRUE;
         }
-        while (target->state != SCM_VM_STOPPED) {
+        while (target->state != SCM_VM_STOPPED && !timedout) {
             if (pts) {
-                timedout = pthread_cond_timedwait(&target->cond,
-                                                  &target->vmlock,
-                                                  pts);
+                timedout = SCM_INTERNAL_COND_TIMEDWAIT(target->cond,
+                                                       target->vmlock,
+                                                       pts);
             } else {
-                pthread_cond_wait(&target->cond, &target->vmlock);
+                SCM_INTERNAL_COND_WAIT(target->cond, target->vmlock);
             }
         }
     }
-    pthread_mutex_unlock(&target->vmlock);
+    SCM_INTERNAL_MUTEX_UNLOCK(target->vmlock);
 
     if (invalid_state) {
         Scm_Error("cannot stop a thread %S since it is in neither runnable nor stopped state",
@@ -299,24 +327,24 @@ ScmObj Scm_ThreadStop(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
         Scm_Error("target %S is already under inspection by %S", target, taker);
     }
 
-    if (timedout == EINTR) { Scm_SigCheck(vm); goto retry; }
-    if (timedout == ETIMEDOUT) return timeoutval;
+    if (timedout == SCM_INTERNAL_COND_INTR) { Scm_SigCheck(vm); goto retry; }
+    if (timedout == SCM_INTERNAL_COND_TIMEDOUT) return timeoutval;
     return SCM_OBJ(target);
-#else  /*!GAUCHE_USE_PTHREADS*/
+#else  /*!GAUCHE_HAS_THREADS*/
     Scm_Error("not implemented!\n");
     return SCM_UNDEFINED;
-#endif /*!GAUCHE_USE_PTHREADS*/
+#endif /*!GAUCHE_HAS_THREADS*/
 }
 
 
 /* Release inspected thread */
 ScmObj Scm_ThreadCont(ScmVM *target)
 {
-#ifdef GAUCHE_USE_PTHREADS
+#ifdef GAUCHE_HAS_THREADS
     int not_stopped = FALSE;
     ScmVM *stopped_by_other = NULL;
 
-    pthread_mutex_lock(&target->vmlock);
+    SCM_INTERNAL_MUTEX_LOCK(target->vmlock);
     if (target->inspector == NULL) {
         not_stopped = TRUE;
     } else if (target->inspector != Scm_VM()
@@ -325,46 +353,70 @@ ScmObj Scm_ThreadCont(ScmVM *target)
     } else {
         target->inspector = NULL;
         target->state = SCM_VM_RUNNABLE;
-        target->stopRequest = FALSE;
-        pthread_cond_broadcast(&target->cond);
+        target->stopRequest = 0;
+        SCM_INTERNAL_COND_BROADCAST(target->cond);
     }
-    pthread_mutex_unlock(&target->vmlock);
+    SCM_INTERNAL_MUTEX_UNLOCK(target->vmlock);
     if (not_stopped) Scm_Error("target %S is not stopped", target);
     if (stopped_by_other) Scm_Error("target %S is stopped by other thread %S",
                                     target, stopped_by_other);
     return SCM_OBJ(target);
-#else  /*!GAUCHE_USE_PTHREADS*/
+#else  /*!GAUCHE_HAS_THREADS*/
     Scm_Error("not implemented!\n");
     return SCM_UNDEFINED;
-#endif /*!GAUCHE_USE_PTHREADS*/
+#endif /*!GAUCHE_HAS_THREADS*/
 }
 
 /* Thread sleep */
 ScmObj Scm_ThreadSleep(ScmObj timeout)
 {
-#ifdef GAUCHE_USE_PTHREADS
+#ifdef GAUCHE_HAS_THREADS
     struct timespec ts, *pts;
-    ScmInternalCond dummyc = PTHREAD_COND_INITIALIZER;
-    ScmInternalMutex dummym = PTHREAD_MUTEX_INITIALIZER;
+    ScmInternalCond dummyc;
+    ScmInternalMutex dummym;
     int intr = FALSE;
+
+    SCM_INTERNAL_COND_INIT(dummyc);
+    SCM_INTERNAL_MUTEX_INIT(dummym);
     pts = Scm_GetTimeSpec(timeout, &ts);
     if (pts == NULL) Scm_Error("thread-sleep! can't take #f as a timeout value");
-    pthread_mutex_lock(&dummym);
-    if (pthread_cond_timedwait(&dummyc, &dummym, pts) == EINTR) {
+    SCM_INTERNAL_MUTEX_LOCK(dummym);
+    if (SCM_INTERNAL_COND_TIMEDWAIT(dummyc, dummym, pts) == SCM_INTERNAL_COND_INTR) {
         intr = TRUE;
     }
-    pthread_mutex_unlock(&dummym);
+    SCM_INTERNAL_MUTEX_UNLOCK(dummym);
     if (intr) Scm_SigCheck(Scm_VM());
-#else  /*!GAUCHE_USE_PTHREADS*/
+#else  /*!GAUCHE_HAS_THREADS*/
     Scm_Error("not implemented!\n");
-#endif /*!GAUCHE_USE_PTHREADS*/
+#endif /*!GAUCHE_HAS_THREADS*/
     return SCM_UNDEFINED;
 }
 
-/* Thread terminate */
+/* Thread termination
+
+   We try to terminate the thread gracefully as possible.
+   First, we use vm->stopRequest mechanism.  If the target thread is
+   in VM loop, it responds to the flag and terminates itself.
+   If that fails, then we use more agressive means.
+
+   TODO: We should probably make it configurable whether to use the
+   forcible termination---it is too dangerous.
+ */
+
+/* Caller must hold target->vmlock.  return TRUE if the target terminates
+   gracefully. */
+static int wait_for_termination(ScmVM *target)
+{
+    struct timespec ts;
+    int r;
+    ScmObj t = Scm_MakeFlonum(0.001); /* 1ms. somewhat arbitrary */
+    Scm_GetTimeSpec(t, &ts);
+    r = SCM_INTERNAL_COND_TIMEDWAIT(target->cond, target->vmlock, &ts);
+    return (r == 0);
+}
+
 ScmObj Scm_ThreadTerminate(ScmVM *target)
 {
-#ifdef GAUCHE_USE_PTHREADS
     ScmVM *vm = Scm_VM();
     if (target == vm) {
         /* self termination */
@@ -375,20 +427,42 @@ ScmObj Scm_ThreadTerminate(ScmVM *target)
         (void)SCM_INTERNAL_MUTEX_UNLOCK(target->vmlock);
         /* Need to unlock before calling pthread_exit(), or the cleanup
            routine can't obtain the lock */
-        pthread_exit(NULL);
-    } else {
-        (void)SCM_INTERNAL_MUTEX_LOCK(target->vmlock);
+        SCM_INTERNAL_THREAD_EXIT();
+        /*NOTREACHED*/
+    }
+
+    (void)SCM_INTERNAL_MUTEX_LOCK(target->vmlock);
+    do {
         /* This ensures only the first call of thread-terminate! on a thread
            is in effect. */
         if (target->canceller == NULL) {
             target->canceller = vm;
+
+            /* First try */
+            target->stopRequest = SCM_VM_REQUEST_TERMINATE;
+            target->attentionRequest = TRUE;
+            if (wait_for_termination(target)) break;
+
+            /* Second try */
+#if defined(GAUCHE_USE_PTHREADS)
+# if defined(GAUCHE_PTHREAD_SIGNAL)
+            pthread_kill(target->thread, GAUCHE_PTHREAD_SIGNAL);
+# endif /*defined(GAUCHE_PTHREAD_SIGNAL)*/
+#elif defined(GAUCHE_USE_WTHREADS)
+            /* TODO: implement signal mechanism using an event */
+#endif  /* defined(GAUCHE_USE_WTHREADS) */
+            if (wait_for_termination(target)) break;
+
+            /* Last resort */
+            thread_cleanup_inner(target);
+#if defined(GAUCHE_USE_PTHREADS)
             pthread_cancel(target->thread);
+#elif defined(GAUCHE_USE_WTHREADS)
+            TerminateThread(target->thread, 0);
+#endif
         }
-        (void)SCM_INTERNAL_MUTEX_UNLOCK(target->vmlock);
-    }
-#else  /*!GAUCHE_USE_PTHREADS*/
-    Scm_Error("not implemented!\n");
-#endif /*!GAUCHE_USE_PTHREADS*/
+    } while (0);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(target->vmlock);
     return SCM_UNDEFINED;
 }
 
@@ -399,6 +473,8 @@ void Scm_Init_threads(ScmModule *mod)
 {
 #ifdef GAUCHE_USE_PTHREADS
     sigfillset(&threadrec.defaultSigmask);
+# if defined(GAUCHE_PTHREAD_SIGNAL)
+    sigdelset(&threadrec.defaultSigmask, GAUCHE_PTHREAD_SIGNAL);
+# endif /*defined(GAUCHE_PTHRAD_SIGNAL)*/
 #endif /*GAUCHE_USE_PTHREADS*/
 }
-
