@@ -1,23 +1,23 @@
 ;;;
 ;;; threads.scm - thread related procedures.  to be autoloaded
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,9 +29,10 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module gauche.threads
+  (use gauche.record)
   (export gauche-thread-type
 
           thread? make-thread thread-name thread-specific-set! thread-specific
@@ -41,6 +42,7 @@
           mutex? make-mutex mutex-name mutex-state
           mutex-specific-set! mutex-specific
           with-locking-mutex mutex-lock! mutex-unlock!
+          mutex-locker mutex-unlocker
 
           condition-variable? make-condition-variable condition-variable-name
           condition-variable-specific condition-variable-specific-set!
@@ -50,8 +52,7 @@
           terminated-thread-exception? uncaught-exception?
           uncaught-exception-reason
 
-          ;; experimenal
-          atom atom-ref atom-swap!))
+          atom atom? atom-ref atomic atomic-update!))
 (select-module gauche.threads)
 
 (inline-stub
@@ -62,18 +63,20 @@
   "extern void Scm_Init_threads(ScmModule*);")
 
  (initcode
-  "Scm_Init_threads(mod);"
-  "Scm_Init_mutex(mod);"))
+  "Scm_Init_threads(Scm_CurrentModule());"
+  "Scm_Init_mutex(Scm_CurrentModule());"))
 
 ;;===============================================================
 ;; System query
 ;;
 
+;; TODO: for the consistency of ./configure --enable-threads,
+;; it should return 'pthreads' instead of 'pthread' on pthreads platform.
 (inline-stub
  (define-cproc gauche-thread-type ()
-   (.if "defined(GAUCHE_USE_PTHREADS)"
-        (result 'pthread)
-        (result 'none))))
+   (.cond ["defined(GAUCHE_USE_PTHREADS)" (result 'pthread)]
+          ["defined(GAUCHE_USE_WTHREADS)" (result 'win32)]
+          [else (result 'none)])))
 
 ;;===============================================================
 ;; Thread
@@ -91,7 +94,7 @@
 
 (define thread-specific
   (getter-with-setter
-   (lambda (thread)
+   (^[thread]
      (check-arg thread? thread)
      (slot-ref thread 'specific))
    thread-specific-set!))
@@ -125,7 +128,7 @@
    Scm_ThreadStop)
 
  (define-cproc thread-cont! (target::<thread>) Scm_ThreadCont)
- ) 
+ )
 
 ;;===============================================================
 ;; Mutex
@@ -147,16 +150,16 @@
 
 (define mutex-specific
   (getter-with-setter
-   (lambda (mutex)
+   (^[mutex]
      (check-arg mutex? mutex)
      (slot-ref mutex 'specific))
    mutex-specific-set!))
 
-(define (with-locking-mutex mutex thunk)
+(define-inline (with-locking-mutex mutex thunk)
   (dynamic-wind
-   (lambda () (mutex-lock! mutex))
-   thunk
-   (lambda () (mutex-unlock! mutex))))
+      (mutex-locker mutex)
+      thunk
+      (mutex-unlocker mutex)))
 
 (inline-stub
  (define-cproc make-mutex (:optional (name #f)) Scm_MakeMutex)
@@ -174,12 +177,15 @@
            [(SCM_UNBOUNDP thread) (set! owner (Scm_VM))]
            [(not (SCM_FALSEP thread)) (SCM_TYPE_ERROR thread "thread or #f")])
      (result (Scm_MutexLock mutex timeout owner))))
- 
+
  (define-cproc mutex-unlock! (mutex::<mutex> :optional (cv #f) (timeout #f))
    (let* ([cond::ScmConditionVariable* NULL])
      (cond [(SCM_CONDITION_VARIABLE_P cv) (set! cond (SCM_CONDITION_VARIABLE cv))]
            [(not (SCM_FALSEP cv)) (SCM_TYPE_ERROR cv "condition variale or #f")])
      (result (Scm_MutexUnlock mutex cond timeout))))
+
+ (define-cproc mutex-locker (mutex::<mutex>) Scm_MutexLocker)
+ (define-cproc mutex-unlocker (mutex::<mutex>) Scm_MutexUnlocker)
  )
 
 ;;===============================================================
@@ -212,8 +218,8 @@
 
   (define-cproc condition-variable-broadcast! (cv::<condition-variable>)
     Scm_ConditionVariableBroadcast)
-  )  
-     
+  )
+
 ;;===============================================================
 ;; Exceptions
 ;;
@@ -238,27 +244,45 @@
 ;; Atom
 ;;
 
-;; EXPERIMENTAL
+(define-record-type <atom> %make-atom atom?
+  (applier atom-applier)
+  (updater atom-updater))
+
 (define (atom . vals)
   (define m (make-mutex))
   (let-syntax ([with-lock
                 (syntax-rules ()
-                  [(_ timeout . form)
+                  [(_ timeout timeout-val . form)
                    (unwind-protect
-                       (and (mutex-lock! m timeout) . form)
+                       (if (mutex-lock! m timeout)
+                         (begin . form)
+                         timeout-val)
                      (when (eq? (mutex-state m) (current-thread))
                        (mutex-unlock! m)))])])
-    (define (atom-internal msg proc timeout)
-      (case msg
-        [(ref)  (with-lock timeout (apply proc vals))]
-        [(swap) (with-lock timeout
-                           (call-with-values (cut apply proc vals)
-                             (^ vs (set! vals vs) (apply values vals))))]
-        [else (error "Unknown message to an atom" msg)]))
-    atom-internal))
+    ;; TODO: we may expand special cases like vals is 1 to 3 elements long,
+    ;; avoiding creation of lists every time updater is called.
+    (%make-atom
+     (^[proc timeout timeout-val]
+       (with-lock timeout timeout-val (apply proc vals)))
+     (^[proc timeout timeout-val]
+       (with-lock timeout timeout-val
+                  (call-with-values (cut apply proc vals)
+                    (^ vs
+                      (unless (= (length vs) (length vals))
+                        (errorf "atomic-update!: procedure returned wrong \
+                                   number of values (~a, while ~a expected)"
+                                (length vs) (length vals)))
+                      (set! vals vs)
+                      (apply values vals))))))))
 
-(define (atom-ref atom :optional (proc values) (timeout #f))
-  (atom 'ref proc timeout))
+(define (atomic atom proc :optional (timeout #f) (timeout-val #f))
+  (unless (atom? atom) (error "atom required, but got:" atom))
+  ((atom-applier atom) proc timeout timeout-val))
 
-(define (atom-swap! atom proc :optional (timeout #f))
-  (atom 'swap proc timeout))
+(define (atomic-update! atom proc :optional (timeout #f) (timeout-val #f))
+  (unless (atom? atom) (error "atom required, but got:" atom))
+  ((atom-updater atom) proc timeout timeout-val))
+
+(define (atom-ref atom :optional (index 0) (timeout #f) (timeout-val #f))
+  (unless (atom? atom) (error "atom required, but got:" atom))
+  ((atom-applier atom) (^ xs (list-ref xs index)) timeout timeout-val))
