@@ -85,7 +85,7 @@
                    :getc (lambda ()
                            (and (pair? x) (pop! x))))))
          (port->string-list p)))
-         
+
 (test* "getb -> read-line" '("abcd" "efg")
        (let* ((x '(97 98 99 100 10 101 102 103 10))
               (p (make <virtual-input-port>
@@ -176,14 +176,15 @@
 
 (let ()
   (define (test-biport file size)
-    (let* ((src (open-input-file file))
+    (let* ((ifile #`",(sys-dirname (current-load-path))/,file")
+           (src (open-input-file ifile))
            (p (apply make <buffered-input-port>
                      :fill  (lambda (buf) (read-block! buf src))
                      :close (lambda () (close-input-port src))
                      (if size
                        (list :buffer-size size)
                        '())))
-           (a (file->string-list file))
+           (a (file->string-list ifile))
            (b (port->string-list p)))
       (close-input-port p)
       (list (equal? a b) (port-closed? src))))
@@ -233,7 +234,8 @@
 
 (let ()
   (define (test-boport file size)
-    (let* ((src  (file->string file))
+    (let* ((ifile #`",(sys-dirname (current-load-path))/,file")
+           (src  (file->string ifile))
            (sink (open-output-string))
            (closed? #f)
            (p (apply make <buffered-output-port>
@@ -248,7 +250,7 @@
       (close-output-port p)
       (list (equal? src (get-output-string sink))
             closed?)))
-  
+
   (test* "vport.c"      '(#t #t) (test-boport "vport.c" #f))
   (test* "vport.h" '(#t #t) (test-boport "vport.h" #f))
 
@@ -361,11 +363,11 @@
   (tester 10 9)
   (tester 10 10)
   (tester 10 15)
-  
+
   (tester 20000 19999)
   (tester 20000 20000)
   (tester 20000 20001)
-  
+
   (tester 0  0)
   (tester 0  1)
   (tester 0  10)
