@@ -82,7 +82,7 @@
             (cond ((break-char-list/pred c))
                   ((eof-object? c) (bad))
                   ((= i limit))
-                  (else                    
+                  (else
                    (display (read-char port))
                    (loop (peek-char port) (+ i 1)))))))
       )
@@ -196,7 +196,7 @@
      (lambda ()
        (list
         (with-output-to-string
-          (lambda () (port-for-each proc read-byte))))))
+          (cut generator-for-each proc read-byte)))))
     ((procedure? count)
      (let ((unpacker (count 'unpacker)))
        (lambda ()
@@ -224,7 +224,7 @@
 
 ;; Skip a value of a given length n, where n could be an integer,
 (define (make-skipper n)
-  (cond 
+  (cond
     ((eq? n #\*)
      (lambda ()
        (while (not (eof-object? (read-block 1024))))))
@@ -362,8 +362,7 @@
         (lambda (v) (for-each writer v) '())
         (lambda ()
           (let ((res '()))
-            (while (not (eof-object? (peek-char)))
-              (push! res (reader)))
+            (until (reader) eof-object? => x (push! res x))
             (reverse res)))
         (make-skipper #\*)))
       ((procedure? count)
@@ -795,7 +794,7 @@
            ((#\q)
             (make-number-pack-dispatcher
              read-binary-sint64 write-binary-sint64 64 count vlp))
-           
+
            ;; Q   An unsigned quad value.
            ((#\Q)
             (make-number-pack-dispatcher
