@@ -1,23 +1,23 @@
 ;;;
 ;;; gauche.cgen.cise - C in S expression
-;;;  
-;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2004-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module gauche.cgen.cise
   (use srfi-1)
@@ -38,7 +38,6 @@
   (use gauche.parameter)
   (use gauche.cgen.unit)
   (use gauche.cgen.literal)
-  (use gauche.experimental.ref)
   (use gauche.experimental.app)
   (use gauche.experimental.lamb)
   (use util.match)
@@ -134,7 +133,7 @@
 
 (define (push-static-decl! stree :optional (context (cise-context)))
   (push! (ref context'static-decls) stree))
-                                
+
 (define (emit-static-decls port :optional (context (cise-context)))
   (dolist [stree (reverse (ref context'static-decls))]
     (render-finalize stree port)))
@@ -311,9 +310,8 @@
   (match form
     [([? symbol? key] . args)
      (cond [(cise-lookup-macro key)
-            => (lambda (expander)
-                 `(,@(source-info form env)
-                   ,@(render-rec (expander form env) env)))]
+            => (^[expander] `(,@(source-info form env)
+                              ,@(render-rec (expander form env) env)))]
            [(or (type-decl-initial? key)
                 (any type-decl-subsequent? args))
             (cise-render-typed-var form "" env)]
@@ -366,11 +364,11 @@
       (emit-static-decls outp)
       (display (get-output-string toutp) outp))
     (newline outp))
-  
+
   (eval '(use gauche.cgen.cise) environment)
   (eval '(use util.match) environment)
   (parameterize ([cise-context context])
-    (let loop ((toutp outp))
+    (let loop ([toutp outp])
       (match (read inp)
         [(? eof-object?) (finish toutp)]
         [('.raw-c-code . cs)
@@ -429,7 +427,7 @@
       [(':static . body) (record-static name args ret-type)
                          (gen-cfn "static" name args ret-type body)]
       [_                 (gen-cfn "" name args ret-type body)]))
-  
+
   (ensure-toplevel-ctx form env)
   (match form
     [(_ name (args ...) ':: ret-type . body)
@@ -466,7 +464,7 @@
        [((var . spec) ...)
         (let1 eenv (expr-env env)
           `(begin
-             ,@(map (lambda (var spec)
+             ,@(map (^[var spec]
                       (receive (type has-init? init)
                           (match spec
                             [()         (values 'ScmObj #f #f)]
@@ -512,7 +510,7 @@
       `("(" ,(render-rec test eenv) ")" ,(render-rec `(begin ,@rest) env)))
     (match form
       [(_ (test . rest) ...)
-       (fold-right (lambda (test rest r)
+       (fold-right (^[test rest r]
                      (cond
                       [(and (null? r) (eq? test 'else))
                        `(" else ",(render-rec `(begin ,@rest) env))]
@@ -532,11 +530,11 @@
     (match form
       [(_ expr (literalss . clauses) ...)
        `("switch (",(render-rec expr eenv)") {"
-         ,@(map (lambda (literals clause)
+         ,@(map (^[literals clause]
                   `(,@(source-info literals env)
                     ,@(if (eq? literals 'else)
                         '("default: ")
-                        (map (lambda (literal)
+                        (map (^[literal]
                                `("case ",(render-rec literal eenv)" : "))
                              literals))
                     ,@(render-rec `(begin ,@clause
@@ -544,7 +542,7 @@
                                   env)))
                 literalss clauses)
          "}")]
-      )))    
+      )))
 
 (define-cise-macro (case form env)
   (ensure-stmt-ctx form env)
@@ -592,8 +590,8 @@
 ;;   The lambda form is a fake; you don't really create a closure.
 (define-cise-macro (for-each form env)
   (ensure-stmt-ctx form env)
-  (let ((eenv (expr-env env))
-        (tmp  (gensym "cise__")))
+  (let ([eenv (expr-env env)]
+        [tmp  (gensym "cise__")])
     (match form
       [(_ ('lambda (var) . body) list-expr)
        (env-decl-add! env `(,tmp ScmObj))
@@ -606,7 +604,7 @@
 ;; [cise stmt] dolist [VAR EXPR] STMT ...
 (define-cise-macro (dolist form env)
   (ensure-stmt-ctx form env)
-  (let ((eenv (expr-env env)))
+  (let1 eenv (expr-env env)
     (match form
       [(_ (var expr) . body)
        `(for-each (lambda (,var) ,@body) ,expr)])))
@@ -616,7 +614,7 @@
 ;;   each element of the list.
 (define-cise-macro (pair-for-each form env)
   (ensure-stmt-ctx form env)
-  (let ((eenv (expr-env env)))
+  (let1 eenv (expr-env env)
     (match form
       [(_ ('lambda (var) . body) list-expr)
        (env-decl-add! env `(,var ScmObj))
@@ -628,7 +626,7 @@
 ;; [cise stmt] dopairs [VAR EXPR] STMT ...
 (define-cise-macro (dopairs form env)
   (ensure-stmt-ctx form env)
-  (let ((eenv (expr-env env)))
+  (let1 eenv (expr-env env)
     (match form
       [(_ (var expr) . body)
        `(pair-for-each (lambda (,var) ,@body) ,expr)])))
@@ -638,8 +636,8 @@
 ;;   to (N-1).
 (define-cise-macro (dotimes form env)
   (ensure-stmt-ctx form env)
-  (let ((eenv (expr-env env))
-        (n    (gensym "cise__")))
+  (let ([eenv (expr-env env)]
+        [n    (gensym "cise__")])
     (match form
       [(_ (var expr) . body)
        `(let* ((,var :: int 0) (,n :: int ,expr))
@@ -683,11 +681,13 @@
   (ensure-stmt-or-toplevel-ctx form env)
   (match form
     [(_ condition stmt1)
-     `("#if " ,(x->string condition) "\n" |#reset-line|
+     `("\n" |#reset-line|               ;make sure we start from the fresh line
+       "#if " ,(x->string condition) "\n" |#reset-line|
        ,(render-rec stmt1 env) "\n"
        "#endif /* " ,(x->string condition) " */\n" |#reset-line|)]
     [(_ condition stmt1 stmt2)
-     `("#if " ,(x->string condition) "\n" |#reset-line|
+     `("\n" |#reset-line|               ;make sure we start from the fresh line
+       "#if " ,(x->string condition) "\n" |#reset-line|
        ,(render-rec stmt1 env) "\n"
        "#else  /* !",(x->string condition) " */\n" |#reset-line|
        ,(render-rec stmt2 env) "\n"
@@ -697,7 +697,7 @@
   (ensure-stmt-or-toplevel-ctx form env)
   (match form
     [(_ (condition . stmts) ...)
-     `("#if 0 /*dummy*/\n"
+     `("\n#if 0 /*dummy*/\n" |#reset-line|
        ,@(fold-right (lambda (c ss seed)
                        `(,(cond [(eq? c 'else) '("#else")]
                                 [else `("#elif " ,(x->string c))])
@@ -707,16 +707,20 @@
                      '("#endif\n" |#reset-line|)
                      condition stmts))]))
 
+(define-cise-macro (.undef form env)
+  (ensure-stmt-or-toplevel-ctx form env)
+  (match form
+    [(_ name) `("#undef " ,(x->string name) "\n" |#reset-line|)]))
+
 (define-cise-macro (.include form env)
   (ensure-stmt-or-toplevel-ctx form env)
   (match form
     [(_ item ...)
-     (map (lambda (f)
-            `("#include "
-              ,(cond [(string? f) (write-to-string f)]
-                     [(symbol? f) (x->string f)]
-                     [else (error "bad argument to .include:" f)])
-              "\n" |#reset-line|))
+     (map (^f `("#include "
+                ,(cond [(string? f) (write-to-string f)]
+                       [(symbol? f) (x->string f)]
+                       [else (error "bad argument to .include:" f)])
+                "\n" |#reset-line|))
           item)]
     [(_ . other) (error "malformed .include:" form)]))
 
@@ -816,7 +820,7 @@
           [(_ a b . x)
            (list* ',op (list ',op a b) x)])
         env))))
-       
+
 (define-nary + "+")
 (define-nary - "-")
 (define-nary * "*")
@@ -905,7 +909,7 @@
      (match form
        [(_ a b ...)
         `("(",(render-rec a eenv)")"
-          ,(append-map (lambda (ind) `("[",(render-rec ind eenv)"]")) b))])
+          ,(append-map (^[ind] `("[",(render-rec ind eenv)"]")) b))])
      env)))
 
 (define-cise-macro (?: form env)
@@ -975,7 +979,7 @@
   [(_ a b c)     `(Scm_Values3 ,a ,b ,c)]
   [(_ a b c d)   `(Scm_Values4 ,a ,b ,c ,d)]
   [(_ a b c d e) `(Scm_Values5 ,a ,b ,c ,d ,e)]
-  [(_ x ...)   `(Scm_Values ,(fold (lambda (elt r) `(Scm_cons ,elt ,r)) '() x))]
+  [(_ x ...)     `(Scm_Values ,(fold (^[elt r] `(Scm_cons ,elt ,r)) '() x))]
   )
 ;; Using quote is a convenient way to embed Scheme constant in C code.
 (define-cise-expr quote
@@ -1004,7 +1008,7 @@
   (or (memq sym '(* &))
       (type-decl-initial? sym)))
 
-      
+
 (define (cise-render-typed-var typespec var env)
   (match typespec
     [('.array spec (dim ...))
