@@ -1,23 +1,23 @@
 ;;;
 ;;; gauche.cgen.literal - static literal data
-;;;  
-;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2004-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module gauche.cgen.literal
   (use srfi-1)
@@ -38,8 +38,6 @@
   (use gauche.parameter)
   (use gauche.sequence)
   (use gauche.cgen.unit)
-  (use gauche.experimental.ref)
-  (use gauche.experimental.lamb)
   (use gauche.experimental.app)
   (use util.match)
   (export <cgen-literal> cgen-c-name cgen-cexpr cgen-make-literal
@@ -92,7 +90,7 @@
    (c-type :init-keyword :c-type)  ; symbol for C type, e.g. 'ScmObj
    (c-member-name :init-form (gensym "d")) ; member name in the C structure
    (count  :init-value 0)          ; # of allocated objs.
-   (cpp-condition :init-form (cgen-cpp-condition)) ; cpp condition
+   (cpp-conditions :init-form (cgen-cpp-conditions)) ; cpp condition
    (init-thunks :init-value '())   ; thunks to generate initializers
                                    ;  constructed in rev order.
    ))
@@ -109,16 +107,16 @@
 
   (define (ensure-static-data-list category c-type)
     (and-let* ([unit (cgen-current-unit)])
-      (let* ([cppc (cgen-cpp-condition)]
-             [dl   (find (^(dl) (and (eq? (~ dl'c-type) c-type)
+      (let* ([cppc (cgen-cpp-conditions)]
+             [dl   (find (^[dl] (and (eq? (~ dl'c-type) c-type)
                                      (eq? (~ dl'category) category)
-                                     (equal? (~ dl'cpp-condition) cppc)))
+                                     (equal? (~ dl'cpp-conditions) cppc)))
                          (~ unit'static-data-list))])
         (or dl
             (rlet1 new (make <cgen-static-data-list>
                          :category category :c-type c-type)
               (push! (~ unit'static-data-list) new))))))
-  
+
   (let ([dl (ensure-static-data-list category c-type)]
         [value-type? (not init-thunk)]
         [ithunk (or init-thunk (if (eq? c-type 'ScmObj) "SCM_UNBOUND" "NULL"))])
@@ -136,7 +134,7 @@
                 count)))))
 
 (define (cgen-allocate-static-array category c-type init-thunks)
-  (fold (lambda (init-thunk seed)
+  (fold (^[init-thunk seed]
           (let1 cexpr (cgen-allocate-static-datum category c-type init-thunk)
             (or seed cexpr)))
         #f init-thunks))
@@ -144,11 +142,11 @@
 (define-method cgen-emit-static-data ((unit <cgen-unit>))
 
   (define (emit-one-category category dls)
-    (let1 dls (filter (^(dl) (eq? (~ dl'category) category)) dls)
+    (let1 dls (filter (^[dl] (eq? (~ dl'category) category)) dls)
       (unless (null? dls)
         (emit-struct-def category dls)
         (print "{")
-        (dolist (dl dls) (emit-initializers dl))
+        (dolist [dl dls] (emit-initializers dl))
         (print "};"))))
 
   (define (emit-struct-def category dls)
@@ -157,21 +155,21 @@
               (if (eq? category 'constant) "SCM_CGEN_CONST " "")
               name)
       (dolist [dl dls]
-        (cond [(~ dl'cpp-condition) => (cut print "#if "<>)])
+        (for-each (cut print "#if "<>) (~ dl'cpp-conditions))
         (format #t "  ~a ~a[~a];\n" (~ dl'c-type) (~ dl'c-member-name)
                 (~ dl'count))
-        (cond [(~ dl'cpp-condition) => (cut print "#endif /*"<>"*/")]))
+        (for-each (cut print "#endif /*"<>"*/") (~ dl'cpp-conditions)))
       (format #t "} ~a = " name)))
 
   (define (emit-initializers dl)
-    (cond [(~ dl'cpp-condition) => (cut print "#if "<>)])
+    (for-each (cut print "#if "<>) (~ dl'cpp-conditions))
     (print "  {   /* "(~ dl'c-type)" "(~ dl'c-member-name)" */")
     (dolist [thunk (reverse (~ dl'init-thunks))]
       (if (string? thunk)
         (format #t "    ~a,\n" thunk)
         (begin (format #t "    ") (thunk) (print ","))))
     (print "  },")
-    (cond [(~ dl'cpp-condition) => (cut print "#endif /*"<>"*/")]))
+    (for-each (cut print "#endif /*"<>"*/") (~ dl'cpp-conditions)))
 
   (and-let* ([dls (~ unit'static-data-list)])
     (unless (null? dls)
@@ -219,7 +217,7 @@
 ;;                            hols the object is returned.
 ;;
 ;;      (extern (self) ...)
-;;      (decl (self) ...) 
+;;      (decl (self) ...)
 ;;      (body (self) ...)
 ;;      (init (self) ...)  -> handle generation of each part.
 ;;                            If slot has <cgen-literal>, they
@@ -229,7 +227,7 @@
 ;;      (static (self) ...) -> used by cgen-literal-static?.
 ;;                            If omitted, #t is returned.
 ;;
-;;   
+;;
 
 ;; <cgen-literal> base class ----------------------------------
 
@@ -326,7 +324,7 @@
     ((define-cgen-literal . _)
      (syntax-error "malformed define-cgen-literal:" (define-cgen-literal . _)))
     ))
-       
+
 ;; method cgen-literal returns a <cgen-literal> node for the
 ;; literal value of given Scheme value.  It first scans the current
 ;; unit's toplevel nodes with the same value, and returns it if found.
@@ -337,7 +335,7 @@
 ;; object-equal? that returns #t with different class's instances.
 
 (define (cgen-literal value)
-  (or (and-let* ((unit (cgen-current-unit)))
+  (or (and-let* ([unit (cgen-current-unit)])
         (lookup-literal-value unit value))
       (cgen-make-literal value)))
 
@@ -398,20 +396,20 @@
 
 (define (register-literal-value unit literal-obj)
   (let ([lh   (ensure-literal-hash unit)]
-        [cppc (~ literal-obj'cpp-condition)]
+        [cppc (~ literal-obj'cpp-conditions)]
         [h    (literal-value-hash (~ literal-obj'value))])
-    (or (and-let* ([entry (find (^(e) (and (equal? (caar e) cppc)
-                                           (literal-value=? (~ literal-obj'value)
-                                                            (cdar e))))
+    (or (and-let* ([entry (find (^e (and (equal? (caar e) cppc)
+                                         (literal-value=? (~ literal-obj'value)
+                                                          (cdar e))))
                                 (vector-ref lh h))])
           (set-cdr! entry literal-obj))
         (push! (vector-ref lh h) (acons cppc (~ literal-obj'value) literal-obj)))))
 
 (define (lookup-literal-value unit val)
   (let ([lh (ensure-literal-hash unit)]
-        [cppc (cgen-cpp-condition)])
-    (and-let* ([entry (find (^(e) (and (equal? (caar e) cppc)
-                                       (literal-value=? val (cdar e))))
+        [cppc (cgen-cpp-conditions)])
+    (and-let* ([entry (find (^e (and (equal? (caar e) cppc)
+                                     (literal-value=? val (cdar e))))
                             (vector-ref lh (literal-value-hash val)))])
       (cdr entry))))
 
@@ -478,12 +476,11 @@
   (with-string-io value
     (lambda ()
       (display "\"")
-      (port-for-each (lambda (b)
-                       (if (or (= #x20 b) (= #x21 b) ; #x22 = #\"
-                               (<= #x23 b #x5b)      ; #x5c = #\\
-                               (<= #x5d b #x7e))
-                         (write-byte b)
-                         (format #t "\\~3,'0o" b)))
+      (port-for-each (^b (if (or (= #x20 b) (= #x21 b) ; #x22 = #\"
+                                 (<= #x23 b #x5b)      ; #x5c = #\\
+                                 (<= #x5d b #x7e))
+                           (write-byte b)
+                           (format #t "\\~3,'0o" b)))
                      read-byte)
       (display "\""))))
 
@@ -497,9 +494,10 @@
           :symbol-name (cgen-literal (symbol->string value))))
   (init (self)
     (print "  " (cgen-c-name self)
-           " = Scm_Intern(SCM_STRING("
+           " = Scm_MakeSymbol(SCM_STRING("
            (cgen-cexpr (~ self'symbol-name))
-           ")); /* "(cgen-safe-comment (~ self'value))" */"))
+           ")," (if (symbol-interned? (~ self'value)) "TRUE" "FALSE")");"
+           " /* "(cgen-safe-comment (~ self'value))" */"))
   (static (self) #f)
   )
 
@@ -635,7 +633,7 @@
                   (list*
                    "SCM_OBJ(SCM_CLASS_STATIC_TAG(Scm_VectorClass)) /* <vector> */"
                    (format "SCM_OBJ(~a)" (length literals))
-                   (map (^(lit) (if (cgen-literal-static? lit)
+                   (map (^[lit] (if (cgen-literal-static? lit)
                                   (cgen-cexpr lit)
                                   "SCM_UNDEFINED"))
                         literals)))])
@@ -645,7 +643,7 @@
         :literals literals)))
   (init (self)
     (for-each-with-index
-     (lambda (ind elt)
+     (^[ind elt]
        (unless (cgen-literal-static? elt)
          (print "  ((ScmObj*)"(cgen-c-name self)")["(+ ind 2)"] = "(cgen-cexpr elt)";")))
      (~ self'literals)))
@@ -674,14 +672,15 @@
       (print (uvector-class->c-type-name class)" "(~ self'elements)"[] = {")
       (dotimes [i (uvector-length value)]
         ($ uvector-class-emit-elt class
-           $ %uvector-ref value (uvector-class->type-enum class) i))
+           $ (with-module gauche.internal %uvector-ref)
+             value (uvector-class->type-enum class) i))
       (print  "};"))]
   [static (self) #f]
   )
 
 (define (uvector-class->tag-name class)
   (rxmatch->string #/<(.\d+)vector>/ (symbol->string (class-name class)) 1))
-      
+
 (define (uvector-class->c-name class)
   #`"Scm_,(string-upcase (uvector-class->tag-name class))VectorClass")
 
@@ -750,7 +749,8 @@
   (init (self)
     (print "  {")
     (print "     ScmCharSet *cs = SCM_CHARSET(Scm_MakeEmptyCharSet());")
-    (dolist (range (%char-set-ranges (~ self'value)))
+    (dolist [range ((with-module gauche.internal %char-set-ranges)
+                    (~ self'value))]
       (format #t "     Scm_CharSetAddRange(cs, SCM_CHAR(~a), SCM_CHAR(~a));\n"
               (car range) (cdr range)))
     (print "     "(cgen-c-name self)" = SCM_OBJ(cs);")
@@ -785,7 +785,7 @@
 ;;  srfi-10 syntax, reader constructor of the inner srfi-10 is invoked
 ;;  in this process.  If the reader constructor have side effects, this
 ;;  routine may cause undesirable side effects as well.  There's no way
-;;  we can know if it is safe. 
+;;  we can know if it is safe.
 
 (define-class <cgen-user-defined-type> (<cgen-literal>)
   ((ctor-name :init-keyword :ctor-name)
@@ -826,6 +826,5 @@
 
 (define cgen-unique-name
   (let1 counter 0
-    (lambda () (format "~5,'0d" (inc! counter)))))
-
+    (^[] (format "~5,'0d" (inc! counter)))))
 
