@@ -1,23 +1,23 @@
 ;;;
 ;;; gauche.cgen.precomp - Precompile Scheme into C data
-;;;  
-;;;   Copyright (c) 2004-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2004-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;;; Precompiler takes compiled VM instruction array and dumps it
 ;;; as C code.
@@ -39,12 +39,12 @@
   (use srfi-13)
   (use gauche.cgen)
   (use gauche.cgen.stub)
+  (use gauche.cgen.tmodule)
   (use gauche.vm.insn)
   (use gauche.parameter)
   (use gauche.sequence)
-  (use gauche.experimental.app)
-  (use gauche.experimental.ref)
   (use gauche.experimental.lamb)
+  (use gauche.experimental.app)
   (use file.util)
   (use util.match)
   (use util.list)
@@ -151,49 +151,10 @@
 ;;      hygienic public macros expands to a call of private macros, and
 ;;      gauche.cgen.precomp cannot detect such dependencies yet.
 
-;; TO BE FIXED AFTER 0.9 RELEASE:
-;;  The keyword arguments marked 'TBF' in cgen-precomp and cgen-precomp-multi
-;;  should be fixed to the newer syntax ((:key var) init).  We use older
-;;  syntax here just to make the source buildable by 0.8.14.  After releasing
-;;  0.9 they should be changed to use the newer syntax.  See also the
-;;  pass1/extended-lambda for another temporary fix.
-(define (cgen-precompile src
-                         :key (out.c #f)
-                              (out.sci #f)
-                              (prefix :strip-prefix #f) ; TBF
-                              (ext-initializer #f)
-                              (dso :dso-name #f)        ; TBF
-                              (initializer-name #f)
-                              (sub-initializers '())
-                              (predef-syms '())
-                              (macros-to-keep '()))
-  (let ([out.c   (or out.c (path-swap-extension (sys-basename src) "c"))]
-        [out.sci (or out.sci
-                     (and (check-first-form-is-define-module src)
-                          (strip-prefix (path-swap-extension src "sci")
-                                        prefix)))])
-    ;; see PARAMETERS section below
-    (parameterize ([cgen-current-unit (get-unit src out.c predef-syms
-                                                ext-initializer)]
-                   [compile-module    (make-module #f)]
-                   [dso-name (cons
-                              (or dso
-                                  (and ext-initializer
-                                       (basename-sans-extension out.c)))
-                              initializer-name)]
-                   [vm-eval-situation SCM_VM_COMPILING]
-                   [private-macros-to-keep macros-to-keep])
-      (cond [out.sci
-             (make-directory* (sys-dirname out.sci))
-             (call-with-output-file out.sci
-               (lambda (p)
-                 (display ";; generated automatically.  DO NOT EDIT\n" p)
-                 (display "#!no-fold-case\n" p)
-                 (parameterize ([ext-module-file p])
-                   (do-it src ext-initializer sub-initializers))))]
-            [else
-             (parameterize ([ext-module-file #f])
-               (do-it src ext-initializer sub-initializers))]))))
+(define (cgen-precompile src . keys)
+  (with-tmodule-recording
+   <ptmodule>
+   (apply %cgen-precompile src keys)))
 
 (define (do-it src ext-initializer sub-initializers)
   (setup ext-initializer sub-initializers)
@@ -208,43 +169,97 @@
 ;; source is used to derive DSO name.
 (define (cgen-precompile-multi srcs
                                :key (ext-initializer #f)
-                                    (prefix :strip-prefix #f) ; TBF
-                                    (dso :dso-name #f)        ; TBF
+                                    ((:strip-prefix prefix) #f)
+                                    ((:dso-name dso) #f)
                                     (predef-syms '())
                                     (macros-to-keep '()))
   (match srcs
     [() #f]
     [(main . subs)
      (clean-output-files srcs prefix)
-     (dolist [src (order-files-by-dependency srcs)]
-       (let* ([out.c ($ xlate-cfilename
-                        $ strip-prefix (path-swap-extension src "c") prefix)]
-              [initname (string-tr (path-sans-extension out.c) "-+." "___")])
-         (cgen-precompile src
-                          :out.c out.c
-                          :dso-name (or dso (basename-sans-extension main))
-                          :predef-syms predef-syms
-                          :strip-prefix prefix
-                          :macros-to-keep macros-to-keep
-                          :ext-initializer (and (equal? src main)
-                                                ext-initializer)
-                          :initializer-name #`"Scm_Init_,initname")))]
+     (with-tmodule-recording
+      <ptmodule>
+      (dolist [src (order-files-by-dependency srcs)]
+        (let* ([out.c ($ xlate-cfilename
+                         $ strip-prefix (path-swap-extension src "c") prefix)]
+               [initname (string-tr (path-sans-extension out.c) "-+." "___")])
+          (%cgen-precompile src
+                            :out.c out.c
+                            :dso-name (or dso (basename-sans-extension main))
+                            :predef-syms predef-syms
+                            :strip-prefix prefix
+                            :macros-to-keep macros-to-keep
+                            :ext-initializer (and (equal? src main)
+                                                  ext-initializer)
+                            :initializer-name #`"Scm_Init_,initname"))))]
     ))
 
+;; Common stuff -- process single source
+(define (%cgen-precompile src
+                          :key (out.c #f)
+                               (out.sci #f)
+                               ((:strip-prefix prefix) #f)
+                               (ext-initializer #f)
+                               ((:dso-name dso) #f)
+                               (initializer-name #f)
+                               (sub-initializers '())
+                               (predef-syms '())
+                               (macros-to-keep '()))
+  (let ([out.c   (or out.c (path-swap-extension (sys-basename src) "c"))]
+        [out.sci (or out.sci
+                     (and (check-first-form-is-define-module src)
+                          (strip-prefix (path-swap-extension src "sci")
+                                        prefix)))])
+    ;; see PARAMETERS section below
+    (parameterize ([cgen-current-unit (get-unit src out.c predef-syms
+                                                ext-initializer)]
+                   [dso-name (cons
+                              (or dso
+                                  (and ext-initializer
+                                       (basename-sans-extension out.c)))
+                              initializer-name)]
+                   [vm-eval-situation SCM_VM_COMPILING]
+                   [private-macros-to-keep macros-to-keep])
+      (select-tmodule 'gauche)
+      (cond [out.sci
+             (make-directory* (sys-dirname out.sci))
+             (call-with-output-file out.sci
+               (^p (display ";; generated automatically.  DO NOT EDIT\n" p)
+                   (display "#!no-fold-case\n" p)
+                   (parameterize ([ext-module-file p])
+                     (do-it src ext-initializer sub-initializers))))]
+            [else
+             (parameterize ([ext-module-file #f])
+               (do-it src ext-initializer sub-initializers))]))))
+
+;;================================================================
+;; Transient modules
+;;
+
+;; To avoid interference between the host compiler and the target code,
+;; the compiler creates a temporary anonymous module and works in it.
+;; The basic mechanism is provided by gauche.cgen.tmodule.  We augument
+;; it with adding special bindings.
+
+(define-class <ptmodule> (<tmodule>) ())
+
+(define-method initialize ((m <ptmodule>) initargs)
+  (next-method)
+  ;; redefine several toplevel syntaxes precompiler needs to recognize
+  (for-each (^f (eval f (~ m'module))) *special-handlers*))
+
+(define (eval-in-current-tmodule expr)
+  (eval expr (~ (current-tmodule)'module)))
+
+(define (compile-in-current-tmodule expr)
+  (compile expr (~ (current-tmodule)'module)))
+
 ;;================================================================
 ;; Parameters
 ;;
 
-;; we compile things within an anonymous module to avoid interference.
-(define compile-module (make-parameter #f))
-
-;; keep the (supposed) name of the current module.  (current-module) syntax
-;; is compiled into (find-module ...) expression to lookup this name at
-;; runtime.
-(define compile-module-name (make-parameter #f))
-
 ;; A pair of the name of the generated DSO (w/o extension) and the name
-;; of the initializer function.  
+;; of the initializer function.
 (define dso-name (make-parameter #f))
 
 ;; keep the list of exported bindings (or #t if export-all)
@@ -269,17 +284,16 @@
 ;;================================================================
 ;; Bridge to the internal stuff
 ;;
-  
+
 ;; compatibility kludge
 (define compile       (with-module gauche.internal compile))
-(define compile-toplevel-lambda
-  (with-module gauche.internal compile-toplevel-lambda))
 (define %procedure-inliner
   (with-module gauche.internal %procedure-inliner))
 (define vm-code->list (with-module gauche.internal vm-code->list))
 (define vm-eval-situation
   (with-module gauche.internal vm-eval-situation))
 (define global-eq?? (with-module gauche.internal global-eq??))
+(define make-identifier (with-module gauche.internal make-identifier))
 
 (define-constant SCM_VM_COMPILING 2) ;; must match with vm.h
 
@@ -290,14 +304,16 @@
 (define (get-unit src out.c predef-syms ext-init?)
   (let* ([base (basename-sans-extension out.c)]
          [safe-name (string-tr base "-+" "__")])
-    (make <cgen-stub-unit>
-      :name base :c-name-prefix safe-name
-      :preamble `(,#`"/* Generated automatically from ,|src|.  DO NOT EDIT */")
-      :pre-decl (map (lambda (s) #`"#define ,s") predef-syms)
-      :init-prologue (format "~avoid Scm_Init_~a() { ScmModule *mod;"
-                             (if ext-init? "SCM_EXTENSION_ENTRY " "")
-                             safe-name)
-      )))
+    (rlet1 u (make <cgen-stub-unit>
+               :name base :c-name-prefix safe-name
+               :preamble `(,#`"/* Generated automatically from ,|src|.  DO NOT EDIT */")
+               :init-prologue (format "~avoid Scm_Init_~a() {"
+                                      (if ext-init? "SCM_EXTENSION_ENTRY " "")
+                                      safe-name)
+               )
+      (parameterize ([cgen-current-unit u])
+        (for-each cgen-define predef-syms)
+        (cgen-include "<gauche.h>")))))
 
 (define (strip-prefix path prefix)
   (cond
@@ -332,7 +348,7 @@
   (match (first-form src)
     [('define-module modname . forms)
      (list modname src
-           (filter-map (^(x)(match x [('use mod . _) mod] [_ #f])) forms))]
+           (filter-map (^x(match x [('use mod . _) mod] [_ #f])) forms))]
     [_ #f]))
 
 ;; Sort the given list of source files so that each file only depends on
@@ -340,9 +356,9 @@
 (define (order-files-by-dependency srcs)
   (let* ([deps (filter-map get-module-dependency srcs)]
          [sorted (topological-sort (map (^.[(n _ ns) (cons n ns)]) deps))]
-         [sorted-srcs (filter-map (^(s) (cond [(assq s deps) => cadr]
-                                               [else #f]))
-                                   sorted)]
+         [sorted-srcs (filter-map (^s (cond [(assq s deps) => cadr]
+                                            [else #f]))
+                                  sorted)]
          [unsorted-srcs (lset-difference string=? srcs sorted-srcs)])
     (append sorted-srcs unsorted-srcs)))
 
@@ -361,7 +377,7 @@
   (cgen-decl "#include <gauche/code.h>")
   (cgen-decl "#include <gauche/macro.h>") ; for MakeMacroTransformerOld. temporary.
   (cond [(and ext-init? (ext-module-file))
-         => (lambda (extm)
+         => (^[extm]
               (cgen-decl "#include <gauche/extend.h>")
               (let* ([extname ($ path-sans-extension
                                  $ sys-basename $ port-name extm)]
@@ -369,8 +385,6 @@
                 (cgen-init #`"SCM_INIT_EXTENSION(,safe-extname);")))])
   (dolist [init subinits]
     (cgen-decl #`"extern void Scm_Init_,init(void);"))
-
-  (setup-compiler-environment (compile-module))
   )
 
 (define (finalize subinits)
@@ -395,7 +409,7 @@
 ;;       compiled by the host compiler.
 ;;    2. use vm-find-insn-info (in gauche.vm.insn module) to map
 ;;       mnemonics to the target compiler's code.
-;;   
+;;
 ;;   For this scheme to work, the following conditions should be satisfied.
 ;;
 ;;    a. gauche.vm.insn should be the one generated from the same
@@ -418,7 +432,8 @@
 (define-syntax define-global-pred
   (syntax-rules ()
     [(_ name sym)
-     (define name (global-eq?? 'sym 'gauche compile-module))]))
+     (define name
+       (global-eq?? 'sym 'gauche (^[](~(current-tmodule)'module))))]))
 (define-global-pred =define-module?   define-module)
 (define-global-pred =select-module?   select-module)
 (define-global-pred =use?             use)
@@ -429,6 +444,16 @@
 (define-global-pred =lambda?          lambda)
 (define-global-pred =declare?         declare)
 
+;; KLUDGE: Include needs to refer to the path of including source file, which
+;; is taken from current-load-path in 0.9.2 and before.  If we were able to
+;; override that, we wouldn't need to intercept 'include'.  That should be the
+;; right thing, since 'include' doesn't need to appear in toplevel.
+;; For 0.9.3, however, we need to keep things compilable by 0.9.2 compiler
+;; whose behavior we can't change.  Hence this hack allows us to make 'include'
+;; work for the time being.
+(define-global-pred =include?         include)
+
+
 ;; compile FORM, and conses the toplevel code (something to be
 ;; executed at toplevel).
 (define (compile-toplevel-form form seed)
@@ -440,7 +465,7 @@
       ;; Module related stuff
       [((? =define-module?) mod . body)
        (write-ext-module form)
-       (parameterize ([compile-module-name mod])
+       (with-tmodule mod
          (fold compile-toplevel-form seed body))]
       [((? =select-module?) mod)
        (write-ext-module form)
@@ -450,37 +475,44 @@
          [(name . initfn)
           (write-ext-module `(dynamic-load ,name :init-function ,initfn))]
          [_ #f])
-       (let1 sym (cgen-literal mod)
-         (cgen-init
-          (format "  mod = Scm_FindModule(SCM_SYMBOL(~a),\
-                                          SCM_FIND_MODULE_CREATE);"
-                  (cgen-cexpr sym))
-          ;; force the current module to be mod
-          "  Scm_SelectModule(mod);"))
-       (compile-module-name mod)
+       (select-tmodule mod)
        seed]
       [((? =use?) mod)
-       (eval `(use ,mod) (compile-module)) seed]
+       (eval-in-current-tmodule `(use ,mod)) seed]
       [((? =export?) . syms)
        (when (list? (compile-module-exports))
          (compile-module-exports
           (lset-union eq? syms (compile-module-exports))))
-       (eval `(export ,@syms) (compile-module)) seed]
+       (eval-in-current-tmodule `(export ,@syms)) seed]
       [((? =export-all?)) (compile-module-exports #t)]
       [((? =export-if-defined?) . _) (write-ext-module form) seed]
       [((? =provide?) arg) (write-ext-module form) seed]
       ;; TODO - we need more general framework supporting various declarations.
       ;; for the time being, this ad-hoc solution suffice our needs.
       [((? =declare?) decls ...)
-       (for-each (^.[('keep-private-macro . macros)
-                     (private-macros-to-keep (append (private-macros-to-keep)
-                                                     macros))]
-                    [other (error "Unknown declaration:" other)])
-                 decls)
+       (dolist [x decls]
+         (match x
+           [('keep-private-macro . macros)
+            (private-macros-to-keep (append (private-macros-to-keep) macros))]
+           [other (error "Unknown declaration:" other)]))
        seed]
+      ;; TODO: Temporary kludge---remove in the future release.  See the
+      ;; above comment about 'include' kludge.
+      [((? =include?) path)
+       (let* ([spath (if (absolute-path? path)
+                       path
+                       (build-path ($ sys-dirname $ port-name
+                                      $ current-input-port)
+                                   path))]
+              [source (any (^[ext] (let1 p #`",|spath|,|ext|"
+                                     (and (file-is-readable? p) p)))
+                           (cons "" *load-suffixes*))])
+         (unless source
+           (errorf "Couldn't read included file ~s" path))
+         (compile-toplevel-form `(begin ,@(file->sexp-list source)) seed))]
       ;; Finally, ordinary expressions.
       [else
-       (let1 compiled-code (compile form (compile-module))
+       (let1 compiled-code (compile-in-current-tmodule form)
          ;; We exclude a compiled code with only CONSTU-RET, which appears
          ;; as the result of macro expansion sometimes.
          (if (toplevel-constu-ret-code? compiled-code)
@@ -526,15 +558,18 @@
 (define *special-handlers*
   '((define-macro (current-module)
       `(find-module ',(with-module gauche.cgen.precomp
-                        (compile-module-name))))
+                        (~(current-tmodule)'name))))
     (define-macro (inline-stub . forms)
-      (for-each (lambda (s)
-                  ((with-module gauche.cgen.stub cgen-stub-parse-form)
-                   (unwrap-syntax s)))
-                forms)
+      (dolist [s forms]
+        ((with-module gauche.cgen.stub cgen-stub-parse-form)
+         (unwrap-syntax s)))
       (undefined))
-    (define-macro (define . f)
-      ((with-module gauche.cgen.precomp handle-define) f))
+    (define-macro (define-cproc . args)
+      ((with-module gauche.cgen.stub cgen-stub-parse-form)
+       (unwrap-syntax (cons 'define-cproc args)))
+      (undefined))
+    ;; (define-macro (define . f)
+    ;;   ((with-module gauche.cgen.precomp handle-define) f))
     (define-macro (define-constant . f)
       ((with-module gauche.cgen.precomp handle-define-constant) f))
     (define-macro (define-syntax . f)
@@ -543,32 +578,34 @@
       ((with-module gauche.cgen.precomp handle-define-macro) f))
     ))
 
-(define (setup-compiler-environment mod)
-  (dolist [form *special-handlers*]
-    (eval form mod)))
+;; Macros are "consumed" by the Gauche's compiler---that is, it is
+;; executed inside the compiler and won't appear in the compiled output.
+;; For exported macros, we should include the macro itself in the compiled
+;; file, so we intercept define-macro and define-syntax.
 
 ;; For the time being, we only compile the legacy macros into C file.
 ;; R5RS macros are put in ext-module file as is.
 (define (handle-define-macro form)
+  (define %define (make-identifier 'define (find-module 'gauche) '()))
+  (define %define-syntax (make-identifier 'define-syntax (find-module 'gauche) '()))
+  (define %lambda (make-identifier 'lambda (find-module 'gauche) '()))
+  (define %begin (make-identifier 'begin (find-module 'gauche) '()))
+  (define %macro (make-identifier 'make-macro-transformer
+                                  (find-module 'gauche.internal) '()))
+  (define (do-handle name expr)
+    (if (or (symbol-exported? name)
+            (memq name (private-macros-to-keep)))
+      `(,%begin
+        (,%define ,name (,%macro ',name ,expr))
+        ((with-module gauche define-macro) ,name ,expr))
+      `((with-module gauche define-macro) ,name ,expr)))
+
   (match form
     [((name . formals) . body)
-     (when (or (symbol-exported? name)
-               (memq name (private-macros-to-keep)))
-       (let* ([body-closure (compile-toplevel-lambda form name formals
-                                                     body (compile-module))]
-              [code (cgen-literal (closure-code body-closure))]
-              [var  (cgen-literal name)])
-         (cgen-init
-          (format "  Scm_Define(mod, SCM_SYMBOL(~a), \
-                             Scm_MakeMacroTransformerOld(SCM_SYMBOL(~a),\
-                                 SCM_PROCEDURE(Scm_MakeClosure(~a, NULL)))); /* ~s */"
-                  (cgen-cexpr var) (cgen-cexpr var)
-                  (cgen-cexpr code) (cgen-safe-comment name)))))]
-    [(name . expr)
-     (when (symbol-exported? name)
-       (write-ext-module `(define-macro . ,form)))]
-    [_ #f])
-  (cons '(with-module gauche define-macro) form))
+     (handle-define-macro `(,name
+                            (,%lambda ,formals ,@body)))]
+    [(name expr) (do-handle name expr)]
+    [_ (error "Malformed define-macro" form)]))
 
 (define (handle-define-syntax form)
   (match form
@@ -577,28 +614,13 @@
                (memq name (private-macros-to-keep)))
        (write-ext-module `(define-syntax . ,form)))]
     [_ #f])
-  (cons '(with-module gauche define-syntax) form))  
-
-(define (handle-define form)
-  (match form
-    [((name . args) . body)
-     (handle-define `(,name (lambda ,args ,@body)))]
-    [((? symbol? name) ((? =lambda?) args . body))
-     (let* ([closure
-             (compile-toplevel-lambda form name args body (compile-module))]
-            [code (cgen-literal (closure-code closure))]
-            [var  (cgen-literal name)])
-       (cgen-init
-        (format "  Scm_Define(mod, SCM_SYMBOL(~a), Scm_MakeClosure(~a, NULL)); /* ~s */"
-                (cgen-cexpr var) (cgen-cexpr code) (cgen-safe-comment name))))
-     (undefined)]
-    [_
-     (cons '(with-module gauche define) form)]))
+  (cons '(with-module gauche define-syntax) form))
 
 (define (handle-define-constant form)
   (match form
     [((? symbol? name) expr)
-     (eval `((with-module gauche define-constant) ,@form) (compile-module))]
+     (eval-in-current-tmodule
+      `((with-module gauche define-constant) ,@form))]
     [_ #f])
   (cons '(with-module gauche define-constant) form))
 
@@ -609,20 +631,19 @@
 
 ;;================================================================
 ;; Compiler-specific literal handling definitions
-;;       
+;;
 (define-cgen-literal <cgen-scheme-code> <compiled-code>
   ((code-name   :init-keyword :code-name)
    (code-vector-c-name :init-keyword :code-vector-c-name)
    (literals    :init-keyword :literals)
    )
   (make (value)
-    (let* ((cv  (vm-code->list value))
-           (lv  (extract-literals cv))
-           (cvn (allocate-code-vector cv lv (~ value'full-name)))
-           (code-name (cgen-literal (~ value'name)))
-           (arg-info (cgen-literal (~ value'arg-info)))
-           (inliner (check-packed-inliner value))
-           )
+    (let* ([cv  (vm-code->list value)]
+           [lv  (extract-literals cv)]
+           [cvn (allocate-code-vector cv lv (~ value'full-name))]
+           [code-name (cgen-literal (~ value'name))]
+           [arg-info (cgen-literal (~ value'arg-info))]
+           [inliner (check-packed-inliner value)])
       (define (init-thunk)
         (format #t "    SCM_COMPILED_CODE_CONST_INITIALIZER(  /* ~a */\n"
                 (cgen-safe-comment (~ value'name)))
@@ -648,10 +669,10 @@
             :code-name code-name
             :literals lv)))
   (init (self)
-    (unless (cgen-literal-static? [~ self'code-name])
-      (print "  SCM_COMPILED_CODE("[~ self'c-name]")->name = "
-             (cgen-cexpr [~ self'code-name])";"
-             "/* "(cgen-safe-comment [~ self'value'full-name])" */"))
+    (unless (cgen-literal-static? (~ self'code-name))
+      (print "  SCM_COMPILED_CODE("(~ self'c-name)")->name = "
+             (cgen-cexpr (~ self'code-name))";"
+             "/* "(cgen-safe-comment (~ self'value'full-name))" */"))
     (fill-code self))
   (static (self) #t)
   )
@@ -660,8 +681,8 @@
 ;; <cgen-literal>s corresponding to the literal values in CODE.
 ;; #f is filled in the places that don't have corresponding litaral value.
 (define (extract-literals code)
-  (let loop ((code code)
-             (lits '()))
+  (let loop ([code code]
+             [lits '()])
     (if (null? code)
       (reverse lits)
       (let* ([insn (car code)]
@@ -741,12 +762,10 @@
   (let ([cvn  (~ code'code-vector-c-name)]
         [lv   (~ code'literals)])
     (for-each-with-index
-     (lambda (index lit)
-       (when (and lit (not (cgen-literal-static? lit)))
-         (format #t "  ((ScmWord*)~a)[~a] = SCM_WORD(~a);\n"
-                 cvn index (cgen-cexpr lit))))
-     lv)
-    ))
+     (^[index lit] (when (and lit (not (cgen-literal-static? lit)))
+                     (format #t "  ((ScmWord*)~a)[~a] = SCM_WORD(~a);\n"
+                             cvn index (cgen-cexpr lit))))
+     lv)))
 
 ;; If the compiled-code has packed IForm for inliner, translate it for
 ;; the target VM insns and returns the packed IForm.
@@ -769,16 +788,29 @@
     (make <cgen-scheme-identifier> :value value
           :c-name (cgen-allocate-static-datum)
           :id-name (cgen-literal (~ value'name))
-          :mod-name (and-let* ([modnam (module-name-fix (~ value'module))])
-                      (cgen-literal modnam))))
+          :mod-name (cond [(module-name-fix (~ value'module))
+                           => cgen-literal]
+                          [(current-tmodule) => (^m (cgen-literal (~ m'name)))]
+                          [else #f])))
   (init (self)
     (let ([name (cgen-cexpr (~ self'id-name))]
           [cname (~ self'c-name)])
       (or (and-let* ([modnam (~ self'mod-name)])
-            (print "  "cname" = Scm_MakeIdentifier(SCM_SYMBOL("name"), "
-                   "Scm_FindModule(SCM_SYMBOL("(cgen-cexpr modnam)"), SCM_FIND_MODULE_CREATE),"
-                   "SCM_NIL);"))
-          (print "  "cname" = Scm_MakeIdentifier(SCM_SYMBOL("name"), mod, SCM_NIL);"))))
+            (format #t "  ~a = Scm_MakeIdentifier(SCM_SYMBOL(~a), \
+                                  Scm_FindModule(SCM_SYMBOL(~a), \
+                                                 SCM_FIND_MODULE_CREATE),
+                                  SCM_NIL); /* ~a#~a */\n"
+                    cname name (cgen-cexpr modnam)
+                    (cgen-safe-comment (~ self'mod-name'value))
+                    (cgen-safe-comment (~ self'id-name'value)))
+            #t)
+          (let1 mod-cname (current-tmodule-cname)
+            (format #t "  ~a = Scm_MakeIdentifier(SCM_SYMBOL(~a), \
+                                                  SCM_MODULE(~a), \
+                                                  SCM_NIL); /* ~a#~a */\n"
+                    cname name mod-cname
+                    (cgen-safe-comment (~(current-tmodule)'name))
+                    (cgen-safe-comment (~ self'id-name'value)))))))
   (static (self) #f)
   )
 
@@ -806,8 +838,27 @@
       :c-name  (cgen-allocate-static-datum)
       :gf-name (cgen-literal (ref value 'name))))
   (init (self)
-    (format #t "  ~a = Scm_SymbolValue(mod, SCM_SYMBOL(~a));\n"
-            (~ self'c-name) (~ self'gf-name'c-name)))
+    (format #t "  ~a = Scm_SymbolValue(~a, SCM_SYMBOL(~a));\n"
+            (~ self'c-name)
+            (current-tmodule-cname)
+            (~ self'gf-name'c-name)))
   (static (self) #f)
   )
 
+;; We allow literal closures if it doens't close environment.
+;; Closures do not have its own class, so we define cgen-literal class
+;; for <procedure>.
+(define-cgen-literal <cgen-closure> <procedure>
+  ([code :init-keyword :code])  ; <cgen-scheme-code>
+  (make (value)
+    (unless (toplevel-closure? value)
+      (error "a procedure (except top-level closure) cannot be \
+              a compile-time constant:" value))
+    (make <cgen-closure>
+      :value value :c-name (cgen-allocate-static-datum)
+      :code (cgen-literal (closure-code value))))
+  (init (self)
+    (format #t "  ~a = Scm_MakeClosure(~a, NULL); /* ~a */\n"
+            (cgen-cexpr self) (cgen-cexpr (~ self'code))
+            (cgen-safe-comment (write-to-string (~ self'value)))))
+  (static (self) #f))
