@@ -1,23 +1,23 @@
 ;;;
 ;;; gauce.cgen.stub - stub forms parsing and code generation
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module gauche.cgen.stub
   (use srfi-1)
@@ -46,8 +46,7 @@
   (use gauche.cgen.unit)
   (use gauche.cgen.literal)
   (use gauche.cgen.cise)
-  (use gauche.experimental.ref)
-  (use gauche.experimental.lamb)
+  (use gauche.cgen.tmodule)
   (export <cgen-stub-unit> <cgen-stub-error>
           cgen-genstub
           cgen-stub-parser cgen-stub-parse-form)
@@ -65,7 +64,7 @@
 ;;
 ;;      Create a subr function.
 ;;
-;;      <args> specifies arguments.   
+;;      <args> specifies arguments.
 ;;
 ;;       (<arg> ... [:rest <var>])
 ;;          Each <arg> is variable name or var::type, specifies required
@@ -89,7 +88,7 @@
 ;;          argument in the ScmObj array.  <max> specifies the maximum number
 ;;          of optional arguments that can be passed in the array form.
 ;;          If more than <max> args are given, a list of excessive arguments
-;;          are passed to the rest <var> if it is specified, or 
+;;          are passed to the rest <var> if it is specified, or
 ;;
 ;;      <Rettype> specifies the return type of SUBR.
 ;;
@@ -235,21 +234,24 @@
       (cgen-safe-name-friendly (path-sans-extension (sys-basename stubfile)))
     (parameterize ([cgen-current-unit
                     (make <cgen-stub-unit>
-                      :name (path-sans-extension stubfile)
+                      :name (path-sans-extension (sys-basename stubfile))
                       :c-name-prefix (string-append prefix "_")
                       :preamble '("/* Generated by genstub.  Do not edit. */")
-                      :pre-decl (map (cut format "#define ~a" <>) predef-syms)
                       :init-prologue (format "void Scm_Init_~a(ScmModule *mod)\
                                               {\n" prefix))])
-      (with-input-from-file stubfile
-        (cut port-fold
-             ;; We treat the initial raw strings specially---they will be
-             ;; in decl part.  The flag decl-strings? tracks that.
-             (lambda (form decl-strings?)
-               (cond [(and decl-strings? (string? form)) (cgen-decl form) #t]
-                     [else (cgen-stub-parse-form form) #f]))
-             #t read))
-      (cgen-emit-c (cgen-current-unit)))))
+      (with-tmodule-recording
+       <tmodule>
+       (for-each cgen-define predef-syms)
+       (cgen-include "<gauche.h>")
+       (with-input-from-file stubfile
+         (cut port-fold
+              ;; We treat the initial raw strings specially---they will be
+              ;; in decl part.  The flag decl-strings? tracks that.
+              (^[form decl-strings?]
+                (cond [(and decl-strings? (string? form)) (cgen-decl form) #t]
+                      [else (cgen-stub-parse-form form) #f]))
+              #t read))
+       (cgen-emit-c (cgen-current-unit))))))
 
 ;; utilities
 (define (f fmt . args) (apply format #t fmt args) (newline))
@@ -277,11 +279,11 @@
    (handler :init-keyword :handler)))
 
 (define-macro (define-form-parser name args . body)
-  `(make <form-parser> :name ',name :args ',args :handler (lambda ,args ,@body)))
+  `(make <form-parser> :name ',name :args ',args :handler (^ ,args ,@body)))
 
 (define-method invoke ((self <form-parser>) form)
   (define (badform)
-    (errorf <cgen-stub-error> "malformed ~a: ~s" (~ self'name) form))
+    (errorf <cgen-stub-error> "stub: malformed ~a: ~s" (~ self'name) form))
   (apply (~ self'handler)
          ;; need to check if given form matches args
          (let loop ([llist (~ self'args)]
@@ -295,7 +297,7 @@
 
 (define (cgen-stub-parser key)
   (cond [(find (^p (eq? key (~ p'name))) (instance-pool->list <form-parser>))
-         => (lambda (parser) (cut invoke parser <>))]
+         => (^[parser] (cut invoke parser <>))]
         [else #f]))
 
 (define (cgen-stub-parse-form form)
@@ -340,12 +342,23 @@
 
 (define-class <stub> (<cgen-node>)
   ((scheme-name :init-keyword :scheme-name :init-form #f)
+      ;; Scheme name (symbol) of the object this stub represents.
+      ;; Can be #f if the object is anonymous.
    (c-name      :init-keyword :c-name)
+      ;; C expression of type ScmObj to refer to the object in C program.
+   (tmodule     :init-form (current-tmodule))
+      ;; The current transient module when this stub is parsed.
+      ;; Can be #f.  See stub-tmodule-cname below
    ))
 
 (define (get-stubs class)
   (filter (cut is-a? <> class) (cgen-unit-toplevel-nodes (cgen-current-unit))))
 
+;; convenience routine.
+(define (stub-tmodule-cname stub)
+  (cond [(~ stub'tmodule) => (cut ~ <> 'cname)]
+        [else (current-tmodule-cname)]))
+
 ;;===================================================================
 ;; Literals
 ;;
@@ -437,12 +450,13 @@
 ;; (define-enum name) - a special case of define-constant
 
 (define (variable-parser-common const? name init opts)
-  (let ((c-name  (get-keyword :c-name opts #f))
-        (symbol  (make-literal name))
-        (initval (make-literal init)))
+  (let ([c-name  (get-keyword :c-name opts #f)]
+        [symbol  (make-literal name)]
+        [initval (make-literal init)])
     (when c-name
       (cgen-decl #`"#define ,c-name (,(cgen-cexpr symbol))"))
-    (cgen-init (format "  Scm_MakeBinding(mod, SCM_SYMBOL(~a), ~a, ~a);\n"
+    (cgen-init (format "  Scm_MakeBinding(SCM_MODULE(~a), SCM_SYMBOL(~a), ~a, ~a);\n"
+                       (current-tmodule-cname)
                        (cgen-cexpr symbol)
                        (cgen-cexpr initval)
                        (if const? "SCM_BINDING_CONST" "0")))
@@ -463,7 +477,7 @@
 
 (define-form-parser define-enum-conditionally (name)
   (check-arg symbol? name)
-  (parameterize ((cgen-cpp-condition #`"defined(,name)"))
+  (cgen-with-cpp-condition `(defined ,name)
     (variable-parser-common #t name (list 'c #`"Scm_MakeInteger(,name)") '())))
 
 ;;-------------------------------------------------------------------
@@ -590,7 +604,7 @@
     (errorf <cgen-stub-error> "bad argument in argspec: ~a in ~a" arg name))
 
   ;; support old &-notation.  will fade away.
-  (define (xlate-old-lambda-keywords specs) 
+  (define (xlate-old-lambda-keywords specs)
     (map (^s (cond [(assq s '((&optional . :optional) (&keyword . :key)
                               (&rest . :rest)
                               (&allow-other-keys . :allow-other-keys)))
@@ -721,7 +735,7 @@
       [(('body . spec) . r) (process-body-spec cproc spec) (loop r)]
       [(('expr . spec) . r) (process-expr-spec cproc spec) (loop r)]
       [(('catch . spec) . r) (process-catch-spec cproc spec) (loop r)]
-      [(('code . stmts) . r) (dolist (s stmts) (push-stmt! cproc s)) (loop r)]
+      [(('code . stmts) . r) (dolist [s stmts] (push-stmt! cproc s)) (loop r)]
       [([? symbol? s]) ; 'call' convention
        ;; If the named C routine returns multiple values, we don't need to
        ;; call Scm_Values*() by ourselves.  We generate an appropriate cise
@@ -761,16 +775,16 @@
     (receive (args keyargs nreqs nopts rest? other-keys?)
         (process-cproc-args (ref cproc'proc-name) (car decl))
       (receive (body rettype) (extract-rettype (cdr decl))
-        (let ((setter (make <cproc>
-                        :scheme-name `(setter ,(~ cproc'scheme-name))
-                        :c-name #`",(~ cproc'c-name)_SETTER"
-                        :proc-name (make-literal (x->string `(setter ,(~ cproc'scheme-name))))
-                        :args args :return-type rettype
-                        :keyword-args keyargs
-                        :num-reqargs nreqs
-                        :num-optargs nopts
-                        :have-rest-arg? rest?
-                        :allow-other-keys? other-keys?)))
+        (let1 setter (make <cproc>
+                       :scheme-name `(setter ,(~ cproc'scheme-name))
+                       :c-name #`",(~ cproc'c-name)_SETTER"
+                       :proc-name (make-literal (x->string `(setter ,(~ cproc'scheme-name))))
+                       :args args :return-type rettype
+                       :keyword-args keyargs
+                       :num-reqargs nreqs
+                       :num-optargs nopts
+                       :have-rest-arg? rest?
+                       :allow-other-keys? other-keys?)
           (set! (~ cproc'setter) #`",(~ setter'c-name)__STUB")
           (process-body setter body)
           (cgen-add! setter))))]))
@@ -803,6 +817,7 @@
     [else (err)]))
 
 (define-method process-body-spec ((cproc <procstub>) form)
+  (define (err) (error <cgen-stub-error> "malformed 'body' spec:" form))
   (match form
     [((? symbol? rettype) . stmts)
      (process-body-inner cproc rettype stmts)]
@@ -813,8 +828,9 @@
 
 (define-method process-expr-spec ((cproc <procstub>) form)
   (define (typed-result rettype expr)
-    (let1 expr (if (string? expr) expr
-                   (call-with-output-string (cut cise-render expr <> 'expr)))
+    (let1 expr (if (string? expr)
+                 expr
+                 (call-with-output-string (cut cise-render expr <> 'expr)))
       (push-stmt! cproc "{")
       (push-stmt! cproc #`",(~ rettype'c-type) SCM_RESULT;")
       (push-stmt! cproc #`" SCM_RESULT = (,expr);")
@@ -867,15 +883,14 @@
   (define (typed-results rettypes stmts)
     (let1 nrets (length rettypes)
       (for-each-with-index
-       (lambda (i rettype)
-         (push-stmt! cproc #`",(~ rettype'c-type) SCM_RESULT,i;"))
+       (^[i rettype] (push-stmt! cproc #`",(~ rettype'c-type) SCM_RESULT,i;"))
        rettypes)
       (push-stmt! cproc "{")
       (for-each expand-stmt stmts)
       (push-stmt! cproc "}")
       (let1 results
           (string-join
-           (map-with-index (lambda (i rettype)
+           (map-with-index (^[i rettype]
                              (cgen-box-expr rettype #`"SCM_RESULT,i"))
                            rettypes)
            ",")
@@ -912,8 +927,15 @@
   ;; argument decl
   (for-each emit-arg-decl (~ cproc'args))
   (for-each emit-arg-decl (~ cproc'keyword-args))
-  (unless (null? (~ cproc'keyword-args))
-    (p "  ScmObj SCM_OPTARGS = SCM_ARGREF(SCM_ARGCNT-1);"))
+  (if (null? (~ cproc'keyword-args))
+    (p "  ScmObj SCM_SUBRARGS["(+ (length (~ cproc'args))
+                                  (~ cproc'num-optargs))"];")
+    (begin
+      (p "  ScmObj SCM_SUBRARGS["(+ (length (~ cproc'args))
+                                    (~ cproc'num-optargs)
+                                    -1)"];")
+      (p "  ScmObj SCM_OPTARGS = SCM_ARGREF(SCM_ARGCNT-1);")))
+  (p "  int SCM_i;")
   (p "  SCM_ENTER_SUBR(\""(~ cproc'scheme-name)"\");")
   ;; argument count check (for optargs)
   (when (and (> (~ cproc'num-optargs) 0)
@@ -926,6 +948,11 @@
        " is expected, %d given.\", "
        "SCM_ARGCNT + Scm_Length(SCM_ARGREF(SCM_ARGCNT-1)) - 1);"))
   ;; argument assertions & unbox op.
+  (let1 k (+ (length (~ cproc'args)) (~ cproc'num-optargs)
+             (if (null? (~ cproc'keyword-args)) 0 -1))
+    (p "  for (SCM_i=0; SCM_i<"k"; SCM_i++) {")
+    (p "    SCM_SUBRARGS[SCM_i] = SCM_ARGREF(SCM_i);")
+    (p "  }"))
   (for-each emit-arg-unbox (~ cproc'args))
   (unless (null? (~ cproc'keyword-args))
     (emit-keyword-args-unbox cproc))
@@ -937,7 +964,7 @@
   (p "  }")
   (unless (null? (~ cproc'c++-handlers))
     (p "}")
-    (dolist (h (~ cproc'c++-handlers))
+    (dolist [h (~ cproc'c++-handlers)]
       (f "catch (~a) {" (car h))
       (for-each p (cdr h))
       (p "}")))
@@ -971,7 +998,8 @@
 
 (define-method cgen-emit-init ((cproc <cproc>))
   (when (symbol? (~ cproc'scheme-name))
-    (f "  Scm_MakeBinding(mod, SCM_SYMBOL(SCM_INTERN(~s)), SCM_OBJ(&~a), ~a);"
+    (f "  Scm_MakeBinding(SCM_MODULE(~a), SCM_SYMBOL(SCM_INTERN(~s)), SCM_OBJ(&~a), ~a);"
+       (stub-tmodule-cname cproc)
        (symbol->string (~ cproc'scheme-name))
        (c-stub-name cproc)
        (if (or (~ cproc'inline-insn) (memq :constant (~ cproc'flags)))
@@ -986,7 +1014,7 @@
   (match (~ cproc'setter)
     [(? string? x) (emit x)]
     [(? symbol? x)
-     (or (and-let* ([setter (find (lambda (z) (eq? (~ z'scheme-name) x))
+     (or (and-let* ([setter (find (^z (eq? (~ z'scheme-name) x))
                                   (get-stubs <stub>))])
            (emit (c-stub-name setter)))
          (errorf <cgen-stub-error>
@@ -1017,29 +1045,29 @@
       (p "  "(~ arg'c-name)" = "(~ arg'scm-name)";"))))
 
 (define-method emit-arg-unbox ((arg <required-arg>))
-  (p "  "(~ arg'scm-name)" = SCM_ARGREF("(~ arg'count)");")
+  (p "  "(~ arg'scm-name)" = SCM_SUBRARGS["(~ arg'count)"];")
   (emit-arg-unbox-rec arg))
 
 (define-method emit-arg-unbox ((arg <optional-arg>))
   (p "  if (SCM_ARGCNT > "(~ arg'count)"+1) {")
-  (p "    "(~ arg'scm-name)" = SCM_ARGREF("(~ arg'count)");")
+  (p "    "(~ arg'scm-name)" = SCM_SUBRARGS["(~ arg'count)"];")
   (p "  } else {")
   (p "    "(~ arg'scm-name)" = "(get-arg-default arg)";")
   (p "  }")
   (emit-arg-unbox-rec arg))
- 
+
 (define-method emit-arg-unbox ((arg <rest-arg>))
-  (p "  "(~ arg'scm-name)" = SCM_ARGREF(SCM_ARGCNT-1);")
+  (p "  "(~ arg'scm-name)" = SCM_SUBRARGS[SCM_ARGCNT-1];")
   (emit-arg-unbox-rec arg))
 
 (define-method emit-arg-unbox ((arg <optarray-arg>))
-  (p "  "(~ arg'c-name)" = &SCM_ARGREF("(~ arg'count)");")
+  (p "  "(~ arg'c-name)" = &SCM_SUBRARGS["(~ arg'count)"];")
   (p "  "(get-c-name "" (~ arg'count-var))" = SCM_ARGCNT-1-"(~ arg'count)";"))
 
 (define (get-arg-default arg)
   (cond [(~ arg'default) => cgen-cexpr]
         [else "SCM_UNBOUND"]))
- 
+
 (define (emit-keyword-args-unbox cproc)
   (let ([args (~ cproc'keyword-args)]
         [other-keys? (~ cproc'allow-other-keys?)])
@@ -1047,15 +1075,14 @@
     (p "    Scm_Error(\"keyword list not even: %S\", SCM_OPTARGS);")
     (p "  while (!SCM_NULLP(SCM_OPTARGS)) {")
     (pair-for-each
-     (lambda (args)
-       (let ([arg (car args)]
-             [tail? (null? (cdr args))])
-         (f "    if (SCM_EQ(SCM_CAR(SCM_OPTARGS), ~a)) {"
-            (cgen-c-name (~ arg'keyword)))
-         (f "      ~a = SCM_CADR(SCM_OPTARGS);" (~ arg'scm-name))
-         (if tail?
-           (p "    }")
-           (p "    } else "))))
+     (^[args] (let ([arg (car args)]
+                    [tail? (null? (cdr args))])
+                (f "    if (SCM_EQ(SCM_CAR(SCM_OPTARGS), ~a)) {"
+                   (cgen-c-name (~ arg'keyword)))
+                (f "      ~a = SCM_CADR(SCM_OPTARGS);" (~ arg'scm-name))
+                (if tail?
+                  (p "    }")
+                  (p "    } else "))))
      args)
     (unless other-keys?
       (p "    else Scm_Warn(\"unknown keyword %S\", SCM_CAR(SCM_OPTARGS));"))
@@ -1086,19 +1113,21 @@
   (p))
 
 (define-method cgen-emit-init ((self <cgeneric>))
-  (f "  Scm_InitBuiltinGeneric(&~a, ~s, mod);"
-     (~ self'c-name) (symbol->string (~ self'scheme-name)))
+  (f "  Scm_InitBuiltinGeneric(&~a, ~s, SCM_MODULE(~a));"
+     (~ self'c-name) (symbol->string (~ self'scheme-name))
+     (stub-tmodule-cname self))
   (next-method))
 
 (define-form-parser define-cgeneric (scheme-name c-name . body)
   (check-arg symbol? scheme-name)
   (check-arg string? c-name)
-  (let ((gf (make <cgeneric> :scheme-name scheme-name :c-name c-name)))
-    (for-each (^.[('extern) (set! [~ gf'extern?] #t)]
-                 [('fallback (? string? fallback))
-                  (set! (~ gf'fallback) (cadr form))]
-                 [('setter . spec) (process-setter gf spec)]
-                 [form (error <cgen-stub-error> "bad gf form:" form)])
+  (let1 gf (make <cgeneric> :scheme-name scheme-name :c-name c-name)
+    (for-each (match-lambda
+                [('extern) (set! [~ gf'extern?] #t)]
+                [('fallback (? string? fallback))
+                 (set! (~ gf'fallback) (cadr form))]
+                [('setter . spec) (process-setter gf spec)]
+                [form (error <cgen-stub-error> "bad gf form:" form)])
               body)
     (cgen-add! gf)))
 
@@ -1131,17 +1160,17 @@
   (receive (args specializers numargs have-optarg?)
       (parse-specialized-args argspec)
     (receive (body rettype) (extract-rettype body)
-      (let ([method (make <cmethod>
-                      :scheme-name scheme-name
-                      :c-name (get-c-name (~(cgen-current-unit)'c-name-prefix)
-                                          (gensym (x->string scheme-name)))
-                      :return-type rettype
-                      :specializers specializers
-                      :num-reqargs numargs
-                      :args args
-                      :have-rest-arg? have-optarg?
-                      )])
-        (let loop ((body body))
+      (let1 method (make <cmethod>
+                     :scheme-name scheme-name
+                     :c-name (get-c-name (~(cgen-current-unit)'c-name-prefix)
+                                         (gensym (x->string scheme-name)))
+                     :return-type rettype
+                     :specializers specializers
+                     :num-reqargs numargs
+                     :args args
+                     :have-rest-arg? have-optarg?
+                     )
+        (let loop ([body body])
           (match body
             [() #f]
             [([? string?] . r) (push-stmt! method stmt) (loop r)]
@@ -1183,8 +1212,16 @@
      (~ method'c-name))
   (p "{")
   (for-each emit-arg-decl (~ method'args))
-  (when (~ method'have-rest-arg?)
-    (p "  ScmObj SCM_OPTARGS = SCM_ARGREF(SCM_ARGCNT-1);"))
+  (if (~ method'have-rest-arg?)
+    (begin (p "  ScmObj SCM_SUBRARGS["(+ (length (~ method'args)) -1)"];")
+           (p "  ScmObj SCM_OPTARGS = SCM_ARGREF(SCM_ARGCNT-1);"))
+    (p "  ScmObj SCM_SUBRARGS["(length (~ method'args))"];"))
+  (p "  int SCM_i;")
+  (let1 k (+ (length (~ method'args))
+             (if (~ method'have-rest-arg?) -1 0))
+    (p "  for (SCM_i=0; SCM_i<"k"; SCM_i++) {")
+    (p "    SCM_SUBRARGS[SCM_i] = SCM_ARGREF(SCM_i);")
+    (p "  }"))
   (for-each emit-arg-unbox (~ method'args))
   ;; body
   (p "  {")
@@ -1193,7 +1230,7 @@
   (p "}")
   (p "")
   (p "static ScmClass *"(~ method'c-name)"__SPEC[] = { ")
-  (for-each (lambda (spec) (p "SCM_CLASS_STATIC_PTR("spec"), "))
+  (for-each (^[spec] (p "SCM_CLASS_STATIC_PTR("spec"), "))
             (reverse (~ method'specializers)))
   (p "};")
   (f "static SCM_DEFINE_METHOD(~a__STUB, &~a, ~a, ~a, ~a__SPEC, ~:*~a, NULL);"
@@ -1209,9 +1246,9 @@
 ;; returns four values: args, specializers, numreqargs, have-optarg?
 (define (parse-specialized-args arglist)
   (define (badlist) (error <cgen-stub-error> "malformed arglist:" arglist))
-  (let loop ((arglist arglist)
-             (args    '())
-             (specs   '()))
+  (let loop ([arglist arglist]
+             [args    '()]
+             [specs   '()])
     (cond [(null? arglist)
            (values (reverse args) specs (length args) #f)]
           [(symbol? arglist)
@@ -1261,7 +1298,7 @@
 ;;                  [:c-spec <c-spec>]
 ;;                  [:getter <proc-spec>]
 ;;                  [:setter <proc-spec>])
-;;                  
+;;
 ;; <proc-spec> := <c-code> | (c <c-name>) | #f | #t
 ;;
 ;; <cpa> := (<string> ...)
@@ -1324,18 +1361,18 @@
      [(lset-difference eqv? quals '(:built-in :base :private)) pair?
       => (cut error <cgen-stub-error> "unknown define-cclass qualifier(s)" <>)])
     (match rest
-      [(c-type c-name cpa slot-spec . more) 
+      [(c-type c-name cpa slot-spec . more)
        (check-arg string? c-name)
        (check-arg list? cpa)
        (check-arg list? slot-spec)
-       (let* ((allocator (cond ((assq 'allocator more) => cadr) (else #f)))
-              (printer   (cond ((assq 'printer more) => cadr) (else #f)))
-              (dsupers   (cond ((assq 'direct-supers more) => cdr) (else '())))
-              (cclass (make <cclass>
+       (let* ([allocator (cond [(assq 'allocator more) => cadr] [else #f])]
+              [printer   (cond [(assq 'printer more) => cadr] [else #f])]
+              [dsupers   (cond [(assq 'direct-supers more) => cdr] [else '()])]
+              [cclass (make <cclass>
                         :scheme-name scm-name :c-type c-type :c-name c-name
                         :qualifiers quals
                         :cpa cpa :direct-supers dsupers
-                        :allocator allocator :printer printer)))
+                        :allocator allocator :printer printer)])
          (set! (~ cclass'slot-spec) (process-cclass-slots cclass slot-spec))
          (cgen-add! cclass))])))
 
@@ -1357,7 +1394,7 @@
     #`",(~ self'c-name)__SLOTS"))
 
 (define (cclass-emit-standard-decls self)
-  (let ((type (cgen-type-from-name (~ self'scheme-name))))
+  (let1 type (cgen-type-from-name (~ self'scheme-name))
     (p "SCM_CLASS_DECL("(~ self'c-name)");")
     (p "#define "(~ type'unboxer)"(obj) (("(~ self'c-type)")obj)")
     (p "#define "(~ type'c-predicate)"(obj) SCM_XTYPEP(obj, (&"(~ self'c-name)"))")
@@ -1397,12 +1434,12 @@
   )
 
 (define-method cgen-emit-init ((self <cclass>))
-  (p "  Scm_InitBuiltinClass(&"(~ self'c-name)", \""(~ self'scheme-name)"\", "(c-slot-spec-name self)", TRUE, mod);")
+  (p "  Scm_InitBuiltinClass(&"(~ self'c-name)", \""(~ self'scheme-name)"\", "(c-slot-spec-name self)", TRUE, SCM_MODULE("(stub-tmodule-cname self)"));")
   ;; adjust direct-supers if necessary
   (let1 ds (~ self'direct-supers)
     (when (not (null? ds))
       (p "  "(~ self'c-name)".directSupers = Scm_List(")
-      (for-each (lambda (s) (p "SCM_OBJ(&"s"), ")) ds)
+      (for-each (^s (p "SCM_OBJ(&"s"), ")) ds)
       (p " NULL);")
       (p))))
 
@@ -1418,7 +1455,7 @@
   (let1 cpa (~ self'cpa)
     (unless (or (null? cpa) (c-literal? cpa))
       (p "static ScmClass *"(~ self'c-name)"_CPL[] = {")
-      (for-each (lambda (class) (p "  SCM_CLASS_STATIC_PTR("class"),")) cpa)
+      (for-each (^[class] (p "  SCM_CLASS_STATIC_PTR("class"),")) cpa)
       (unless (equal? (car (last-pair cpa)) "Scm_TopClass")
         (p "  SCM_CLASS_STATIC_PTR(Scm_TopClass),"))
       (p "  NULL")
@@ -1427,14 +1464,14 @@
 ;; slot ---------
 
 (define (process-cclass-slots cclass slot-spec)
-  (map (lambda (spec)
+  (map (^[spec]
          (unless (list? spec) (error <cgen-stub-error> "bad slot spec" spec))
-         (let* ((name (car spec))
-                (type   (get-keyword :type (cdr spec) '<top>))
-                (c-name (get-keyword :c-name (cdr spec) (get-c-name "" name)))
-                (c-spec (get-keyword :c-spec (cdr spec) #f))
-                (getter (get-keyword :getter (cdr spec) #t))
-                (setter (get-keyword :setter (cdr spec) #t)))
+         (let* ([name (car spec)]
+                [type   (get-keyword :type (cdr spec) '<top>)]
+                [c-name (get-keyword :c-name (cdr spec) (get-c-name "" name))]
+                [c-spec (get-keyword :c-spec (cdr spec) #f)]
+                [getter (get-keyword :getter (cdr spec) #t)]
+                [setter (get-keyword :setter (cdr spec) #t)])
            (make <cslot>
              :cclass cclass :scheme-name name :c-name c-name
              :c-spec c-spec :type (name->type type)
@@ -1497,7 +1534,7 @@
 
 ;;===================================================================
 ;; Foreign pointers
-;;  NOT FINISHED YET.  
+;;  NOT FINISHED YET.
 
 ;; (define-class <c-ptr> (<stub>)
 ;;   ((c-type     :init-keyword :c-type)
@@ -1571,17 +1608,17 @@
 ;;
 
 (define-form-parser if (test then . maybe-else)
-  (parameterize ((cgen-cpp-condition test))
+  (cgen-with-cpp-condition test
     (cgen-stub-parse-form then))
   (unless (null? maybe-else)
-    (parameterize ((cgen-cpp-condition #`"!(,test)"))
+    (cgen-with-cpp-condition `(not ,test)
       (cgen-stub-parse-form (car maybe-else)))))
 
 (define-form-parser begin forms
   (for-each cgen-stub-parse-form forms))
 
 (define-form-parser when (test . forms)
-  (parameterize ((cgen-cpp-condition test))
+  (cgen-with-cpp-condition test
     (for-each cgen-stub-parse-form forms)))
 
 (define-form-parser include (file)
@@ -1589,17 +1626,17 @@
     ;; TODO: search path
     (error <cgen-stub-error> "couldn't find include file: " file))
   (with-input-from-file file
-    (lambda () (port-for-each cgen-stub-parse-form read)))
+    (^[] (port-for-each cgen-stub-parse-form read)))
   )
 
 (define-form-parser initcode codes
-  (dolist (c codes)
+  (dolist [c codes]
     (if (string? c)
       (cgen-init c)
       (cgen-init (call-with-output-string (cut cise-render c <>))))))
 
 (define-form-parser declcode codes
-  (dolist (c codes)
+  (dolist [c codes]
     (if (string? c)
       (cgen-decl c)
       (cgen-decl (call-with-output-string (cut cise-render c <>))))))
@@ -1609,7 +1646,7 @@
 
 (define-form-parser eval* exprs
   (let* ([m (~ (cgen-current-unit)'temporary-module)]
-         [r (fold (lambda (f r) (eval f m)) #f exprs)])
+         [r (fold (^[f r] (eval f m)) #f exprs)])
     (when (or (pair? r) (string? r))
       (cgen-stub-parse-form r))))
 
