@@ -1,23 +1,23 @@
 ;;;
 ;;; port related utility functions.  to be autoloaded.
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,12 +29,13 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module gauche.portutil
   (export port->string port->list port->string-list port->sexp-list
-          copy-port port-fold port-fold-right port-for-each port-map
-          port-position-prefix port-tell)
+          copy-port port-position-prefix port-tell
+          ;; transient until 0.9.4 release:
+          port-fold port-fold-right port-for-each port-map)
   )
 (select-module gauche.portutil)
 
@@ -43,13 +44,8 @@
 ;;   TODO: allow caller to specify reading units
 (define (port->string port)
   (let1 out (open-output-string :private? #t)
-    (with-port-locking port
-      (lambda ()
-        (let loop ((ch (read-char port)))
-          (unless (eof-object? ch)
-            (write-char ch out)
-            (loop (read-char port))))
-        (get-output-string out)))))
+    (copy-port port out :unit 'byte)
+    (get-output-string out)))
 
 (define (port->list reader port)
   (with-port-locking port
@@ -60,7 +56,7 @@
           (reverse! result)
           (loop (reader port) (cons obj result)))))))
 
-(define (port->string-list port) (port->list read-line port))
+(define (port->string-list port) (port->list (cut read-line <> #t) port))
 (define (port->sexp-list port)   (port->list read port))
 
 ;;-----------------------------------------------------
@@ -133,39 +129,6 @@
         [else (error "unit must be 'char, 'byte, or non-negative integer" unit)]
         ))
 
-;;-----------------------------------------------------
-;; Iterators on the input stream
-;;   These constructs take reader function that expects to return
-;;   new data for each invocation, and EOF whe exhausted.  It can
-;;   be any function; need not be bound to a port.
-
-(define (port-fold fn knil reader)
-  (let loop ((item (reader))
-             (r    knil))
-    (if (eof-object? item)
-      r
-      (loop (reader) (fn item r)))))
-
-;; This will consume large stack if input file is large.
-(define (port-fold-right fn knil reader)
-  (let loop ((item (reader)))
-    (if (eof-object? item)
-      knil
-      (fn item (loop (reader))))))
-
-(define (port-for-each fn reader)
-  (let loop ((item (reader)))
-    (unless (eof-object? item)
-      (fn item)
-      (loop (reader)))))
-
-(define (port-map fn reader)
-  (let loop ((item (reader))
-             (r    '()))
-    (if (eof-object? item)
-      (reverse! r)
-      (loop (reader) (cons (fn item) r)))))
-
 ;; useful for error messages
 (define (port-position-prefix port)
   (let ((n (port-name port))
@@ -176,9 +139,12 @@
         (format #f "~s: " n))
       "")))
 
-;;-----------------------------------------------------
 ;; useful alias
-;;
-
 (define (port-tell p) (port-seek p 0 SEEK_CUR))
 
+;; in order to compile post-0.9.3 source with 0.9.3, we need these
+;; to be in this module.  Remove them after 0.9.4 release.
+(define port-fold generator-fold)
+(define port-fold-right generator-fold-right)
+(define port-for-each generator-for-each)
+(define port-map generator-map)
