@@ -1,23 +1,23 @@
 ;;;
 ;;; regexp.scm - auxiliary macros and procedures for regexp.  autoloaded.
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;; NB: refrain from using util.match, for it could cause nasty dependency
 ;; problem.  (Should be resolved once we incorporate match into the core).
@@ -40,72 +40,66 @@
 
 (define-syntax rxmatch-bind*
   (syntax-rules ()
-    ((rxmatch-bind* ?n ?match () ?form ...)
-     (begin ?form ...))
-    ((rxmatch-bind* ?n ?match (#f ?vars ...) ?form ...)
-     (rxmatch-bind* (+ ?n 1) ?match (?vars ...) ?form ...))
-    ((rxmatch-bind* ?n ?match (?var ?vars ...) ?form ...)
-     (let ((?var (rxmatch-substring ?match ?n)))
-       (rxmatch-bind* (+ ?n 1) ?match (?vars ...) ?form ...)))
-    ))
+    [(rxmatch-bind* ?n ?match () ?form ...)
+     (begin ?form ...)]
+    [(rxmatch-bind* ?n ?match (#f ?vars ...) ?form ...)
+     (rxmatch-bind* (+ ?n 1) ?match (?vars ...) ?form ...)]
+    [(rxmatch-bind* ?n ?match (?var ?vars ...) ?form ...)
+     (let1 ?var (rxmatch-substring ?match ?n)
+       (rxmatch-bind* (+ ?n 1) ?match (?vars ...) ?form ...))]))
 
 (define-syntax rxmatch-let
   (syntax-rules ()
-    ((rxmatch-let ?expr (?var ...) ?form ...)
-     (cond (?expr
-            => (lambda (match)
-                 (rxmatch-bind* 0 match (?var ...) ?form ...)))
-           (else (error "rxmatch-let: match failed:" '?expr))))))
+    [(rxmatch-let ?expr (?var ...) ?form ...)
+     (cond [?expr => (^(match) (rxmatch-bind* 0 match (?var ...) ?form ...))]
+           [else (error "rxmatch-let: match failed:" '?expr)])]))
 
 (define-syntax rxmatch-if
   (syntax-rules ()
-    ((rxmatch-if ?expr (?var ...) ?then ?else)
-     (cond (?expr
-            => (lambda (match)
-                 (rxmatch-bind* 0 match (?var ...) ?then)))
-           (else ?else)))))
+    [(rxmatch-if ?expr (?var ...) ?then ?else)
+     (cond [?expr => (^(match) (rxmatch-bind* 0 match (?var ...) ?then))]
+           [else ?else])]))
 
 (define-syntax rxmatch-cond
   (syntax-rules (test else =>)
-    ((rxmatch-cond)
-     #f)
-    ((rxmatch-cond (else ?form ...))
-     (begin ?form ...))
-    ((rxmatch-cond (test ?expr => ?obj) ?clause ...)
-     (cond (?expr => ?obj) (else (rxmatch-cond ?clause ...))))
-    ((rxmatch-cond (test ?expr ?form ...) ?clause ...)
-     (if ?expr (begin ?form ...) (rxmatch-cond ?clause ...)))
-    ((rxmatch-cond (?matchexp ?bind ?form ...) ?clause ...)
+    [(rxmatch-cond) #f]
+    [(rxmatch-cond (else ?form ...))
+     (begin ?form ...)]
+    [(rxmatch-cond (test ?expr => ?obj) ?clause ...)
+     (cond (?expr => ?obj) (else (rxmatch-cond ?clause ...)))]
+    [(rxmatch-cond (test ?expr ?form ...) ?clause ...)
+     (if ?expr (begin ?form ...) (rxmatch-cond ?clause ...))]
+    [(rxmatch-cond (?matchexp ?bind ?form ...) ?clause ...)
      (rxmatch-if ?matchexp ?bind
-               (begin ?form ...)
-               (rxmatch-cond ?clause ...)))))
+       (begin ?form ...)
+       (rxmatch-cond ?clause ...))]))
 
 (define-syntax rxmatch-case
   (syntax-rules (test else =>)
-    ((rxmatch-case #t ?temp ?strp)
-     #f)
-    ((rxmatch-case #t ?temp ?strp (else ?form ...))
-     (begin ?form ...))
-    ((rxmatch-case #t ?temp ?strp (test ?proc => ?obj) ?clause ...)
-     (cond ((?proc ?temp) => ?obj)
-           (else (rxmatch-case #t ?temp ?strp ?clause ...))))
-    ((rxmatch-case #t ?temp ?strp (test ?proc ?form ...) ?clause ...)
+    [(rxmatch-case #t ?temp ?strp) #f]
+    [(rxmatch-case #t ?temp ?strp (else => ?proc))
+     (?proc ?temp)]
+    [(rxmatch-case #t ?temp ?strp (else ?form ...))
+     (begin ?form ...)]
+    [(rxmatch-case #t ?temp ?strp (test ?proc => ?obj) ?clause ...)
+     (cond [(?proc ?temp) => ?obj]
+           [else (rxmatch-case #t ?temp ?strp ?clause ...)])]
+    [(rxmatch-case #t ?temp ?strp (test ?proc ?form ...) ?clause ...)
      (if (?proc ?temp)
-         (begin ?form ...)
-         (rxmatch-case #t ?temp ?strp ?clause ...)))
-    ((rxmatch-case #t ?temp ?strp (?re ?bind ?form ...) ?clause ...)
-     (rxmatch-if (and ?strp (rxmatch ?re ?temp))
-          ?bind
        (begin ?form ...)
-       (rxmatch-case #t ?temp ?strp ?clause ...)))
-    ((rxmatch-case #t ?temp ?strip ?clause ...)
-     (syntax-error "malformed rxmatch-case"))
+       (rxmatch-case #t ?temp ?strp ?clause ...))]
+    [(rxmatch-case #t ?temp ?strp (?re ?bind ?form ...) ?clause ...)
+     (rxmatch-if (and ?strp (rxmatch ?re ?temp))
+         ?bind
+       (begin ?form ...)
+       (rxmatch-case #t ?temp ?strp ?clause ...))]
+    [(rxmatch-case #t ?temp ?strip ?clause ...)
+     (syntax-error "malformed rxmatch-case")]
     ;; main entry
-    ((rxmatch-case ?str ?clause ...)
-     (let* ((temp ?str)
-            (strp (string? temp)))
-       (rxmatch-case #t temp strp ?clause ...)))
-    ))
+    [(rxmatch-case ?str ?clause ...)
+     (let* ([temp ?str]
+            [strp (string? temp)])
+       (rxmatch-case #t temp strp ?clause ...))]))
 
 ;; NB: This should eventually be defined in regexp.c, and Scm_RegComp
 ;; should throw this on invalid AST tree.
@@ -165,7 +159,7 @@
 
   (define (unparse-assert-like op . asst)
     (let ((ch (if (eq? op 'assert) "=" "!")))
-      (if (and (pair? (car asst)) (eq? (caar asst) 'lookbehind))
+      (if (and (pair? asst) (pair? (car asst)) (eq? (caar asst) 'lookbehind))
         (if (null? (cdr asst))
           (between (format "(?<~a" ch) (cdar asst) ")")
           (err "invalid AST within assert or nassert" asst))
@@ -182,9 +176,10 @@
 
   (define (unparse n)
     (cond [(char? n)     (cond [(memv n '(#\. #\^ #\$ #\( #\) #\{ #\} #\[ #\]
-                                          #\\ #\* #\+ #\?))
+                                          #\\ #\* #\+ #\? #\|))
                                 (disp "\\") (disp n)]
-                               ;; TODO: nonprintable chars.
+                               [(eq? (char-general-category n) 'Cc)
+                                (disp (format "\\u~4,'0x" (char->ucs n)))]
                                [else (disp n)])]
           [(char-set? n) (disp (substring (write-to-string n) 1 -1))]
           [(eq? n 'any)  (disp #\.)]
@@ -199,7 +194,10 @@
                   [(seq)  (seq (cdr n))]
                   [(seq-uncase) (between "(?i:" (cdr n) ")")]
                   [(seq-case)   (between "(?-i:" (cdr n) ")")]
-                  [(alt)        (intersp (cdr n) "|")]
+                  [(alt)
+                   ;; empty alt always fails, and we represent it by
+                   ;; empty negative lookahead assertion.
+                   (if (null? (cdr n)) (disp "(?!)") (intersp (cdr n) "|"))]
                   [(rep rep-min rep-while) (apply unparse-rep n)]
                   [(cpat) (apply unparse-cpat (cdr n))]
                   [(backref)
