@@ -1,23 +1,23 @@
 ;;;
 ;;; http.scm - HTTP 1.1
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;; HTTP handling routines.
 
@@ -58,34 +58,34 @@
   (use gauche.partcont)
   (use util.match)
   (use util.list)
+  (use text.tree)
   (export <http-error>
-          http-user-agent make-http-connection http-compose-query
-          http-compose-form-data
-          
-          http-request
-          http-null-receiver http-string-receiver http-oport-receiver 
+          http-user-agent make-http-connection reset-http-connection
+          http-compose-query http-compose-form-data
+
+          http-proxy http-request
+          http-null-receiver http-string-receiver http-oport-receiver
           http-file-receiver http-cond-receiver
           http-null-sender http-string-sender http-blob-sender
           http-file-sender http-multipart-sender
-          
+
           http-get http-head http-post http-put http-delete
           http-default-auth-handler
+          http-default-redirect-handler
 
           http-secure-connection-available?
           )
   )
 (select-module rfc.http)
 
-;; 
+;;
 (autoload rfc.mime
           mime-compose-message
           mime-compose-message-string
           mime-compose-parameters
           mime-parse-content-type)
-
-(autoload gauche.process
-          run-process process-input process-output process-error
-          process-wait process-kill)
+(autoload rfc.tls
+          make-tls tls-connect tls-input-port tls-output-port tls-close)
 
 (autoload file.util file-size find-file-in-paths null-device)
 
@@ -99,10 +99,32 @@
 ;; Global parameters
 ;;
 
-;; default string to be used for user-agent.  
+;; default string to be used for user-agent.
 (define http-user-agent
   (make-parameter #`"gauche.http/,(gauche-version)"))
 
+;; global proxy settings.  can be overridden by :proxy keyword
+;; argument.
+(define http-proxy (make-parameter #f))
+
+;; The default redirect handler
+;;
+(define http-default-redirect-handler
+  (make-parameter
+   (^[method code headers body]
+     (and-let* ([loc (rfc822-header-ref headers "location")])
+       (case (x->integer code)
+         [(300 301 305 307)
+          (case method [(GET HEAD) `(,method . ,loc)] [else #f])]
+         [(302 303)                     ;found / see other
+          ;; See rfc2616 notes - the agent isn't supposed to automatically
+          ;; redirect 302 response of POST, but redirect with GET for it is
+          ;; the de-facto behavior.
+          (case method
+            [(GET HEAD) `(,method . ,loc)]
+            [else `(GET . ,loc)])]
+         [else #f])))))
+
 ;;==============================================================
 ;; Higher-level API
 ;;
@@ -142,6 +164,11 @@
 ;;             The retriever procedure returns zero when body is exhausted.
 ;;             In that case the value(s) the receiver returns will be
 ;;             the return value of http-request.
+;;             The retriever thunk can return #f as size, if we don't know the
+;;             chunk of response size.  In that case, the receiver may read
+;;             up to EOF, or as much data as it wants, then call the
+;;             retriever.  The second time the retriever returns size=0,
+;;             so you can do cleanup work.
 ;;             The retriever procedure returns -1 if an error occurs
 ;;             during comminucation.  In that case, the receiver can do
 ;;             whatever cleanup work.  After the receiver returns, an
@@ -171,49 +198,68 @@
 ;;             the sender needs to call BODY-SINK with argument 0.
 ;;
 ;;   host    - the host name passed to the 'host' header field.
-;;   secure  - if true, using secure connection (via external stunnel
-;;             process).
-;;   no-redirect - if true, the procedures won't attempt to issue
-;;             the request specified by 3xx response headers.
+;;   secure  - if true, using secure connection (via gauche.tls).
 ;;   auth-user, auth-password, auth-handler - authentication parameters.
 ;;   request-encoding - when http-* is to construct request-uri and/or
 ;;             request body, this argument specifies the character encoding
 ;;             to be used as the external encoding.
+;;   redirect-handler - Called if the server responds with 3xx status.
+;;             The argument is the request method, a status code, list of
+;;             headers and response body (can be #f).  It may return a
+;;             (METHOD . URL) or #f.   For the first case,
+;;             http-request re-attempts to fetch the URL with the given method.
+;;             (unless we're not looping).  If it returns #f, the original
+;;             code, headers and body are returned from http-request.
+;;             If given #t (default), the procedure bound to the parameter
+;;             http-default-redirect-handler is called.
+;;   no-redirect - If true, we ignore the value of redirect-handler and
+;;             returns without attempting retrying.
+;;             This is provided for the backward compatibility; newer code
+;;             should use :redirect-handler #f
 ;;
 ;; Other unrecognized options are passed as request headers.
 
 (define (http-request method server request-uri
                       :key (host #f)
+                           (redirect-handler #t)
                            (no-redirect #f)
                            auth-handler
                            auth-user
                            auth-password
-                           proxy
+                           (proxy (http-proxy))
                            extra-headers
                            (user-agent (http-user-agent))
                            (secure #f)
                            (receiver (http-string-receiver))
                            (sender #f)
-                           (enc :request-encoding (gauche-character-encoding))
+                           ((:request-encoding enc) (gauche-character-encoding))
                       :allow-other-keys opts)
   (let1 conn (ensure-connection server auth-handler auth-user auth-password
                                 proxy secure extra-headers)
     (let loop ([history '()]
                [host host]
+               [method method]
                [request-uri (ensure-request-uri request-uri enc)])
       (receive (code headers body)
           (request-response method conn host request-uri sender receiver
                             `(:user-agent ,user-agent ,@opts) enc)
         (or (and-let* ([ (not no-redirect) ]
                        [ (string-prefix? "3" code) ]
-                       [loc (assoc "location" headers)])
+                       [h (case redirect-handler
+                            [(#t) (http-default-redirect-handler)]
+                            [(#f) #f]
+                            [else => identity])]
+                       [r (h method code headers body)]
+                       [method (car r)]
+                       [loc (cdr r)])
               (receive (uri proto new-server path*)
-                  (canonical-uri conn (cadr loc) (ref conn'server))
+                  (canonical-uri conn loc (ref conn'server))
                 (when (or (member uri history)
                           (> (length history) 20))
                   (errorf <http-error> "redirection is looping via ~a" uri))
                 (loop (cons uri history)
                       (ref (redirect conn proto new-server)'server)
+                      method
                       path*)))
             (values code headers body))))))
 
@@ -225,35 +271,37 @@
     ;; TODO: check headers for encoding
     (let loop ([sink (open-output-string)])
       (receive (remote size) (retr)
-        (cond [(= size 0) (get-output-string sink)]
-              [(> size 0) (copy-port remote sink :size size) (loop sink)])))))
+        (cond [(eqv? size 0) (get-output-string sink)]
+              [(or (not size) (> size 0))
+               (copy-port remote sink :size size) (loop sink)])))))
 
 (define (http-null-receiver)
   (lambda (code hdrs total retr)
     (let loop ([sink (open-output-file (cond-expand [gauche.os.windows "NUL"]
                                                     [else "/dev/null"]))])
       (receive (remote size) (retr)
-        (cond [(= size 0) (close-output-port sink)]
+        (cond [(and size (<= size 0)) (close-output-port sink)]
               [else (copy-port remote sink :size size) (loop sink)])))))
 
 (define (http-oport-receiver sink flusher)
   (lambda (code hdrs total retr)
     (let loop ()
       (receive (remote size) (retr)
-        (cond [(= size 0) (flusher sink hdrs)]
-              [(> size 0) (copy-port remote sink :size size) (loop)])))))
+        (cond [(and size (<= size 0)) (flusher sink hdrs)]
+              [else (copy-port remote sink :size size) (loop)])))))
 
 (define (http-file-receiver filename :key (temporary #f))
   (lambda (code hdrs total retr)
     (receive (port tmpname) (sys-mkstemp filename)
       (let loop ()
         (receive (remote size) (retr)
-          (cond [(= size 0)
+          (cond [(or (not size) (> size 0))
+                 (copy-port remote port :size size) (loop)]
+                [(= size 0)
                  (close-output-port port)
                  (if temporary
                    tmpname
                    (begin (sys-rename tmpname filename) filename))]
-                [(> size 0) (copy-port remote port :size size) (loop)]
                 [else (close-output-port port) (sys-unlink tmpname)]))))))
 
 (define-syntax http-cond-receiver
@@ -385,29 +433,31 @@
 ;; the rfc.http APIs create a temporary connection under the hood.)
 
 (define-class <http-connection> ()
-  ;; All the slots are private.
+  ;; All slots are private.
   ((server :init-keyword :server)       ; server[:port]
    (socket :init-value #f)              ; A <socket> for persistent connection.
                                         ; If it is shutdown by the server,
                                         ; the APIs attempt to reconnect.
    (secure-agent  :init-value #f)       ; When using secure connection via
-                                        ; external process, this slot holds
-                                        ; its handle.
+                                        ; tls, this slot holds its handle.
+   (persistent    :init-keyword :persistent)   ; true for persistent connection.
    (auth-handler  :init-keyword :auth-handler) ; unused yet
    (auth-user     :init-keyword :auth-user)    ; unused yet
    (auth-password :init-keyword :auth-password); unused yet
    (proxy         :init-keyword :proxy)
    (extra-headers :init-keyword :extra-headers)
-   (secure        :init-keyword :secure)
+   (secure        :init-keyword :secure) ; boolean
    ))
 
 (define (make-http-connection server :key
+                              (persistent #t)
                               (auth-handler  http-default-auth-handler)
                               (auth-user     #f)
                               (auth-password #f)
                               (proxy #f)
                               (extra-headers '()))
   (make <http-connection>
+    :persistent persistent
     :server server :auth-handler auth-handler :auth-user auth-user
     :auth-password auth-password :proxy proxy
     :extra-headers extra-headers))
@@ -433,7 +483,7 @@
 ;; NB: Query string syntax (aka application/x-www-form-urlencoded) is
 ;; not defined in RFC2616.  In fact, it is not clearly defined at all
 ;; in RFC.  The most definitive source might be the HTML4 specification,
-;; section 17.13.4 "Form content types", 
+;; section 17.13.4 "Form content types",
 ;; <http://www.w3.org/TR/html4/interact/forms.html#h-17.13.4.1>.
 ;; In reality it is used very frequently with http, so we put it here.
 (define (http-compose-query path params
@@ -492,7 +542,7 @@
   (if (not port)
     (mime-compose-message-string (map translate-param params))
     (mime-compose-message (map translate-param params) port)))
-  
+
 ;;==============================================================
 ;; internal utilities
 ;;
@@ -518,9 +568,11 @@
 ;; Always returns a connection object.
 (define (ensure-connection server auth-handler auth-user auth-password
                            proxy secure extra-headers)
-  (rlet1 conn (cond [(is-a? server <http-connection>) server]
-                    [(string? server) (make-http-connection server)]
-                    [else (error "bad type of argument for server: must be an <http-connection> object or a string of the server's name, but got:" server)])
+  (rlet1 conn (cond
+               [(is-a? server <http-connection>) server]
+               [(string? server) (make-http-connection server :persistent #f)]
+               [else (error "bad type of argument for server: must be an <http-connection> object or a string of the server's name, but got:" server)])
+    ;; TODO: Might need to reset connections if parameters are changed
     (let-syntax ([check-override
                   (syntax-rules ()
                     [(_ id)
@@ -532,39 +584,59 @@
       (check-override extra-headers)
       (check-override secure))))
 
-(define (server->socket server)
-  (cond ((#/([^:]+):(\d+)/ server)
-         => (lambda (m) (make-client-socket (m 1) (x->integer (m 2)))))
-        (else (make-client-socket server 80))))
+(define (reset-http-connection conn)
+  (shutdown-socket-connection conn)
+  (shutdown-secure-agent conn))
+
+(define (start-socket-connection conn)
+  (let1 server (or (~ conn'proxy) (~ conn'server))
+    (set! (~ conn'socket)
+          (cond [(#/([^:]+):(\d+)/ server)
+                 => (^m (make-client-socket (m 1) (x->integer (m 2))))]
+                [else
+                 (make-client-socket server (if (~ conn'secure) 443 80))]))))
+
+(define (shutdown-socket-connection conn)
+  (when (~ conn'socket)
+    (guard (e [(<system-error> e) #f])
+      (socket-shutdown (~ conn'socket)))
+    (socket-close (~ conn'socket))
+    (set! (~ conn'socket) #f)))
 
 (define (with-connection conn proc)
-  (cond [(~ conn'secure)
-         (start-secure-agent conn)
-         (unwind-protect
-             (proc (process-output (~ conn'secure-agent))
-                   (process-input (~ conn'secure-agent)))
-           (shutdown-secure-agent conn))]
-        [else
-         (let1 s (server->socket (or (ref conn'proxy) (ref conn'server)))
-           (unwind-protect
-               (proc (socket-input-port s) (socket-output-port s))
-             (socket-close s)))]))
+  (unless (~ conn'persistent)
+    (unless (~ conn'socket) (start-socket-connection conn))
+    (when (~ conn'secure) (start-secure-agent conn)))
+  (unwind-protect
+      (apply proc (if (~ conn'secure)
+                    `(,(tls-input-port (~ conn'secure-agent))
+                      ,(tls-output-port (~ conn'secure-agent)))
+                    `(,(socket-input-port (~ conn'socket))
+                      ,(socket-output-port (~ conn'socket)))))
+    (unless (~ conn'persistent)
+      (when (~ conn'secure) (shutdown-secure-agent conn))
+      (shutdown-socket-connection conn))))
 
 (define (request-response method conn host request-uri
                           sender receiver options enc)
   (define no-body-replies '("204" "304"))
-  (receive (host uri)
-      (consider-proxy conn (or host (~ conn'server)) request-uri)
-    (with-connection
-     conn
-     (lambda (in out)
-       (send-request out method host uri sender options enc)
-       (receive (code headers) (receive-header in)
-         (values code
-                 headers
-                 (and (not (eq? method 'HEAD))
-                      (not (member code no-body-replies))
-                      (receive-body in code headers receiver))))))))
+  (receive (host uri) (consider-proxy conn (or host (~ conn'server)) request-uri)
+    (let1 req-headers `(,@(if (~ conn'persistent)
+                            (if (~ conn'proxy)
+                              '(:proxy-connection keep-alive)
+                              '(:connection keep-alive))
+                            '())
+                        :host ,host ,@options)
+      (with-connection
+       conn
+       (lambda (in out)
+         (send-request out method uri sender req-headers enc)
+         (receive (code rep-headers) (receive-header in)
+           (values code
+                   rep-headers
+                   (and (not (eq? method 'HEAD))
+                        (not (member code no-body-replies))
+                        (receive-body in code rep-headers receiver)))))))))
 
 ;; canonicalize uri for the sake of redirection.
 ;; URI is a request-uri given to the API, or the redirect location specified
@@ -592,13 +664,13 @@
     (values host uri)))
 
 ;; send
-(define (send-request out method host uri sender options enc)
-  (display #`",method ,uri HTTP/1.1\r\n" out)
+(define (send-request out method uri sender headers enc)
+  (define request-line #`",method ,uri HTTP/1.1\r\n")
   (case method
     [(POST PUT)
-     (sender (options->request-headers `(:host ,host ,@options)) enc
+     (sender (options->request-headers headers) enc
              (lambda (hdrs)
-               (send-headers hdrs out)
+               (send-headers request-line hdrs out)
                (let ([chunked?
                       (equal? (rfc822-header-ref hdrs "transfer-encoding")
                               "chunked")]
@@ -610,13 +682,20 @@
                    (flush out)
                    out))))]
     [else
-     (send-headers (options->request-headers `(:host ,host ,@options)) out)]))
+     (send-headers request-line (options->request-headers headers) out)]))
 
-(define (send-headers hdrs out)
-  (dolist [hdr hdrs] (format out "~a: ~a\r\n" (car hdr) (cadr hdr)))
-  (display "\r\n" out)
+;; NB: We try to send the request line and headers in one packet if possible,
+;; since some http servers assumes important headers can be read in single
+;; read() call.
+(define (send-headers request-line hdrs out)
+  (display (tree->string
+            `(,request-line
+              ,@(map (^h `(,(car h)": ",(cadr h)"\r\n")) hdrs)
+              "\r\n"))
+           out)
   (flush out))
 
+;; convert key-value list to (("key" "value") ...)
 (define (options->request-headers options)
   (let loop ([options options] [r '()])
     (if (or (null? options) (null? (cdr options))) ;be permissive
@@ -647,16 +726,9 @@
      [total
       (when (> total 0) (set! handler (handler remote total)))
       (handler remote 0)]
-     [else
-      ;; length is unknown.  we should gradually read from remote as
-      ;; the data arrives.  but for now, we take a dumb approach---read
-      ;; everything into a string, then call the receiver with a string
-      ;; input port.
-      (let* ([content (port->string remote)]
-             [size (string-size content)]
-             [p (open-input-string content)])
-        (when (> size 0) (set! handler (handler p size)))
-        (handler p 0))])))
+     [else      ; length is unknown.  we call the handler with size=#f.
+      (set! handler (handler remote #f))
+      (handler remote 0)])))
 
 ;; NB: chunk extension and trailer are ignored for now.
 (define (receive-body-chunked remote handler)
@@ -683,83 +755,28 @@
 ;; secure agent handling
 ;;
 
-;; NB: In future, this part should be splitted into an individual
-;; module so that it can be used as an infrastructure for
-;; secure connection.  It should allow selection and customization
-;; of various subsystems.   For now, we just assumes stunnel version
-;; 4 or 3, and use some heuristics to find out which is available.
-
 (define (shutdown-secure-agent conn)
   (when (~ conn'secure-agent)
-    (close-output-port (process-input (~ conn'secure-agent)))
-    (or (process-wait (~ conn'secure-agent) #t)
-        (begin (sys-nanosleep #e1e8)    ;0.1s
-               (process-wait (~ conn'secure-agent) #t))
-        (begin (sys-nanosleep #e1e8)    ;0.1s
-               (process-kill (~ conn'secure-agent))))
+    (tls-close (~ conn'secure-agent))
     (set! (~ conn'secure-agent) #f)))
 
 (define (start-secure-agent conn)
+  (unless (http-secure-connection-available?)
+    (error "Secure connection is not available on this platform"))
   (when (~ conn'secure-agent) (shutdown-secure-agent conn))
-  (let* ([rhost      (or (~ conn'proxy) (~ conn'server))]
-         [rhost:port (if (string-index rhost #\:) rhost #`",|rhost|:https")]
-         [proc (probe-stunnel)])
-    (unless proc
-      ;; NB: It's better to raise more descriptive condition, but <http-error>
-      ;; isn't appropriate.  Some kind of network-layer error is good.  Let's
-      ;; wait til we have condition hierarchy in gauche.net.
-      (error "secure connection isn't available on this system."))
-    (set! (~ conn'secure-agent) (proc rhost:port))))
-
-;; Returns a closure to run the process.
-(define probe-stunnel
-  (let1 result #f
-    (define (run-stunnel3 path)
-      (lambda (host:port)
-        (run-process `(,path "-c" "-r" ,host:port) :input :pipe :output :pipe
-                     :error :null :wait #f)))
-    (define (run-stunnel4 path)
-      (lambda (host:port)
-        (receive (in out) (sys-pipe)
-          (rlet1 p (run-process `(,path "-fd" 3)
-                                :redirects `((< 0 stdin)
-                                             (> 1 stdout)
-                                             (> 2 :null)
-                                             (< 3 ,(port-file-number in)))
-                                :wait #f)
-            (format out "client = yes\n")
-            (format out "connect = ~a" host:port)
-            (close-output-port out)))))
-
-    (lambda (:key (force #f))
-      (if (and result (not force))
-        result
-        (and-let* ([path (or (find-file-in-paths "stunnel4") ;ubuntu has this
-                             (find-file-in-paths "stunnel"))]
-                   [p (run-process `(,path "-version")
-                                   :error :pipe :output :pipe)]
-                   [vers (read-line (process-error p))])
-          (process-wait p)
-          (rlet1 proc (cond
-                       [(eof-object? vers) (run-stunnel3 path)]
-                       [(#/stunnel (\d+)\.\d+/ vers)
-                        => (^m (if (>= (x->integer (m 1)) 4)
-                                 (run-stunnel4 path)
-                                 (run-stunnel3 path)))]
-                       [(#/exec failed/i vers) #f]
-                       [else (run-stunnel3 path)])
-            (set! result proc)))))))
+  (let1 tls (make-tls)
+    (tls-connect tls (socket-fd (~ conn'socket)))
+    (set! (~ conn'secure-agent) tls)))
 
 ;; for external api
 (define (http-secure-connection-available?)
-  ;; eventually this will check availability of multiple subsystems.
-  (boolean (probe-stunnel)))
-  
-     
+  (cond-expand
+   [gauche.net.tls #t]
+   [else #f]))
+
 ;;==============================================================
 ;; authentication handling
 ;;
 
 ;; dummy - to be written
 (define (http-default-auth-handler . _) #f)
-
