@@ -1,23 +1,23 @@
 ;;;
 ;;; srfi-14.scm - character set
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;; Basic operators are built in the Gauche kernel.  This module
 ;; defines auxiliary procedures.
@@ -44,9 +44,9 @@
 
 ;; Functions supported by the core interpreter:
 ;;    char-set char-set? char-set-contains? char-set-copy
+;;    char-set-complement char-set-complement!
 ;;    %char-set-equal? %char-set-add-chars! %char-set-add-range!
-;;    %char-set-add! %char-set-complement!
-;;    %char-set-ranges
+;;    %char-set-add! %char-set-ranges
 ;; Functions not in SRFI but defined here
 ;;    integer-range->char-set integer-range->char-set!
 
@@ -78,6 +78,18 @@
   )
 (select-module srfi-14)
 
+;; some built-in support
+(define %char-set-equal? (with-module gauche.internal %char-set-equal?))
+(define %char-set<=? (with-module gauche.internal %char-set<=?))
+(define %char-set-add-chars! (with-module gauche.internal %char-set-add-chars!))
+(define %char-set-add-range! (with-module gauche.internal %char-set-add-range!))
+(define %char-set-add! (with-module gauche.internal %char-set-add!))
+(define %char-set-ranges (with-module gauche.internal %char-set-ranges))
+(define %char-set-predefined (with-module gauche.internal %char-set-predefined))
+
+(define char-set-complement  (with-module gauche char-set-complement))
+(define char-set-complement! (with-module gauche char-set-complement!))
+
 ;;-------------------------------------------------------------------
 ;; Comparison
 (define (char-set= . args)
@@ -259,7 +271,7 @@
             )))
       )
     ))
-                  
+
 (define (->char-set obj)
   (cond ((list? obj)   (list->char-set obj))
         ((string? obj) (string->char-set obj))
@@ -281,7 +293,6 @@
 
 (autoload "srfi-14/set"   char-set-adjoin char-set-adjoin!
                           char-set-delete char-set-delete!
-                          char-set-complement char-set-complement!
                           char-set-union char-set-union!
                           char-set-intersection char-set-intersection!
                           char-set-difference char-set-difference!
@@ -313,5 +324,5 @@
 (define char-set:symbol      (string->char-set "$+<=>^`|~"))
 (define char-set:ascii       (integer-range->char-set 0 128))
 (define char-set:empty       (char-set))
-(define char-set:full        (%char-set-complement! (char-set)))
+(define char-set:full        (char-set-complement! (char-set)))
 
