@@ -11,7 +11,7 @@
 ;
 ; sebastian.egner@philips.com, Eindhoven, The Netherlands, Feb-2003.
 ; Scheme R5RS (incl. macros), SRFI-23 (error).
-; 
+;
 ; Loading the implementation into Scheme48 0.57:
 ;   ,open srfi-23
 ;   ,load ec.scm
@@ -20,15 +20,15 @@
 ;   ; File > Open ... "ec.scm", click Execute
 ;
 ; Loading the implementation into SCM 5d7:
-;   (require 'macro) (require 'record) 
+;   (require 'macro) (require 'record)
 ;   (load "ec.scm")
 ;
 ; Implementation comments:
 ;   * All local (not exported) identifiers are named ec-<something>.
-;   * This implementation focuses on portability, performance, 
+;   * This implementation focuses on portability, performance,
 ;     readability, and simplicity roughly in this order. Design
 ;     decisions related to performance are taken for Scheme48.
-;   * Alternative implementations, Comments and Warnings are 
+;   * Alternative implementations, Comments and Warnings are
 ;     mentioned after the definition with a heading.
 
 (define-module srfi-42
@@ -51,7 +51,7 @@
 ;; yet.  Using this version, the SRFI-42 examples.scm tests fail first
 ;; on the following form:
 ;
-; (my-check 
+; (my-check
 ;  (list-ec (:parallel (:range i 1 10) (:list x '(a b c))) (list i x))
 ;  => '((1 a) (2 b) (3 c)) )
 
@@ -107,26 +107,30 @@
 ;; use the same signature as the hygienic version for compatibility, we
 ;; ignore the middle "collector" arg
 (define-macro (%replace-keywords syn _ . args)
+  (define (rename symbol)               ; bandage for hygiene
+    ((with-module gauche.internal make-identifier)
+     symbol (current-module) '()))
   (define (rewrite x)
     (if (pair? x)
       (if (keyword? (car x))
         (cons
          (case (car x)
-           ((:)            'srfi-42-)
-           ((:list)        'srfi-42-list)
-           ((:string)      'srfi-42-string)
-           ((:vector)      'srfi-42-vector)
-           ((:integers)    'srfi-42-integers)
-           ((:range)       'srfi-42-range)
-           ((:real-range)  'srfi-42-real-range)
-           ((:char-range)  'srfi-42-char-range)
-           ((:port)        'srfi-42-port)
-           ((:dispatched)  'srfi-42-dispatched)
-           ((:do)          'srfi-42-do)
-           ((:let)         'srfi-42-let)
-           ((:parallel)    'srfi-42-parallel)
-           ((:while)       'srfi-42-while)
-           ((:until)       'srfi-42-until)
+           ((:)            (rename 'srfi-42-))
+           ((:list)        (rename 'srfi-42-list))
+           ((:string)      (rename 'srfi-42-string))
+           ((:vector)      (rename 'srfi-42-vector))
+           ((:integers)    (rename 'srfi-42-integers))
+           ((:range)       (rename 'srfi-42-range))
+           ((:real-range)  (rename 'srfi-42-real-range))
+           ((:char-range)  (rename 'srfi-42-char-range))
+           ((:port)        (rename 'srfi-42-port))
+           ((:generator)   (rename 'srfi-42-generator))
+           ((:dispatched)  (rename 'srfi-42-dispatched))
+           ((:do)          (rename 'srfi-42-do))
+           ((:let)         (rename 'srfi-42-let))
+           ((:parallel)    (rename 'srfi-42-parallel))
+           ((:while)       (rename 'srfi-42-while))
+           ((:until)       (rename 'srfi-42-until))
            (else (car x)))
          (rewrite (cdr x)))
         (cons (rewrite (car x)) (rewrite (cdr x))))
@@ -139,7 +143,7 @@
 ; ==========================================================================
 ;
 ; All eager comprehensions are reduced into do-ec and
-; all generators are reduced to :do. 
+; all generators are reduced to :do.
 ;
 ; We use the following short names for syntactic variables
 ;   q    - qualifier
@@ -159,7 +163,7 @@
 ;   etc  - more arguments of mixed type
 
 ; (do-ec q ... cmd)
-;   handles nested, if/not/and/or, begin, :let, and calls generator 
+;   handles nested, if/not/and/or, begin, :let, and calls generator
 ;   macros in CPS to transform them into fully decorated :do.
 ;   The code generation for a :do is delegated to do-ec:do.
 
@@ -224,12 +228,12 @@
   (syntax-rules (srfi-42-do let)
 
     ; reentry point (*) -> generate code
-    ((do-ec:do cmd 
-               (srfi-42-do (let obs oc ...) 
-                    lbs 
-                    ne1? 
-                    (let ibs ic ...) 
-                    ne2? 
+    ((do-ec:do cmd
+               (srfi-42-do (let obs oc ...)
+                    lbs
+                    ne1?
+                    (let ibs ic ...)
+                    ne2?
                     (ls ...) ))
      (ec-simplify
        (let obs
@@ -245,7 +249,7 @@
                         (if ne2?
                             (loop ls ...) )))))))))) ))
 
-    
+
 ; (ec-simplify <expression>)
 ;   generates potentially more efficient code for <expression>.
 ;   The macro handles if, (begin <command>*), and (let () <command>*)
@@ -272,13 +276,13 @@
     ((ec-simplify (if (not (not test)) consequent alternate))
      (ec-simplify (if test consequent alternate)) )
 
-; (let () <command>*) 
+; (let () <command>*)
 
     ; empty <binding spec>*
     ((ec-simplify (let () command ...))
      (ec-simplify (begin command ...)) )
 
-; begin 
+; begin
 
     ; flatten use helper (ec-simplify 1 done to-do)
     ((ec-simplify (begin command ...))
@@ -316,7 +320,7 @@
     ; short form -> fill in default values
     ((_ cc lbs ne1? lss)
      (srfi-42-do cc (let ()) lbs ne1? (let ()) #t lss) )))
-    
+
 
 (define-syntax srfi-42-let
   (syntax-rules (index)
@@ -335,10 +339,10 @@
      (g (srfi-42-parallel-1 cc (gen ...)) arg1 arg ...) )))
 
 ; (:parallel-1 cc (to-do ...) result [ next ] )
-;    iterates over to-do by converting the first generator into 
+;    iterates over to-do by converting the first generator into
 ;    the :do-generator next and merging next into result.
 
-(define-syntax srfi-42-parallel-1  ; used as 
+(define-syntax srfi-42-parallel-1  ; used as
   ;;(syntax-rules (:do let)
   (syntax-rules (srfi-42-do let)
 
@@ -348,28 +352,28 @@
 
     ; reentry point (**) -> merge next into result
     ((_
-       cc 
-       gens 
-       (srfi-42-do (let (ob1 ...) oc1 ...) 
-            (lb1 ...) 
-            ne1?1 
-            (let (ib1 ...) ic1 ...) 
-            ne2?1 
+       cc
+       gens
+       (srfi-42-do (let (ob1 ...) oc1 ...)
+            (lb1 ...)
+            ne1?1
+            (let (ib1 ...) ic1 ...)
+            ne2?1
             (ls1 ...) )
-       (srfi-42-do (let (ob2 ...) oc2 ...) 
-            (lb2 ...) 
-            ne1?2 
-            (let (ib2 ...) ic2 ...) 
-            ne2?2 
+       (srfi-42-do (let (ob2 ...) oc2 ...)
+            (lb2 ...)
+            ne1?2
+            (let (ib2 ...) ic2 ...)
+            ne2?2
             (ls2 ...) ))
      (srfi-42-parallel-1
-       cc 
-       gens 
-       (srfi-42-do (let (ob1 ... ob2 ...) oc1 ... oc2 ...) 
-            (lb1 ... lb2 ...) 
-            (and ne1?1 ne1?2) 
-            (let (ib1 ... ib2 ...) ic1 ... ic2 ...) 
-            (and ne2?1 ne2?2) 
+       cc
+       gens
+       (srfi-42-do (let (ob1 ... ob2 ...) oc1 ... oc2 ...)
+            (lb1 ... lb2 ...)
+            (and ne1?1 ne1?2)
+            (let (ib1 ... ib2 ...) ic1 ... ic2 ...)
+            (and ne2?1 ne2?2)
             (ls1 ... ls2 ...) )))
 
     ; no more gens -> continue with cc, reentry at (*)
@@ -389,27 +393,27 @@
 
 (define-syntax srfi-42-while-2
   (syntax-rules (srfi-42-do let)
-    ((srfi-42-while-2 cc 
-                      test 
+    ((srfi-42-while-2 cc
+                      test
                       (ib-let     ...)
                       (ib-save    ...)
                       (ib-restore ...)
-                      (srfi-42-do olet 
-                                  lbs 
-                                  ne1? 
+                      (srfi-42-do olet
+                                  lbs
+                                  ne1?
                                   (let ((ib-var ib-rhs) ib ...) ic ...)
-                                  ne2? 
+                                  ne2?
                                   lss))
-     (srfi-42-while-2 cc 
-                      test 
+     (srfi-42-while-2 cc
+                      test
                       (ib-let     ... (ib-tmp #f))
                       (ib-save    ... (ib-var ib-rhs))
                       (ib-restore ... (ib-var ib-tmp))
-                      (srfi-42-do olet 
-                                  lbs 
-                                  ne1? 
-                                  (let (ib ...) ic ... (set! ib-tmp ib-var)) 
-                                  ne2? 
+                      (srfi-42-do olet
+                                  lbs
+                                  ne1?
+                                  (let (ib ...) ic ... (set! ib-tmp ib-var))
+                                  ne2?
                                   lss)))
     ((srfi-42-while-2 cc
                       test
@@ -466,7 +470,7 @@
   (syntax-rules (index)
     ((_ cc var (index i) arg)
      (srfi-42-do cc
-          (let ((str arg) (len 0)) 
+          (let ((str arg) (len 0))
             (set! len (string-length str)))
           ((i 0))
           (< i len)
@@ -489,7 +493,7 @@
      (srfi-42-vector cc var (index i) arg) )
     ((_ cc var (index i) arg)
      (srfi-42-do cc
-          (let ((vec arg) (len 0)) 
+          (let ((vec arg) (len 0))
             (set! len (vector-length vec)))
           ((i 0))
           (< i len)
@@ -555,7 +559,7 @@
      (srfi-42-do cc
           (let ((b arg2))
             (if (not (and (integer? b) (exact? b)))
-                (error 
+                (error
                    "arguments of :range are not exact integer "
                    "(use :real-range?)" 0 b 1 )))
           ((var 0))
@@ -568,7 +572,7 @@
      (srfi-42-do cc
           (let ((b arg2))
             (if (not (and (integer? b) (exact? b)))
-                (error 
+                (error
                    "arguments of :range are not exact integer "
                    "(use :real-range?)" 0 b 1 )))
           ((var 0))
@@ -582,7 +586,7 @@
           (let ((a arg1) (b arg2))
             (if (not (and (integer? a) (exact? a)
                           (integer? b) (exact? b) ))
-                (error 
+                (error
                    "arguments of :range are not exact integer "
                    "(use :real-range?)" a b 1 )) )
           ((var a))
@@ -596,7 +600,7 @@
           (let ((a arg1) (b arg2) (s -1) (stop 0))
             (if (not (and (integer? a) (exact? a)
                           (integer? b) (exact? b) ))
-                (error 
+                (error
                    "arguments of :range are not exact integer "
                    "(use :real-range?)" a b -1 )) )
           ((var a))
@@ -613,7 +617,7 @@
             (if (not (and (integer? a) (exact? a)
                           (integer? b) (exact? b)
                           (integer? s) (exact? s) ))
-                (error 
+                (error
                    "arguments of :range are not exact integer "
                    "(use :real-range?)" a b s ))
             (if (zero? s)
@@ -626,7 +630,7 @@
           ((+ var s)) ))))
 
 ; Comment: The macro :range inserts some code to make sure the values
-;   are exact integers. This overhead has proven very helpful for 
+;   are exact integers. This overhead has proven very helpful for
 ;   saving users from themselves.
 
 
@@ -664,7 +668,7 @@
 ;   value in case any of the other values is inexact. This is a
 ;   precaution to avoid (list-ec (: x 0 3.0) x) => '(0 1.0 2.0).
 
-    
+
 (define-syntax srfi-42-char-range
   (syntax-rules (index)
     ((_ cc var (index i) arg1 arg2)
@@ -678,26 +682,39 @@
           #t
           ((+ i 1)) ))))
 
-; Warning: There is no R5RS-way to implement the :char-range generator 
-;   because the integers obtained by char->integer are not necessarily 
+; Warning: There is no R5RS-way to implement the :char-range generator
+;   because the integers obtained by char->integer are not necessarily
 ;   consecutive. We simply assume this anyhow for illustration.
 
 
 (define-syntax srfi-42-port
   (syntax-rules (index)
-    ((_ cc var (index i) arg1 arg ...)
-     (srfi-42-parallel cc (srfi-42-port var arg1 arg ...) (srfi-42-integers i)) )
-    ((_ cc var arg)
-     (srfi-42-port cc var arg read) )
-    ((_ cc var arg1 arg2)
+    [(_ cc var (index i) arg1 arg ...)
+     (srfi-42-parallel cc (srfi-42-port var arg1 arg ...) (srfi-42-integers i))]
+    [(_ cc var arg)
+     (srfi-42-port cc var arg read)]
+    [(_ cc var arg1 arg2)
      (srfi-42-do cc
           (let ((port arg1) (read-proc arg2)))
           ((var (read-proc port)))
           (not (eof-object? var))
           (let ())
           #t
-          ((read-proc port)) ))))
+          ((read-proc port)) )]))
 
+;; Gauche extension
+(define-syntax srfi-42-generator
+  (syntax-rules (index)
+    [(_ cc var (index i) expr)
+     (srfi-42-parallel cc (srfi-42-generator var expr) (srfi-42-integers i))]
+    [(_ cc var expr)
+     (srfi-42-do cc
+          (let ([gen expr]))
+          ([var (gen)])
+          (not (eof-object? var))
+          (let ())
+          #t
+          [(gen)])]))
 
 ; ==========================================================================
 ; The typed generator :dispatched and utilities for constructing dispatchers
@@ -706,19 +723,19 @@
 (define-syntax srfi-42-dispatched
   (syntax-rules (index)
     ((_ cc var (index i) dispatch arg1 arg ...)
-     (srfi-42-parallel cc 
+     (srfi-42-parallel cc
                 (srfi-42-integers i)
                 (srfi-42-dispatched var dispatch arg1 arg ...) ))
     ((_ cc var dispatch arg1 arg ...)
      (srfi-42-do cc
-          (let ((d dispatch) 
-                (args (list arg1 arg ...)) 
-                (g #f) 
+          (let ((d dispatch)
+                (args (list arg1 arg ...))
+                (g #f)
                 (empty (list #f)) )
             (set! g (d args))
             (if (not (procedure? g))
-                (error "unrecognized arguments in dispatching" 
-                       args 
+                (error "unrecognized arguments in dispatching"
+                       args
                        (d '()) )))
           ((var (g empty)))
           (not (eq? var empty))
@@ -741,14 +758,14 @@
 
     ; reentry point (**) -> make the code from a single :do
     ((_
-       var 
-       (srfi-42-do (let obs oc ...) 
-            ((lv li) ...) 
-            ne1? 
-            (let ((i v) ...) ic ...) 
-            ne2? 
+       var
+       (srfi-42-do (let obs oc ...)
+            ((lv li) ...)
+            ne1?
+            (let ((i v) ...) ic ...)
+            ne2?
             (ls ...)) )
-     (ec-simplify 
+     (ec-simplify
       (let obs
           oc ...
           (let ((lv li) ... (ne2 #t))
@@ -757,13 +774,13 @@
                (lambda (empty)
                  (if (and ne1? ne2)
                      (ec-simplify
-                      (begin 
+                      (begin
                         (set! i v) ...
                         ic ...
                         (let ((value var))
                           (ec-simplify
                            (if ne2?
-                               (ec-simplify 
+                               (ec-simplify
                                 (begin (set! lv ls) ...) )
                                (set! ne2 #f) ))
                           value )))
@@ -778,9 +795,9 @@
   (lambda (args)
     (let ((g1 (d1 args)) (g2 (d2 args)))
       (if g1
-          (if g2 
+          (if g2
               (if (null? args)
-                  (append (if (list? g1) g1 (list g1)) 
+                  (append (if (list? g1) g1 (list g1))
                           (if (list? g2) g2 (list g2)) )
                   (error "dispatching conflict" args (d1 '()) (d2 '())) )
               g1 )
@@ -794,73 +811,64 @@
 (define (make-initial-:-dispatch)
   (lambda (args)
     (case (length args)
-      ((0) 'SRFI42)
-      ((1) (let ((a1 (car args)))
+      [(0) 'SRFI42]
+      [(1) (let ([a1 (car args)])
              (cond
-              ((list? a1)
-               (srfi-42-generator-proc (srfi-42-list a1)) )
-              ((string? a1)
-               (srfi-42-generator-proc (srfi-42-string a1)) )
-              ((vector? a1)
-               (srfi-42-generator-proc (srfi-42-vector a1)) )
-              ((and (integer? a1) (exact? a1))
-               (srfi-42-generator-proc (srfi-42-range a1)) )
-              ((real? a1)
-               (srfi-42-generator-proc (srfi-42-real-range a1)) )
-              ((input-port? a1)
-               (srfi-42-generator-proc (srfi-42-port a1)) )
-              (else
-               #f ))))
-      ((2) (let ((a1 (car args)) (a2 (cadr args)))
+              [(list? a1)
+               (srfi-42-generator-proc (srfi-42-list a1))]
+              [(string? a1)
+               (srfi-42-generator-proc (srfi-42-string a1))]
+              [(vector? a1)
+               (srfi-42-generator-proc (srfi-42-vector a1))]
+              [(and (integer? a1) (exact? a1))
+               (srfi-42-generator-proc (srfi-42-range a1))]
+              [(real? a1)
+               (srfi-42-generator-proc (srfi-42-real-range a1))]
+              [(input-port? a1)
+               (srfi-42-generator-proc (srfi-42-port a1))]
+              [(applicable? a1)
+               (srfi-42-generator-proc (srfi-42-generator a1))]
+              [else #f]))]
+      [(2) (let ([a1 (car args)] [a2 (cadr args)])
              (cond
-              ((and (list? a1) (list? a2))
-               (srfi-42-generator-proc (srfi-42-list a1 a2)) )
-              ((and (string? a1) (string? a1))
-               (srfi-42-generator-proc (srfi-42-string a1 a2)) )
-              ((and (vector? a1) (vector? a2))
-               (srfi-42-generator-proc (srfi-42-vector a1 a2)) )
-              ((and (integer? a1) (exact? a1) (integer? a2) (exact? a2))
-               (srfi-42-generator-proc (srfi-42-range a1 a2)) )
-              ((and (real? a1) (real? a2))
-               (srfi-42-generator-proc (srfi-42-real-range a1 a2)) )
-              ((and (char? a1) (char? a2))
-               (srfi-42-generator-proc (srfi-42-char-range a1 a2)) )
-              ((and (input-port? a1) (procedure? a2))
-               (srfi-42-generator-proc (srfi-42-port a1 a2)) )
-              (else
-               #f ))))
-      ((3) (let ((a1 (car args)) (a2 (cadr args)) (a3 (caddr args)))
+              [(and (list? a1) (list? a2))
+               (srfi-42-generator-proc (srfi-42-list a1 a2)) ]
+              [(and (string? a1) (string? a1))
+               (srfi-42-generator-proc (srfi-42-string a1 a2)) ]
+              [(and (vector? a1) (vector? a2))
+               (srfi-42-generator-proc (srfi-42-vector a1 a2)) ]
+              [(and (integer? a1) (exact? a1) (integer? a2) (exact? a2))
+               (srfi-42-generator-proc (srfi-42-range a1 a2)) ]
+              [(and (real? a1) (real? a2))
+               (srfi-42-generator-proc (srfi-42-real-range a1 a2)) ]
+              [(and (char? a1) (char? a2))
+               (srfi-42-generator-proc (srfi-42-char-range a1 a2)) ]
+              [(and (input-port? a1) (procedure? a2))
+               (srfi-42-generator-proc (srfi-42-port a1 a2)) ]
+              [else #f]))]
+      [(3) (let ([a1 (car args)] [a2 (cadr args)] [a3 (caddr args)])
              (cond
-              ((and (list? a1) (list? a2) (list? a3))
-               (srfi-42-generator-proc (srfi-42-list a1 a2 a3)) )
-              ((and (string? a1) (string? a1) (string? a3))
-               (srfi-42-generator-proc (srfi-42-string a1 a2 a3)) )
-              ((and (vector? a1) (vector? a2) (vector? a3))
-               (srfi-42-generator-proc (srfi-42-vector a1 a2 a3)) )
-              ((and (integer? a1) (exact? a1) 
+              [(and (list? a1) (list? a2) (list? a3))
+               (srfi-42-generator-proc (srfi-42-list a1 a2 a3)) ]
+              [(and (string? a1) (string? a1) (string? a3))
+               (srfi-42-generator-proc (srfi-42-string a1 a2 a3)) ]
+              [(and (vector? a1) (vector? a2) (vector? a3))
+               (srfi-42-generator-proc (srfi-42-vector a1 a2 a3)) ]
+              [(and (integer? a1) (exact? a1)
                     (integer? a2) (exact? a2)
                     (integer? a3) (exact? a3))
-               (srfi-42-generator-proc (srfi-42-range a1 a2 a3)) )
-              ((and (real? a1) (real? a2) (real? a3))
-               (srfi-42-generator-proc (srfi-42-real-range a1 a2 a3)) )
-              (else
-               #f ))))
-      (else
-       (letrec ((every? 
-                 (lambda (pred args)
-                   (if (null? args)
-                       #t
-                       (and (pred (car args))
-                            (every? pred (cdr args)) )))))
-         (cond
-          ((every? list? args)
-           (srfi-42-generator-proc (srfi-42-list (apply append args))) )
-          ((every? string? args)
-           (srfi-42-generator-proc (srfi-42-string (apply string-append args))) )
-          ((every? vector? args)
-           (srfi-42-generator-proc (srfi-42-list (apply append (map vector->list args)))) )
-          (else
-           #f )))))))
+               (srfi-42-generator-proc (srfi-42-range a1 a2 a3)) ]
+              [(and (real? a1) (real? a2) (real? a3))
+               (srfi-42-generator-proc (srfi-42-real-range a1 a2 a3)) ]
+              [else #f]))]
+      [else (cond
+             [(every list? args)
+              (srfi-42-generator-proc (srfi-42-list (apply append args))) ]
+             [(every string? args)
+              (srfi-42-generator-proc (srfi-42-string (apply string-append args)))]
+             [(every vector? args)
+              (srfi-42-generator-proc (srfi-42-list (apply append (map vector->list args))))]
+             [else #f])])))
 
 (define srfi-42--dispatch
   (make-initial-:-dispatch) )
@@ -947,7 +955,7 @@
      (list->string (list-ec etc1 etc ...)) )))
 
 ; Alternative: For very long strings, the intermediate list may be a
-;   problem. A more space-aware implementation collect the characters 
+;   problem. A more space-aware implementation collect the characters
 ;   in an intermediate list and when this list becomes too large it is
 ;   converted into an intermediate string. At the end, the intermediate
 ;   strings are concatenated with string-append.
@@ -1046,8 +1054,8 @@
 
     ((%first-ec default qualifier expression)
      (let ((result default) (stop #f))
-       (ec-guarded-do-ec 
-         stop 
+       (ec-guarded-do-ec
+         stop
          (nested qualifier)
          (begin (set! result expression)
                 (set! stop #t) ))
@@ -1081,8 +1089,8 @@
      (begin etc ... (ec-guarded-do-ec stop (nested q ...) cmd)) )
 
     ((ec-guarded-do-ec stop (nested gen q ...) cmd)
-     (do-ec 
-       (srfi-42-until gen stop) 
+     (do-ec
+       (srfi-42-until gen stop)
        (ec-guarded-do-ec stop (nested q ...) cmd) ))
 
     ((ec-guarded-do-ec stop (nested) cmd)
@@ -1091,10 +1099,10 @@
 ; Alternative: Instead of modifying the generator with :until, it is
 ;   possible to use call-with-current-continuation:
 ;
-;   (define-synatx first-ec 
+;   (define-synatx first-ec
 ;     ...same as above...
 ;     ((first-ec default qualifier expression)
-;      (call-with-current-continuation 
+;      (call-with-current-continuation
 ;       (lambda (cc)
 ;        (do-ec qualifier (cc expression))
 ;        default ))) ))
