@@ -1,23 +1,23 @@
 ;;;
 ;;; html-lite.scm - lightweight HTML construction
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 (define-module text.html-lite
   (use text.tree)
@@ -40,15 +40,13 @@
 
 ;; Escaping ---------------------------------------------
 (define (html-escape)
-  (port-for-each (lambda (c)
-                   (case c
-                     ((#\<) (display "&lt;"))
-                     ((#\>) (display "&gt;"))
-                     ((#\&) (display "&amp;"))
-                     ((#\") (display "&quot;"))
-                     (else (display c))
-                     ))
-                 read-char))
+  (generator-for-each (^c (case c
+                            [(#\<) (display "&lt;")]
+                            [(#\>) (display "&gt;")]
+                            [(#\&) (display "&amp;")]
+                            [(#\") (display "&quot;")]
+                            [else (display c)]))
+                      read-char))
 
 (define (html-escape-string string)
   (with-string-io (x->string string) html-escape))
