@@ -1,23 +1,23 @@
 ;;;
 ;;; sequence.scm - sequence operations
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;; This module defines an unified way to treat sequence-like objects
 ;; (that is, a collection object that can be accessed by integer index).
@@ -132,7 +132,7 @@
         (if (end?)
           seed
           (let1 elt (next)
-            (proc elt (rec)))))) 
+            (proc elt (rec))))))
     (call-with-iterators
      (cons seq more)
      (lambda (ends? nexts)
@@ -342,7 +342,7 @@
 (define-method permute! ((seq <string>) (ord <sequence>))
   ;; for string, this one is faster.
   (%permute!-arg-check seq ord)
-  (%string-replace-body! seq (permute seq ord)))
+  ((with-module gauche.internal %string-replace-body!) seq (permute seq ord)))
 
 (define (%permute!-arg-check seq ord)
   (unless (= (size-of seq) (size-of ord))
@@ -386,5 +386,6 @@
 
 (define-method shuffle! ((seq <string>) . args)
   ;; string-set! is super-slow, so we provide the alternative.
-  (%string-replace-body! seq (apply shuffle seq args)))
+  ((with-module gauche.internal %string-replace-body!)
+   seq (apply shuffle seq args)))
 
