@@ -16,34 +16,54 @@
 ;; which is autoloaded.  The user doesn't need to see this change.
 
 (define-module srfi-1
-  (export xcons cons* list-tabulate circular-list iota
+  (export xcons list-tabulate circular-list
           not-pair?
           list=
           first second third fourth fifth sixth seventh eighth
-          ninth tenth car+cdr take drop take-right drop-right
-          take! drop-right! split-at! last
+          ninth tenth car+cdr take-right drop-right
+          take! drop-right!
           concatenate concatenate!
           append-reverse append-reverse!
           zip unzip1 unzip2 unzip3 unzip4 unzip5
           count unfold pair-fold reduce unfold-right
           pair-fold-right reduce-right append-map append-map!
-          map! pair-for-each filter-map map-in-order
-          filter partition remove filter! partition! remove!
-          member find-tail list-index
+          map! pair-for-each map-in-order
+          partition partition!
+          list-index
           take-while drop-while take-while! span break span! break!
-          delete delete-duplicates delete! delete-duplicates!
-          assoc alist-cons alist-copy alist-delete alist-delete!
+          alist-cons
           lset<= lset= lset-adjoin lset-union lset-union!
           lset-intersection lset-intersection! lset-difference
           lset-difference! lset-xor lset-xor!
-          lset-diff+intersection lset-diff+intersection!))
+          lset-diff+intersection lset-diff+intersection!
+
+          ;; These are defined in gauche module, but they're not exported.
+          ;; We need to export them here so that (use srfi-1 :only (fold))
+          ;; will work.
+          null-list? cons* last member
+          take drop take-right drop-right take! drop-right!
+          delete delete! delete-duplicates delete-duplicates!
+          assoc alist-copy alist-delete alist-delete!
+          any every filter filter! remove remove! filter-map
+          fold fold-right find find-tail
+          split-at split-at! iota
+          ))
 (select-module srfi-1)
 
 (define (xcons a b) (cons b a))
-(define cons* list*)
 
 (define-inline (not-pair? x) (not (pair? x)))
 
+(define-macro (re-export . syms)
+  `(begin ,@(map (^s `(define ,s (with-module gauche ,s))) syms)))
+
+(re-export null-list? cons* last member
+           take drop take-right drop-right take! drop-right!
+           delete delete! delete-duplicates delete-duplicates!
+           assoc alist-copy alist-delete alist-delete!
+           any every filter filter! remove remove! filter-map
+           fold fold-right find find-tail
+           split-at split-at! iota)
 
 ;;;
 ;;; List generators of SRFI-1
@@ -56,20 +76,6 @@
        (ans '() (cons (proc i) ans)))
       ((< i 0) ans)))
 
-;;; IOTA count [start step]	(start start+step ... start+(count-1)*step)
-
-(define (iota count . args)
-  (if (< count 0) (error "Negative step count" iota count))
-  (let ((start (if (pair? args) (car args) 0))
-        (step  (if (and (pair? args) (pair? (cdr args))) (cadr args) 1)))
-    (check-arg number? start)
-    (check-arg number? step)
-    (let ((last-val (+ start (* (- count 1) step))))
-      (do ((count count (- count 1))
-	   (val last-val (- val step))
-	   (ans '() (cons val ans)))
-	  ((<= count 0)  ans)))))
-	  
 (define (circular-list val1 . vals)
   (let ((ans (cons val1 vals)))
     (set-cdr! (last-pair ans) ans)
@@ -96,7 +102,7 @@
                     (and (not (null-list? list-c))
                          (= (car list-a) (car list-c))
                          (lp2 (cdr list-a) (cdr list-c)))))))))))
-			
+
 
 ;;;
 ;;; Zipper and unzipper of SRFI-1
@@ -159,79 +165,13 @@
 (define (ninth   x) (car  (cddddr (cddddr x))))
 (define (tenth   x) (cadr (cddddr (cddddr x))))
 
-;;;
-;;; Selectors of SRFI-1
-;;;
-;;; take & drop
-
-(define (take lis k)
-  (check-arg integer? k)
-  (let recur ((lis lis) (k k))
-    (if (zero? k)
-      '()
-      (cons (car lis)
-            (recur (cdr lis) (- k 1))))))
-
-(define drop list-tail)  ; Gauche has list-tail
-
-(define (take! lis k)
-  (check-arg integer? k)
-  (if (zero? k)
-    '()
-    (begin (set-cdr! (drop lis (- k 1)) '())
-           lis)))
-
-;;; TAKE-RIGHT and DROP-RIGHT work by getting two pointers into the list, 
-;;; off by K, then chasing down the list until the lead pointer falls off
-;;; the end.
-
-(define (take-right lis k)
-  (check-arg integer? k)
-  (let lp ((lag lis)  (lead (drop lis k)))
-    (if (pair? lead)
-      (lp (cdr lag) (cdr lead))
-      lag)))
-
-(define (drop-right lis k)
-  (check-arg integer? k)
-  (let recur ((lag lis) (lead (drop lis k)))
-    (if (pair? lead)
-      (cons (car lag) (recur (cdr lag) (cdr lead)))
-      '())))
-
-;;; In this function, LEAD is actually K+1 ahead of LAG. This lets
-;;; us stop LAG one step early, in time to smash its cdr to ().
-(define (drop-right! lis k)
-  (check-arg integer? k)
-  (let ((lead (drop lis k)))
-    (if (pair? lead)
-
-      (let lp ((lag lis)  (lead (cdr lead)))	; Standard case
-        (if (pair? lead)
-          (lp (cdr lag) (cdr lead))
-          (begin (set-cdr! lag '())
-                 lis)))
-
-      '())))	; Special case dropping everything -- no cons to side-effect.
-
-(define (split-at! x k)
-  (check-arg integer? k)
-  (if (zero? k)
-    (values '() x)
-    (let* ((prev (drop x (- k 1)))
-           (suffix (cdr prev)))
-      (set-cdr! prev '())
-      (values x suffix))))
-
-(define (last lis) (car (last-pair lis)))
-
 ;;;
 ;;; Utility functions for n-ary operation in SRFI-1
 ;;;
 
 (define (car+cdr pair) (values (car pair) (cdr pair)))
 
-;;; Return (map cdr lists). 
+;;; Return (map cdr lists).
 ;;; However, if any element of LISTS is empty, just abort and return '().
 (define (%cdrs lists)
   (call-with-current-continuation
@@ -263,20 +203,6 @@
                    (values (cons a cars) (cons d cdrs))))))
          (values '() '()))))))
 
-;;; Like %CARS+CDRS, but we pass in a final elt tacked onto the end of the
-;;; cars list. What a hack.
-(define (%cars+cdrs+ lists cars-final)
-  (call-with-current-continuation
-   (lambda (abort)
-     (let recur ((lists lists))
-       (if (pair? lists)
-         (receive (list other-lists) (car+cdr lists)
-           (if (null-list? list) (abort '() '()) ; LIST is empty -- bail out
-               (receive (a d) (car+cdr list)
-                 (receive (cars cdrs) (recur other-lists)
-                   (values (cons a cars) (cons d cdrs))))))
-         (values (list cars-final) '()))))))
-
 ;;; Like %CARS+CDRS, but blow up if any list is empty.
 (define (%cars+cdrs/no-test lists)
   (let recur ((lists lists))
@@ -291,19 +217,8 @@
 ;;; Concatenators of SRFI-1
 ;;;
 
-(define (append-reverse rev-head tail)
-  (let lp ((rev-head rev-head) (tail tail))
-    (if (null-list? rev-head)
-      tail
-      (lp (cdr rev-head) (cons (car rev-head) tail)))))
-
-(define (append-reverse! rev-head tail)
-  (let lp ((rev-head rev-head) (tail tail))
-    (if (null-list? rev-head)
-      tail
-      (let ((next-rev (cdr rev-head)))
-        (set-cdr! rev-head tail)
-        (lp next-rev rev-head)))))
+(define (append-reverse list tail)  (reverse list tail))
+(define (append-reverse! list tail) (reverse! list tail))
 
 (define (concatenate  lists) (reduce-right append  '() lists))
 (define (concatenate! lists) (reduce-right append! '() lists))
@@ -351,7 +266,7 @@
       (if (p seed) '()
           (cons (f seed) (recur (g seed)))))))
 
-;; fold and fold-right are built-in      
+;; fold and fold-right are built-in
 (define (pair-fold-right f zero lis1 . lists)
   (if (pair? lists)
     (let recur ((lists (cons lis1 lists)))		; N-ary case
@@ -374,7 +289,7 @@
           (let ((tail (cdr lis)))		; Grab the cdr now,
             (lp tail (f lis ans)))))))	; in case F SET-CDR!s LIS.
 
-      
+
 
 ;;; REDUCE and REDUCE-RIGHT only use RIDENTITY in the empty-list case.
 ;;; These cannot meaningfully be n-ary.
@@ -398,7 +313,7 @@
 
 (define (append-map f lis1 . lists)
   (really-append-map append-map  append  f lis1 lists))
-(define (append-map! f lis1 . lists) 
+(define (append-map! f lis1 . lists)
   (really-append-map append-map! append! f lis1 lists))
 
 (define (really-append-map who appender f lis1 lists)
@@ -447,67 +362,10 @@
     (pair-for-each (lambda (pair) (set-car! pair (f (car pair)))) lis1))
   lis1)
 
-
-;;; Map F across L, and save up all the non-false results.
-(define (filter-map f lis1 . lists)
-  (if (pair? lists)
-    (let recur ((lists (cons lis1 lists))
-                (r '()))
-      (receive (cars cdrs) (%cars+cdrs lists)
-        (cond ((null-list? cars) (reverse! r))
-              ((apply f cars) => (lambda (x) (recur cdrs (cons x r))))
-              (else (recur cdrs r)))))
-    
-    ;; Fast path.
-    (let recur ((lis lis1)
-                (r   '()))
-      (cond ((null-list? lis) (reverse! r))
-            ((f (car lis)) => (lambda (x) (recur (cdr lis) (cons x r))))
-            (else (recur (cdr lis) r))))))
-
 ;;;
 ;;; Filters of SRFI-1
 ;;;
 
-(define (filter pred lis)
-  (let loop ((lis lis) (r '()))
-    (cond
-     ((null-list? lis) (reverse! r))
-     ((pred (car lis))
-      (loop (cdr lis) (cons (car lis) r)))
-     (else
-      (loop (cdr lis) r)))))
-
-(define (filter! pred lis)
-  (let lp ((ans lis))
-    (cond ((null-list? ans) ans)			; Scan looking for
-	  ((not (pred (car ans))) (lp (cdr ans)))	; first cons of result.
-
-	  ;; ANS is the eventual answer.
-	  ;; SCAN-IN: (CDR PREV) = LIS and (CAR PREV) satisfies PRED.
-	  ;;          Scan over a contiguous segment of the list that
-	  ;;          satisfies PRED.
-	  ;; SCAN-OUT: (CAR PREV) satisfies PRED. Scan over a contiguous
-	  ;;           segment of the list that *doesn't* satisfy PRED.
-	  ;;           When the segment ends, patch in a link from PREV
-	  ;;           to the start of the next good segment, and jump to
-	  ;;           SCAN-IN.
-	  (else (letrec ((scan-in (lambda (prev lis)
-				    (if (pair? lis)
-                                      (if (pred (car lis))
-                                        (scan-in lis (cdr lis))
-                                        (scan-out prev (cdr lis))))))
-			 (scan-out (lambda (prev lis)
-				     (let lp ((lis lis))
-				       (if (pair? lis)
-                                         (if (pred (car lis))
-                                           (begin (set-cdr! prev lis)
-                                                  (scan-in lis (cdr lis)))
-                                           (lp (cdr lis)))
-                                         (set-cdr! prev lis))))))
-		  (scan-in ans (cdr ans))
-		  ans)))))
-
 ;; Avoid non-tail recursion.
 (define (partition pred lis)
   (let recur ((lis lis)
@@ -562,20 +420,10 @@
                    (values l lis))		; Done.
                   (else (lp l (cdr l)))))))))
 
-(define (remove! pred l) (filter! (lambda (x) (not (pred x))) l))
-(define (remove  pred l) (filter  (lambda (x) (not (pred x))) l))
-
 ;;;
 ;;; Find and alike of SRFI-1
 ;;;
 
-(define (find-tail pred list)
-  (let lp ((list list))
-    (and (not (null-list? list))
-	 (if (pred (car list))
-           list
-           (lp (cdr list))))))
-
 ;; use tail-recursion
 (define (take-while pred lis)
   (let recur ((lis lis)
@@ -823,91 +671,5 @@
 
 (define map-in-order map) ; Gauche's map is already in order
 
-;; In the common case, these procs uses Gauche native, even not loading
-;; the generic filter routine.
-
-(define-syntax %case-by-cmp
-  (syntax-rules ()
-    ((_ args = eq-case eqv-case equal-case default-case)
-     (let ((= (if (pair? args) (car args) equal?)))
-       (cond ((eq? = eq?)    eq-case)
-             ((eq? = eqv?)   eqv-case)
-             ((eq? = equal?) equal-case)
-             (else default-case))))))
-
-(define (delete x lis . args)
-  (%case-by-cmp args =
-                (%delete x lis 'eq?)
-                (%delete x lis 'eqv?)
-                (%delete x lis 'equal?)
-                (filter (lambda (y) (not (= x y))) lis)))
-
-(define (delete! x lis . args)
-  (%case-by-cmp args =
-                (%delete! x lis 'eq?)
-                (%delete! x lis 'eqv?)
-                (%delete! x lis 'equal?)
-                (filter! (lambda (y) (not (= x y))) lis)))
-
-;;; Extended from R4RS to take an optional comparison argument.
-(define (member x lis . args)
-  (let ((%member (with-module scheme member))) ;save original func
-    (%case-by-cmp args =
-                  (memq x lis)
-                  (memv x lis)
-                  (%member x lis)
-                  (find-tail (lambda (y) (= x y)) lis))))
-
-(define (delete-duplicates lis . args)
-  (%case-by-cmp args =
-                (%delete-duplicates lis 'eq?)
-                (%delete-duplicates lis 'eqv?)
-                (%delete-duplicates lis 'equal?)
-                (let recur ((lis lis))
-                  (if (null-list? lis) lis
-                      (let* ((x (car lis))
-                             (tail (cdr lis))
-                             (new-tail (recur (delete x tail =))))
-                        (if (eq? tail new-tail) lis (cons x new-tail)))))))
-
-(define (delete-duplicates! lis . args)
-  (%case-by-cmp args =
-                (%delete-duplicates! lis 'eq?)
-                (%delete-duplicates! lis 'eqv?)
-                (%delete-duplicates! lis 'equal?)
-                (let recur ((lis lis))
-                  (if (null-list? lis) lis
-                      (let* ((x (car lis))
-                             (tail (cdr lis))
-                             (new-tail (recur (delete! x tail =))))
-                        (if (eq? tail new-tail) lis (cons x new-tail)))))))
-
-;;; Extended from R4RS to take an optional comparison argument.
-(define (assoc x lis . args)
-  (let ((%assoc (with-module scheme assoc)))
-    (%case-by-cmp args =
-                  (assq x lis)
-                  (assv x lis)
-                  (%assoc x lis)
-                  (find (lambda (entry) (= x (car entry))) lis))))
-
 (define alist-cons acons)
 
-(define (alist-copy alist)
-  (map (lambda (elt) (cons (car elt) (cdr elt)))
-       alist))
-
-(define (alist-delete key alist . args)
-  (%case-by-cmp args =
-                (%alist-delete key alist 'eq?)
-                (%alist-delete key alist 'eqv?)
-                (%alist-delete key alist 'equal?)
-                (filter (lambda (elt) (not (= key (car elt)))) alist)))
-
-(define (alist-delete! key alist . args)
-  (%case-by-cmp args =
-                (%alist-delete! key alist 'eq?)
-                (%alist-delete! key alist 'eqv?)
-                (%alist-delete! key alist 'equal?)
-                (filter! (lambda (elt) (not (= key (car elt)))) alist)))
-
