@@ -1,23 +1,23 @@
 ;;;
 ;;; srfi-13.scm - string library
-;;;  
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;;   
+;;;
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;;   
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
-;;;  
+;;;
 ;;;   2. Redistributions in binary form must reproduce the above copyright
 ;;;      notice, this list of conditions and the following disclaimer in the
 ;;;      documentation and/or other materials provided with the distribution.
-;;;  
+;;;
 ;;;   3. Neither the name of the authors nor the names of its contributors
 ;;;      may be used to endorse or promote products derived from this
 ;;;      software without specific prior written permission.
-;;;  
+;;;
 ;;;   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 ;;;   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 ;;;   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -29,7 +29,7 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
+;;;
 
 ;; Natively implemented functions:
 ;;   string? make-string string string->list list->string
@@ -90,6 +90,10 @@
 
 (define (%char-pred/pred c/s/p x) (c/s/p x))
 
+(define %maybe-substring (with-module gauche.internal %maybe-substring))
+(define %hash-string (with-module gauche.internal %hash-string))
+(define %string-replace-body! (with-module gauche.internal %string-replace-body!))
+
 ;;;
 ;;; Predicates
 ;;;
@@ -934,11 +938,11 @@
   (let ((patlen (vector-length rv)))
     (let lp ((si start)
              (vi i))
-      (cond ((= vi patlen) (- si)) 
+      (cond ((= vi patlen) (- si))
             ((= si end) vi)
             (else (lp (+ si 1)
                       (kmp-step pat rv (string-ref s si)
                                 vi c= p-start)))))))
-    
 
-          
+
+
