@@ -164,6 +164,13 @@
 (define match-lambda*. (%match-id 'match-lambda*))
 (define match-letrec.  (%match-id 'match-letrec))
 (define match-define.  (%match-id 'match-define))
+(define match:length>=?. (%match-id 'match:length>=?))
+
+;; In the original code, length>= is introduced as a local binding
+;; so that optimizer can inline the call.   In Gauche we can use
+;; define-inline, so we don't need to bother introducing the local
+;; bindings.
+(define-inline (match:length>=? n) (lambda (l) (>= (length l) n)))
 
 ;;; [SK] End of black magic
 
@@ -209,8 +216,7 @@
             (slot-set! obj (slot-definition-name slot) val))))))
 
 (define (genmatch x clauses match-expr)
-  (let* ((length>= (gensym))
-         (eb-errf (error-maker match-expr))
+  (let* ((eb-errf (error-maker match-expr))
          (blist (car eb-errf))
          (plist (map (lambda (c)
                        (let* ((x (bound (validate-pattern (car c))))
@@ -233,18 +239,13 @@
                                      (append bindings blist)))
                          (list p code bv (and fail (gensym)) #f)))
                      clauses))
-         (code (gen x '() plist (cdr eb-errf) length>= (gensym))))
+         (code (gen x '() plist (cdr eb-errf) (gensym))))
     (unreachable plist match-expr)
-    (inline-let
-     `(let ((,length>= (lambda (n)
-                         (lambda (l)
-                           (>= (length l) n))))
-            ,@blist)
-        ,code))))
+    `(let ,blist
+       ,code)))
 
 (define (genletrec pat exp body match-expr)
-  (let* ((length>= (gensym))
-         (eb-errf (error-maker match-expr))
+  (let* ((eb-errf (error-maker match-expr))
          (x (bound (validate-pattern pat)))
          (p (car x))
          (bv (cadr x))
@@ -252,13 +253,10 @@
          (code (gensym))
          (plist (list (list p code bv #f #f)))
          (x (gensym))
-         (m (gen x '() plist (cdr eb-errf) length>= (gensym)))
+         (m (gen x '() plist (cdr eb-errf) (gensym)))
          (gs (map (lambda (_) (gensym)) bv)))
     (unreachable plist match-expr)
-    `(letrec ((,length>= (lambda (n)
-                           (lambda (l)
-                             (>= (length l) n))))
-              ,@(map (lambda (v) `(,v #f)) bv)
+    `(letrec (,@(map (lambda (v) `(,v #f)) bv)
               (,x ,exp)
               (,code (lambda ,gs
                        ,@(map (lambda (v g) `(set! ,v ,g)) bv gs)
@@ -268,8 +266,7 @@
        ,m)))
 
 (define (gendefine pat exp match-expr)
-  (let* ((length>= (gensym))
-         (eb-errf (error-maker match-expr))
+  (let* ((eb-errf (error-maker match-expr))
          (x (bound (validate-pattern pat)))
          (p (car x))
          (bv (cadr x))
@@ -277,21 +274,22 @@
          (code (gensym))
          (plist (list (list p code bv #f #f)))
          (x (gensym))
-         (m (gen x '() plist (cdr eb-errf) length>= (gensym)))
+         (m (gen x '() plist (cdr eb-errf) (gensym)))
          (gs (map (lambda (_) (gensym)) bv)))
     (unreachable plist match-expr)
     `(begin ,@(map (lambda (v) `(define ,v #f)) bv)
-            ,(inline-let
-              `(let ((,length>= (lambda (n)
-                                  (lambda (l)
-                                    (>= (length l) n))))
-                     (,x ,exp)
-                     (,code (lambda ,gs
-                              ,@(map (lambda (v g) `(set! ,v ,g)) bv gs)
-                              (cond (#f #f))))
-                     ,@bindings
-                     ,@(car eb-errf))
-                 ,m)))))
+            `(let ((,x ,exp)
+                   (,code (lambda ,gs
+                            ,@(map (lambda (v g) `(set! ,v ,g)) bv gs)
+                            (cond (#f #f))))
+                   ,@bindings
+                   ,@(car eb-errf))
+               ,m))))
+
+(define (symbolize x)
+  (cond [(symbol? x) x]
+        [(identifier? x) (unwrap-syntax x)]
+        [else x]))
 
 (define (pattern-var? x)
   (and-let* ([x (cond [(symbol? x) x]
@@ -342,7 +340,7 @@
        ((equal? p '_) '_)
        ((pattern-var? p) p)
        ((pair? p)
-        (case (car p)
+        (case (symbolize (car p))
           ((quasiquote)
            (if (and (pair? (cdr p)) (null? (cddr p)))
              (quasi (cadr p))
@@ -415,7 +413,7 @@
        ((simple? p) p)
        ((symid? p) `',p)
        ((pair? p)
-        (cond 
+        (cond
          ((eq? (car p) 'unquote)
           (if (and (pair? (cdr p)) (null? (cddr p)))
             (ordinary (cadr p))
@@ -606,277 +604,142 @@
          '()
          (lambda (p a) (list p (reverse a) pred-bodies))))
 
-(define (inline-let let-exp)
-  (letrec ((occ (lambda (x e)
-                  (let loop ((e e))
-                    (cond
-                     ((pair? e)
-                      (+ (loop (car e)) (loop (cdr e))))
-                     ((eq? x e) 1)
-                     (else 0)))))
-           (subst (lambda (e old new)
-                    (let loop ((e e))
-                      (cond
-                       ((pair? e)
-                        (cons (loop (car e)) (loop (cdr e))))
-                       ((eq? old e) new)
-                       (else e)))))
-           (const? (lambda (sexp)
-                     (or (symid? sexp)
-                         (boolean? sexp)
-                         (string? sexp)
-                         (char? sexp)
-                         (number? sexp)
-                         (null? sexp)
-                         (and (pair? sexp)
-                              (eq? (car sexp) 'quote)
-                              (pair? (cdr sexp))
-                              (symid? (cadr sexp))
-                              (null? (cddr sexp))))))
-           (isval? (lambda (sexp)
-                     (or (const? sexp)
-                         (and (pair? sexp)
-                              (memq (car sexp)
-                                    '(lambda quote
-                                       match-lambda
-                                       match-lambda*))))))
-           (small? (lambda (sexp)
-                     (or (const? sexp)
-                         (and (pair? sexp)
-                              (eq? (car sexp) 'lambda)
-                              (pair? (cdr sexp))
-                              (pair? (cddr sexp))
-                              (const? (caddr sexp))
-                              (null? (cdddr sexp)))))))
-    (let loop ((b (cadr let-exp))
-               (new-b '())
-               (e (caddr let-exp)))
-      (cond
-       ((null? b)
-        (if (null? new-b)
-          e
-          `(let ,(reverse new-b) ,e)))
-       ((isval? (cadr (car b)))
-        (let* ((x (caar b))
-               (n (occ x e)))
-          (cond
-           ((= 0 n) (loop (cdr b) new-b e))
-           ((or (= 1 n)
-                (small?
-                 (cadr (car b))))
-            (loop (cdr b) new-b (subst e x (cadr (car b)))))
-           (else
-            (loop (cdr b) (cons (car b) new-b) e)))))
-       (else (loop (cdr b) (cons (car b) new-b) e))))))
-
-(define (gen x sf plist erract length>= eta)
-  (define (gen-rec plist memo)
-    (if (null? plist)
-      (erract x)
-      (let* ((v '())
-             (val (lambda (x) (cdr (assq x v))))
-             (fail (lambda (sf)
-                     (or (hash-table-get memo (cdr plist) #f)
-                         (rlet1 code (gen-rec (cdr plist) memo)
-                           (hash-table-put! memo (cdr plist) code)))))
-             (success (lambda (sf)
-                        (set-car! (cddddr (car plist)) #t)
-                        (let* ((code (cadr (car plist)))
-                               (bv (caddr (car plist)))
-                               (fail-sym (cadddr (car plist))))
-                          (if fail-sym
-                            (let ((ap `(,code ,fail-sym ,@(map val bv))))
-                              `(call-with-current-continuation
-                                (lambda (,fail-sym)
-                                  (let ((,fail-sym
-                                         (lambda ()
-                                           (call-with-values
-                                               (lambda () ,(fail sf))
-                                             ,fail-sym))))
-                                    ,ap))))
-                            `(,code ,@(map val bv)))))))
-        (let next ((p (caar plist))
-                   (e x)
-                   (sf sf)
-                   (kf fail)
-                   (ks success))
-          (cond
-           ((eq? '_ p) (ks sf))
-           ((symid? p) (set! v (cons (cons p e) v))
-            (ks sf))
-           ((null? p) (emit `(null? ,e) sf kf ks))
-           ((equal? p ''()) (emit `(null? ,e) sf kf ks))
-           ((string? p) (emit `(equal? ,e ,p) sf kf ks))
-           ((boolean? p) (emit `(equal? ,e ,p) sf kf ks))
-           ((char? p) (emit `(equal? ,e ,p) sf kf ks))
-           ((number? p) (emit `(equal? ,e ,p) sf kf ks))
-           ((keyword? p) (emit `(equal? ,e ,p) sf kf ks))
-           ((and (pair? p) (eq? 'quote (car p)))
-            (emit `(equal? ,e ,p) sf kf ks))
-           ((and (pair? p) (eq? '? (car p)))
-            (let ((tst `(,(cadr p) ,e)))
-              (emit tst sf kf ks)))
-           ((and (pair? p) (eq? '= (car p)))
-            (if (and (pair? (cadr p))
-                     (eq? (caadr p) 'quote))
-              (next (caddr p) `(ref ,(cadr p) ,e) sf kf ks)
-              (next (caddr p) `(,(cadr p) ,e) sf kf ks)))
-           ((and (pair? p) (eq? 'and (car p)))
+(define (gen x sf plist erract eta)
+  (if (null? plist)
+    (erract x)
+    (let* ((v '())
+           (val (lambda (x) (cdr (assq x v))))
+           (fail (lambda (sf)
+                   (gen x sf (cdr plist) erract eta)))
+           (success (lambda (sf)
+                      (set-car! (cddddr (car plist)) #t)
+                      (let* ((code (cadr (car plist)))
+                             (bv (caddr (car plist)))
+                             (fail-sym (cadddr (car plist))))
+                        (if fail-sym
+                          (let ((ap `(,code ,fail-sym ,@(map val bv))))
+                            `(call-with-current-continuation
+                              (lambda (,fail-sym)
+                                (let ((,fail-sym
+                                       (lambda ()
+                                         (call-with-values
+                                             (lambda () ,(fail sf))
+                                           ,fail-sym))))
+                                  ,ap))))
+                          `(,code ,@(map val bv)))))))
+      (let next ((p (caar plist))
+                 (e x)
+                 (sf sf)
+                 (kf fail)
+                 (ks success))
+        (cond
+         ((eq? '_ p) (ks sf))
+         ((symid? p) (set! v (cons (cons p e) v))
+          (ks sf))
+         ((null? p) (emit `(null? ,e) sf kf ks))
+         ((equal? p ''()) (emit `(null? ,e) sf kf ks))
+         ((string? p) (emit `(equal? ,e ,p) sf kf ks))
+         ((boolean? p) (emit `(equal? ,e ,p) sf kf ks))
+         ((char? p) (emit `(equal? ,e ,p) sf kf ks))
+         ((number? p) (emit `(equal? ,e ,p) sf kf ks))
+         ((keyword? p) (emit `(equal? ,e ,p) sf kf ks))
+         ((and (pair? p) (eq? 'quote (car p)))
+          (emit `(equal? ,e ,p) sf kf ks))
+         ((and (pair? p) (eq? '? (car p)))
+          (let ((tst `(,(cadr p) ,e)))
+            (emit tst sf kf ks)))
+         ((and (pair? p) (eq? '= (car p)))
+          (if (and (pair? (cadr p))
+                   (eq? (caadr p) 'quote))
+            (next (caddr p) `(ref ,(cadr p) ,e) sf kf ks)
+            (next (caddr p) `(,(cadr p) ,e) sf kf ks)))
+         ((and (pair? p) (eq? 'and (car p)))
+          (let loop ((p (cdr p))
+                     (sf sf))
+            (if (null? p)
+              (ks sf)
+              (next (car p) e sf kf (lambda (sf) (loop (cdr p) sf))))))
+         ((and (pair? p) (eq? 'or (car p)))
+          (let ((or-v v))
             (let loop ((p (cdr p))
                        (sf sf))
               (if (null? p)
-                (ks sf)
-                (next (car p) e sf kf (lambda (sf) (loop (cdr p) sf))))))
-           ((and (pair? p) (eq? 'or (car p)))
-            (let ((or-v v))
-              (let loop ((p (cdr p))
-                         (sf sf))
-                (if (null? p)
-                  (kf sf)
-                  (begin (set! v or-v)
-                         (next (car p) e sf (lambda (sf) (loop (cdr p) sf))
-                               ks))))))
-           ((and (pair? p) (eq? 'not (car p)))
-            (next (cadr p) e sf ks kf))
-           ((and (pair? p) (eq? '$ (car p)))
-            (let* ((tag (cadr p))
-                   (fields (cdr p))
-                   (rlen (length fields))
-                   (tst `(is-a? ,e ,tag)))
-              (emit tst sf kf
-                    (let rloop ((n 1))
-                      (lambda (sf)
-                        (if (= n rlen)
-                          (ks sf)
-                          (next (list-ref fields n)
-                                `(match:$-ref ,tag ,(- n 1) ,e)
-                                sf kf (rloop (+ 1 n)))))))))
-           ((and (pair? p) (eq? 'object (car p)))  ;; Gauche extension
-            (let* ((tag (cadr p))
-                   (fields (cddr p))
-                   (tst `(is-a? ,e ,tag)))
-              (emit tst sf kf
-                    (let rloop ((fields fields))
-                      (lambda (sf)
-                        (if (null? fields)
-                          (ks sf)
-                          (next (cadar fields)
-                                `(ref ,e ',(caar fields))
-                                sf kf (rloop (cdr fields)))))))))
-           ((and (pair? p) (eq? 'set! (car p)))
-            (set! v (cons (cons (cadr p) (get-setter e p)) v))
-            (ks sf))
-           ((and (pair? p) (eq? 'get! (car p)))
-            (set! v (cons (cons (cadr p) (get-getter e p)) v))
-            (ks sf))
-           ((and (pair? p)
-                 (pair? (cdr p))
-                 (dot-dot-k? (cadr p)))
-            (emit `(list? ,e) sf kf
-                  (lambda (sf)
-                    (let* ((k (dot-dot-k? (cadr p)))
-                           (ks (lambda (sf)
-                                 (let ((bound (list-ref p 2)))
-                                   (cond
-                                    ((eq? (car p) '_) (ks sf))
-                                    ((null? bound)
-                                     (let* ((eta (gensym))
-                                            (ptst (next (car p) eta sf
-                                                        (lambda (sf) #f)
-                                                        (lambda (sf) #t)))
-                                            (tst (if (and (pair? ptst)
-                                                          (symid? (car ptst))
-                                                          (pair? (cdr ptst))
-                                                          (eq? eta (cadr ptst))
-                                                          (null? (cddr ptst)))
-                                                   (car ptst)
-                                                   `(lambda (,eta) ,ptst))))
-                                       (assm `(match:every ,tst ,e)
-                                             (kf sf)
-                                             (ks sf))))
-                                    ((and (symid? (car p))
-                                          (equal? (list (car p)) bound))
-                                     (next (car p) e sf kf ks))
-                                    (else
-                                     (let* ((gloop (list-ref p 3))
-                                            (ge (list-ref p 4))
-                                            (fresh (list-ref p 5))
-                                            (p1 (next (car p)
-                                                      `(car ,ge)
-                                                      sf
-                                                      kf
-                                                      (lambda (sf)
-                                                        `(,gloop
-                                                          (cdr ,ge)
-                                                          ,@(map (lambda (b f)
-                                                                   `(cons ,(val b)
-                                                                          ,f))
-                                                                 bound
-                                                                 fresh))))))
-                                       (set! v
-                                             (append
-                                              (map cons
-                                                   bound
-                                                   (map (lambda (x)
-                                                          `(reverse ,x))
-                                                        fresh))
-                                              v))
-                                       `(let ,gloop ((,ge ,e)
-                                                     ,@(map (lambda (x)
-                                                              `(,x '()))
-                                                            fresh))
-                                             (if (null? ,ge)
-                                               ,(ks sf)
-                                               ,p1)))))))))
-                      (case k
-                        ((0) (ks sf))
-                        ((1) (emit `(pair? ,e) sf kf ks))
-                        (else (emit `((,length>= ,k) ,e) sf kf ks)))))))
-           ((pair? p) (emit `(pair? ,e)
-                            sf
-                            kf
-                            (lambda (sf)
-                              (next (car p)
-                                    (add-a e)
-                                    sf
-                                    kf
-                                    (lambda (sf)
-                                      (next (cdr p) (add-d e) sf kf ks))))))
-           ((and (vector? p)
-                 (>= (vector-length p) 6)
-                 (dot-dot-k? (vector-ref p (- (vector-length p) 5))))
-            (let* ((vlen (- (vector-length p) 6))
-                   (k (dot-dot-k? (vector-ref p (+ vlen 1))))
-                   (minlen (+ vlen k))
-                   (bound (vector-ref p (+ vlen 2))))
-              (emit `(vector? ,e)
-                    sf
-                    kf
+                (kf sf)
+                (begin (set! v or-v)
+                       (next (car p) e sf (lambda (sf) (loop (cdr p) sf))
+                             ks))))))
+         ((and (pair? p) (eq? 'not (car p)))
+          (next (cadr p) e sf ks kf))
+         ((and (pair? p) (eq? '$ (car p)))
+          (let* ((tag (cadr p))
+                 (fields (cdr p))
+                 (rlen (length fields))
+                 (tst `(is-a? ,e ,tag)))
+            (emit tst sf kf
+                  (let rloop ((n 1))
                     (lambda (sf)
-                      (assm `(>= (vector-length ,e) ,minlen)
-                            (kf sf)
-                            ((let vloop ((n 0))
-                               (lambda (sf)
+                      (if (= n rlen)
+                        (ks sf)
+                        (next (list-ref fields n)
+                              `(match:$-ref ,tag ,(- n 1) ,e)
+                              sf kf (rloop (+ 1 n)))))))))
+         ((and (pair? p) (eq? 'object (car p)))  ;; Gauche extension
+          (let* ((tag (cadr p))
+                 (fields (cddr p))
+                 (tst `(is-a? ,e ,tag)))
+            (emit tst sf kf
+                  (let rloop ((fields fields))
+                    (lambda (sf)
+                      (if (null? fields)
+                        (ks sf)
+                        (next (cadar fields)
+                              `(ref ,e ',(caar fields))
+                              sf kf (rloop (cdr fields)))))))))
+         ((and (pair? p) (eq? 'set! (car p)))
+          (set! v (cons (cons (cadr p) (get-setter e p)) v))
+          (ks sf))
+         ((and (pair? p) (eq? 'get! (car p)))
+          (set! v (cons (cons (cadr p) (get-getter e p)) v))
+          (ks sf))
+         ((and (pair? p)
+               (pair? (cdr p))
+               (dot-dot-k? (cadr p)))
+          (emit `(list? ,e) sf kf
+                (lambda (sf)
+                  (let* ((k (dot-dot-k? (cadr p)))
+                         (ks (lambda (sf)
+                               (let ((bound (list-ref p 2)))
                                  (cond
-                                  ((not (= n vlen))
-                                   (next (vector-ref p n)
-                                         `(vector-ref ,e ,n)
-                                         sf
-                                         kf
-                                         (vloop (+ 1 n))))
-                                  ((eq? (vector-ref p vlen) '_)
-                                   (ks sf))
+                                  ((eq? (car p) '_) (ks sf))
+                                  ((null? bound)
+                                   (let* ((eta (gensym))
+                                          (ptst (next (car p) eta sf
+                                                      (lambda (sf) #f)
+                                                      (lambda (sf) #t)))
+                                          (tst (if (and (pair? ptst)
+                                                        (symid? (car ptst))
+                                                        (pair? (cdr ptst))
+                                                        (eq? eta (cadr ptst))
+                                                        (null? (cddr ptst)))
+                                                 (car ptst)
+                                                 `(lambda (,eta) ,ptst))))
+                                     (assm `(match:every ,tst ,e)
+                                           (kf sf)
+                                           (ks sf))))
+                                  ((and (symid? (car p))
+                                        (equal? (list (car p)) bound))
+                                   (next (car p) e sf kf ks))
                                   (else
-                                   (let* ((gloop (vector-ref p (+ vlen 3)))
-                                          (ind (vector-ref p (+ vlen 4)))
-                                          (fresh (vector-ref p (+ vlen 5)))
-                                          (p1 (next (vector-ref p vlen)
-                                                    `(vector-ref ,e ,ind)
+                                   (let* ((gloop (list-ref p 3))
+                                          (ge (list-ref p 4))
+                                          (fresh (list-ref p 5))
+                                          (p1 (next (car p)
+                                                    `(car ,ge)
                                                     sf
                                                     kf
                                                     (lambda (sf)
                                                       `(,gloop
-                                                        (- ,ind 1)
+                                                        (cdr ,ge)
                                                         ,@(map (lambda (b f)
                                                                  `(cons ,(val b)
                                                                         ,f))
@@ -884,38 +747,106 @@
                                                                fresh))))))
                                      (set! v
                                            (append
-                                            (map cons bound fresh)
+                                            (map cons
+                                                 bound
+                                                 (map (lambda (x)
+                                                        `(reverse ,x))
+                                                      fresh))
                                             v))
-                                     `(let ,gloop
-                                        ((,ind (- (vector-length ,e) 1))
-                                         ,@(map (lambda (x) `(,x '()))
-                                                fresh))
-                                        (if (> ,minlen ,ind)
+                                     `(let ,gloop ((,ge ,e)
+                                                   ,@(map (lambda (x)
+                                                            `(,x '()))
+                                                          fresh))
+                                        (if (null? ,ge)
                                           ,(ks sf)
-                                          ,p1)))))))
-                             sf))))))
-           ((vector? p)
-            (let ((vlen (vector-length p)))
-              (emit `(vector? ,e)
-                    sf
-                    kf
-                    (lambda (sf)
-                      (emit `(equal? (vector-length ,e) ,vlen)
-                            sf
-                            kf
-                            (let vloop ((n 0))
-                              (lambda (sf)
-                                (if (= n vlen)
-                                  (ks sf)
-                                  (next (vector-ref p n)
-                                        `(vector-ref ,e ,n)
-                                        sf
-                                        kf
-                                        (vloop (+ 1 n)))))))))))
-           (else (display "FATAL ERROR IN PATTERN MATCHER")
-                 (newline)
-                 (error #f "THIS NEVER HAPPENS")))))))
-  (gen-rec plist (make-hash-table 'eq?)))
+                                          ,p1)))))))))
+                    (case k
+                      ((0) (ks sf))
+                      ((1) (emit `(pair? ,e) sf kf ks))
+                      (else (emit `((,match:length>=?. ,k) ,e) sf kf ks)))))))
+         ((pair? p) (emit `(pair? ,e)
+                          sf
+                          kf
+                          (lambda (sf)
+                            (next (car p)
+                                  (add-a e)
+                                  sf
+                                  kf
+                                  (lambda (sf)
+                                    (next (cdr p) (add-d e) sf kf ks))))))
+         ((and (vector? p)
+               (>= (vector-length p) 6)
+               (dot-dot-k? (vector-ref p (- (vector-length p) 5))))
+          (let* ((vlen (- (vector-length p) 6))
+                 (k (dot-dot-k? (vector-ref p (+ vlen 1))))
+                 (minlen (+ vlen k))
+                 (bound (vector-ref p (+ vlen 2))))
+            (emit `(vector? ,e)
+                  sf
+                  kf
+                  (lambda (sf)
+                    (assm `(>= (vector-length ,e) ,minlen)
+                          (kf sf)
+                          ((let vloop ((n 0))
+                             (lambda (sf)
+                               (cond
+                                ((not (= n vlen))
+                                 (next (vector-ref p n)
+                                       `(vector-ref ,e ,n)
+                                       sf
+                                       kf
+                                       (vloop (+ 1 n))))
+                                ((eq? (vector-ref p vlen) '_)
+                                 (ks sf))
+                                (else
+                                 (let* ((gloop (vector-ref p (+ vlen 3)))
+                                        (ind (vector-ref p (+ vlen 4)))
+                                        (fresh (vector-ref p (+ vlen 5)))
+                                        (p1 (next (vector-ref p vlen)
+                                                  `(vector-ref ,e ,ind)
+                                                  sf
+                                                  kf
+                                                  (lambda (sf)
+                                                    `(,gloop
+                                                      (- ,ind 1)
+                                                      ,@(map (lambda (b f)
+                                                               `(cons ,(val b)
+                                                                      ,f))
+                                                             bound
+                                                             fresh))))))
+                                        (set! v
+                                              (append
+                                               (map cons bound fresh)
+                                               v))
+                                        `(let ,gloop
+                                           ((,ind (- (vector-length ,e) 1))
+                                            ,@(map (lambda (x) `(,x '()))
+                                                   fresh))
+                                           (if (> ,minlen ,ind)
+                                             ,(ks sf)
+                                             ,p1)))))))
+                           sf))))))
+         ((vector? p)
+          (let ((vlen (vector-length p)))
+            (emit `(vector? ,e)
+                  sf
+                  kf
+                  (lambda (sf)
+                    (emit `(equal? (vector-length ,e) ,vlen)
+                          sf
+                          kf
+                          (let vloop ((n 0))
+                            (lambda (sf)
+                              (if (= n vlen)
+                                (ks sf)
+                                (next (vector-ref p n)
+                                      `(vector-ref ,e ,n)
+                                      sf
+                                      kf
+                                      (vloop (+ 1 n)))))))))))
+         (else (display "FATAL ERROR IN PATTERN MATCHER")
+               (newline)
+               (error #f "THIS NEVER HAPPENS")))))))
 
 (define (emit tst sf kf ks)
   (cond
@@ -967,12 +898,12 @@
       `(if (and ,tst ,(cadr s))
          ,(caddr s)
          ,f)))
-   (;; (call/cc (lambda (X) 
-    ;;            (let ((Y (lambda () (call-with-values (lambda () f) X)))) 
+   (;; (call/cc (lambda (X)
+    ;;            (let ((Y (lambda () (call-with-values (lambda () f) X))))
     ;;              BODY)))
     ;; =>
-    ;; (call/cc (lambda (X) 
-    ;;            (let ((Y (lambda () (call-with-values (lambda () f) X)))) 
+    ;; (call/cc (lambda (X)
+    ;;            (let ((Y (lambda () (call-with-values (lambda () f) X))))
     ;;              (assm tst (Y) BODY))))
     (and (pair? s)
          (equal? (car s) 'call-with-current-continuation)
@@ -995,7 +926,7 @@
          (null? (cadadr (caadar (cddadr s))))
          (pair? (cddadr (caadar (cddadr s))))
          (pair? (car (cddadr (caadar (cddadr s)))))
-         (equal? (caar (cddadr (caadar (cddadr s)))) 'call-with-values) 
+         (equal? (caar (cddadr (caadar (cddadr s)))) 'call-with-values)
          (pair? (cdar (cddadr (caadar (cddadr s)))))
          (pair? (cadar (cddadr (caadar (cddadr s)))))
          (equal? (caadar (cddadr (caadar (cddadr s)))) 'lambda)
