@@ -1,12 +1,12 @@
 /*
  * char.c - character and character set operations
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -34,6 +34,9 @@
 #include <ctype.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/char_attr.h"
+
+#include "char_attr.c"          /* generated tables */
 
 /*=======================================================================
  * Character functions
@@ -218,7 +221,7 @@ static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
             charset_print_ch(out, code, first);
             prev = code;
             first = FALSE;
-        } 
+        }
         else if (!MASK_ISSET(cs, code) && prev >= 0) {
             if (code - prev > 1) {
                 if (code - prev > 2) Scm_Printf(out, "-");
@@ -312,7 +315,7 @@ ScmChar Scm_ReadXdigitsFromPort(ScmPort *port, int ndigits,
                                 char *buf, int *nread)
 {
     int i, c, val = 0, dig;
-    
+
     for (i = 0; i < ndigits; i++) {
         SCM_GETC(c, port);
         if (c == EOF) break;
@@ -339,7 +342,7 @@ static int charset_compare(ScmObj x, ScmObj y, int equalp)
 {
     ScmCharSet *xx = SCM_CHAR_SET(x);
     ScmCharSet *yy = SCM_CHAR_SET(y);
-    
+
     if (equalp) {
         return (Scm_CharSetEq(xx, yy)? 0 : 1);
     } else {
@@ -397,7 +400,7 @@ int Scm_CharSetLE(ScmCharSet *x, ScmCharSet *y)
 ScmObj Scm_CharSetAddRange(ScmCharSet *cs, ScmChar from, ScmChar to)
 {
     ScmDictEntry *e, *lo, *hi;
-    
+
     if (to < from) return SCM_OBJ(cs);
     if (from < SCM_CHAR_SET_SMALL_CHARS) {
         if (to < SCM_CHAR_SET_SMALL_CHARS) {
@@ -443,7 +446,7 @@ ScmObj Scm_CharSetAdd(ScmCharSet *dst, ScmCharSet *src)
     ScmDictEntry *e;
 
     if (dst == src) return SCM_OBJ(dst);  /* precaution */
-    
+
     Scm_BitsOperate(dst->small, SCM_BIT_IOR, dst->small, src->small,
                     0, SCM_CHAR_SET_SMALL_CHARS);
     Scm_TreeIterInit(&iter, &src->large, NULL);
@@ -477,17 +480,31 @@ ScmObj Scm_CharSetComplement(ScmCharSet *cs)
     return SCM_OBJ(cs);
 }
 
-/* Make charset case-insensitive.  For now, we only deal with
-   ASCII range. */
+/* Make charset case-insensitive. */
 ScmObj Scm_CharSetCaseFold(ScmCharSet *cs)
 {
+    ScmCharSet *copy = SCM_CHAR_SET(Scm_CharSetCopy(cs));
+    ScmChar c, uch, lch;
+    ScmTreeIter iter;
+    ScmDictEntry *e;
     int ch;
+
     for (ch='a'; ch<='z'; ch++) {
         if (MASK_ISSET(cs, ch) || MASK_ISSET(cs, (ch-('a'-'A')))) {
             MASK_SET(cs, ch);
             MASK_SET(cs, (ch-('a'-'A')));
         }
     }
+
+    Scm_TreeIterInit(&iter, &cs->large, NULL);
+    while ((e = Scm_TreeIterNext(&iter)) != NULL) {
+        for (c = e->key; c <= e->value; c++) {
+            uch = Scm_CharUpcase(c);
+            lch = Scm_CharDowncase(c);
+            Scm_CharSetAddRange(cs, uch, uch);
+            Scm_CharSetAddRange(cs, lch, lch);
+        }
+    }
     return SCM_OBJ(cs);
 }
 
@@ -540,7 +557,6 @@ ScmObj Scm_CharSetRanges(ScmCharSet *cs)
     return h;
 }
 
-#if SCM_DEBUG_HELPER
 void Scm_CharSetDump(ScmCharSet *cs, ScmPort *port)
 {
     int i;
@@ -556,7 +572,6 @@ void Scm_CharSetDump(ScmCharSet *cs, ScmPort *port)
     Scm_TreeCoreDump(&cs->large, port);
     Scm_Printf(port, "\n");
 }
-#endif /* SCM_DEBUG_HELPER */
 
 /*-----------------------------------------------------------------
  * Reader
@@ -602,7 +617,7 @@ static ScmChar read_charset_xdigits(ScmPort *port, int ndigs, int key)
    charset (except the complimenting caret) is taken as a literal
    character, instead of terminating the charset.  It should be TRUE
    during reading the regexp syntax for compatibility to POSIX regexp.
-   
+
    If complement_p is not NULL, the location get a boolean value of
    whether complement character (caret in the beginning) appeared or not.
    In that case, the returned charset is not complemented. */
@@ -798,6 +813,198 @@ ScmObj read_predef_charset(ScmPort *input, ScmObj *chars, int error_p)
     return SCM_FALSE;
 }
 
+/*-----------------------------------------------------------------
+ * Character attributes
+ */
+
+int Scm_CharGeneralCategory(ScmChar ch)
+{
+    return (int)(Scm__LookupCharCategory(ch) & SCM_CHAR_CATEGORY_MASK);
+}
+
+int Scm_CharAlphabeticP(ScmChar ch)
+{
+    return (SCM_CHAR_ALPHA_MASK & Scm__LookupCharCategory(ch)) != 0;
+}
+
+int Scm_CharUppercaseP(ScmChar ch)
+{
+    return ((SCM_CHAR_ALPHA_MASK & Scm__LookupCharCategory(ch))
+            == SCM_CHAR_UPPERCASE_BITS);
+}
+
+int Scm_CharLowercaseP(ScmChar ch)
+{
+    return ((SCM_CHAR_ALPHA_MASK & Scm__LookupCharCategory(ch))
+            == SCM_CHAR_LOWERCASE_BITS);
+}
+
+int Scm_CharTitlecaseP(ScmChar ch)
+{
+    return (Scm_CharGeneralCategory(ch) == SCM_CHAR_CATEGORY_Lt);
+}
+
+int Scm_CharNumericP(ScmChar ch)
+{
+    return (Scm_CharGeneralCategory(ch) == SCM_CHAR_CATEGORY_Nd);
+}
+
+/* An internal entry to extract case mapping info.
+ * Internal table is compressed, so the caller must provide
+ * the buffer for ScmCharCaseMap.
+ * The function returns either the pointer to the given buffer
+ * with information filled, or a pointer to a static read-only
+ * data structure in the internal table.
+ */
+static const ScmCharCaseMap casemap_identity = {
+    0, 0, 0, {-1}, {-1}, {-1}
+};
+
+const ScmCharCaseMap *Scm__CharCaseMap(ScmChar ch,
+                                       ScmCharCaseMap *buf,
+                                       int full)
+{
+    if (ch < 0x10000) {
+        int subtable = casemap_000[(ch >> 8) & 0xff];
+        unsigned short cmap;
+
+        if (subtable == 255) return &casemap_identity;
+
+        cmap = casemap_subtable[subtable][(unsigned char)(ch & 0xff)];
+        if (cmap == SCM_CHAR_NO_CASE_MAPPING) return &casemap_identity;
+        if (cmap & 0x8000) {
+            /* mapping is extended. */
+            return &(extended_casemaps[cmap & 0x7fff]);
+        } else {
+            /* mapping is simple */
+            int off = (cmap & 0x2000)? (signed int)(cmap|~0x1fff) : cmap&0x1fff;
+            if (cmap & 0x4000) {
+                buf->to_upper_simple = off;
+                buf->to_lower_simple = 0;
+                buf->to_title_simple = off;
+            } else {
+                buf->to_upper_simple = 0;
+                buf->to_lower_simple = off;
+                buf->to_title_simple = 0;
+            }
+            if (full) {
+                /* indicate no special mappings */
+                buf->to_upper_full[0] = -1;
+                buf->to_lower_full[0] = -1;
+                buf->to_title_full[0] = -1;
+            }
+            return buf;
+        }
+    } else {
+        /* TODO: 104xx*/
+        return &casemap_identity;
+    }
+}
+
+/*
+ * Case conversion API.  For the time being, CharCaseMap works on Unicode
+ * codepoints, so we have to convert from/to ScmChar if the internal encoding
+ * is either EUC-JP or SJIS.
+ */
+#define SIMPLE_CASE(code, buf, field) \
+    (ScmChar)((code) + Scm__CharCaseMap((code), (buf), FALSE)->SCM_CPP_CAT3(to_, field, _simple))
+
+#define SIMPLE_CASE_CV(code, buf, field)    \
+    ((code) = (ScmChar)Scm_CharToUcs((int)(code)), \
+     (code) = SIMPLE_CASE(code, buf, field),       \
+     Scm_UcsToChar((int)(code)))
+
+ScmChar Scm_CharUpcase(ScmChar ch)
+{
+    ScmCharCaseMap cm;
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
+    if (ch < 0x80) return SIMPLE_CASE(ch, &cm, upper);
+    else if (Scm__CharInUnicodeP(ch)) return SIMPLE_CASE_CV(ch, &cm, upper);
+    else           return ch;
+#elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
+    return SIMPLE_CASE(ch, &cm, upper);
+#else
+    /* Latin-1 mapping and Unicode mapping differ in U+00B5 (MICRO SIGN)
+       and U+00FF (LATIN SMALL LETTER Y WITH DIAERESIS).  In Unicode
+       they map to U+039C and U+0178, respectively.  In Latin-1 we don't
+       have those characters, so we leave them alone. */
+    if (ch == 0xb5 || ch == 0xff) return ch;
+    else return SIMPLE_CASE(ch, &cm, upper);
+#endif
+}
+
+ScmChar Scm_CharDowncase(ScmChar ch)
+{
+    ScmCharCaseMap cm;
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
+    if (ch < 0x80) return SIMPLE_CASE(ch, &cm, lower);
+    else if (Scm__CharInUnicodeP(ch)) return SIMPLE_CASE_CV(ch, &cm, lower);
+    else           return ch;
+#else
+    return SIMPLE_CASE(ch, &cm, lower);
+#endif
+}
+
+ScmChar Scm_CharTitlecase(ScmChar ch)
+{
+    ScmCharCaseMap cm;
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
+    if (ch < 0x80) return SIMPLE_CASE(ch, &cm, title);
+    else if (Scm__CharInUnicodeP(ch)) return SIMPLE_CASE_CV(ch, &cm, title);
+    else           return ch;
+#elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
+    return SIMPLE_CASE(ch, &cm, title);
+#else
+    /* In Latin-1, titlecase is the same as upcase. */
+    return Scm_CharUpcase(ch);
+#endif
+}
+
+ScmChar Scm_CharFoldcase(ScmChar ch)
+{
+    ScmCharCaseMap cm;
+    const ScmCharCaseMap *pcm;
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
+    if (Scm__CharInUnicodeP(ch)) {
+        ScmChar ucs = (ScmChar)Scm_CharToUcs(ch);
+        pcm = Scm__CharCaseMap(ucs, &cm, FALSE);
+        if (pcm->to_lower_simple == 0 && pcm->to_upper_simple == 0) {
+            /* we don't have case folding */
+            return ch;
+        }
+        /* Otherwise, we do (char-downcase (char-upcase ch)) */
+        if (pcm->to_upper_simple != 0) {
+            ucs += pcm->to_upper_simple;
+            pcm = Scm__CharCaseMap(ucs, &cm, FALSE);
+        }
+        return Scm_UcsToChar((int)(ucs + pcm->to_lower_simple));
+    } else {
+        return ch;
+    }
+#elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
+    if (ch == 0x130 || ch == 0x131) {
+        /* char-foldcase is identity for
+           U+0130 Turkish I (LATIN CAPITAL LETTER I WITH DOT ABOVE) and
+           U+0131 Turkish i (LATIN SMALL LETTER DOTLESS I) */
+        return ch;
+    }
+    pcm = Scm__CharCaseMap(ch, &cm, FALSE);
+    if (pcm->to_lower_simple == 0 && pcm->to_upper_simple == 0) {
+        /* we don't have case folding */
+        return ch;
+    }
+    /* Otherwise, we do (char-downcase (char-upcase ch)) */
+    if (pcm->to_upper_simple != 0) {
+        ch += pcm->to_upper_simple;
+        pcm = Scm__CharCaseMap(ch, &cm, FALSE);
+    }
+    return ch + pcm->to_lower_simple;
+#else
+    /* In Latin-1 range, foldcase is the same as donwcase. */
+    return SIMPLE_CASE(ch, &cm, lower);
+#endif
+}
+
 /*-----------------------------------------------------------------
  * Pre-defined charset
  */
