@@ -1,12 +1,12 @@
 /*
  * code.c - compiled code builder/handler
  *
- *   Copyright (c) 2005-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2005-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -121,18 +121,18 @@ static ScmObj execute_toplevels(ScmObj *args, int nargs, void *cv)
 /*----------------------------------------------------------------------
  * Disassembler
  */
+static ScmObj check_lifted_closure(ScmWord *p, ScmObj lifted);
+static void print_header(const char *prefix, ScmObj name, ScmCompiledCode *cc);
+
 void Scm_CompiledCodeDump(ScmCompiledCode *cc)
 {
     int i;
     ScmWord *p;
-    ScmObj closures = SCM_NIL, cp;
-    int clonum = 0;
+    ScmObj closures = SCM_NIL, lifted = SCM_NIL, shown_lifted = SCM_NIL, cp;
+    int clonum = 0, more = FALSE;
 
-    Scm_Printf(SCM_CUROUT, "main_code (name=%S, code=%p, size=%d, const=%d, stack=%d):\n",
-               cc->name, cc->code, cc->codeSize, cc->constantSize,
-               cc->maxstack);
+    print_header("main_code", SCM_MAKE_STR(""), cc);
     do {
-      loop:
         p = cc->code;
         Scm_Printf(SCM_CUROUT, "args: %S\n", cc->argInfo);
         for (i=0; i < cc->codeSize; i++) {
@@ -145,7 +145,7 @@ void Scm_CompiledCodeDump(ScmCompiledCode *cc)
             info = Scm_Assq(SCM_MAKE_INT(i), cc->info);
             code = SCM_VM_INSN_CODE(insn);
             insn_name = Scm_VMInsnName(code);
-            
+
             switch (Scm_VMInsnNumParams(code)) {
             case 0:
                 Scm_Printf(out, "  %4d %s ", i, insn_name);
@@ -165,6 +165,8 @@ void Scm_CompiledCodeDump(ScmCompiledCode *cc)
                 i++;
                 break;
             case SCM_VM_OPERAND_OBJ:
+                /* Check if we're referring to a lifted closure. */
+                lifted = check_lifted_closure(p+i, lifted);
                 Scm_Printf(out, "%S", p[i+1]);
                 i++;
                 break;
@@ -220,15 +222,81 @@ void Scm_CompiledCodeDump(ScmCompiledCode *cc)
                 }
             }
         }
+        more = FALSE;
         if (!SCM_NULLP(closures)) {
             cc = SCM_COMPILED_CODE(SCM_CAAR(closures));
-            Scm_Printf(SCM_CUROUT, "internal_closure_%S (name=%S, code=%p, size=%d, const=%d stack=%d):\n",
-                       SCM_CDAR(closures), cc->name, cc->code,
-                       cc->codeSize, cc->constantSize, cc->maxstack);
+            print_header("closure:", SCM_CDAR(closures), cc);
             closures = SCM_CDR(closures);
-            goto loop;
+            more = TRUE;
+        } else if (!SCM_NULLP(lifted)) {
+            while (!SCM_NULLP(lifted)) {
+                if (SCM_FALSEP(Scm_Memq(SCM_CAAR(lifted), shown_lifted))) {
+                    cc = SCM_COMPILED_CODE(SCM_CAAR(lifted));
+                    print_header("lifted:", SCM_CDAR(lifted), cc);
+                    shown_lifted = Scm_Cons(SCM_CAAR(lifted), shown_lifted);
+                    lifted = SCM_CDR(lifted);
+                    more = TRUE;
+                    break;
+                } else {
+                    lifted = SCM_CDR(lifted);
+                    continue;
+                }
+            }
         }
-    } while (0);
+    } while (more);
+}
+
+void print_header(const char *prefix, ScmObj name, ScmCompiledCode *cc)
+{
+    Scm_Printf(SCM_CUROUT, "=== %s%A (name=%S, code=%p, size=%d, const=%d stack=%d):\n",
+               prefix, name, cc->name, cc->code,
+               cc->codeSize, cc->constantSize, cc->maxstack);
+}
+
+/* The compiler may have lifted an internal closure to a global procedure.
+   We can tel so if the opcode is GREF_x, and the operand is an identifier,
+   whose name is an uninterned symbol and it is globally bound to a procedure.
+
+   If we indeed have a lifted closure, we chain the closure's code and
+   the identifier into the lifted list, returns the updated list.
+   Otherwise, we return lifted list as is.
+ */
+ScmObj check_lifted_closure(ScmWord *p, ScmObj lifted)
+{
+    ScmIdentifier *id;
+    ScmObj g;
+    ScmWord code = SCM_VM_INSN_CODE(p[0]);
+    int i;
+    static ScmWord gref_insns[] = {
+        SCM_VM_GREF,
+        SCM_VM_GREF_PUSH,
+        SCM_VM_GREF_CALL,
+        SCM_VM_GREF_TAIL_CALL,
+        SCM_VM_PUSH_GREF,
+        SCM_VM_PUSH_GREF_CALL,
+        SCM_VM_PUSH_GREF_TAIL_CALL
+    };
+
+    if (!SCM_IDENTIFIERP(p[1])) return lifted;
+    id = SCM_IDENTIFIER(p[1]);
+    if (SCM_SYMBOL_INTERNED(id->name)) return lifted;
+
+    for (i=0; i < sizeof(gref_insns)/sizeof(ScmWord); i++) {
+        if (code == gref_insns[i]) {
+            g = Scm_GlobalVariableRef(id->module, id->name,
+                                      SCM_BINDING_STAY_IN_MODULE);
+            if (SCM_CLOSUREP(g)) {
+                if (SCM_FALSEP(Scm_Assq(SCM_CLOSURE(g)->code, lifted))) {
+                    return Scm_Acons(SCM_CLOSURE(g)->code,
+                                     SCM_OBJ(id->name),
+                                     lifted);
+                } else {
+                    return lifted;
+                }
+            }
+        }
+    }
+    return lifted;
 }
 
 /*------------------------------------------------------------------
@@ -351,7 +419,7 @@ static void finish_transition(cc_builder *b);
 static void cc_builder_flush(cc_builder *b)
 {
     u_int code;
-    
+
     if ((b)->currentInsn == CC_BUILDER_BUFFER_EMPTY) return;
     if ((b)->currentInsn == CC_BUILDER_BUFFER_TRANS) {
         finish_transition(b);
@@ -477,7 +545,7 @@ ScmObj Scm_CompiledCodeNewLabel(ScmCompiledCode *cc)
 void Scm_CompiledCodeSetLabel(ScmCompiledCode *cc, ScmObj label)
 {
     cc_builder *b;
-    
+
     CC_BUILDER_GET(b, cc);
 
     /* Flush buffered insn first. */
@@ -553,7 +621,7 @@ void Scm_CompiledCodeFinishBuilder(ScmCompiledCode *cc, int maxstack)
 
     /* set max stack depth */
     cc->maxstack = maxstack;
-    
+
     /* make sure this code is 'fixed'---no more building */
     cc->builder = NULL;
 }
@@ -728,14 +796,14 @@ ScmObj Scm_CompiledCodeToList(ScmCompiledCode *cc)
 {
     u_int i, off;
     ScmObj h = SCM_NIL, t = SCM_NIL;
-    
+
     for (i=0; i<(u_int)cc->codeSize; i++) {
         ScmWord insn = cc->code[i];
         u_int code = SCM_VM_INSN_CODE(insn);
         const char *name = Scm_VMInsnName(code);
-        
+
         switch (Scm_VMInsnNumParams(code)) {
-        case 0: 
+        case 0:
             SCM_APPEND1(h, t, SCM_LIST1(SCM_INTERN(name)));
             break;
         case 1:
@@ -874,7 +942,7 @@ int Scm_VMInsnNameToCode(ScmObj name)
     const char *n;
     struct insn_info *info;
     int i;
-    
+
     if (SCM_SYMBOLP(name))  name = SCM_OBJ(SCM_SYMBOL_NAME(name));
     else if (!SCM_STRINGP(name)) {
         Scm_Error("vm-insn-name->code: requires a symbol or a string, but got %S", name);
@@ -894,10 +962,10 @@ int Scm_VMInsnNameToCode(ScmObj name)
 ScmWord Scm_VMInsnBuild(ScmObj obj)
 {
     int len = Scm_Length(obj), code, arg0, arg1;
-    
+
     if (len < 1 || len > 3 || !SCM_SYMBOLP(SCM_CAR(obj))) goto badspec;
     code = Scm_VMInsnNameToCode(SCM_CAR(obj));
-    
+
     switch (Scm_VMInsnNumParams(code)) {
     case 0:
         if (len != 1) {
@@ -925,7 +993,7 @@ ScmWord Scm_VMInsnBuild(ScmObj obj)
         return SCM_VM_INSN2(code, arg0, arg1);
     }
     /*FALLTHROUGH*/
-  badspec:    
+  badspec:
     Scm_Error("Bad VM insn spec: %S", obj);
     return 0;       /* dummy */
 }
