@@ -1,12 +1,12 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -77,6 +77,7 @@ extern void Scm__InitWrite(void);
 extern void Scm__InitCompaux(void);
 extern void Scm__InitMacro(void);
 extern void Scm__InitLoad(void);
+extern void Scm__InitParameter(void);
 extern void Scm__InitProc(void);
 extern void Scm__InitRegexp(void);
 extern void Scm__InitRead(void);
@@ -88,14 +89,27 @@ extern void Scm__InitRepl(void);
 extern void Scm__InitAutoloads(void);
 extern void Scm__InitCollection(void);
 
-extern void Scm_Init_stdlib(ScmModule *);
-extern void Scm_Init_extlib(ScmModule *);
-extern void Scm_Init_syslib(ScmModule *);
-extern void Scm_Init_intlib(ScmModule *);
-
-extern void Scm_Init_scmlib(void);
+extern void Scm_Init_libalpha(void);
+extern void Scm_Init_libbool(void);
+extern void Scm_Init_libchar(void);
+extern void Scm_Init_libdict(void);
+extern void Scm_Init_libeval(void);
+extern void Scm_Init_libexc(void);
+extern void Scm_Init_libio(void);
+extern void Scm_Init_liblazy(void);
+extern void Scm_Init_liblist(void);
+extern void Scm_Init_libmisc(void);
+extern void Scm_Init_libmod(void);
+extern void Scm_Init_libnum(void);
+extern void Scm_Init_libobj(void);
+extern void Scm_Init_libproc(void);
+extern void Scm_Init_librx(void);
+extern void Scm_Init_libstr(void);
+extern void Scm_Init_libsym(void);
+extern void Scm_Init_libsys(void);
+extern void Scm_Init_libvec(void);
 extern void Scm_Init_compile(void);
-extern void Scm_Init_objlib(void);
+extern void Scm_Init_libomega(void);
 
 static void finalizable(void);
 static void init_cond_features(void);
@@ -131,6 +145,7 @@ void Scm_Init(const char *signature)
 
     /* Initialize components.  The order is important, for some components
        rely on the other components to be initialized. */
+    Scm__InitParameter();
     Scm__InitVM();
     Scm__InitSymbol();
     Scm__InitModule();
@@ -153,14 +168,27 @@ void Scm_Init(const char *signature)
     Scm__InitSystem();
     Scm__InitRepl();
 
-    Scm_Init_stdlib(Scm_SchemeModule());
-    Scm_Init_extlib(Scm_GaucheModule());
-    Scm_Init_syslib(Scm_GaucheModule());
-    Scm_Init_intlib(Scm_GaucheInternalModule());
-
-    Scm_Init_scmlib();
+    Scm_Init_libalpha();
+    Scm_Init_libbool();
+    Scm_Init_libchar();
+    Scm_Init_libdict();
+    Scm_Init_libeval();
+    Scm_Init_libexc();
+    Scm_Init_libio();
+    Scm_Init_liblazy();
+    Scm_Init_liblist();
+    Scm_Init_libmisc();
+    Scm_Init_libmod();
+    Scm_Init_libnum();
+    Scm_Init_libobj();
+    Scm_Init_libproc();
+    Scm_Init_librx();
+    Scm_Init_libstr();
+    Scm_Init_libsym();
+    Scm_Init_libsys();
+    Scm_Init_libvec();
     Scm_Init_compile();
-    Scm_Init_objlib();
+    Scm_Init_libomega();
 
     Scm__InitCompaux();
 
@@ -244,12 +272,14 @@ void Scm_UnregisterFinalizer(ScmObj z)
                                    &ofn, &ocd);
 }
 
-/* GC calls this back when finalizers are queued */
+/* GC calls this back when finalizers are queued. */
 void finalizable(void)
 {
     ScmVM *vm = Scm_VM();
-    vm->finalizerPending = TRUE;
-    vm->attentionRequest = TRUE;
+    if (vm != NULL) {
+        vm->finalizerPending = TRUE;
+        vm->attentionRequest = TRUE;
+    }
 }
 
 /* Called from VM loop.  Queue is not empty. */
@@ -273,7 +303,7 @@ struct cleanup_handler_rec {
 static struct {
     int dirty;                  /* Flag to avoid cleaning up more than once. */
     struct cleanup_handler_rec *handlers;
-} cleanup = { TRUE, NULL }; 
+} cleanup = { TRUE, NULL };
 
 /* Add cleanup handler.  Returns an opaque handle, which can be
    passed to DeleteCleanupHandler. */
@@ -305,14 +335,13 @@ void Scm_DeleteCleanupHandler(void *handle)
     }
 }
 
-/* pthread specific cleanup handler to unlock mutex */
-#ifdef GAUCHE_USE_PTHREADS
+#if defined(GAUCHE_HAS_THREADS)
 void Scm__MutexCleanup(void *mutex_)
 {
     ScmInternalMutex *mutex = mutex_;
     (void)SCM_INTERNAL_MUTEX_UNLOCK(*mutex);
 }
-#endif /* GAUCHE_USE_PTHREADS */
+#endif /* GAUCHE_HAS_THREADS */
 
 
 /* Scm_Cleanup and Scm_Exit
@@ -338,7 +367,7 @@ void Scm_Cleanup(void)
 
     if (!cleanup.dirty) return;
     cleanup.dirty = FALSE;
-    
+
     /* Execute pending dynamic handlers.
        NB: we ignore errors here.  It may not be accurate behavior, though.
        (A handler may intentionally raise an error to skip the rest of
@@ -353,7 +382,7 @@ void Scm_Cleanup(void)
     for (ch = cleanup.handlers; ch; ch = ch->next) {
         ch->handler(ch->data);
     }
-    
+
     /* Flush Scheme ports. */
     Scm_FlushAllPorts(TRUE);
 }
@@ -393,8 +422,8 @@ Scm_GetFeatures()
  * Append FEATURE to the cond-features alist.  Once added, the symbol FEATURE
  * will be recognized by cond-expand.
  *
- *  (cond-expand (FEATURE body ...)) 
- *   
+ *  (cond-expand (FEATURE body ...))
+ *
  * If loading a module is required in order to make FEATURE available,
  * such module name can be specified in MODULE argument.  It can be NULL
  * if the feature is built-in.
@@ -412,7 +441,7 @@ Scm_AddFeature(const char *feature, const char *module)
     } else {
         cell = SCM_LIST1(SCM_INTERN(feature));
     }
-    
+
     (void)SCM_INTERNAL_MUTEX_LOCK(cond_features.mutex);
     cond_features.alist = Scm_Cons(cell, cond_features.alist);
     (void)SCM_INTERNAL_MUTEX_UNLOCK(cond_features.mutex);
@@ -428,16 +457,35 @@ init_cond_features()
         const char *module;
     } init_features[] = {
         { "gauche", NULL },
-#ifdef GAUCHE_WINDOWS
+
+        /* Platform */
+#if   defined(GAUCHE_WINDOWS)
         { "gauche.os.windows", NULL },
         { "gauche-windows", NULL }, /* for backward compatibility */
-#endif
-#ifdef __CYGWIN__              /* cygwin is different enough to deserve this */
+#elif defined(__CYGWIN__)  /* cygwin is different enough to deserve this */
         { "gauche.os.cygwin", NULL },
 #endif
-#ifdef GAUCHE_USE_PTHREADS
+
+        /* Threads */
+#if   defined(GAUCHE_USE_PTHREADS)
+        { "gauche.sys.threads", "gauche.threads" },
         { "gauche.sys.pthreads", "gauche.threads" },
+#elif defined(GAUCHE_USE_WTHREADS)
+        { "gauche.sys.threads", "gauche.threads" },
+        { "gauche.sys.wthreads", "gauche.threads" },
 #endif
+
+        /* TLS/SSL.  This nees to be in the core in order to switch
+           code _before_ loading rfc.tls */
+#if defined(GAUCHE_USE_AXTLS)
+        { "gauche.net.tls", "rfc.tls" },
+        { "gauche.net.tls.axtls", "rfc.tls" },
+#elif defined(GAUCHE_USE_OPENSSL)
+        { "gauche.net.tls", "rfc.tls" },
+        { "gauche.net.tls.openssl", "rfc.tls" },
+#endif
+
+        /* SRFIs */
         { "srfi-0", NULL },         /* autoloaded */
         { "srfi-1", "srfi-1" },
         { "srfi-2", NULL },         /* builtin */
@@ -551,7 +599,7 @@ ScmObj Scm_SiteArchitectureDirectory(void)
    the returned values of other Scm_*Directory functions above.
    This is needed to augument the returned values of gauche.config
    module.  Some refactorization is needed in future.
-   
+
    Aside from the current purpose, it will be useful to have a procedure
    to obtain the directory of the running executable.  This function
    can eventually evlove to it.
@@ -596,9 +644,9 @@ void Scm_SimpleMain(int argc, const char *argv[],
     ScmLoadPacket lpak;
 
     SCM_ASSERT(argc > 0);
-    if (Scm_Load("gauche-init.scm", 0, NULL)) {
+    if (Scm_Load("gauche-init.scm", 0, &lpak)) {
         Scm_Printf(SCM_CURERR, "%s: Couldn't load gauche-init.scm: %A(%A).\n",
-                   argv[0], 
+                   argv[0],
                    Scm_ConditionMessage(lpak.exception),
                    Scm_ConditionTypeName(lpak.exception));
         Scm_Exit(1);
