@@ -1,7 +1,7 @@
 /*
  * gauche.h - Gauche scheme system header
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -40,6 +40,11 @@
    get pthread-compatible sigwait()---but we may encounter more of such
    instances. */
 #include <gauche/config.h>
+#include <gauche/config_threads.h>
+
+#ifndef GAUCHE_API_0_9
+#define GAUCHE_API_0_9   1      /* 0.9 API */
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -55,6 +60,24 @@
 #include <gauche/float.h>
 #include <gauche/arch.h>
 
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
+/* For Windows platforms, we need some compatibility tricks.
+   This defines GAUCHE_WINDOWS preprocessor symbol.
+   (This should come before including gc.h) */
+#if defined(__MINGW32__) || defined(MSVC)
+#include <gauche/win-compat.h>
+#endif /* MINGW32 || WINDOWS */
+
 #if defined(LIBGAUCHE_BODY)
 #if !defined(GC_DLL)
 #define GC_DLL    /* for gc.h to handle Win32 crazyness */
@@ -77,27 +100,10 @@
 
 SCM_DECL_BEGIN
 
-#ifdef TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# ifdef HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif /*HAVE_UNISTD_H*/
 
-/* For Windows platforms, we need some compatibility tricks.
-   This defines GAUCHE_WINDOWS preprocessor symbol. */
-#if defined(__MINGW32__) || defined(MSVC)
-#include <gauche/win-compat.h>
-#endif /* MINGW32 || WINDOWS */
-
 /* Defines SCM_EXTERN magic. */
 #include <gauche/extern.h>
 
@@ -109,20 +115,20 @@ SCM_DECL_BEGIN
 #define TRUE (!FALSE)
 #endif
 
-/* This defines several auxiliary routines that are useful for debugging */
-#ifndef SCM_DEBUG_HELPER
-#define SCM_DEBUG_HELPER      TRUE
-#endif
-
 /* Define this to 0 to turn off fast flonum extension.  See the comment in
    gauche/number.h for the details. */
 #define GAUCHE_FFX 1
 
+/* Define this to 0 to turn off lazy-pair feature. */
+#define GAUCHE_LAZY_PAIR 1
+
 /* Include appropriate threading interface.  Threading primitives are
    abstracted with SCM_INTERNAL_* macros and ScmInternal* typedefs.
    See gauche/uthread.h for the semantics of these primitives. */
 #ifdef GAUCHE_USE_PTHREADS
 # include <gauche/pthread.h>
+#elif  GAUCHE_USE_WTHREADS
+# include <gauche/wthread.h>
 #else  /* !GAUCHE_USE_PTHREADS */
 # include <gauche/uthread.h>
 #endif /* !GAUCHE_USE_PTHREADS */
@@ -324,10 +330,12 @@ typedef struct ScmFlonumRec {
 #define SCM_CHAR_MAX            (0xffffff)
 
 #define SCM_CHAR_ASCII_P(ch)    ((ch) < 0x80)
-#define SCM_CHAR_UPPER_P(ch)    (('A' <= (ch)) && ((ch) <= 'Z'))
-#define SCM_CHAR_LOWER_P(ch)    (('a' <= (ch)) && ((ch) <= 'z'))
-#define SCM_CHAR_UPCASE(ch)     (SCM_CHAR_LOWER_P(ch)?((ch)-('a'-'A')):(ch))
-#define SCM_CHAR_DOWNCASE(ch)   (SCM_CHAR_UPPER_P(ch)?((ch)+('a'-'A')):(ch))
+
+/* The following four macros are obsoleted; use API version instead.*/
+#define SCM_CHAR_UPPER_P(ch)    Scm_CharUppercaseP(ch)
+#define SCM_CHAR_LOWER_P(ch)    Scm_CharLowercaseP(ch)
+#define SCM_CHAR_UPCASE(ch)     Scm_CharUpcase(ch)
+#define SCM_CHAR_DOWNCASE(ch)   Scm_CharDowncase(ch)
 
 SCM_EXTERN int Scm_DigitToInt(ScmChar ch, int radix);
 SCM_EXTERN ScmChar Scm_IntToDigit(int n, int radix);
@@ -337,6 +345,18 @@ SCM_EXTERN ScmObj Scm_CharEncodingName(void);
 SCM_EXTERN const char **Scm_SupportedCharacterEncodings(void);
 SCM_EXTERN int Scm_SupportedCharacterEncodingP(const char *encoding);
 
+SCM_EXTERN int Scm_CharGeneralCategory(ScmChar ch);
+SCM_EXTERN int Scm_CharAlphabeticP(ScmChar ch);
+SCM_EXTERN int Scm_CharUppercaseP(ScmChar ch);
+SCM_EXTERN int Scm_CharLowercaseP(ScmChar ch);
+SCM_EXTERN int Scm_CharTitlecaseP(ScmChar ch);
+SCM_EXTERN int Scm_CharNumericP(ScmChar ch);
+
+SCM_EXTERN ScmChar Scm_CharUpcase(ScmChar ch);
+SCM_EXTERN ScmChar Scm_CharDowncase(ScmChar ch);
+SCM_EXTERN ScmChar Scm_CharTitlecase(ScmChar ch);
+SCM_EXTERN ScmChar Scm_CharFoldcase(ScmChar ch);
+
 SCM_EXTERN void Scm__InstallCharconvHooks(ScmChar (*u2c)(int),
                                           int (*c2u)(ScmChar));
 
@@ -454,6 +474,7 @@ SCM_EXTERN void Scm_UnregisterFinalizer(ScmObj z);
 typedef struct ScmVMRec        ScmVM;
 typedef struct ScmPairRec      ScmPair;
 typedef struct ScmExtendedPairRec ScmExtendedPair;
+typedef struct ScmLazyPairRec  ScmLazyPair;
 typedef struct ScmCharSetRec   ScmCharSet;
 typedef struct ScmStringRec    ScmString;
 typedef struct ScmDStringRec   ScmDString;
@@ -592,6 +613,22 @@ SCM_EXTERN ScmObj Scm_VMGetResult(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStackLite(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStack(ScmVM *vm);
 
+/* A box is to keep a reference.  It is mainly used for mutable local variables.
+ */
+
+typedef struct ScmBoxRec {
+    SCM_HEADER;
+    ScmObj value;
+} ScmBox;
+
+SCM_CLASS_DECL(Scm_BoxClass);
+#define SCM_CLASS_BOX            (&Scm_BoxClass)
+#define SCM_BOX(obj)             ((ScmBox*)(obj))
+#define SCM_BOXP(obj)            (SCM_XTYPEP(obj, SCM_CLASS_BOX))
+#define SCM_BOX_VALUE(obj)       (SCM_BOX(obj)->value)
+#define SCM_BOX_SET(obj, val)    (SCM_BOX(obj)->value = (val))
+
+SCM_EXTERN ScmBox *Scm_MakeBox(ScmObj value);
 
 /*---------------------------------------------------------
  * CLASS
@@ -722,13 +759,23 @@ enum {
 
     /* A special flag that only be used for "natively applicable"
        objects, which basically inherits ScmProcedure. */
-    SCM_CLASS_APPLICABLE = 0x04
+    SCM_CLASS_APPLICABLE = 0x04,
+
+    /* If this flag is set, important slots such as class-precedence-list
+       or class-slots becomes settable.
+       We reset this flag at the end of class initialization, so that
+       we can avoid the behavior of a class from being accidentally
+       chnaged.  The flag may be set during updating a class metaobject
+       triggered by metaclass change (see lib/gauche/redefutil.scm).
+     */
+    SCM_CLASS_MALLEABLE = 0x08
 };
 
-#define SCM_CLASS_FLAGS(obj)     (SCM_CLASS(obj)->flags)
+#define SCM_CLASS_FLAGS(obj)        (SCM_CLASS(obj)->flags)
 #define SCM_CLASS_APPLICABLE_P(obj) (SCM_CLASS_FLAGS(obj)&SCM_CLASS_APPLICABLE)
 
-#define SCM_CLASS_CATEGORY(obj)  (SCM_CLASS_FLAGS(obj)&3)
+#define SCM_CLASS_CATEGORY(obj)     (SCM_CLASS_FLAGS(obj)&3)
+#define SCM_CLASS_MALLEABLE_P(obj)  (SCM_CLASS_FLAGS(obj)&SCM_CLASS_MALLEABLE)
 
 SCM_EXTERN void Scm_InitStaticClass(ScmClass *klass, const char *name,
                                     ScmModule *mod,
@@ -759,12 +806,16 @@ SCM_EXTERN int Scm_SubtypeP(ScmClass *sub, ScmClass *type);
 SCM_EXTERN int Scm_TypeP(ScmObj obj, ScmClass *type);
 SCM_EXTERN ScmClass *Scm_BaseClassOf(ScmClass *klass);
 
+SCM_EXTERN void   Scm_ClassMalleableSet(ScmClass *klass, int flag);
+
 SCM_EXTERN ScmObj Scm_VMSlotRef(ScmObj obj, ScmObj slot, int boundp);
 SCM_EXTERN ScmObj Scm_VMSlotSet(ScmObj obj, ScmObj slot, ScmObj value);
 SCM_EXTERN ScmObj Scm_VMSlotBoundP(ScmObj obj, ScmObj slot);
 
+
 /* built-in classes */
 SCM_CLASS_DECL(Scm_TopClass);
+SCM_CLASS_DECL(Scm_BottomClass);
 SCM_CLASS_DECL(Scm_BoolClass);
 SCM_CLASS_DECL(Scm_CharClass);
 SCM_CLASS_DECL(Scm_ClassClass);
@@ -776,6 +827,7 @@ SCM_CLASS_DECL(Scm_ForeignPointerClass);
 
 
 #define SCM_CLASS_TOP              (&Scm_TopClass)
+#define SCM_CLASS_BOTTOM           (&Scm_BottomClass)
 #define SCM_CLASS_BOOL             (&Scm_BoolClass)
 #define SCM_CLASS_CHAR             (&Scm_CharClass)
 #define SCM_CLASS_CLASS            (&Scm_ClassClass)
@@ -946,7 +998,12 @@ struct ScmExtendedPairRec {
     ScmObj attributes;          /* should be accessed via API func. */
 };
 
-#define SCM_PAIRP(obj)          (SCM_HPTRP(obj)&&SCM_HTAG(obj)!=7)
+#if GAUCHE_LAZY_PAIR
+#define SCM_PAIRP(obj)  \
+    (SCM_HPTRP(obj)&&(SCM_HTAG(obj)!=7||Scm_PairP(SCM_OBJ(obj))))
+#else  /*!GAUCHE_LAZY_PAIR*/
+#define SCM_PAIRP(obj)          (SCM_HPTRP(obj)&&(SCM_HTAG(obj)!=7))
+#endif /*!GAUCHE_LAZY_PAIR*/
 
 #define SCM_PAIR(obj)           ((ScmPair*)(obj))
 #define SCM_CAR(obj)            (SCM_PAIR(obj)->car)
@@ -1044,6 +1101,8 @@ SCM_EXTERN ScmObj Scm_Append2(ScmObj list, ScmObj obj);
 SCM_EXTERN ScmObj Scm_Append(ScmObj args);
 SCM_EXTERN ScmObj Scm_ReverseX(ScmObj list);
 SCM_EXTERN ScmObj Scm_Reverse(ScmObj list);
+SCM_EXTERN ScmObj Scm_Reverse2X(ScmObj list, ScmObj tail);
+SCM_EXTERN ScmObj Scm_Reverse2(ScmObj list, ScmObj tail);
 SCM_EXTERN ScmObj Scm_ListTail(ScmObj list, int i, ScmObj fallback);
 SCM_EXTERN ScmObj Scm_ListRef(ScmObj list, int i, ScmObj fallback);
 SCM_EXTERN ScmObj Scm_LastPair(ScmObj list);
@@ -1064,6 +1123,7 @@ SCM_EXTERN ScmObj Scm_DeleteDuplicates(ScmObj list, int cmpmode);
 SCM_EXTERN ScmObj Scm_DeleteDuplicatesX(ScmObj list, int cmpmode);
 
 SCM_EXTERN ScmObj Scm_MonotonicMerge(ScmObj start, ScmObj sequences);
+SCM_EXTERN ScmObj Scm_MonotonicMerge1(ScmObj sequences);
 SCM_EXTERN ScmObj Scm_Union(ScmObj list1, ScmObj list2);
 SCM_EXTERN ScmObj Scm_Intersection(ScmObj list1, ScmObj list2);
 
@@ -1479,7 +1539,36 @@ SCM_CLASS_DECL(Scm_NextMethodClass);
 #define SCM_NEXT_METHODP(obj)      SCM_XTYPEP(obj, SCM_CLASS_NEXT_METHOD)
 #define SCM_NEXT_METHOD(obj)       ((ScmNextMethod*)obj)
 
-/* Other APIs */
+/* Calling a Scheme function from C
+ *
+ *  static ScmObj proc = SCM_UNDEFINED;
+ *
+ *  SCM_BIND_PROC(proc, "scheme-proc-name", module);
+ *
+ *  Scm_ApplyRec(proc, args);
+ *   or
+ *  Scm_Apply(proc, args, &result);
+ *
+ * SCM_BIND_PROC macro initializes the C variable proc to the value of
+ * the global Scheme variable scheme-proc-name in the module.
+ * It is idempotent operation, so it's MT-safe.
+ */
+#define SCM_BIND_PROC(var, name, module)                                \
+    do {                                                                \
+        if (SCM_UNDEFINEDP(var)) {                                      \
+            ScmObj v__ =                                                \
+                Scm_GlobalVariableRef(module,                           \
+                                      SCM_SYMBOL(SCM_INTERN(name)),     \
+                                      0);                               \
+            if (SCM_UNBOUNDP(v__)) {                                    \
+                Scm_Error("Procedure %s is unbound", name);             \
+            }                                                           \
+            var = v__;                                                  \
+        }                                                               \
+    } while (0)
+
+
+/* OBSOLETED - These are defined in Scheme now. */
 SCM_EXTERN ScmObj Scm_ForEach1(ScmObj proc, ScmObj args);
 SCM_EXTERN ScmObj Scm_ForEach(ScmObj proc, ScmObj arg1, ScmObj args);
 SCM_EXTERN ScmObj Scm_Map1(ScmObj proc, ScmObj args);
@@ -1542,8 +1631,22 @@ SCM_CLASS_DECL(Scm_PromiseClass);
 SCM_EXTERN ScmObj Scm_MakePromise(int forced, ScmObj code);
 SCM_EXTERN ScmObj Scm_Force(ScmObj p);
 
+/* Lazy pair structur is opaque to public.  Whenever you apply to an
+   ScmObj SCM_PAIRP, a lazy pair morphs itself to a pair, so the normal
+   code never see lazy pairs. */
+
+SCM_CLASS_DECL(Scm_LazyPairClass);
+#define SCM_CLASS_LAZY_PAIR        (&Scm_LazyPairClass)
+#define SCM_LAZY_PAIR(obj)         ((ScmLazyPair*)(obj))
+#define SCM_LAZY_PAIR_P(obj)       SCM_XTYPEP(obj, SCM_CLASS_LAZY_PAIR)
+
+SCM_EXTERN ScmObj Scm_MakeLazyPair(ScmObj item, ScmObj generator);
+SCM_EXTERN int    Scm_DecomposeLazyPair(ScmObj obj, ScmObj *item, ScmObj *generator);
+SCM_EXTERN ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *lp);
+SCM_EXTERN int Scm_PairP(ScmObj x);
+
 /*--------------------------------------------------------
- * CONDITION
+ * condition
  */
 
 /* Condition classes are defined in a separate file */
