@@ -1,12 +1,12 @@
 /*
  * char-none.h
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -36,7 +36,7 @@
  * Header part
  */
 
-/* The name of the encoding.  Scheme procedure 
+/* The name of the encoding.  Scheme procedure
  * gauche-character-encoding returns a symbol with this name.
  */
 #define SCM_CHAR_ENCODING_NAME "none"
@@ -119,4 +119,14 @@ static const char *supportedCharacterEncodings[] = {
     NULL
 };
 
+/*
+ * Lookup character category.  The tables are in char_attr.c, automatically
+ * generated by gen-unicode.scm.
+ */
+static inline unsigned char Scm__LookupCharCategory(ScmChar ch)
+{
+    if (ch == SCM_CHAR_INVALID || ch > 0xff) return SCM_CHAR_CATEGORY_Cn;
+    return ucs_general_category_00000[ch];
+}
+
 #endif /* !SCM_CHAR_ENCODING_BODY */
