@@ -1,12 +1,12 @@
 /*
  * char-sjis.h
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -36,7 +36,7 @@
  * Header part
  */
 
-/* The name of the encoding.  Scheme procedure 
+/* The name of the encoding.  Scheme procedure
  * gauche-character-encoding returns a symbol with this name.
  */
 #define SCM_CHAR_ENCODING_NAME "sjis"
@@ -107,7 +107,10 @@
     } while (0)
 
 /* C is an ScmChar > 0x80.  Returns true if C is a whitespace character. */
-#define SCM_CHAR_EXTRA_WHITESPACE(c)  ((c) == 0x8140) /* zenkaku space */
+#define SCM_CHAR_EXTRA_WHITESPACE(c) \
+    (((c) == 0x8140)                       /* zenkaku space */ \
+     || ((c) == 0x8541))                   /* NBSP */
+
 /* Like SCM_CHAR_EXTRA_WHITESPACE, but excludes Zl and Zp.
    See R6RS on the intraline whitespaces. */
 #define SCM_CHAR_EXTRA_WHITESPACE_INTRALINE(c) SCM_CHAR_EXTRA_WHITESPACE(c)
@@ -128,4 +131,51 @@ static const char *supportedCharacterEncodings[] = {
     NULL
 };
 
+/*
+ * Lookup character category.  The tables are in char_attr.c, automatically
+ * generated by gen-unicode.scm.
+ */
+static inline unsigned char Scm__LookupCharCategory(ScmChar ch)
+{
+    if (ch == SCM_CHAR_INVALID) return SCM_CHAR_CATEGORY_Cn;
+    if (ch < 0x80) return sjis_general_category_00[ch];
+    else if (ch < 0xa0)   return SCM_CHAR_CATEGORY_Cn;
+    else if (ch < 0xe0)   return sjis_general_category_a0[ch-0xa0];
+    else if (ch < 0x8040) return SCM_CHAR_CATEGORY_Cn;
+    else if (ch < 0x9ffd) {
+        unsigned char b0 = (ch >> 8) - 0x80;
+        unsigned char b1 = (ch & 0xff) - 0x40;
+        if (b0 >= (0xa0 - 0x80)) return SCM_CHAR_CATEGORY_Cn;
+        if (b1 >= (0xfd - 0x40)) return SCM_CHAR_CATEGORY_Cn;
+        SCM_ASSERT(0 <= b0 && b0 < (0xa0 - 0x80));
+        SCM_ASSERT(0 <= b1 && b1 < (0xfd - 0x40));
+        return sjis_general_category_8000[b0 * (0xfd-0x40) + b1];
+    } else if (ch < 0xe000) {
+        return SCM_CHAR_CATEGORY_Cn;
+    } else if (ch < 0xfffd) {
+        unsigned char b0 = (ch >> 8) - 0xe0;
+        unsigned char b1 = (ch & 0xff) - 0x40;
+        if (b0 >= (0x100 - 0xe0)) return SCM_CHAR_CATEGORY_Cn;
+        if (b1 >= (0xfd - 0x40))  return SCM_CHAR_CATEGORY_Cn;
+        return sjis_general_category_e000[b0 * (0xfd-0x40) + b1];
+    } else return SCM_CHAR_CATEGORY_Cn;
+}
+
+/*
+ * Returns true if the character isn't supported in Unicode.
+ */
+static int Scm__CharInUnicodeP(ScmChar ch)
+{
+    if (ch < 0x82f5 || ch > 0x8686) return TRUE;
+    if (ch < 0x8600) {
+        if (ch <= 0x82f9 || (ch >= 0x8397 && ch <= 0x839e)
+            || ch == 0x83f6) return FALSE;
+        else return TRUE;
+    } else {
+        if (ch == 0x8663 || (ch >= 0x8667 && ch <= 0x866e)
+            || ch == 0x8685 || ch == 0x8686) return FALSE;
+        else return TRUE;
+    }
+}
+
 #endif /* !SCM_CHAR_ENCODING_BODY */
