@@ -1,12 +1,12 @@
 /*
  * class.h - Gauche object system private header
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -112,6 +112,9 @@ SCM_EXTERN ScmObj Scm__AllocateAndInitializeInstance(ScmClass *klass,
                                                      int numInits,
                                                      u_long flags);
 SCM_EXTERN ScmObj Scm_ComputeCPL(ScmClass *klass);
+SCM_EXTERN int    Scm_MethodApplicableForClasses(ScmMethod *m,
+                                                 ScmClass *types[],
+                                                 int nargs);
 SCM_EXTERN ScmObj Scm_ComputeApplicableMethods(ScmGeneric *gf,
                                                ScmObj *argv,
                                                int argc,
