@@ -1,12 +1,12 @@
 /*
  * string.h - Public API for Scheme strings
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -225,13 +225,15 @@ enum {
     SCM_STRING_JOIN_PREFIX
 };
 
-/* 
+/*
  * Searching
  */
 
 SCM_EXTERN ScmObj  Scm_StringSplitByChar(ScmString *str, ScmChar ch);
 SCM_EXTERN ScmObj  Scm_StringScan(ScmString *s1, ScmString *s2, int retmode);
 SCM_EXTERN ScmObj  Scm_StringScanChar(ScmString *s1, ScmChar ch, int retmode);
+SCM_EXTERN ScmObj  Scm_StringScanRight(ScmString *s1, ScmString *s2, int retmode);
+SCM_EXTERN ScmObj  Scm_StringScanCharRight(ScmString *s1, ScmChar ch, int retmode);
 
 /* "retmode" argument for string scan */
 enum {
@@ -249,7 +251,8 @@ enum {
 SCM_EXTERN int     Scm_MBLen(const char *str, const char *stop);
 
 /* INTERNAL */
-SCM_EXTERN const char *Scm_StringPosition(ScmString *str, int k);
+SCM_EXTERN const char *Scm_StringPosition(ScmString *str, int k); /*DEPRECATED*/
+SCM_EXTERN const char *Scm_StringBodyPosition(const ScmStringBody *str, int k);
 SCM_EXTERN ScmObj  Scm_MaybeSubstring(ScmString *x, ScmObj start, ScmObj end);
 
 /*
@@ -358,10 +361,7 @@ SCM_EXTERN ScmObj Scm_StringPointerPrev(ScmStringPointer *sp);
 SCM_EXTERN ScmObj Scm_StringPointerSet(ScmStringPointer *sp, int index);
 SCM_EXTERN ScmObj Scm_StringPointerSubstring(ScmStringPointer *sp, int beforep);
 SCM_EXTERN ScmObj Scm_StringPointerCopy(ScmStringPointer *sp);
-
-#ifdef SCM_DEBUG_HELPER
 SCM_EXTERN void   Scm_StringPointerDump(ScmStringPointer *sp);
-#endif
 
 #endif /* GAUCHE_STRING_H */
 
