@@ -1,12 +1,12 @@
 /*
  * uthread.h - user level thread primitives
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -43,15 +43,38 @@
 
 /* Abstract thread API */
 
+#undef GAUCHE_HAS_THREADS  /*define this if the system supports real threads*/
+
 /* ScmInternalThread - a handle to the system's thread.
-   
+
    The variable of type ScmInternalThread must be initialized by
    SCM_INTERNAL_THREAD_INIT.  You can check if the variable is
    initialized or not by SCM_INTERNAL_THREAD_INITIALIZED_P.
+
+   SCM_INTERNAL_THREAD_GETCURRENT() should return the current thread.
+   SCM_INTERNAL_THREAD_SETSPECIFIC(key, val) sets the value to the
+   thread-local storage identified by key.  It should evaluate
+   to TRUE on success, FALSE on error.
+
+   SCM_INTERNAL_THREAD_CLEANUP_{PUSH|POP} is an abstraction similar
+   to pthread_cleanup_{push|pop}.  We assume no Scheme error is
+   signalled between these macros.
+
+   SCM_INTERNAL_THREAD_PROC_RET{TYPE|VAL} specify return type and value
+   of the thread entry procedure.  See ext/threads/threads.c.
  */
 typedef int ScmInternalThread;
 #define SCM_INTERNAL_THREAD_INIT(thr)      (0)
 #define SCM_INTERNAL_THREAD_INITIALIZED_P(thr)  TRUE
+#define SCM_INTERNAL_THRAED_GETCURRENT()   (0)
+#define SCM_INTERNAL_THREAD_SETSPECIFIC(key, val) TRUE
+#define SCM_INTERNAL_THREAD_EXIT()         _exit(0)
+
+#define SCM_INTERNAL_THREAD_CLEANUP_PUSH(fn, data) {
+#define SCM_INTERNAL_THREAD_CLEANUP_POP()          }
+
+#define SCM_INTERNAL_THREAD_PROC_RETTYPE   void*
+#define SCM_INTERNAL_THREAD_PROC_RETVAL    NULL
 
 /* ScmInternalMutex - a mutex structure.
 
@@ -99,19 +122,24 @@ typedef int ScmInternalFastlock;
 #define SCM_INTERNAL_FASTLOCK_UNLOCK(fl)   (0)
 #define SCM_INTENRAL_FASTLOCK_DESTROY(fl)  (0)
 
-/* ScmInternalCond - condition varaiable      
+/* ScmInternalCond - condition varaiable
  *
  *  Pthreads-style condition variables.  SCM_INTERNAL_COND_INITIALIZER
  *  may be used for the placeholder of static variables (but the
  *  variable still needs to be initialized by SCM_INTERNAL_COND_INIT.
+ *
+ *  SCM_INTERNAL_COND_TIMEDWAIT should return either 0 (signalled),
+ *  SCM_INTERNAL_COND_TIMEDOUT, or SCM_INTERNAL_COND_INTR.
  */
 typedef int ScmInternalCond;
 #define SCM_INTERNAL_COND_INIT(cond)       (0)
 #define SCM_INTERNAL_COND_SIGNAL(cond)     (0)
 #define SCM_INTERNAL_COND_BROADCAST(cond)  (0)
 #define SCM_INTERNAL_COND_WAIT(cond, mutex) (0)
+#define SCM_INTERNAL_COND_TIMEDWAIT(cond, mutex, timespec) (0)
 #define SCM_INTERNAL_COND_DESTROY(cond)    (0)
 #define SCM_INTERNAL_COND_INITIALIZER      (0)
-
+#define SCM_INTERNAL_COND_TIMEDOUT         (0)
+#define SCM_INTERNAL_COND_INTR             (0)
 
 #endif /* GAUCHE_UTHREAD_H */
