@@ -1,8 +1,6 @@
 /*
  * getdir_win.c - get the library directory at runtime (fow windows)
  *  included from paths.c
- *
- * $Id: getdir_win.c,v 1.6 2007/10/05 20:35:13 shirok Exp $
  */
 
 #include <string.h>
@@ -15,7 +13,7 @@ static int get_install_dir(char *buf, int buflen,
     int len;
     const char *mbpath;
     TCHAR path[MAX_PATH];
-    const TCHAR *libname = _T("libgauche.dll");
+    const TCHAR *libname = _T("libgauche-"GAUCHE_ABI_VERSION".dll");
 
     if ((mod = GetModuleHandle(libname)) == NULL) {
         errfn("GetModuleHandle failed");
