@@ -1,12 +1,12 @@
 /*
  * list.c - List related functions
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -85,7 +85,7 @@ ScmObj Scm_List(ScmObj elt, ...)
     ScmObj cdr;
 
     if (elt == NULL) return SCM_NIL;
-        
+
     va_start(pvar, elt);
     cdr = Scm_VaList(pvar);
     va_end(pvar);
@@ -99,7 +99,7 @@ ScmObj Scm_Conses(ScmObj elt, ...)
     ScmObj cdr;
 
     if (elt == NULL) return SCM_NIL;
-    
+
     va_start(pvar, elt);
     cdr = Scm_VaCons(pvar);
     va_end(pvar);
@@ -111,7 +111,7 @@ ScmObj Scm_Conses(ScmObj elt, ...)
 ScmObj Scm_VaList(va_list pvar)
 {
     ScmObj start = SCM_NIL, cp = SCM_NIL, obj;
-    
+
     for (obj = va_arg(pvar, ScmObj);
 	 obj != NULL;
 	 obj = va_arg(pvar, ScmObj))
@@ -221,7 +221,7 @@ int Scm_Length(ScmObj obj)
     for (;;) {
         if (SCM_NULLP(obj)) break;
         if (!SCM_PAIRP(obj)) return SCM_LIST_DOTTED;
-	
+
 	obj = SCM_CDR(obj);
 	len++;
         if (SCM_NULLP(obj)) break;
@@ -245,7 +245,7 @@ ScmObj Scm_CopyList(ScmObj list)
     ScmObj start = SCM_NIL, last = SCM_NIL;
 
     if (!SCM_PAIRP(list)) return list;
-    
+
     SCM_FOR_EACH(list, list) {
         SCM_APPEND1(start, last, SCM_CAR(list));
     }
@@ -269,8 +269,8 @@ ScmObj Scm_MakeList(int len, ScmObj fill)
     }
     return start;
 }
-     
-    
+
+
 /* Scm_Append2X(list, obj)
  *    Replace cdr of last pair of LIST for OBJ.
  *    If LIST is not a pair, return OBJ.
@@ -326,21 +326,24 @@ ScmObj Scm_Append(ScmObj args)
     return start;
 }
 
-/* Scm_Reverse(list)
- *    Reverse LIST.  If LIST is not a pair, return LIST itself.
- *    If LIST is improper list, cdr of the last pair is ignored.
+/* Scm_Reverse2(list, tail)
+ *    Reverse LIST, and append TAIL to the result.
+ *    If LIST is an improper list, cdr of the last pair is ignored.
+ *    If LIST is not a pair, TAIL is returned.
+ * Scm_Reverse(list)
+ *    Scm_Reverse2(list, SCM_NIL).  Just for the backward compatibility.
  */
 
-ScmObj Scm_Reverse(ScmObj list)
+ScmObj Scm_Reverse2(ScmObj list, ScmObj tail)
 {
     ScmObj cp, result;
     ScmPair *p;
 
-    if (!SCM_PAIRP(list)) return list;
+    if (!SCM_PAIRP(list)) return tail;
 
     p = SCM_NEW(ScmPair);
     SCM_SET_CAR(p, SCM_NIL);
-    SCM_SET_CDR(p, SCM_NIL);
+    SCM_SET_CDR(p, tail);
     result = SCM_OBJ(p);
     SCM_FOR_EACH(cp, list) {
 	SCM_SET_CAR(result, SCM_CAR(cp));
@@ -352,17 +355,23 @@ ScmObj Scm_Reverse(ScmObj list)
     return SCM_CDR(result);
 }
 
-    
-/* Scm_ReverseX(list)
+ScmObj Scm_Reverse(ScmObj list)
+{
+    return Scm_Reverse2(list, SCM_NIL);
+}
+
+
+/* Scm_Reverse2X(list, tail)
  *   Return reversed list of LIST.  Pairs in previous LIST is used to
- *   create new list.  If LIST is not a pair, return LIST itself.
+ *   create new list.  TAIL is appended to the result.
+ *   If LIST is not a pair, returns TAIL.
  *   If LIST is an improper list, cdr of the last cell is ignored.
  */
 
-ScmObj Scm_ReverseX(ScmObj list)
+ScmObj Scm_Reverse2X(ScmObj list, ScmObj tail)
 {
-    ScmObj first, next, result = SCM_NIL;
-    if (!SCM_PAIRP(list)) return list;
+    ScmObj first, next, result = tail;
+    if (!SCM_PAIRP(list)) return result;
     for (first = list; SCM_PAIRP(first); first = next) {
         next = SCM_CDR(first);
         SCM_SET_CDR(first, result);
@@ -371,6 +380,11 @@ ScmObj Scm_ReverseX(ScmObj list)
     return result;
 }
 
+ScmObj Scm_ReverseX(ScmObj list)
+{
+    return Scm_Reverse2X(list, SCM_NIL);
+}
+
 /* Scm_ListTail(list, i, fallback)
  * Scm_ListRef(list, i, fallback)
  *    Note that i is C-INTEGER.  If i is out of bound, signal error.
@@ -385,7 +399,7 @@ ScmObj Scm_ListTail(ScmObj list, int i, ScmObj fallback)
         list = SCM_CDR(list);
     }
     return list;
-  err:  
+  err:
     if (SCM_UNBOUNDP(fallback)) Scm_Error("argument out of range: %d", i);
     return fallback;
 }
@@ -540,7 +554,7 @@ ScmObj Scm_AssocDelete(ScmObj elt, ScmObj alist, int cmpmode)
         Scm_Error("assoc-delete: list required, but got %S", alist);
     }
     if (SCM_NULLP(alist)) return SCM_NIL;
-    
+
     SCM_FOR_EACH(cp, alist) {
         p = SCM_CAR(cp);
         if (SCM_PAIRP(p)) {
@@ -613,7 +627,7 @@ ScmObj Scm_DeleteDuplicatesX(ScmObj list, int cmpmode)
  *  list.   If there's more than one way to order an element, choose the
  *  first one appears in the given list of lists.
  *  Returns SCM_FALSE if the lists are inconsistent to be ordered
- *  in the way. 
+ *  in the way.
  *
  *  START is an item of the starting point.  It is inserted into the result
  *  first.  SEQUENCES is a list of lists describing the order of preference.
@@ -623,11 +637,26 @@ ScmObj Scm_DeleteDuplicatesX(ScmObj list, int cmpmode)
  *    http://www.webcom.com/~haahr/dylan/linearization-oopsla96.html.
  *  Since the algorithm is generally useful, I implement the core routine
  *  of the algorithm here.
+ *
+ *  TODO at 1.0: I noticed START argument acutally isn't used in the
+ *  algorithm at all.  We can drop it and the caller can just say
+ *  Scm_Cons(start, Scm_MonotonicMerge(sequences)).   We can't change it
+ *  now because of ABI compatibility, but it will be nice to do so when
+ *  releasing 1.0.
  */
 
+/* DEPRECATED 1.0 */
 ScmObj Scm_MonotonicMerge(ScmObj start, ScmObj sequences)
 {
-    ScmObj result = Scm_Cons(start, SCM_NIL), next, h;
+    ScmObj r = Scm_MonotonicMerge1(sequences);
+    if (!SCM_FALSEP(r)) r = Scm_Cons(start, r);
+    return r;
+}
+
+/* WILL RENAME IN 1.0 */
+ScmObj Scm_MonotonicMerge1(ScmObj sequences)
+{
+    ScmObj result = SCM_NIL, next, h;
     ScmObj *seqv, *sp, *tp;
     int nseqs = Scm_Length(sequences);
 
@@ -701,7 +730,7 @@ ScmObj Scm_PairAttrGet(ScmPair *pair, ScmObj key, ScmObj fallback)
     if (!SCM_EXTENDED_PAIR_P(pair)) {
         goto fallback;
     }
-    
+
     p = Scm_Assq(key, SCM_EXTENDED_PAIR(pair)->attributes);
     if (SCM_PAIRP(p)) return SCM_CDR(p);
   fallback:
@@ -718,7 +747,7 @@ ScmObj Scm_PairAttrSet(ScmPair *pair, ScmObj key, ScmObj value)
         Scm_Error("Cannot set pair attribute (%S) to non-extended pair: %S",
                   key, SCM_OBJ(pair));
     }
-    
+
     p = Scm_Assq(key, SCM_EXTENDED_PAIR(pair)->attributes);
     if (SCM_PAIRP(p)) SCM_SET_CDR(p, value);
     else SCM_EXTENDED_PAIR(pair)->attributes
