@@ -1,12 +1,12 @@
 /*
  * load.c - load a program
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -50,9 +50,10 @@ typedef struct dlobj_initfn_rec dlobj_initfn;
 /* Static parameters */
 static struct {
     /* Load path list */
-    ScmGloc *load_path_rec;     /* *load-path*         */
-    ScmGloc *dynload_path_rec;  /* *dynamic-load-path* */
-    ScmGloc *load_suffixes_rec; /* *load-suffixes*     */
+    ScmGloc *load_path_rec;      /* *load-path*         */
+    ScmGloc *dynload_path_rec;   /* *dynamic-load-path* */
+    ScmGloc *load_suffixes_rec;  /* *load-suffixes*     */
+    ScmGloc *load_path_hooks_rec; /* *load-path-hooks*   */
     ScmInternalMutex path_mutex;
 
     /* Provided features */
@@ -65,6 +66,17 @@ static struct {
     ScmInternalMutex prov_mutex;
     ScmInternalCond  prov_cv;
 
+    /* Dynamic enviornments kept during specific `load'.  They are
+       thread-specific, and we use ScmParameter mechanism. */
+    ScmParameterLoc load_history;      /* history of the nested load */
+    ScmParameterLoc load_next;         /* list of the directories to be
+                                          searched. */
+    ScmParameterLoc load_port;         /* current port from which we are
+                                          loading */
+    ScmParameterLoc load_main_script;  /* yields #t during loading main script.
+                                          see SCM_LOAD_MAIN_SCRIPT flag
+                                          in load.h. */
+
     /* Dynamic linking */
     ScmObj dso_suffixes;
     dlobj *dso_list;              /* List of dynamically loaded objects. */
@@ -75,6 +87,12 @@ static struct {
 static ScmObj key_error_if_not_found = SCM_UNBOUND;
 static ScmObj key_macro              = SCM_UNBOUND;
 static ScmObj key_ignore_coding      = SCM_UNBOUND;
+static ScmObj key_main_script        = SCM_UNBOUND;
+static ScmObj key_paths              = SCM_UNBOUND;
+static ScmObj key_environment        = SCM_UNBOUND;
+
+#define PARAM_REF(vm, loc)      Scm_ParameterRef(vm, &ldinfo.loc)
+#define PARAM_SET(vm, loc, val) Scm_ParameterSet(vm, &ldinfo.loc, val)
 
 /*
  * ScmLoadPacket is the way to communicate to Scm_Load facility.
@@ -104,7 +122,7 @@ void Scm_LoadPacketInit(ScmLoadPacket *p)
 /*--------------------------------------------------------------------
  * Scm_LoadFromPort
  * Scm_VMLoadFromPort
- * 
+ *
  *   The most basic function in the load()-family.  Read an expression
  *   from the given port and evaluates it repeatedly, until it reaches
  *   EOF.  Then the port is closed.   The port is locked by the calling
@@ -139,6 +157,7 @@ struct load_info {
     ScmObj prev_port;
     ScmObj prev_history;
     ScmObj prev_next;
+    ScmObj prev_main_script;
     int    prev_situation;
 };
 
@@ -161,9 +180,10 @@ static ScmObj load_after(ScmObj *args, int nargs, void *data)
     Scm_ClosePort(p->port);
     PORT_UNLOCK(p->port);
     Scm_SelectModule(p->prev_module);
-    vm->load_port = p->prev_port;
-    vm->load_history = p->prev_history;
-    vm->load_next = p->prev_next;
+    PARAM_SET(vm, load_port, p->prev_port);
+    PARAM_SET(vm, load_history, p->prev_history);
+    PARAM_SET(vm, load_next, p->prev_next);
+    PARAM_SET(vm, load_main_script, p->prev_main_script);
     vm->evalSituation = p->prev_situation;
     return SCM_UNDEFINED;
 }
@@ -210,17 +230,19 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
 
     p = SCM_NEW(struct load_info);
     p->port = port;
-    p->prev_module = vm->module;
-    p->prev_port = vm->load_port;
-    p->prev_history = vm->load_history;
-    p->prev_next = vm->load_next;
+    p->prev_module    = vm->module;
+    p->prev_port      = PARAM_REF(vm, load_port);
+    p->prev_history   = PARAM_REF(vm, load_history);
+    p->prev_next      = PARAM_REF(vm, load_next);
+    p->prev_main_script = PARAM_REF(vm, load_main_script);
     p->prev_situation = vm->evalSituation;
 
     p->ctx = Scm_MakeReadContext(NULL);
     p->ctx->flags = SCM_READ_LITERAL_IMMUTABLE | SCM_READ_SOURCE_INFO;
-    
-    vm->load_next = next_paths;
-    vm->load_port = SCM_OBJ(port);
+
+    PARAM_SET(vm, load_next, next_paths);
+    PARAM_SET(vm, load_port, SCM_OBJ(port));
+    PARAM_SET(vm, load_main_script, SCM_MAKE_BOOL(flags&SCM_LOAD_MAIN_SCRIPT));
     vm->module = module;
     vm->evalSituation = SCM_VM_LOADING;
     if (SCM_PORTP(p->prev_port)) {
@@ -229,7 +251,7 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
     } else {
         port_info = SCM_LIST1(SCM_FALSE);
     }
-    vm->load_history = Scm_Cons(port_info, vm->load_history);
+    PARAM_SET(vm,load_history, Scm_Cons(port_info, PARAM_REF(vm,load_history)));
 
     PORT_LOCK(port, vm);
     return Scm_VMDynamicWindC(NULL, load_body, load_after, p);
@@ -239,24 +261,23 @@ int Scm_LoadFromPort(ScmPort *port, u_long flags, ScmLoadPacket *packet)
 {
     static ScmObj load_from_port = SCM_UNDEFINED;
     ScmEvalPacket eresult;
+    ScmObj args = SCM_NIL;
     int r;
+    SCM_BIND_PROC(load_from_port, "load-from-port", Scm_GaucheModule());
+    load_packet_prepare(packet);
 
-    if (SCM_UNDEFINEDP(load_from_port)) {
-        /* This should be an idempotent operation, so we don't need to
-           worry about MT-safety. */
-        load_from_port =
-            Scm_GlobalVariableRef(Scm_GaucheModule(),
-                                  SCM_SYMBOL(SCM_INTERN("load-from-port")),
-                                  0);
+    if (flags&SCM_LOAD_MAIN_SCRIPT) {
+        args = Scm_Cons(key_main_script, Scm_Cons(SCM_TRUE, args));
     }
 
-    load_packet_prepare(packet);
+    args = Scm_Cons(SCM_OBJ(port), args);
+
     if (flags&SCM_LOAD_PROPAGATE_ERROR) {
-        Scm_ApplyRec(load_from_port, SCM_LIST1(SCM_OBJ(port)));
+        Scm_ApplyRec(load_from_port, args);
         if (packet) packet->loaded = TRUE;
         return 0;
     } else {
-        r = Scm_Apply(load_from_port, SCM_LIST1(SCM_OBJ(port)), &eresult);
+        r = Scm_Apply(load_from_port, args, &eresult);
         if (packet) {
             packet->exception = eresult.exception;
             packet->loaded = (r >= 0);
@@ -265,190 +286,95 @@ int Scm_LoadFromPort(ScmPort *port, u_long flags, ScmLoadPacket *packet)
     }
 }
 
-/*---------------------------------------------------------------------
- * Scm_FindFile
- *
- *   Core function to search specified file from the search path *PATH.
- *   Search rules are:
- *   
- *    (1) If given filename begins with "/", "./" or "../", the file is
- *        searched.
- *    (2) If given filename begins with "~", unix-style username
- *        expansion is done, then the resulting file is searched.
- *    (3) Otherwise, the file is searched for each directory in
- *        *load-path*.
- *
- *   If a file is found, it's pathname is returned.  *PATH is modified
- *   to contain the remains of *load-path*, which can be used again to
- *   find next matching filename.
- *   If SUFFIXES is given, filename is assumed not to have suffix,
- *   and suffixes listed in SUFFIXES are tried one by one.
- *   The element in SUFFIXES is directly appended to the FILENAME;
- *   so usually it begins with dot.
- */
-
-static int regfilep(ScmObj path)
-{
-    struct stat statbuf;
-    int r = stat(Scm_GetStringConst(SCM_STRING(path)), &statbuf);
-    if (r < 0) return FALSE;
-    return S_ISREG(statbuf.st_mode);
-}
-
-static ScmObj try_suffixes(ScmObj base, ScmObj suffixes)
-{
-    ScmObj sp, fpath;
-    if (regfilep(base)) return base;
-    SCM_FOR_EACH(sp, suffixes) {
-        fpath = Scm_StringAppend2(SCM_STRING(base), SCM_STRING(SCM_CAR(sp)));
-        if (regfilep(fpath)) return fpath;
-    }
-    return SCM_FALSE;
-}
-
-ScmObj Scm_FindFile(ScmString *filename, ScmObj *paths,
-                    ScmObj suffixes, int flags)
-{
-    u_int size;
-    const char *ptr = Scm_GetStringContent(filename, &size, NULL, NULL);
-    int use_load_paths = TRUE;
-    ScmObj file = SCM_OBJ(filename), fpath = SCM_FALSE;
-
-    if (size == 0) Scm_Error("bad filename to load: \"\"");
-    if (*ptr == '~') {
-        file = Scm_NormalizePathname(filename, SCM_PATH_EXPAND);
-        use_load_paths = FALSE;
-    } else if (*ptr == '/'
-               || (*ptr == '.' && *(ptr+1) == '/')
-               || (*ptr == '.' && *(ptr+1) == '.' && *(ptr+2) == '/')
-#if defined(__CYGWIN__) || defined(GAUCHE_WINDOWS)
-	       /* support for wicked legacy DOS drive letter */
-	       || (isalpha(*ptr) && *(ptr+1) == ':')
-#endif /* __CYGWIN__ || GAUCHE_WINDOWS */
-	       ) {
-        use_load_paths = FALSE;
-    }
-
-    if (use_load_paths) {
-        ScmObj lpath;
-        SCM_FOR_EACH(lpath, *paths) {
-            if (!SCM_STRINGP(SCM_CAR(lpath))) {
-                Scm_Warn("*load-path* contains invalid element: %S", *paths);
-            }
-            fpath = Scm_StringAppendC(SCM_STRING(SCM_CAR(lpath)), "/", 1, 1);
-            fpath = Scm_StringAppend2(SCM_STRING(fpath), SCM_STRING(file));
-            fpath = try_suffixes(fpath, suffixes);
-            if (!SCM_FALSEP(fpath)) break;
-        }
-        if (SCM_PAIRP(lpath)) {
-            *paths = SCM_CDR(lpath);
-            return SCM_OBJ(fpath);
-        } else if (!(flags&SCM_LOAD_QUIET_NOFILE)) {
-            Scm_Error("cannot find file %S in *load-path* %S", file, *paths);
-        } else {
-            *paths = SCM_NIL;
-        }
-    } else {
-        *paths = SCM_NIL;
-        fpath = try_suffixes(file, suffixes);
-        if (!SCM_FALSEP(fpath)) return fpath;
-        if (!(flags&SCM_LOAD_QUIET_NOFILE)) {
-            Scm_Error("cannot find file %S to load", file);
-        }
-    }
-    return SCM_FALSE;
-}
-
 /*---------------------------------------------------------------------
  * Scm_Load
  * Scm_VMLoad
  *
  *  Scheme's load().
- * 
+ *
  *  filename   - name of the file.  can be sans suffix.
  *  load_paths - list of pathnames or #f.   If #f, system's load path
  *               is used.
  *  env        - a module where the forms are evaluated, or #f.
  *               If #f, the current module is used.
- *  flags      - combination of bit flags
- *               SCM_LOAD_QUIET_NOFILE, SCM_LOAD_IGNORE_CODING
+ *  flags      - combination of ScmLoadFlags.
  */
 
-ScmObj Scm_VMLoad(ScmString *filename, ScmObj load_paths,
-                  ScmObj env, int flags)
+/* internal auxiliary function called from Scheme `load' */
+void Scm__RecordLoadStart(ScmObj load_file_path)
 {
-    ScmObj port, truename, suffixes;
     ScmVM *vm = Scm_VM();
-    int errorp = !(flags&SCM_LOAD_QUIET_NOFILE);
-    int ignore_coding = flags&SCM_LOAD_IGNORE_CODING;
-
-    suffixes = SCM_GLOC_GET(ldinfo.load_suffixes_rec);
-    if (!SCM_PAIRP(load_paths)) load_paths = Scm_GetLoadPath();
-    truename = Scm_FindFile(filename, &load_paths, suffixes, flags);
-    if (SCM_FALSEP(truename)) return SCM_FALSE;
-
 #ifdef HAVE_GETTIMEOFDAY
     if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_COLLECT_LOAD_STATS)) {
         struct timeval t0;
         gettimeofday(&t0, NULL);
         vm->stat.loadStat =
-            Scm_Acons(truename,
+            Scm_Acons(load_file_path,
                       Scm_MakeIntegerU(t0.tv_sec*1000000+t0.tv_usec),
                       vm->stat.loadStat);
     }
 #endif /*HAVE_GETTIMEOFDAY*/
     if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_LOAD_VERBOSE)) {
-        int len = Scm_Length(vm->load_history);
+        int len = Scm_Length(PARAM_REF(vm, load_history));
         SCM_PUTZ(";;", 2, SCM_CURERR);
         while (len-- > 0) SCM_PUTC(' ', SCM_CURERR);
-        Scm_Printf(SCM_CURERR, "Loading %A...\n", truename);
+        Scm_Printf(SCM_CURERR, "Loading %A...\n", load_file_path);
     }
+}
 
-    port = Scm_OpenFilePort(Scm_GetStringConst(SCM_STRING(truename)),
-                            O_RDONLY, SCM_PORT_BUFFER_FULL, 0);
-    if (SCM_FALSEP(port)) {
-        if (errorp) Scm_Error("file %S exists, but couldn't open.", truename);
-        else        return SCM_FALSE;
+/* The real `load' function is moved to Scheme.  This is a C stub to
+   call it. */
+ScmObj Scm_VMLoad(ScmString *filename, ScmObj paths, ScmObj env, int flags)
+{
+    ScmObj opts = SCM_NIL;
+    static ScmObj load_proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(load_proc, "load", Scm_SchemeModule());
+
+    if (flags&SCM_LOAD_QUIET_NOFILE) {
+        opts = Scm_Cons(key_error_if_not_found, Scm_Cons(SCM_FALSE, opts));
     }
-    if (!ignore_coding) {
-        port = Scm_MakeCodingAwarePort(SCM_PORT(port));
+    if (flags&SCM_LOAD_IGNORE_CODING) {
+        opts = Scm_Cons(key_ignore_coding, Scm_Cons(SCM_TRUE, opts));
     }
-    return Scm_VMLoadFromPort(SCM_PORT(port), load_paths, env, flags);
+    if (flags&SCM_LOAD_MAIN_SCRIPT) {
+        opts = Scm_Cons(key_main_script, Scm_Cons(SCM_TRUE, opts));
+    }
+    if (SCM_NULLP(paths) || SCM_PAIRP(paths)) {
+        opts = Scm_Cons(key_paths, Scm_Cons(paths, opts));
+    }
+    if (!SCM_FALSEP(env)) {
+        opts = Scm_Cons(key_environment, Scm_Cons(env, opts));
+    }
+    return Scm_VMApply(load_proc, Scm_Cons(SCM_OBJ(filename), opts));
 }
 
 int Scm_Load(const char *cpath, u_long flags, ScmLoadPacket *packet)
 {
-    static ScmObj load_stub = SCM_UNDEFINED;
+    static ScmObj load_proc = SCM_UNDEFINED;
     ScmObj f = SCM_MAKE_STR_COPYING(cpath);
-    ScmObj options = SCM_NIL;
+    ScmObj opts = SCM_NIL;
     ScmEvalPacket eresult;
+    SCM_BIND_PROC(load_proc, "load", Scm_SchemeModule());
 
-    if (load_stub == SCM_UNDEFINED) {
-        /* This should be an idempotent operation, so we don't need to
-           worry about MT-safety. */
-        load_stub = Scm_GlobalVariableRef(Scm_SchemeModule(),
-                                          SCM_SYMBOL(SCM_INTERN("load")),
-                                          0);
-    }
-    
     if (flags&SCM_LOAD_QUIET_NOFILE) {
-        options = Scm_Cons(key_error_if_not_found,
-                           Scm_Cons(SCM_FALSE, options));
+        opts = Scm_Cons(key_error_if_not_found, Scm_Cons(SCM_FALSE, opts));
     }
     if (flags&SCM_LOAD_IGNORE_CODING) {
-        options = Scm_Cons(key_ignore_coding,
-                           Scm_Cons(SCM_TRUE, options));
+        opts = Scm_Cons(key_ignore_coding, Scm_Cons(SCM_TRUE, opts));
+    }
+    if (flags&SCM_LOAD_MAIN_SCRIPT) {
+        opts = Scm_Cons(key_main_script, Scm_Cons(SCM_TRUE, opts));
     }
 
     load_packet_prepare(packet);
     if (flags&SCM_LOAD_PROPAGATE_ERROR) {
-        ScmObj r = Scm_ApplyRec(load_stub, Scm_Cons(f, options));
+        ScmObj r = Scm_ApplyRec(load_proc, Scm_Cons(f, opts));
         if (packet) {
             packet->loaded = !SCM_FALSEP(r);
         }
         return 0;
     } else {
-        int r = Scm_Apply(load_stub, Scm_Cons(f, options), &eresult);
+        int r = Scm_Apply(load_proc, Scm_Cons(f, opts), &eresult);
         if (packet) {
             packet->exception = eresult.exception;
             packet->loaded = (r > 0 && !SCM_FALSEP(eresult.results[0]));
@@ -499,6 +425,17 @@ static ScmObj break_env_paths(const char *envname)
     }
 }
 
+static ScmObj add_list_item(ScmObj orig, ScmObj item, int afterp)
+{
+    if (afterp) {
+        return Scm_Append2(orig, SCM_LIST1(item));
+    } else {
+        return Scm_Cons(item, orig);
+    }
+}
+#define ADD_LIST_ITEM(list, item, afterp) \
+    list = add_list_item(list, item, afterp)
+
 /* Add CPATH to the current list of load path.  The path is
  * added before the current list, unless AFTERP is true.
  * The existence of CPATH is not checked.
@@ -529,30 +466,31 @@ ScmObj Scm_AddLoadPath(const char *cpath, int afterp)
     }
 
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
-    if (!SCM_PAIRP(ldinfo.load_path_rec->value)) {
-        ldinfo.load_path_rec->value = SCM_LIST1(spath);
-    } else if (afterp) {
-        ldinfo.load_path_rec->value =
-            Scm_Append2(ldinfo.load_path_rec->value, SCM_LIST1(spath));
-    } else {
-        ldinfo.load_path_rec->value = Scm_Cons(spath, ldinfo.load_path_rec->value);
-    }
+    ADD_LIST_ITEM(ldinfo.load_path_rec->value, spath, afterp);
+    ADD_LIST_ITEM(ldinfo.dynload_path_rec->value, dpath, afterp);
     r = ldinfo.load_path_rec->value;
-
-    if (!SCM_PAIRP(ldinfo.dynload_path_rec->value)) {
-        ldinfo.dynload_path_rec->value = SCM_LIST1(dpath);
-    } else if (afterp) {
-        ldinfo.dynload_path_rec->value =
-            Scm_Append2(ldinfo.dynload_path_rec->value, SCM_LIST1(dpath));
-    } else {
-        ldinfo.dynload_path_rec->value =
-            Scm_Cons(dpath, ldinfo.dynload_path_rec->value);
-    }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
-    
+
     return r;
 }
 
+void Scm_AddLoadPathHook(ScmObj proc, int afterp)
+{
+    (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
+    ADD_LIST_ITEM(ldinfo.load_path_hooks_rec->value, proc, afterp);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
+}
+
+void Scm_DeleteLoadPathHook(ScmObj proc)
+{
+    (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.path_mutex);
+    /* we should use Scm_Delete, instead of Scm_DeleteX,
+       to avoid race with reader of the list */
+    ldinfo.load_path_hooks_rec->value
+        = Scm_Delete(proc, ldinfo.load_path_hooks_rec->value, SCM_CMP_EQ);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.path_mutex);
+}
+
 /*------------------------------------------------------------------
  * Dynamic linking
  */
@@ -725,7 +663,7 @@ static const char *derive_dynload_initfn(const char *filename)
     return name;
 }
 
-/* find dlobj_initfn from the given dlobj with name. 
+/* find dlobj_initfn from the given dlobj with name.
    Assuming the caller holding the lock of OBJ. */
 static dlobj_initfn *find_initfn(dlobj *dlo, const char *name)
 {
@@ -739,18 +677,23 @@ static dlobj_initfn *find_initfn(dlobj *dlo, const char *name)
     fns->initialized = FALSE;
     fns->next = dlo->initfns;
     dlo->initfns = fns;
-    return fns;    
+    return fns;
 }
 
 /* From the given DSO name, find out the path of the actual DSO */
 static const char *find_dso_path(ScmString *dsoname)
 {
-    ScmObj lpaths = Scm_GetDynLoadPath();
-    ScmObj spath = Scm_FindFile(dsoname, &lpaths, ldinfo.dso_suffixes, TRUE);
+    ScmObj spath;
+    static ScmObj find_file = SCM_UNDEFINED;
+    SCM_BIND_PROC(find_file, "find-load-file", Scm_GaucheInternalModule());
+
+    spath = Scm_ApplyRec5(find_file, SCM_OBJ(dsoname), Scm_GetDynLoadPath(),
+                          ldinfo.dso_suffixes, SCM_FALSE, SCM_FALSE);
     if (SCM_FALSEP(spath)) {
         Scm_Error("can't find dlopen-able module %S", dsoname);
     }
-    return Scm_GetStringConst(SCM_STRING(spath));
+    SCM_ASSERT(SCM_STRINGP(SCM_CAR(spath)));
+    return Scm_GetStringConst(SCM_STRING(SCM_CAR(spath)));
 }
 
 /* Obtain the initializer function name (with '_' prepended) */
@@ -772,7 +715,7 @@ static void load_dlo(dlobj *dlo)
 {
     ScmVM *vm = Scm_VM();
     if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_LOAD_VERBOSE)) {
-        int len = Scm_Length(vm->load_history);
+        int len = Scm_Length(PARAM_REF(vm, load_history));
         SCM_PUTZ(";;", 2, SCM_CURERR);
         while (len-- > 0) SCM_PUTC(' ', SCM_CURERR);
         Scm_Printf(SCM_CURERR, "Dynamically Loading %s...\n", dlo->path);
@@ -815,7 +758,7 @@ static void call_initfn(dlobj *dlo, const char *name)
             }
         }
     }
-    
+
     /* Call initialization function.  note that there can be arbitrary
        complex stuff done within func(), including evaluation of
        Scheme procedures and/or calling dynamic-load for other
@@ -904,7 +847,7 @@ ScmObj Scm_DynLoad(ScmString *filename, ScmObj initfn, u_long flags/*reserved*/)
  *   there's no 'provide' form in X.scm, the feature "X" is automatically
  *   provided upon a successful loading of X.scm.
  *
- *   If a 'provide' form appears in X.scm, the autoprovide feature is 
+ *   If a 'provide' form appears in X.scm, the autoprovide feature is
  *   turned off.  It is allowed that X.scm provides features other than
  *   "X".   As a special case, (provide #f) causes the autoprovide feature
  *   to be turned of without providing any feature.
@@ -936,7 +879,7 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
 
     /* Check provided, providing and waiting list.  See the comment above. */
     (void)SCM_INTERNAL_MUTEX_LOCK(ldinfo.prov_mutex);
-    do {
+    for (;;) {
         provided = Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL);
         if (!SCM_FALSEP(provided)) break;
         providing = Scm_Assoc(feature, ldinfo.providing, SCM_CMP_EQUAL);
@@ -946,7 +889,7 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
         p = providing;
         SCM_ASSERT(SCM_PAIRP(p) && SCM_PAIRP(SCM_CDR(p)));
         if (SCM_CADR(p) == SCM_OBJ(vm)) { loop = TRUE; break; }
-        
+
         for (;;) {
             q = Scm_Assq(SCM_CDR(p), ldinfo.waiting);
             if (SCM_FALSEP(q)) break;
@@ -959,8 +902,7 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
         ldinfo.waiting = Scm_Acons(SCM_OBJ(vm), feature, ldinfo.waiting);
         (void)SCM_INTERNAL_COND_WAIT(ldinfo.prov_cv, ldinfo.prov_mutex);
         ldinfo.waiting = Scm_AssocDeleteX(SCM_OBJ(vm), ldinfo.waiting, SCM_CMP_EQ);
-        continue;
-    } while (0);
+    }
     if (!loop && SCM_FALSEP(provided)) {
         ldinfo.providing =
             Scm_Acons(feature, SCM_LIST1(SCM_OBJ(vm)), ldinfo.providing);
@@ -975,7 +917,7 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
             return -1;
         }
     }
-        
+
     if (!SCM_FALSEP(provided)) return 0; /* no work to do */
     r = Scm_Load(Scm_GetStringConst(SCM_STRING(feature)), 0, &xresult);
     if (packet) packet->exception = xresult.exception;
@@ -995,7 +937,7 @@ int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *packet)
     p = Scm_Assoc(feature, ldinfo.providing, SCM_CMP_EQUAL);
     ldinfo.providing = Scm_AssocDeleteX(feature, ldinfo.providing, SCM_CMP_EQUAL);
     /* `Autoprovide' feature */
-    if (SCM_NULLP(SCM_CDDR(p)) 
+    if (SCM_NULLP(SCM_CDDR(p))
         && SCM_FALSEP(Scm_Member(feature, ldinfo.provided, SCM_CMP_EQUAL))) {
         ldinfo.provided = Scm_Cons(feature, ldinfo.provided);
     }
@@ -1009,7 +951,7 @@ ScmObj Scm_Provide(ScmObj feature)
 {
     ScmObj cp;
     ScmVM *self = Scm_VM();
-    
+
     if (!SCM_STRINGP(feature)&&!SCM_FALSEP(feature)) {
         SCM_TYPE_ERROR(feature, "string");
     }
@@ -1113,7 +1055,7 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
     int circular = FALSE;
     ScmModule *prev_module;
     ScmVM *vm = Scm_VM();
-    
+
     /* shortcut in case if somebody else already did the job. */
     if (adata->loaded) return adata->value;
 
@@ -1158,7 +1100,7 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
         /* ok, somebody did the work for me.  just use the result. */
         return adata->value;
     }
-    
+
     if (circular) {
         /* Since we have already checked recursive loading, it isn't normal
            if we reach here.  Right now I have no idea how this happens, but
@@ -1174,7 +1116,7 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
         vm->module = adata->module;
         Scm_Require(SCM_OBJ(adata->path), SCM_LOAD_PROPAGATE_ERROR, NULL);
         vm->module = prev_module;
-    
+
         if (adata->import_from) {
             /* autoloaded file defines import_from module.  we need to
                import the binding individually. */
@@ -1219,6 +1161,14 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
     return adata->value;
 }
 
+/*------------------------------------------------------------------
+ * Dynamic parameter access
+ */
+ScmObj Scm_CurrentLoadHistory() { return PARAM_REF(Scm_VM(), load_history); }
+ScmObj Scm_CurrentLoadNext()    { return PARAM_REF(Scm_VM(), load_next); }
+ScmObj Scm_CurrentLoadPort()    { return PARAM_REF(Scm_VM(), load_port); }
+ScmObj Scm_LoadMainScript()     { return PARAM_REF(Scm_VM(), load_main_script); }
+
 /*------------------------------------------------------------------
  * Compatibility stuff
  */
@@ -1247,6 +1197,7 @@ ScmObj Scm__RequireCompat(ScmObj feature)
 void Scm__InitLoad(void)
 {
     ScmModule *m = Scm_SchemeModule();
+    ScmVM *vm = Scm_VM();
     ScmObj init_load_path, init_dynload_path, init_load_suffixes, t;
 
     init_load_path = t = SCM_NIL;
@@ -1271,13 +1222,17 @@ void Scm__InitLoad(void)
     key_error_if_not_found = SCM_MAKE_KEYWORD("error-if-not-found");
     key_macro = SCM_MAKE_KEYWORD("macro");
     key_ignore_coding = SCM_MAKE_KEYWORD("ignore-coding");
-    
+    key_main_script = SCM_MAKE_KEYWORD("main-script");
+    key_paths = SCM_MAKE_KEYWORD("paths");
+    key_environment = SCM_MAKE_KEYWORD("environment");
+
 #define DEF(rec, sym, val) \
     rec = SCM_GLOC(Scm_Define(m, SCM_SYMBOL(sym), val))
 
     DEF(ldinfo.load_path_rec,    SCM_SYM_LOAD_PATH, init_load_path);
     DEF(ldinfo.dynload_path_rec, SCM_SYM_DYNAMIC_LOAD_PATH, init_dynload_path);
     DEF(ldinfo.load_suffixes_rec, SCM_SYM_LOAD_SUFFIXES, init_load_suffixes);
+    DEF(ldinfo.load_path_hooks_rec, SCM_SYM_LOAD_PATH_HOOKS, SCM_NIL);
 
     ldinfo.provided =
         SCM_LIST5(SCM_MAKE_STR("srfi-2"), /* and-let* */
@@ -1291,4 +1246,10 @@ void Scm__InitLoad(void)
     ldinfo.dso_suffixes = SCM_LIST2(SCM_MAKE_STR(".la"),
                                     SCM_MAKE_STR("." SHLIB_SO_SUFFIX));
     ldinfo.dso_list = NULL;
+
+#define PARAM_INIT(name, val) Scm_InitParameterLoc(vm, &ldinfo.name, val)
+    PARAM_INIT(load_history, SCM_NIL);
+    PARAM_INIT(load_next, SCM_NIL);
+    PARAM_INIT(load_port, SCM_FALSE);
+    PARAM_INIT(load_main_script, SCM_FALSE);
 }
