@@ -1,12 +1,12 @@
 /*
  * main.c - interpreter main program
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -59,10 +59,14 @@ ScmObj pre_cmds = SCM_NIL;      /* assoc list of commands that needs to be
                                    Each car has either #\I, #\A, #\u, #\l
                                    or #\e, according to the given cmdargs. */
 
+ScmObj main_module = SCM_FALSE; /* The name of the module where we
+                                   look for 'main'.  If #f, 'user' module
+                                   is used. */
+
 void usage(void)
 {
     fprintf(stderr,
-            "Usage: gosh [-biqV][-I<path>][-A<path>][-u<module>][-l<file>][-L<file>][-e<expr>][-E<expr>][-p<type>][-F<feature>][-f<flag>][--] [file]\n"
+            "Usage: gosh [-biqV][-I<path>][-A<path>][-u<module>][-m<module>][-l<file>][-L<file>][-e<expr>][-E<expr>][-p<type>][-F<feature>][-f<flag>][--] [file]\n"
             "options:\n"
             "  -V       Prints version and exits.\n"
             "  -b       Batch mode.  Doesn't print prompts.  Supersedes -i.\n"
@@ -78,6 +82,11 @@ void usage(void)
             "           the script file or entering repl.\n"
             "  -E<expr> Similar to -e, but reads <expr> as if it is surrounded\n"
             "           by parenthesis.\n"
+            "  -m<module> When the script file is given, this option specifies the\n"
+            "           name of the module in which the 'main' procedure is defined.\n"
+            "           By default, the 'main' procedure in the user module is called\n"
+            "           after loading the script (srfi-22).  This option allows to call\n"
+            "           a main procedure in the different module\n"
             "  -p<type> Turns on the profiler.  Currently <type> can only be\n"
             "           'time'.\n"
             "  -F<feature> Makes <feature> available in cond-expand forms\n"
@@ -98,15 +107,17 @@ void usage(void)
 }
 
 #ifdef GAUCHE_USE_PTHREADS
-#define PTHREAD_OPT ",pthreads"
+#define THREAD_OPT ",pthreads"
+#elif  GAUCHE_USE_WTHREADS
+#define THREAD_OPT ",wthreads"
 #else
-#define PTHREAD_OPT ""
+#define THREAD_OPT ""
 #endif
 
 void version(void)
 {
     printf("Gauche scheme shell, version %s [%s%s], %s\n",
-           GAUCHE_VERSION, SCM_CHAR_ENCODING_NAME, PTHREAD_OPT, GAUCHE_ARCH);
+           GAUCHE_VERSION, SCM_CHAR_ENCODING_NAME, THREAD_OPT, GAUCHE_ARCH);
     exit(0);
 }
 
@@ -128,7 +139,10 @@ void further_options(const char *optarg)
         SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NOINLINE_CONSTS);
     }
     else if (strcmp(optarg, "no-post-inline-pass") == 0) {
-        SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NO_PASS2POST);
+        SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NO_POST_INLINE_OPT);
+    }
+    else if (strcmp(optarg, "no-lambda-lifting-pass") == 0) {
+        SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NO_LIFTING);
     }
     else if (strcmp(optarg, "no-source-info") == 0) {
         SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NOSOURCE);
@@ -136,6 +150,9 @@ void further_options(const char *optarg)
     else if (strcmp(optarg, "load-verbose") == 0) {
         SCM_VM_RUNTIME_FLAG_SET(vm, SCM_LOAD_VERBOSE);
     }
+    else if (strcmp(optarg, "include-verbose") == 0) {
+        SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_INCLUDE_VERBOSE);
+    }
     else if (strcmp(optarg, "case-fold") == 0) {
         SCM_VM_RUNTIME_FLAG_SET(vm, SCM_CASE_FOLD);
     }
@@ -161,7 +178,7 @@ void further_options(const char *optarg)
     }
     else {
         fprintf(stderr, "unknown -f option: %s\n", optarg);
-        fprintf(stderr, "supported options are: -fcase-fold or -fload-verbose, -fno-inline, -fno-inline-globals, -fno-inline-locals, -fno-inline-constants, -fno-source-info, -ftest\n");
+        fprintf(stderr, "supported options are: -fcase-fold or -fload-verbose, -fno-inline, -fno-inline-globals, -fno-inline-locals, -fno-inline-constants, -fno-source-info, -fno-post-inline-pass, -fno-lambda-lifting-pass, -ftest\n");
         exit(1);
     }
 }
@@ -169,7 +186,7 @@ void further_options(const char *optarg)
 void profiler_options(const char *optarg)
 {
     ScmVM *vm = Scm_VM();
-    
+
     if (strcmp(optarg, "time") == 0) {
         profiling_mode = TRUE;
     }
@@ -190,7 +207,7 @@ void feature_options(const char *optarg)
 int parse_options(int argc, char *argv[])
 {
     int c;
-    while ((c = getopt(argc, argv, "+be:E:ip:ql:L:u:VF:f:I:A:-")) >= 0) {
+    while ((c = getopt(argc, argv, "+be:E:ip:ql:L:m:u:VF:f:I:A:-")) >= 0) {
         switch (c) {
         case 'b': batch_mode = TRUE; break;
         case 'i': interactive_mode = TRUE; break;
@@ -199,6 +216,9 @@ int parse_options(int argc, char *argv[])
         case 'f': further_options(optarg); break;
         case 'p': profiler_options(optarg); break;
         case 'F': feature_options(optarg); break;
+        case 'm':
+            main_module = Scm_Intern(SCM_STRING(SCM_MAKE_STR_COPYING(optarg)));
+            break;
         case 'u': /*FALLTHROUGH*/;
         case 'l': /*FALLTHROUGH*/;
         case 'L': /*FALLTHROUGH*/;
@@ -259,13 +279,13 @@ void test_paths_setup(void)
        be searched first (hence it should come latter), since some
        extension modules are built from the file in src then linked
        from lib, and we want to test the one in lib. */
-    if (access("../src/stdlib.stub", R_OK) == 0
+    if (access("../src/gauche-init.scm", R_OK) == 0
         && access("../libsrc/srfi-1.scm", R_OK) == 0
         && access("../lib/srfi-0.scm", R_OK) == 0) {
         Scm_AddLoadPath("../src", FALSE);
         Scm_AddLoadPath("../libsrc", FALSE);
         Scm_AddLoadPath("../lib", FALSE);
-    } else if (access("../../src/stdlib.stub", R_OK) == 0
+    } else if (access("../../src/gauche-init.scm", R_OK) == 0
                && access("../../libsrc/srfi-1.scm", R_OK) == 0
                && access("../../lib/srfi-0.scm", R_OK) == 0) {
         Scm_AddLoadPath("../../src", FALSE);
@@ -293,10 +313,10 @@ void cleanup_main(void *data)
     if (profiling_mode) {
         Scm_ProfilerStop();
         Scm_EvalCString("(profiler-show)",
-                        SCM_OBJ(SCM_FIND_MODULE("gauche.vm.profiler", 0)),
+                        SCM_OBJ(Scm_GaucheModule()),
                         NULL); /* ignore errors */
     }
-    
+
     /* EXPERIMENTAL */
     if (stats_mode) {
         fprintf(stderr, "\n;; Statistics (*: main thread only):\n");
@@ -317,7 +337,7 @@ void cleanup_main(void *data)
         Scm_Eval(SCM_LIST2(SCM_INTERN("profiler-show-load-stats"),
                            SCM_LIST2(SCM_INTERN("quote"),
                                      vm->stat.loadStat)),
-                 SCM_OBJ(SCM_FIND_MODULE("gauche.vm.profiler", 0)),
+                 SCM_OBJ(Scm_GaucheModule()),
                  NULL);    /* ignore errors */
     }
 }
@@ -339,14 +359,23 @@ static int init_console(void)
 {
 #if defined(GAUCHE_WINDOWS)
 #  if defined(GAUCHE_WINDOWS_NOCONSOLE)
-    close(0);               /* just in case */
-    close(1);               /* ditto */
-    close(2);               /* ditto */
-    open("NUL", O_RDONLY);
-    open("NUL", O_WRONLY);
-    open("NUL", O_WRONLY);
+    char buf[100];
+    int in_fd, out_fd;
+#define ERR(msg) do {sprintf(buf, msg, strerror(errno));goto fail;} while(0)
+
+    if ((in_fd = open("NUL", O_RDONLY)) < 0) ERR("couldn't open NUL: %s");
+    if ((out_fd = open("NUL", O_WRONLY))< 0) ERR("couldn't open NUL: %s");
+    if (_dup2(in_fd, 0) < 0)  ERR("dup2(0) failed (%s)");
+    if (_dup2(out_fd, 1) < 0) ERR("dup2(1) failed (%s)");
+    if (_dup2(out_fd, 2) < 0) ERR("dup2(2) failed (%s)");
+    close(in_fd);
+    close(out_fd);
     return FALSE;
-#  else /*!defined(GAUCHE_WINDOS_NOCONSOLE)*/
+#undef ERR
+ fail:
+    MessageBoxA(NULL, buf, "gosh-noconsole", MB_OK|MB_ICONERROR);
+    Scm_Exit(1);
+#  else /*!defined(GAUCHE_WINDOWS_NOCONSOLE)*/
     /* This saves so much trouble */
     _setmode(_fileno(stdin),  _O_BINARY);
     _setmode(_fileno(stdout), _O_BINARY);
@@ -357,7 +386,7 @@ static int init_console(void)
     return TRUE;
 #endif /*!defined(GAUCHE_WINDOWS)*/
 }
-    
+
 /*-----------------------------------------------------------------
  * MAIN
  */
@@ -438,7 +467,7 @@ int main(int argc, char **argv)
     SCM_FOR_EACH(cp, Scm_Reverse(pre_cmds)) {
         ScmObj p = SCM_CAR(cp);
         ScmObj v = SCM_CDR(p);
-        
+
         switch (SCM_CHAR_VALUE(SCM_CAR(p))) {
         case 'I':
             Scm_AddLoadPath(Scm_GetStringConst(SCM_STRING(v)), FALSE);
@@ -481,7 +510,7 @@ int main(int argc, char **argv)
                                 SCM_OBJ(Scm_UserModule()),
                                 &epak) < 0) {
                 error_exit(epak.exception);
-            }   
+            }
             break;
         }
     }
@@ -499,17 +528,23 @@ int main(int argc, char **argv)
 
     if (scriptfile != NULL) {
         /* If script file is specified, load it. */
-        ScmObj mainproc;
+        ScmModule *mainmod;
+        ScmObj mainproc = SCM_FALSE;
         ScmEvalPacket epak;
 
         if (Scm_Load(scriptfile, 0, &lpak) < 0) {
             error_exit(lpak.exception);
         }
 
-        /* if symbol 'main is bound to a procedure in the user module,
-           call it.  (SRFI-22) */
-        mainproc = Scm_SymbolValue(Scm_UserModule(),
-                                   SCM_SYMBOL(SCM_INTERN("main")));
+        /* If symbol 'main is bound, call it (SRFI-22).   */
+        mainmod = (SCM_SYMBOLP(main_module)
+                   ? Scm_FindModule(SCM_SYMBOL(main_module), 0)
+                   : Scm_UserModule());
+        if (mainmod) {
+            mainproc = Scm_GlobalVariableRef(mainmod,
+                                             SCM_SYMBOL(SCM_INTERN("main")),
+                                             SCM_BINDING_STAY_IN_MODULE);
+        }
         if (SCM_PROCEDUREP(mainproc)) {
 #if 0 /* Temporarily turned off due to the bug that loses stack traces. */
             int r = Scm_Apply(mainproc, SCM_LIST1(av), &epak);
@@ -563,7 +598,7 @@ int getopt(int argc, char **argv, const char *spec)
     static int clusterind = 0;
 
     char optchar, *optspec;
-    
+
     do {
         if (optind >= argc) return -1;
         if (clusterind > 0) {
@@ -590,7 +625,7 @@ int getopt(int argc, char **argv, const char *spec)
         /* now we have option char in optchar.  we know spec[0] == '+' so
            we skip it. */
         if ((optspec = strchr(spec+1, optchar)) == NULL) return '?';
-        
+
         if (optspec[1] != ':') return optchar;
 
         /* look for the argument. */
