@@ -8,16 +8,13 @@
 ;;
 ;;  gosh -l./preload -I./src -I./lib ....
 ;;
-;;
-;; $Id: preload.scm,v 1.4 2007/11/20 12:46:35 shirok Exp $
-;;
 
 (use gauche.collection)
 (use gauche.sequence)
 (use gauche.hook)
 (use gauche.parameter)
 (use gauche.uvector)
-(use gauche.regexp)  ; need to compile 0.8.13_pre with 0.8.12
+(use gauche.regexp)
 (use srfi-1)
 (use srfi-13)
 (use file.util)
