@@ -1,12 +1,12 @@
 /*
  * symbol.c - symbol implementation
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -47,7 +47,7 @@ static ScmSymbol *make_sym(ScmObj name, int interned)
     ScmSymbol *sym = SCM_NEW(ScmSymbol);
     SCM_SET_CLASS(sym, SCM_CLASS_SYMBOL);
     sym->name = SCM_STRING(name);
-    sym->interned = interned;
+    sym->flags = interned? SCM_SYMBOL_FLAG_INTERNED : 0;
     return sym;
 }
 
@@ -165,7 +165,7 @@ void Scm_WriteSymbolName(ScmString *snam, ScmPort *port, ScmWriteContext *ctx,
     int siz = SCM_STRING_BODY_SIZE(b), i;
     int escape = FALSE;
     int spmask = ((SCM_WRITE_CASE(ctx) == SCM_WRITE_CASE_FOLD)? 0x12 : 0x02);
-        
+
     if (siz == 0) {         /* special case */
         if (!(flags & SCM_SYMBOL_WRITER_NOESCAPE_EMPTY)) {
             SCM_PUTZ("||", -1, port);
@@ -238,6 +238,7 @@ static void symbol_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 
 void Scm__InitSymbol(void)
 {
+    SCM_INTERNAL_MUTEX_INIT(obtable_mutex);
     obtable = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_STRING, 4096));
     init_builtin_syms();
 }
