@@ -1,12 +1,12 @@
 /*
  * system.c - system interface
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -45,13 +45,33 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <math.h>
+
 #if !defined(_MSC_VER)
 #include <dirent.h>
 #endif /* !_MSC_VER */
+
+#if defined(_MSC_VER)
+/* This is a kludge to patch the defect of config.h misconfiguration. */
+#undef HAVE_SETENV
+#undef HAVE_UNSETENV
+#undef HAVE_GETPGID
+#endif /*_MSC_VER*/
+
 #if !defined(GAUCHE_WINDOWS)
 #include <grp.h>
 #include <pwd.h>
+#include <sys/times.h>
 #include <sys/wait.h>
+
+# if !defined(HAVE_CRT_EXTERNS_H)
+/* POSIX defines environ, and ISO C defines __environ.
+   Modern C seems to have the latter declared in unistd.h */
+extern char **environ;
+# else  /* HAVE_CRT_EXTERNS_H */
+/* On newer OSX, we can't directly access global 'environ' variable.
+   We need to use _NSGetEnviron(), and this header defines it. */
+#include <crt_externs.h>
+# endif /* HAVE_CRT_EXTERNS_H */
 #else   /* GAUCHE_WINDOWS */
 #include <lm.h>
 #include <tlhelp32.h>
@@ -74,7 +94,7 @@ static int win_wait_for_handles(HANDLE *handles, int nhandles, int options,
 
 /*===============================================================
  * Conversion between off_t and Scheme integer.
- * off_t can be either 32bit or 64bit.  
+ * off_t can be either 32bit or 64bit.
  */
 off_t Scm_IntegerToOffset(ScmObj i)
 {
@@ -186,7 +206,7 @@ ScmObj Scm_ReadDirectory(ScmString *pathname)
     ScmVM *vm = Scm_VM();
     struct dirent *dire;
     DIR *dirp = opendir(Scm_GetStringConst(pathname));
-    
+
     if (dirp == NULL) {
         SCM_SIGCHECK(vm);
         Scm_SysError("couldn't open directory %S", pathname);
@@ -438,7 +458,24 @@ ScmObj Scm_NormalizePathname(ScmString *pathname, int flags)
 	return Scm_DStringGet(&buf, 0);
     }
 #else /* GAUCHE_WINDOWS */
-    if (endp > srcp+1 && isalpha(*srcp) && *(srcp+1) == ':') {
+    if ((flags & SCM_PATH_EXPAND) && size >= 1 && *str == '~') {
+        const char *home;
+        if (size >= 2 && strchr("/\\", str[1]) == NULL) {
+            Scm_Error("On windows native platforms, getting other user's home "
+                      "directory is not supported (yet): %S",
+                      SCM_OBJ(pathname));
+        }
+        srcp++;
+        if ((home = getenv("HOME")) != NULL) { /* MSYS */
+            Scm_DStringPutz(&buf, home, -1);
+        } else if ((home = getenv("HOMEDRIVE")) != NULL) { /* cmd.exe */
+            Scm_DStringPutz(&buf, home, -1);
+            if ((home = getenv("HOMEPATH")) != NULL) {
+                Scm_DStringPutz(&buf, home, -1);
+            }
+        }
+        /*FALLTHROUGH - if no env is set we use root dir. */
+    } else if (endp > srcp+1 && isalpha(*srcp) && *(srcp+1) == ':') {
         /* We first process the Evil Drive Letter */
         Scm_DStringPutc(&buf, *srcp++);
         Scm_DStringPutc(&buf, *srcp++);
@@ -515,7 +552,7 @@ ScmObj Scm_NormalizePathname(ScmString *pathname, int flags)
             }
         }
     }
-    return Scm_DStringGet(&buf, 0);        
+    return Scm_DStringGet(&buf, 0);
 }
 
 /* Returns system's temporary directory. */
@@ -547,7 +584,7 @@ ScmObj Scm_TmpDir(void)
 /* Basename and dirname.
    On Win32, we need to treat drive names specially, e.g.:
    (sys-dirname "C:/a") == (sys-dirname "C:/") == (sys-dirname "C:") == "C:\\"
-   (sys-basename "C:/") == (sys-basename "C:) == "" 
+   (sys-basename "C:/") == (sys-basename "C:) == ""
 */
 
 ScmObj Scm_BaseName(ScmString *filename)
@@ -670,7 +707,7 @@ ScmObj Scm_SysMkstemp(ScmString *templat)
     u_int siz;
     int fd;
     const char *t = Scm_GetStringContent(templat, &siz, NULL, NULL);
-    if (siz >= MKSTEMP_PATH_MAX-6) { 
+    if (siz >= MKSTEMP_PATH_MAX-6) {
 	Scm_Error("pathname too long: %S", templat);
     }
     memcpy(name, t, siz);
@@ -790,14 +827,23 @@ static ScmObj time_allocate(ScmClass *klass, ScmObj initargs)
 static void time_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     ScmTime *t = SCM_TIME(obj);
-    Scm_Printf(port, "#<%S %S.%09lu>", t->type, Scm_MakeInteger64(t->sec), t->nsec);
+    ScmObj sec = Scm_MakeInteger64(t->sec);
+    long nsec = t->nsec;
+    /* t->sec can be negative for time-difference. */
+    if (Scm_Sign(sec) < 0 && t->nsec > 0) {
+        sec = Scm_Abs(Scm_Add(sec, SCM_MAKE_INT(1)));
+        nsec = 1000000000L - nsec;
+        Scm_Printf(port, "#<%S -%S.%09lu>", t->type, sec, nsec);
+    } else {
+        Scm_Printf(port, "#<%S %S.%09lu>", t->type, sec, nsec);
+    }
 }
 
 static int time_compare(ScmObj x, ScmObj y, int equalp)
 {
     ScmTime *tx = SCM_TIME(x);
     ScmTime *ty = SCM_TIME(y);
-    
+
     if (equalp) {
         if (SCM_EQ(tx->type, ty->type)
             && SCM_INT64_EQV(tx->sec, ty->sec)
@@ -815,7 +861,7 @@ static int time_compare(ScmObj x, ScmObj y, int equalp)
             if (tx->nsec < ty->nsec) return -1;
             if (tx->nsec == ty->nsec) return 0;
             else return 1;
-        } 
+        }
         else return 1;
     }
 }
@@ -1016,8 +1062,7 @@ ScmObj Scm_TimeToSeconds(ScmTime *t)
     }
 }
 
-#if defined(HAVE_STRUCT_TIMESPEC) || defined (GAUCHE_USE_PTHREADS)
-/* Scheme time -> timespec conversion, used by pthread routines.*/
+/* Scheme time -> timespec conversion */
 struct timespec *Scm_GetTimeSpec(ScmObj t, struct timespec *spec)
 {
     if (SCM_FALSEP(t)) return NULL;
@@ -1029,7 +1074,7 @@ struct timespec *Scm_GetTimeSpec(ScmObj t, struct timespec *spec)
             Scm_Error("cannot convert Scheme time to struct timespec: out of range: %S", t);
         }
         spec->tv_sec = SCM_TIME(t)->sec.lo;
-        
+
 #else  /*!SCM_EMULATE_INT64*/
         /* TODO: we might want to check if tv_sec can handle 64bit integer */
         spec->tv_sec = SCM_TIME(t)->sec;
@@ -1045,9 +1090,11 @@ struct timespec *Scm_GetTimeSpec(ScmObj t, struct timespec *spec)
         spec->tv_sec = ct->sec;
 #endif /*!SCM_EMULATE_INT64*/
         spec->tv_nsec = ct->nsec;
-        if (SCM_EXACTP(t)) {
+        if (SCM_INTP(t)) {
             spec->tv_sec += Scm_GetUInteger(t);
-        } else if (SCM_FLONUMP(t)) {
+        } else if (!SCM_REALP(t)) {
+            Scm_Panic("implementation error: Scm_GetTimeSpec: something wrong");
+        } else {
             double s;
             spec->tv_nsec += (unsigned long)(modf(Scm_GetDouble(t), &s)*1.0e9);
             spec->tv_sec += (unsigned long)s;
@@ -1055,13 +1102,10 @@ struct timespec *Scm_GetTimeSpec(ScmObj t, struct timespec *spec)
                 spec->tv_nsec -= 1000000000;
                 spec->tv_sec += 1;
             }
-        } else {
-            Scm_Panic("implementation error: Scm_GetTimeSpec: something wrong");
         }
     }
     return spec;
 }
-#endif /* defined(HAVE_STRUCT_TIMESPEC) || defined (GAUCHE_USE_PTHREADS) */
 
 /* <sys-tm> object */
 
@@ -1144,13 +1188,14 @@ Scm_YieldCPU(void)
     spec.tv_sec = 0;
     spec.tv_nsec = 1;
     nanosleep(&spec, NULL);
+#elif defined(GAUCHE_WINDOWS)
+    /* Windows have select(), but it doesn't allow all fds are NULL. */
+    Sleep(10);
 #elif defined(HAVE_SELECT)
     struct timeval tv;
     tv.tv_sec = 0;
     tv.tv_usec = 1;
     select(0, NULL, NULL, NULL, &tv);
-#elif defined(GAUCHE_WINDOWS)
-    Sleep(10);
 #else /* the last resort */
     sleep(1);
 #endif
@@ -1172,7 +1217,7 @@ static ScmObj make_group(struct group *g)
 {
     ScmSysGroup *sg = SCM_NEW(ScmSysGroup);
     SCM_SET_CLASS(sg, SCM_CLASS_SYS_GROUP);
-    
+
     sg->name = SCM_MAKE_STR_COPYING(g->gr_name);
 #ifdef HAVE_STRUCT_GROUP_GR_PASSWD
     sg->passwd = SCM_MAKE_STR_COPYING(g->gr_passwd);
@@ -1415,9 +1460,8 @@ char *win_create_command_line(ScmObj args)
 {
     ScmObj ap, out;
     ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
-    ScmObj proc = Scm_GlobalVariableRef(Scm_GaucheModule(),
-                                        SCM_SYMBOL(SCM_INTERN("%sys-escape-windows-command-line")),
-                                        0);
+    static ScmObj proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(proc, "%sys-escape-windows-command-line", Scm_GaucheModule());
     SCM_FOR_EACH(ap, args) {
       ScmObj escaped = Scm_ApplyRec1(proc, SCM_CAR(ap));
       Scm_Printf(SCM_PORT(ostr), "%A ", escaped);
@@ -1444,7 +1488,7 @@ char *win_create_command_line(ScmObj args)
  *   show the children's pid.   If fork arg is FALSE, this procedure
  *   of course never returns.
  *
- *   On Windows port, this returns a process handle obejct instead of   
+ *   On Windows port, this returns a process handle obejct instead of
  *   pid of the child process in fork mode.  We need to keep handle, or
  *   the process exit status will be lost when the child process terminates.
  */
@@ -1456,20 +1500,27 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
     const char *program, *cdir = NULL;
     pid_t pid = 0;
     int forkp = flags & SCM_EXEC_WITH_FORK;
+    int detachp = flags & SCM_EXEC_DETACHED;
     int *fds;
 
     if (argc < 1) {
         Scm_Error("argument list must have at least one element: %S", args);
     }
 
-    /* make a C array of C strings */    
+    /* make a C array of C strings */
     argv = Scm_ListToCStringArray(args, TRUE, NULL);
     program = Scm_GetStringConst(file);
 
     /* setting up iomap table */
     fds = Scm_SysPrepareFdMap(iomap);
-    
+
+    /*
+     * From now on, we have totally different code for Unix and Windows.
+     */
 #if !defined(GAUCHE_WINDOWS)
+    /*
+     * Unix path
+     */
     if (dir != NULL) cdir = Scm_GetStringConst(dir);
 
     /* When requested, call fork() here. */
@@ -1478,8 +1529,18 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
         if (pid < 0) Scm_SysError("fork failed");
     }
 
-    /* Now we swap file descriptors and exec(). */
-    if (!forkp || pid == 0) {
+    if (!forkp || pid == 0) {   /* possibly the child process */
+
+        /* If we're running the daemon, we fork again to detach the parent,
+           and also reset the session id. */
+        if (detachp) {
+            SCM_SYSCALL(pid, fork());
+            if (pid < 0) Scm_SysError("fork failed");
+            if (pid > 0) exit(0);   /* not Scm_Exit(), for we don't want to
+                                       run the cleanup stuff. */
+            setsid();
+        }
+
         if (cdir != NULL) {
             if (chdir(cdir) < 0) {
                 Scm_Panic("chdir to %s failed before executing %s: %s",
@@ -1501,11 +1562,14 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
     /* We come here only when fork is requested. */
     return Scm_MakeInteger(pid);
 #else  /* GAUCHE_WINDOWS */
+    /*
+     * Windows path
+     */
     if (dir != NULL) {
         /* we need full path for CreateProcess. */
         dir = SCM_STRING(Scm_NormalizePathname(dir, SCM_PATH_ABSOLUTE|SCM_PATH_CANONICALIZE));
         cdir = Scm_GetStringConst(dir);
-        
+
         /* If the program is given in relative pathname,
            it must be adjusted relative to the specified directory. */
         if (program[0] != '/' && program[0] != '\\'
@@ -1519,7 +1583,7 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
             program = Scm_DStringGetz(&ds);
         }
     }
-    
+
     if (forkp) {
         TCHAR  program_path[MAX_PATH+1], *filepart;
         HANDLE *hs = win_prepare_handles(fds);
@@ -1527,6 +1591,7 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
         STARTUPINFO si;
         PROCESS_INFORMATION pi;
         LPCTSTR curdir = NULL;
+        DWORD creation_flags = 0;
 
         pathlen = SearchPath(NULL, SCM_MBS2WCS(program),
                              _T(".exe"), MAX_PATH, program_path,
@@ -1544,12 +1609,16 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
 
         if (cdir != NULL) curdir = SCM_MBS2WCS(cdir);
 
+        if (detachp) {
+            creation_flags |= CREATE_NEW_PROCESS_GROUP;
+        }
+
         r = CreateProcess(program_path,
                           SCM_MBS2WCS(win_create_command_line(args)),
                           NULL, /* process attr */
                           NULL, /* thread addr */
                           TRUE, /* inherit handles */
-                          0,    /* creation flags */
+                          creation_flags, /* creation flags */
                           NULL, /* nenvironment */
                           curdir, /* current dir */
                           &si,  /* startup info */
@@ -1564,8 +1633,10 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
                 Scm_SysError("Couldn't chdir to %s", cdir);
             }
         }
+        /* TODO: We should probably use Windows API to handle various
+           options consistently with fork-and-exec case above. */
 	execvp(program, (const char *const*)argv);
-	Scm_Panic("exec failed: %s: %s", program, strerror(errno));	
+	Scm_Panic("exec failed: %s: %s", program, strerror(errno));
     }
     return SCM_FALSE; /* dummy */
 #endif /* GAUCHE_WINDOWS */
@@ -1585,7 +1656,7 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
    invalid entries.  On the other hand, Scm_SysSwapFds just aborts if
    things goes wrong---not only because of the MT-safety issue, but also
    it is generally impossible to handle errors reasonably since we don't
-   even sure we have stdios.   And the client code is supposed to call  
+   even sure we have stdios.   And the client code is supposed to call
    fork() between these functions.
 
    The client code should treat the returned pointer of Scm_SysPrepareFdMap
@@ -1598,7 +1669,7 @@ int *Scm_SysPrepareFdMap(ScmObj iomap)
         ScmObj iop;
         int iollen = Scm_Length(iomap), i = 0;
         int *tofd, *fromfd;
-        
+
         /* check argument vailidity before duping file descriptors, so that
            we can still use Scm_Error */
         if (iollen < 0) {
@@ -1643,7 +1714,7 @@ int *Scm_SysPrepareFdMap(ScmObj iomap)
 void Scm_SysSwapFds(int *fds)
 {
     int *tofd, *fromfd, nfds, maxfd, i, j, fd;
-    
+
     if (fds == NULL) return;
 
     nfds   = fds[0];
@@ -1673,7 +1744,7 @@ void Scm_SysSwapFds(int *fds)
         if (dup2(fromfd[i], tofd[i]) < 0)
             Scm_Panic("dup2 failed: %s", strerror(errno));
     }
-    
+
     /* Close unused fds */
     for (fd=0; fd<maxfd; fd++) {
         for (j=0; j<nfds; j++) if (fd == tofd[j]) break;
@@ -1703,7 +1774,7 @@ static HANDLE *win_prepare_handles(int *fds)
                                      (HANDLE)_get_osfhandle(from),
                                      GetCurrentProcess(),
                                      &zh,
-                                     0, TRUE, 
+                                     0, TRUE,
                                      DUPLICATE_CLOSE_SOURCE
                                      |DUPLICATE_SAME_ACCESS)) {
                     Scm_SysError("DuplicateHandle failed");
@@ -1755,7 +1826,7 @@ void Scm_SysKill(ScmObj process, int signal)
     } else {
         SCM_TYPE_ERROR(process, "process handle or integer process id");
     }
-    
+
     if (signal == SIGKILL) {
         if (pid_given) {
             p = OpenProcess(PROCESS_TERMINATE, FALSE, pid);
@@ -1773,7 +1844,7 @@ void Scm_SysKill(ScmObj process, int signal)
         return;
     }
     /* another idea; we may map SIGTERM to WM_CLOSE message. */
-    
+
     if (signal == 0) {
         /* We're supposed to do the error check without actually sending
            the signal.   For now we just pretend nothing's wrong. */
@@ -2003,7 +2074,7 @@ static ScmObj select_int(ScmSysFdset *rfds, ScmSysFdset *wfds,
     if (wfds && wfds->maxfd > maxfds) maxfds = wfds->maxfd;
     if (efds && efds->maxfd > maxfds) maxfds = efds->maxfd;
 
-    SCM_SYSCALL(numfds, 
+    SCM_SYSCALL(numfds,
                 select(maxfds+1,
                        (rfds? &rfds->fdset : NULL),
                        (wfds? &wfds->fdset : NULL),
@@ -2037,6 +2108,116 @@ ScmObj Scm_SysSelectX(ScmObj rfds, ScmObj wfds, ScmObj efds, ScmObj timeout)
 
 #endif /* HAVE_SELECT */
 
+/*===============================================================
+ * Environment
+ */
+
+/* POSIX has putenv but not setenv.  However, the XPG spec of
+   putenv is broken w.r.t freeing the passed string; there's no
+   way to know when the pointer can be freed.
+   Some Unix appears to violate the spec and copies the passed pointer,
+   which solves the leak, but the code will break if we bring it to
+   the platform with "proper" putenv(3).
+
+   So we do not trust putenv(3).
+
+   Our strategy: If the system has setenv(3) we use it, since we know
+   it copies the passed string.  If the system doesn't have setenv(3),
+   we fall back to putenv(3) and let it leak.
+*/
+
+void Scm_SetEnv(const char *name, const char *value, int overwrite)
+{
+#if defined(HAVE_SETENV)
+    int r;
+
+    SCM_SYSCALL(r, setenv(name, value, overwrite));
+    if (r < 0) Scm_SysError("setenv failed on '%s=%s'", name, value);
+#elif defined(HAVE_PUTENV)
+    char *nameval;
+    int nlen, vlen, r;
+
+    if (!overwrite) {
+        /* check the existence of NAME first. */
+        if (getenv(name) != NULL) return;
+    }
+    nlen = (int)strlen(name);
+    vlen = (int)strlen(value);
+    /* we need malloc, since the pointer will be owned by the system */
+    nameval = (char*)malloc(nlen+vlen+2);
+    if (nameval == NULL) {
+        Scm_Error("sys-setenv: out of memory");
+    }
+    strcpy(nameval, name);
+    strcpy(nameval+nlen, "=");
+    strcpy(nameval+nlen+1, value);
+    SCM_SYSCALL(r, putenv(nameval));
+    if (r < 0) Scm_SysError("putenv failed on '%s'", nameval);
+#else /* !HAVE_SETENV && !HAVE_PUTENV */
+    /* We can't do much.  we may replace environ by ourselves, but
+       it is unlikely that the system have extern environ and not putenv.
+    */
+    Scm_Error("neither setenv nor putenv is supported on this platform.");
+#endif
+}
+
+/* Returns the system's environment table as a list of strings.
+   Each string is in the format of "key=value". */
+ScmObj Scm_Environ(void)
+{
+#if !defined(GAUCHE_WINDOWS)
+#  if defined(HAVE_CRT_EXTERNS_H)
+    char **environ = *_NSGetEnviron(); /* OSX Hack */
+#  endif /*HAVE_CRT_EXTERNS_H*/
+    if (environ == NULL) return SCM_NIL;
+    else return Scm_CStringArrayToList((const char**)environ, -1,
+                                       SCM_STRING_COPYING);
+#else  /*GAUCHE_WINDOWS*/
+#define ENV_BUFSIZ 64
+    LPVOID ss = GetEnvironmentStrings();
+    ScmObj h = SCM_NIL, t = SCM_NIL;
+    TCHAR *cp = (TCHAR*)ss, *pp;
+    TCHAR sbuf[ENV_BUFSIZ], *buf=sbuf;
+    int bsize = ENV_BUFSIZ, size;
+
+    do {
+        for (pp=cp; *pp; pp++) /*proceed ptr*/;
+        size = (int)(pp - cp) + 1;
+        if (size >= bsize) {
+            buf = SCM_NEW_ATOMIC_ARRAY(TCHAR, size);
+            bsize = size;
+        }
+        memcpy(buf, cp, size*sizeof(TCHAR));
+        SCM_APPEND1(h, t, SCM_MAKE_STR_COPYING(SCM_WCS2MBS(buf)));
+        cp = pp+1;
+    } while (pp[1] != 0);
+    FreeEnvironmentStrings(ss);
+    return h;
+#endif /*GAUCHE_WINDOWS*/
+}
+
+void Scm_UnsetEnv(const char *name)
+{
+#if defined(HAVE_UNSETENV)
+    int r = 0;
+    SCM_SYSCALL(r, unsetenv(name));
+    if (r < 0) Scm_SysError("unsetenv failed on %s", name);
+#else  /*!HAVE_UNSETENV*/
+    Scm_Error("sys-unsetenv is not supported on this platform.");
+#endif /*!HAVE_UNSETENV*/
+}
+
+void Scm_ClearEnv()
+{
+#if defined(HAVE_CLEARENV)
+    int r = 0;
+    SCM_SYSCALL(r, clearenv());
+    if (r < 0) Scm_SysError("clearenv failed");
+#else  /*!HAVE_UNSETENV*/
+    Scm_Error("sys-clearenv is not supported on this platform.");
+#endif /*!HAVE_UNSETENV*/
+}
+
 /*===============================================================
  * Emulation layer for Windows
  */
@@ -2084,7 +2265,7 @@ static ScmObj get_relative_processes(int childrenp)
     DWORD myid = GetCurrentProcessId(), parentid;
     int found = FALSE;
     ScmObj h = SCM_NIL, t = SCM_NIL; /* children pids */
-    
+
     snapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
     if (snapshot == INVALID_HANDLE_VALUE) {
 	Scm_Error("couldn't take process snapshot in getppid()");
@@ -2240,6 +2421,41 @@ pid_t getppid(void)
     return Scm_GetInteger(ppid);
 }
 
+const char *getlogin(void)
+{
+    static TCHAR buf[256]; /* this isn't thread-safe, but getlogin() is
+                              inherently thread-unsafe call anyway */
+    DWORD size = sizeof(buf)/sizeof(TCHAR);
+    BOOL r;
+    r = GetUserName(buf, &size);
+    if (r) {
+	return SCM_WCS2MBS(buf);
+    } else {
+	return NULL;
+    }
+}
+
+clock_t times(struct tms *info)
+{
+    HANDLE process = GetCurrentProcess();
+    FILETIME ctime, xtime, utime, stime;
+    int64_t val;
+    const int factor = 10000000/CLK_TCK;
+    const int bias   = factor/2;
+
+    if (!GetProcessTimes(process, &ctime, &xtime, &stime, &utime)) {
+        Scm_SysError("GetProcessTimes failed");
+    }
+    val = ((int64_t)stime.dwHighDateTime << 32) + stime.dwLowDateTime;
+    info->tms_stime = (u_int)((val+bias) / factor);
+    val = ((int64_t)utime.dwHighDateTime << 32) + utime.dwLowDateTime;
+    info->tms_utime = (u_int)((val+bias) / factor);
+
+    info->tms_cstime = 0;
+    info->tms_cutime = 0;
+    return 0;
+}
+
 /*
  * Other obscure stuff
  */
@@ -2300,7 +2516,7 @@ int truncate(const char *path, off_t len)
 {
     HANDLE file;
     int r;
-    
+
     file = CreateFile(SCM_MBS2WCS(path), GENERIC_WRITE,
                       FILE_SHARE_READ|FILE_SHARE_WRITE,
                       NULL, OPEN_EXISTING, 0, NULL);
@@ -2414,8 +2630,206 @@ static void fini_winsock(void *data)
     (void)WSACleanup();
 }
 
-#endif /* GAUCHE_WINDOWS */
+/* Win32 thread support.  See also gauche/wthread.h */
 
+#if defined(GAUCHE_USE_WTHREADS)
+
+HANDLE Scm__WinCreateMutex()
+{
+    HANDLE m = CreateMutex(NULL, FALSE, NULL);
+    if (m == NULL) Scm_SysError("couldn't create a mutex");
+    return m;
+}
+
+int Scm__WinMutexLock(HANDLE mutex)
+{
+    DWORD r = WaitForSingleObject(mutex, INFINITE);
+    if (r == WAIT_OBJECT_0) return 0;
+    else return 1;              /* TODO: proper error handling */
+}
+
+/* Win32 conditional variable emulation.
+   Native condition variable support is only available on Windows Vista
+   and later.  We don't want to drop XP support (yet), so we avoid using
+   it.  Instead we emulate posix condition variable semantics.
+   We enhanced the implementation described as the SignalObjectAndWait
+   solution shown in
+   <http://www1.cse.wustl.edu/~schmidt/win32-cv-1.html>
+ */
+void Scm__InternalCondInit(ScmInternalCond *cond)
+{
+    cond->numWaiters = 0;
+    cond->broadcast = FALSE;
+    cond->mutex = NULL;         /* set by the first CondWait */
+    cond->sem = CreateSemaphore(NULL,          /* no security */
+                                0, 0x7fffffff, /* initial and max val */
+                                NULL);         /* name */
+    if (cond->sem == NULL) {
+        Scm_SysError("couldn't create a semaphore for a condition variable");
+    }
+    cond->done = CreateEvent(NULL,  /* no security */
+                             FALSE, /* auto-reset */
+                             FALSE, /* initially non-signalled */
+                             NULL); /* name */
+    if (cond->done == NULL) {
+        DWORD err = GetLastError();
+        CloseHandle(cond->sem);
+        SetLastError(err);
+        Scm_SysError("couldn't create event for a condition variable");
+    }
+    InitializeCriticalSection(&cond->numWaitersLock);
+}
+
+int Scm__InternalCondWait(ScmInternalCond *cond, ScmInternalMutex *mutex,
+                          struct timespec *pts)
+{
+    DWORD r0, r1;
+    DWORD timeout_msec;
+    int badMutex = FALSE, lastWaiter;
+
+    if (pts) {
+        u_long now_sec, now_usec;
+        u_long target_sec, target_usec;
+        Scm_GetTimeOfDay(&now_sec, &now_usec);
+        target_sec = pts->tv_sec;
+        target_usec = pts->tv_nsec / 1000;
+        if (target_sec < now_sec
+            || (target_sec == now_sec && target_usec <= now_usec)) {
+            timeout_msec = 0;
+        } else if (target_usec >= now_usec) {
+            timeout_msec = ceil((target_sec - now_sec) * 1000
+                                + (target_usec - now_usec)/1000.0);
+        } else {
+            timeout_msec = ceil((target_sec - now_sec - 1) * 1000
+                                + (1.0e6 + target_usec - now_usec)/1000.0);
+        }
+    } else {
+        timeout_msec = INFINITE;
+    }
+
+    EnterCriticalSection(&cond->numWaitersLock);
+    /* If we're the first one to wait on this cond var, set cond->mutex.
+       We don't allow to use multiple mutexes together with single cond var.
+     */
+    if (cond->mutex != NULL && cond->mutex != mutex) {
+        badMutex = TRUE;
+    } else {
+        cond->numWaiters++;
+        if (cond->mutex == NULL) cond->mutex = mutex;
+    }
+    LeaveCriticalSection(&cond->numWaitersLock);
+
+    if (badMutex) {
+        Scm_Error("Attempt to wait on condition variable %p with different"
+                  " mutex %p\n", cond, mutex);
+    }
+
+    /* Signals mutex and atomically waits on the semaphore */
+    r0 = SignalObjectAndWait(*mutex, cond->sem, timeout_msec, FALSE);
+
+    /* We're signaled, or timed out.   There can be a case that cond is
+       broadcasted between the timeout of SignalObjectAndWait and the
+       following EnterCriticalSection.  So we should check lastWaiter
+       anyway. */
+    EnterCriticalSection(&cond->numWaitersLock);
+    cond->numWaiters--;
+    lastWaiter = cond->broadcast && cond->numWaiters == 0;
+    LeaveCriticalSection(&cond->numWaitersLock);
+
+    if (lastWaiter) {
+        /* tell the broadcaster that all the waiters have gained
+           control, and wait to aquire mutex. */
+        r1 = SignalObjectAndWait(cond->done, *mutex, INFINITE, FALSE);
+    } else {
+        /* Aquire mutex */
+        r1 = WaitForSingleObject(*mutex, INFINITE);
+    }
+    if (r0 == WAIT_TIMEOUT) return SCM_INTERNAL_COND_TIMEDOUT;
+    if (r0 != WAIT_OBJECT_0 || r1 != WAIT_OBJECT_0) return -1;
+    return 0;
+}
+
+int Scm__InternalCondSignal(ScmInternalCond *cond)
+{
+    int haveWaiters;
+    BOOL r = TRUE;
+
+    if (!cond->mutex) return 0; /* nobody ever waited on this cond var. */
+
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(cond->mutex);
+
+    EnterCriticalSection(&cond->numWaitersLock);
+    haveWaiters = (cond->numWaiters > 0);
+    LeaveCriticalSection(&cond->numWaitersLock);
+
+    if (haveWaiters) {
+        r = ReleaseSemaphore(cond->sem, 1, 0);
+    }
+
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
+    if (!r) return -1;
+    return 0;
+}
+
+int Scm__InternalCondBroadcast(ScmInternalCond *cond)
+{
+    int haveWaiters;
+    DWORD err = 0;
+    BOOL r0 = TRUE;
+    DWORD r1 = WAIT_OBJECT_0;
+
+    if (!cond->mutex) return 0; /* nobody ever waited on this cond var. */
+
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(cond->mutex);
+
+    EnterCriticalSection(&cond->numWaitersLock);
+    cond->broadcast = haveWaiters = (cond->numWaiters > 0);
+
+    if (haveWaiters) {
+        r0 = ReleaseSemaphore(cond->sem, cond->numWaiters, 0);
+        if (!r0) err = GetLastError();
+        LeaveCriticalSection(&cond->numWaitersLock);
+
+        if (r0) {
+            /* Each waiter aquires mutex in turn, until the last waiter
+               who will signal on 'done'. */
+            r1 = WaitForSingleObject(cond->done, INFINITE);
+            cond->broadcast = FALSE; /* safe; nobody will check this */
+        }
+    } else {
+        /* nobody's waiting */
+        LeaveCriticalSection(&cond->numWaitersLock);
+    }
+
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
+
+    if (!r0) { SetLastError(err); return -1; }
+    if (r1 != WAIT_OBJECT_0) return -1;
+    return 0;
+}
+
+void Scm__InternalCondDestroy(ScmInternalCond *cond)
+{
+    CloseHandle(cond->sem);
+    cond->sem = NULL;
+    CloseHandle(cond->done);
+    cond->done = NULL;
+}
+
+void Scm__WinThreadExit()
+{
+    ScmVM *vm = Scm_VM();
+    ScmWinCleanup *cup = vm->winCleanup;
+    while (cup) {
+        cup->cleanup(cup->data);
+        cup = cup->prev;
+    }
+    GC_ExitThread(0);
+}
+
+#endif /* GAUCHE_USE_WTHREADS */
+
+#endif /* GAUCHE_WINDOWS */
 
 /*===============================================================
  * Initialization
@@ -2434,6 +2848,7 @@ void Scm__InitSystem(void)
 
 #ifdef GAUCHE_WINDOWS
     init_winsock();
+    SCM_INTERNAL_MUTEX_INIT(process_mgr.mutex);
     Scm_AddCleanupHandler(fini_winsock, NULL);
     Scm_AddCleanupHandler(win_process_cleanup, NULL);
 #endif
