@@ -1,12 +1,12 @@
 /*
  * vm.c - virtual machine
  *
- *   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
- * 
+ *   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+ *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
  *   are met:
- * 
+ *
  *   1. Redistributions of source code must retain the above copyright
  *      notice, this list of conditions and the following disclaimer.
  *
@@ -71,16 +71,25 @@ static ScmWord boundaryFrameMark = SCM_VM_INSN(SCM_VM_NOP);
 static ScmWord return_code[] = { SCM_VM_INSN(SCM_VM_RET) };
 #define PC_TO_RETURN  return_code
 
+/* A dummy env frame to indicate C Continuation */
+static ScmEnvFrame ccEnvMark = {
+    NULL,                       /* up */
+    SCM_FALSE,                  /* info */
+    0                           /* size */
+};
+
+#define C_CONTINUATION_P(cont)  ((cont)->env == &ccEnvMark)
+
 /* A dummy compiled code structure used as 'fill-in', when Scm_Apply
    is called without any VM code running.  See Scm_Apply below. */
-static ScmCompiledCode internal_apply_compiled_code = 
+static ScmCompiledCode internal_apply_compiled_code =
     SCM_COMPILED_CODE_CONST_INITIALIZER(NULL, 0, 0, 0, 0,
                                         SCM_SYM_INTERNAL_APPLY,
                                         SCM_NIL, SCM_FALSE,
                                         SCM_FALSE, SCM_FALSE);
 
 /*
- * The VM. 
+ * The VM.
  *
  *   VM encapsulates the dynamic status of the current exection.
  *   In Gauche, there's always one active virtual machine per thread,
@@ -91,13 +100,22 @@ static ScmCompiledCode internal_apply_compiled_code =
  */
 
 static ScmVM *rootVM = NULL;         /* VM for primodial thread */
+static ScmHashCore vm_table;         /* VMs other than primordial one is
+                                        registered to this hashtalbe, in order
+                                        to avoid being GC-ed. */
+static ScmInternalMutex vm_table_mutex;
+static void vm_register(ScmVM *vm);
+static void vm_unregister(ScmVM *vm);
 
 #ifdef GAUCHE_USE_PTHREADS
 static pthread_key_t vm_key;
 #define theVM   ((ScmVM*)pthread_getspecific(vm_key))
-#else
+#elif  GAUCHE_USE_WTHREADS
+static DWORD vm_key;
+#define theVM   ((ScmVM*)TlsGetValue(vm_key))
+#else  /* !GAUCHE_USE_PTHREADS && !GAUCHE_USE_WTHREADS */
 static ScmVM *theVM;
-#endif  /* !GAUCHE_USE_PTHREADS */
+#endif /* !GAUCHE_USE_PTHREADS */
 
 static void save_stack(ScmVM *vm);
 
@@ -107,6 +125,7 @@ static ScmObj throw_cont_calculate_handlers(ScmEscapePoint *, ScmVM *);
 static ScmObj throw_cont_body(ScmObj, ScmEscapePoint*, ScmObj);
 static void   process_queued_requests(ScmVM *vm);
 static void   vm_finalize(ScmObj vm, void *data);
+static int    check_arglist_tail_for_apply(ScmVM *vm, ScmObj restargs);
 
 static ScmEnvFrame *get_env(ScmVM *vm);
 
@@ -126,7 +145,7 @@ static ScmEnvFrame *get_env(ScmVM *vm);
  *   at the initialization stage (see Scm__InitVM()).   For other threads,
  *   it depends on whether the thread is created from Gauche side or not.
  *
- *   If the thread is created from Gauche side (i.e. by Scm_MakeThread() 
+ *   If the thread is created from Gauche side (i.e. by Scm_MakeThread()
  *   C API or make-thread Scheme API), attaching is handled automatically
  *   by Gauche.
  *
@@ -144,7 +163,7 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
 {
     ScmVM *v = SCM_NEW(ScmVM);
     int i;
-    
+
     SCM_SET_CLASS(v, SCM_CLASS_VM);
     v->state = SCM_VM_NEW;
     (void)SCM_INTERNAL_MUTEX_INIT(v->vmlock);
@@ -157,8 +176,8 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
     v->result = SCM_UNDEFINED;
     v->resultException = SCM_UNDEFINED;
     v->module = proto ? proto->module : Scm_SchemeModule();
-    v->cstack = proto ? proto->cstack : NULL;
-    
+    v->cstack = NULL;
+
     v->curin  = proto? proto->curin  : SCM_PORT(Scm_Stdin());
     v->curout = proto? proto->curout : SCM_PORT(Scm_Stdout());
     v->curerr = proto? proto->curerr : SCM_PORT(Scm_Stderr());
@@ -196,7 +215,7 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
     v->val0 = SCM_UNDEFINED;
     for (i=0; i<SCM_VM_MAX_VALUES; i++) v->vals[i] = SCM_UNDEFINED;
     v->numVals = 1;
-    
+
     v->handlers = SCM_NIL;
 
     v->exceptionHandler = DEFAULT_EXCEPTION_HANDLER;
@@ -206,9 +225,6 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
     v->escapeData[1] = NULL;
     v->defaultEscapeHandler = SCM_FALSE;
 
-    v->load_history = SCM_NIL;
-    v->load_next = SCM_NIL;
-    v->load_port = SCM_FALSE;
     v->evalSituation = SCM_VM_EXECUTING;
 
     sigemptyset(&v->sigMask);
@@ -223,6 +239,10 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
 
     (void)SCM_INTERNAL_THREAD_INIT(v->thread);
 
+#if defined(GAUCHE_USE_WTHREADS)
+    v->winCleanup = NULL;
+#endif /*defined(GAUCHE_USE_WTHREADS)*/
+
     Scm_RegisterFinalizer(SCM_OBJ(v), vm_finalize, NULL);
     return v;
 }
@@ -232,18 +252,56 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
    Returns TRUE on success, FALSE on failure. */
 int Scm_AttachVM(ScmVM *vm)
 {
-#ifdef GAUCHE_USE_PTHREADS
-    if (SCM_INTERNAL_THREAD_INITIALIZED_P(vm->thread)) return FALSE;
-    if (theVM != NULL) return FALSE;
+#ifdef GAUCHE_HAS_THREADS
+    if (theVM != NULL) {
+        /* The current thread already has another VM attached. */
+        return FALSE;
+    }
+    /* NB: We want to check if this VM has already attached to another
+       thread or not. */
 
-    if (pthread_setspecific(Scm_VMKey(), vm) != 0) return FALSE;
+    if (!SCM_INTERNAL_THREAD_SETSPECIFIC(Scm_VMKey(), vm)) return FALSE;
 
-    vm->thread = pthread_self();
+    if (!SCM_INTERNAL_THREAD_INITIALIZED_P(vm->thread)) {
+#ifdef GAUCHE_USE_WTHREADS
+        /* GetCurrentThread() on Windows returns a pseudo handle
+           indicating 'myself', which can't be usable from other thread.
+           We need a special care.  The resulting HANDLE should be closed,
+           which is done in the finalizer. */
+        HANDLE t;
+        BOOL r = DuplicateHandle(GetCurrentProcess(), /* source process */
+                                 GetCurrentThread(),  /* source handle */
+                                 GetCurrentProcess(), /* target process */
+                                 &t,
+                                 0, FALSE, DUPLICATE_SAME_ACCESS);
+        if (!r) {
+            Scm_SysError("Getting current thread handle failed");
+        }
+        vm->thread = t;
+#else  /* GAUCHE_USE_PTHREADS */
+        vm->thread = SCM_INTERNAL_THREAD_GETCURRENT();
+#endif /* GAUCHE_USE_PTHREADS */
+    }
     vm->state = SCM_VM_RUNNABLE;
+    vm_register(vm);
     return TRUE;
-#else  /*!GAUCHE_USE_PTHREADS*/
+#else  /* no threads */
     return FALSE;
-#endif /*!GAUCHE_USE_PTHREADS*/
+#endif /* no threads */
+}
+
+/* If the current VM is attached by Scm_AttachVM() rather than Scheme
+   creating a thread, this needs to be called once you've done with the
+   VM (typically just before the thread terminates).
+ */
+void Scm_DetachVM(ScmVM *vm)
+{
+#ifdef GAUCHE_HAS_THREADS
+    if (vm != NULL) {
+        SCM_INTERNAL_THREAD_SETSPECIFIC(Scm_VMKey(), NULL);
+        vm_unregister(vm);
+    }
+#endif /* GAUCHE_HAS_THREADS */
 }
 
 int Scm_VMGetNumResults(ScmVM *vm)
@@ -285,12 +343,17 @@ ScmVM *Scm_VM(void)
 /*
  * Get VM key
  */
-#ifdef GAUCHE_USE_PTHREADS
+#if   defined(GAUCHE_USE_PTHREADS)
 pthread_key_t Scm_VMKey(void)
 {
     return vm_key;
 }
-#endif /*GAUCHE_USE_PTHREADS*/
+#elif defined(GAUCHE_USE_WTHREADS)
+DWORD Scm_VMKey(void)
+{
+    return vm_key;
+}
+#endif /*GAUCHE_USE_WTHREADS*/
 
 /* Warn if VM is terminated by uncaught exception, and GC-ed without
    joining.  It is cleary an unexpected case and worth reporting. */
@@ -303,6 +366,30 @@ static void vm_finalize(ScmObj obj, void *data)
         Scm_Warn("A thread %S died a lonely death with uncaught exception %S.",
                  vm->name, SCM_THREAD_EXCEPTION(re)->data);
     }
+#ifdef GAUCHE_USE_WTHREADS
+    if (vm->thread != INVALID_HANDLE_VALUE) {
+        CloseHandle(vm->thread);
+        vm->thread = INVALID_HANDLE_VALUE;
+    }
+#endif /*GAUCHE_USE_WTHREADS*/
+}
+
+/* Thread specific storage may not be scanned by GC.  We keep pointer
+   to the live VM in the global hashtable. */
+static void vm_register(ScmVM *vm)
+{
+    ScmDictEntry *e;
+    SCM_INTERNAL_MUTEX_LOCK(vm_table_mutex);
+    e = Scm_HashCoreSearch(&vm_table, (intptr_t)vm, SCM_DICT_CREATE);
+    SCM_DICT_SET_VALUE(e, SCM_TRUE);
+    SCM_INTERNAL_MUTEX_UNLOCK(vm_table_mutex);
+}
+
+static void vm_unregister(ScmVM *vm)
+{
+    SCM_INTERNAL_MUTEX_LOCK(vm_table_mutex);
+    (void)Scm_HashCoreSearch(&vm_table, (intptr_t)vm, SCM_DICT_DELETE);
+    SCM_INTERNAL_MUTEX_UNLOCK(vm_table_mutex);
 }
 
 /*====================================================================
@@ -334,7 +421,7 @@ static void vm_finalize(ScmObj obj, void *data)
 #define CHECK_STACK_PARANOIA(n)  /*empty*/
 #endif
 
-/* Hint for gcc -- at this moment, using __builtin_expect doesn't 
+/* Hint for gcc -- at this moment, using __builtin_expect doesn't
    do any good.  I'll try this later on. */
 #if 0
 #define MOSTLY_FALSE(expr)  __builtin_expect(expr, 0)
@@ -344,10 +431,7 @@ static void vm_finalize(ScmObj obj, void *data)
 
 /* Find the stack bottom next to the continuation frame.
    This macro should be applied only if CONT is in stack. */
-#define CONT_FRAME_END(cont)                                            \
-    ((cont)->argp?                                                      \
-     ((ScmObj*)(cont) + CONT_FRAME_SIZE) :          /*Scheme continuation*/ \
-     ((ScmObj*)(cont) + CONT_FRAME_SIZE + (cont)->size)) /*C continuation*/
+#define CONT_FRAME_END(cont)  ((ScmObj*)(cont) + CONT_FRAME_SIZE)
 
 /* check if *pc is an return instruction.  if so, some
    shortcuts are taken. */
@@ -398,7 +482,6 @@ static void vm_finalize(ScmObj obj, void *data)
         ScmContFrame *newcont = (ScmContFrame*)SP;      \
         newcont->prev = CONT;                           \
         newcont->env = ENV;                             \
-        newcont->argp = ARGP;                           \
         newcont->size = (int)(SP - ARGP);               \
         newcont->pc = next_pc;                          \
         newcont->base = BASE;                           \
@@ -407,34 +490,33 @@ static void vm_finalize(ScmObj obj, void *data)
         ARGP = SP;                                      \
     } while (0)
 
-#define CALL_CCONT(p, v, d) p(v, d)
-
 /* pop a continuation frame, i.e. return from a procedure. */
 #define POP_CONT()                                                      \
     do {                                                                \
-        if (CONT->argp == NULL) {                                       \
+        if (C_CONTINUATION_P(CONT)) {                                   \
             void *data__[SCM_CCONT_DATA_SIZE];                          \
             ScmObj v__ = VAL0;                                          \
             ScmCContinuationProc *after__;                              \
             void **d__ = data__;                                        \
-            void **s__ = (void**)((ScmObj*)CONT + CONT_FRAME_SIZE);     \
-            int i__ = CONT->size;                                       \
-            while (i__-- > 0) {                                         \
+            void **s__ = (void**)CONT - CONT->size;                     \
+            while (s__ < (void**)CONT) {                                \
                 *d__++ = *s__++;                                        \
             }                                                           \
             after__ = (ScmCContinuationProc*)CONT->pc;                  \
-            if (IN_STACK_P((ScmObj*)CONT)) SP = (ScmObj*)CONT;          \
-            ENV = CONT->env;                                            \
+            if (IN_STACK_P((ScmObj*)CONT)) {                            \
+                SP = (ScmObj*)CONT - CONT->size;                        \
+            }                                                           \
+            ENV = NULL;                                                 \
             ARGP = SP;                                                  \
             PC = PC_TO_RETURN;                                          \
             BASE = CONT->base;                                          \
             CONT = CONT->prev;                                          \
             SCM_FLONUM_ENSURE_MEM(v__);                                 \
-            VAL0 = CALL_CCONT(after__, v__, data__);                    \
+            VAL0 = after__(v__, data__);                                \
         } else if (IN_STACK_P((ScmObj*)CONT)) {                         \
-            SP   = CONT->argp + CONT->size;                             \
+            SP   = (ScmObj*)CONT;                                       \
             ENV  = CONT->env;                                           \
-            ARGP = CONT->argp;                                          \
+            ARGP = SP - CONT->size;                                     \
             PC   = CONT->pc;                                            \
             BASE = CONT->base;                                          \
             CONT = CONT->prev;                                          \
@@ -444,12 +526,13 @@ static void vm_finalize(ScmObj obj, void *data)
             ENV = CONT->env;                                            \
             PC = CONT->pc;                                              \
             BASE = CONT->base;                                          \
-            if (CONT->argp && size__) {                                 \
-                ScmObj *s__ = CONT->argp, *d__ = SP;                    \
-                SP += size__;                                           \
-                while (size__-- > 0) {                                  \
+            if (size__) {                                               \
+                ScmObj *s__ = (ScmObj*)CONT - size__;                   \
+                ScmObj *d__ = SP;                                       \
+                while (s__ < (ScmObj*)CONT) {                           \
                     *d__++ = *s__++;                                    \
                 }                                                       \
+                SP = d__;                                               \
             }                                                           \
             CONT = CONT->prev;                                          \
         }                                                               \
@@ -552,7 +635,7 @@ static void vm_finalize(ScmObj obj, void *data)
 
 /* Discard the current procedure's local frame before performing a tail call.
    Just before the tail call, the typical stack position is like this:
-   
+
    SP  >|      |
         | argN |
         |   :  |
@@ -690,7 +773,7 @@ static void run_loop()
 {
     ScmVM *vm = theVM;
     ScmWord code = 0;
-    
+
 #ifdef __GNUC__
     static void *dispatch_table[256] = {
 #define DEFINSN(insn, name, nargs, type)   && SCM_CPP_CAT(LABEL_, insn),
@@ -772,12 +855,12 @@ static void run_loop()
    the order of 10^6 times.   I'm not sure I can optimize this routine
    further without a radical change in stack management code.
 
-   Better strategy is to put an effort in the compiler to avoid closure 
+   Better strategy is to put an effort in the compiler to avoid closure
    creation as much as possible.  */
 
 /* Move the chain of env frames from the stack to the heap,
    replacing the in-stack frames for forwarding env frames.
-   
+
    This routine just moves the env frames, but leaves pointers that
    point to moved frames intact (such pointers are found only in
    the in-stack contniuation frames, chained from vm->cont).
@@ -838,7 +921,8 @@ static void save_cont(ScmVM *vm)
     /* First pass */
     do {
         int size = (CONT_FRAME_SIZE + c->size) * sizeof(ScmObj);
-        ScmContFrame *csave = SCM_NEW2(ScmContFrame*, size);
+        ScmObj *heap = SCM_NEW2(ScmObj*, size);
+        ScmContFrame *csave = (ScmContFrame*)(heap + c->size);
 
         /* update env ptr if necessary */
         if (FORWARDED_ENV_P(c->env)) {
@@ -848,22 +932,20 @@ static void save_cont(ScmVM *vm)
         }
 
         /* copy cont frame */
-        if (c->argp) {
-            *csave = *c; /* copy the frame */
+        if (!C_CONTINUATION_P(c)) {
+            s = (ScmObj*)c - c->size;
+            d = heap;
             if (c->size) {
-                /* copy the args */
-                s = c->argp;
-                d = (ScmObj*)csave + CONT_FRAME_SIZE;
                 for (i=c->size; i>0; i--) {
                     SCM_FLONUM_ENSURE_MEM(*s);
                     *d++ = *s++;
                 }
             }
-            csave->argp = ((ScmObj*)csave + CONT_FRAME_SIZE);
+            *(ScmContFrame*)d = *c; /* copy the frame */
         } else {
             /* C continuation */
-            s = (ScmObj*)c;
-            d = (ScmObj*)csave;
+            s = (ScmObj*)c - c->size;
+            d = heap;
             for (i=CONT_FRAME_SIZE + c->size; i>0; i--) {
                 /* NB: C continuation frame contains opaque pointer,
                    so we shouldn't ENSURE_MEM. */
@@ -874,13 +956,13 @@ static void save_cont(ScmVM *vm)
         /* make the orig frame forwarded */
         if (prev) prev->prev = csave;
         prev = csave;
-        
+
         tmp = c->prev;
         c->prev = csave;
         c->size = -1;
         c = tmp;
     } while (IN_STACK_P((ScmObj*)c));
-    
+
     /* Second pass */
     if (FORWARDED_CONT_P(vm->cont)) {
         vm->cont = FORWARDED_CONT(vm->cont);
@@ -936,7 +1018,7 @@ static ScmEnvFrame *get_env(ScmVM *vm)
 {
     ScmEnvFrame *e;
     ScmContFrame *c;
-    
+
     e = save_env(vm, vm->env);
     if (e != vm->env) {
         vm->env = e;
@@ -949,6 +1031,51 @@ static ScmEnvFrame *get_env(ScmVM *vm)
     return e;
 }
 
+/* When VM stack has incomplete stack frame (that is, SP != ARGP or
+ * *PC != SCM_VM_RET), and we need to run something on VM, we should
+ * preserve this incomplete frame.  Pushing an extra continuation
+ * frame does the job.  We set the PC to point to RET instruction,
+ * so the next time the control returns to the calling VM loop,
+ * the first thing it would do is to pop this extra continuation
+ * frame (unless other thigs are pushed onto the VM stack by VMPushCC).
+ * Returns TRUE if a new extra frame is pushed, FALSE if not.
+ *
+ * Caveat: This function changes the next instruction to be executed.
+ * It is a problem if this is called during VM insturctions such as
+ * CAR-PUSH, which expects the next instruction to make use of the
+ * pushed value---if we make RET to be executed instead of the original
+ * instruction, the pushed value will be lost.  So, the caller of this
+ * function needs to see if the frame is actually pushed, and call
+ * Scm__VMUnprotectStack below to recover the original instruction.
+ */
+int Scm__VMProtectStack(ScmVM *vm)
+{
+    if (vm->sp != vm->argp || *vm->pc != SCM_VM_INSN(SCM_VM_RET)) {
+        CHECK_STACK(CONT_FRAME_SIZE);
+        PUSH_CONT(PC);
+        vm->pc = PC_TO_RETURN;
+        return TRUE;
+    } else {
+        return FALSE;
+    }
+}
+
+/* The inverse of Scm__VMProtectStack.  This is required if you called
+ * Scm__VMProtectStack _in_the_middle_of_VM_instruction_execution_.
+ * The VM instruciton may push things after that, counting on the fact
+ * that subsequent instructoins use the pushed item.  However,
+ * Scm__VMProtectStack makes the next instruction to be executed
+ * to RET.  This makes the thing pushed by the current instruction be
+ * discarded immediately, before the original subsequent instructions
+ * seeing it.  Calling Scm__VMUnprotectStack restores the original next
+ * instruction (assuming it's properly paired up with Scm__VMProtectStack).
+ */
+void Scm__VMUnprotectStack(ScmVM *vm)
+{
+    SCM_ASSERT(vm->pc == PC_TO_RETURN);
+    POP_CONT();
+}
+
 #if GAUCHE_FFX
 /* Move all the FLONUM_REGs to heap and clear the fpstack.
    We cache small number of visited env frames to avoid duplicate scanning
@@ -1000,7 +1127,7 @@ void Scm_VMFlushFPStack(ScmVM *vm)
         if (visited_index < ENV_CACHE_SIZE) {
             visited[visited_index++] = e;
         }
-        
+
         for (i = 0; i < e->size; i++) {
             p = &ENV_DATA(e, i);
             SCM_FLONUM_ENSURE_MEM(*p);
@@ -1027,8 +1154,8 @@ void Scm_VMFlushFPStack(ScmVM *vm)
           next2:
             e = e->up;
         }
-        if (IN_STACK_P(c->argp) && c->size > 0) {
-            p = c->argp;
+        if (IN_STACK_P((ScmObj*)c) && c->size > 0) {
+            p = (ScmObj*)c - c->size;
             for (i=0; i<c->size; i++, p++) SCM_FLONUM_ENSURE_MEM(*p);
         }
         c = c->prev;
@@ -1191,7 +1318,7 @@ ScmObj Scm_VMEval(ScmObj expr, ScmObj e)
     if (restore_module) {
         /* if we swap the module, we need to make sure it is recovered
            after eval */
-        ScmObj body = Scm_MakeClosure(v, get_env(vm));
+        ScmObj body = Scm_MakeClosure(v, NULL);
         ScmObj before = Scm_MakeSubr(eval_restore_env, SCM_MODULE(e),
                                      0, 0, SCM_SYM_EVAL_BEFORE);
         ScmObj after = Scm_MakeSubr(eval_restore_env, (void*)vm->module,
@@ -1220,17 +1347,16 @@ void Scm_VMPushCC(ScmCContinuationProc *after,
 
     CHECK_STACK(CONT_FRAME_SIZE+datasize);
     s = SP;
-    cc = (ScmContFrame*)s;
-    s += CONT_FRAME_SIZE;
-    cc->prev = CONT;
-    cc->argp = NULL;
-    cc->size = datasize;
-    cc->pc = (ScmWord*)after;
-    cc->base = BASE;
-    cc->env = ENV;
     for (i=0; i<datasize; i++) {
         *s++ = SCM_OBJ(data[i]);
     }
+    cc = (ScmContFrame*)s;
+    s += CONT_FRAME_SIZE;
+    cc->prev = CONT;
+    cc->size = datasize;
+    cc->pc = (ScmWord*)after;
+    cc->base = BASE;
+    cc->env = &ccEnvMark;
     CONT = cc;
     ARGP = SP = s;
 }
@@ -1283,8 +1409,7 @@ static ScmObj user_eval_inner(ScmObj program, ScmWord *codevec)
     cstack.prev = vm->cstack;
     cstack.cont = vm->cont;
     vm->cstack = &cstack;
-    Scm_GetSigmask(&cstack.mask);
-    
+
   restart:
     vm->escapeReason = SCM_VM_ESCAPE_NONE;
     if (sigsetjmp(cstack.jbuf, FALSE) == 0) {
@@ -1305,7 +1430,6 @@ static ScmObj user_eval_inner(ScmObj program, ScmWord *codevec)
             Scm_Error("attempt to return from a ghost continuation.");
         }
     } else {
-        Scm_SetSigmask(&cstack.mask);
         /* An escape situation happened. */
         if (vm->escapeReason == SCM_VM_ESCAPE_CONT) {
             ScmEscapePoint *ep = (ScmEscapePoint*)vm->escapeData[0];
@@ -1393,7 +1517,7 @@ ScmObj Scm_ApplyRec(ScmObj proc, ScmObj args)
     ScmVM *vm = theVM;
 
     if (nargs < 0) {
-        Scm_Error("improper list not allowed: %S", args);        
+        Scm_Error("improper list not allowed: %S", args);
     }
 
     for (i=0; i<nargs; i++) {
@@ -1508,7 +1632,7 @@ static ScmObj safe_eval_thunk(ScmObj *args, int nargs, void *data)
 static ScmObj safe_eval_int(ScmObj *args, int nargs, void *data)
 {
     ScmObj thunk, handler;
-    
+
     thunk   = Scm_MakeSubr(safe_eval_thunk, data, 0, 0, SCM_FALSE);
     handler = Scm_MakeSubr(safe_eval_handler, data, 1, 0, SCM_FALSE);
     return Scm_VMWithErrorHandler(handler, thunk);
@@ -1570,6 +1694,52 @@ int Scm_Apply(ScmObj proc, ScmObj args, ScmEvalPacket *packet)
     return safe_eval_wrap(SAFE_APPLY, proc, args, NULL, SCM_FALSE, packet);
 }
 
+#if GAUCHE_LAZY_PAIR
+/*
+ * A subroutine to be called while executing apply instruction.
+ * Apply needs to check the argument tail is a valid list.  However,
+ * naively using ordinary procedures can trigger forcing of lazy
+ * pair, which breaks VM state.
+ *
+ * When called, we know vm->sp[-1] contains the tail list of arguments.
+ * It may be a lazy pair.  If so, we should force it in the safe environment.
+ * Returns the length of the tail list.
+ *
+ * Currently, we force the lazy argument tail in very inefficient way,
+ * assuming that such case is very rare.
+ */
+
+int check_arglist_tail_for_apply(ScmVM *vm, ScmObj z)
+{
+    int count = 0;
+    static ScmObj length_proc = SCM_UNDEFINED;
+    for (;;) {
+        if (SCM_NULLP(z)) {
+            return count;
+        } else if (SCM_LAZY_PAIR_P(z)) {
+            ScmEvalPacket result;
+            SCM_BIND_PROC(length_proc, "length", Scm_GaucheModule());
+            int nres = Scm_Apply(length_proc, SCM_LIST1(z), &result);
+            if (nres == -1) Scm_Raise(result.exception);
+            SCM_ASSERT(nres == 1);
+            SCM_ASSERT(SCM_INTP(result.results[0]));
+            count += SCM_INT_VALUE(result.results[0]);
+            return count;
+        } else if (!SCM_PAIRP(z)) {
+            return -1;
+        } else {
+            z = SCM_CDR(z);
+            count++;
+        }
+    }
+}
+#else  /* !GAUCHE_LAZY_PAIR */
+int check_arglist_tail_for_apply(ScmVM *vm, ScmObj z)
+{
+    return Scm_Length(z);
+}
+#endif /* !GAUCHE_LAZY_PAIR */
+
 
 /*=================================================================
  * Dynamic handlers
@@ -1634,7 +1804,7 @@ static ScmObj dynwind_after_cc(ScmObj result, void **data)
     ScmObj val0 = SCM_OBJ(data[0]);
     int nvals = (int)(intptr_t)data[1];
     ScmVM *vm = theVM;
-    
+
     vm->numVals = nvals;
     if (nvals > 1) {
         SCM_ASSERT(nvals <= SCM_VM_MAX_VALUES);
@@ -1656,7 +1826,7 @@ ScmObj Scm_VMDynamicWindC(ScmSubrProc *before,
         after ? Scm_MakeSubr(after, data, 0, 0, SCM_FALSE) : Scm_NullProc();
     bodyproc =
         body ? Scm_MakeSubr(body, data, 0, 0, SCM_FALSE) : Scm_NullProc();
-    
+
     return Scm_VMDynamicWind(beforeproc, bodyproc, afterproc);
 }
 
@@ -1816,7 +1986,7 @@ void Scm_VMDefaultExceptionHandler(ScmObj e)
             SCM_NEXT_HANDLER;
         }
         SCM_END_PROTECT;
-        
+
         /* Install the continuation */
         for (i=0; i<numVals; i++) vm->vals[i] = rvals[i];
         vm->numVals = numVals;
@@ -1868,7 +2038,7 @@ static SCM_DEFINE_SUBR(default_exception_handler_rec, 1, 0,
  * Entry point of throwing exception.
  *
  *  This function can be called from Scheme function raise,
- *  or C-function Scm_Error families and signal handler. 
+ *  or C-function Scm_Error families and signal handler.
  *  So there may be a raw C code in the continuation of this C call.
  *  Thus we can't use Scm_VMApply to call the user-defined exception
  *  handler.
@@ -1952,7 +2122,7 @@ static ScmObj with_error_handler(ScmVM *vm, ScmObj handler,
     ep->errorReporting =
         SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_ERROR_BEING_REPORTED);
     ep->rewindBefore = rewindBefore;
-    
+
     vm->escapePoint = ep; /* This will be done in install_ehandler, but
                              make sure ep is visible from save_cont
                              to redirect ep->cont */
@@ -1972,7 +2142,7 @@ ScmObj Scm_VMWithGuardHandler(ScmObj handler, ScmObj thunk)
 }
 
 
-/* 
+/*
  * with-exception-handler
  *
  *   This primitive gives the programmer whole responsibility of
@@ -2046,7 +2216,7 @@ static ScmObj throw_cont_body(ScmObj handlers,    /* after/before thunks
         SCM_ASSERT(SCM_PAIRP(SCM_CAR(handlers)));
         handler = SCM_CAAR(handlers);
         chain   = SCM_CDAR(handlers);
-        
+
         data[0] = (void*)SCM_CDR(handlers);
         data[1] = (void*)ep;
         data[2] = (void*)args;
@@ -2064,7 +2234,7 @@ static ScmObj throw_cont_body(ScmObj handlers,    /* after/before thunks
      * won't be overwritten by execution of the partial continuation.
      */
     if (ep->cstack == NULL) save_cont(vm);
-    
+
     /*
      * now, install the target continuation
      */
@@ -2156,13 +2326,13 @@ ScmObj Scm_VMCallPC(ScmObj proc)
     ScmEscapePoint *ep;
     ScmContFrame *c, *cp;
     ScmVM *vm = theVM;
-    
+
     /* save the continuation.  we only need to save the portion above the
        latest boundary frame (+environmentns pointed from them), but for now,
        we save everything to make things easier.  If we want to squeeze
        performance we'll optimize it later. */
     save_cont(vm);
-    
+
     /* find the latest boundary frame */
     for (c = vm->cont, cp = NULL;
          c && !BOUNDARY_FRAME_P(c);
@@ -2224,7 +2394,7 @@ ScmObj Scm_VMValues(ScmVM *vm, ScmObj args)
 {
     ScmObj cp;
     int nvals;
-    
+
     if (!SCM_PAIRP(args)) {
         vm->numVals = 0;
         return SCM_UNDEFINED;
@@ -2328,7 +2498,7 @@ static ScmObj process_queued_requests_cc(ScmObj result, void **data)
     int i;
     ScmObj cp;
     ScmVM *vm = theVM;
-    
+
     vm->numVals = (int)(intptr_t)data[0];
     vm->val0 = data[1];
     if (vm->numVals > 1) {
@@ -2380,16 +2550,27 @@ static void process_queued_requests(ScmVM *vm)
         SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(vm->vmlock);
         /* Double check, since stopRequest can be canceled between the above
            two lines. */
-        if (vm->stopRequest) {
-            vm->stopRequest = FALSE;
+        switch (vm->stopRequest) {
+        case SCM_VM_REQUEST_SUSPEND:
+            vm->stopRequest = 0;
             vm->state = SCM_VM_STOPPED;
             (void)SCM_INTERNAL_COND_BROADCAST(vm->cond);
             while (vm->state == SCM_VM_STOPPED) {
                 /* Here the inspector thread examines VM state */
                 (void)SCM_INTERNAL_COND_WAIT(vm->cond, vm->vmlock);
             }
+            break;
+        case SCM_VM_REQUEST_TERMINATE:
+            vm->state = SCM_VM_TERMINATED;
+            break;
+        default:
+            Scm_Panic("Unkown value in vm->stopRequest (%d).  Aborting.",
+                      vm->stopRequest);
         }
         SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
+        if (vm->state == SCM_VM_TERMINATED) {
+            SCM_INTERNAL_THREAD_EXIT(); /* this'll notify vm->cond. */
+        }
     }
 }
 
@@ -2440,7 +2621,7 @@ static ScmObj env2vec(ScmEnvFrame *env, struct EnvTab *etab)
 {
     int i;
     ScmObj vec;
-    
+
     if (!env) return SCM_FALSE;
     for (i=0; i<etab->numEntries; i++) {
         if (etab->entries[i].env == env) {
@@ -2474,7 +2655,7 @@ ScmObj Scm_VMGetStack(ScmVM *vm)
         info = Scm_VMGetSourceInfo(vm->pc);
         SCM_APPEND1(stack, tail, Scm_Cons(info, env2vec(vm->env, &envTab)));
     }
-    
+
     for (; c; c = c->prev) {
         if (!SCM_PAIRP(c->info)) continue;
         info = Scm_VMGetSourceInfo(c->info);
@@ -2562,13 +2743,13 @@ void Scm_VMDump(ScmVM *vm)
         dump_env(env, out);
         env = env->up;
     }
-    
+
     Scm_Printf(out, "conts:\n");
     while (cont) {
         Scm_Printf(out, "   %p\n", cont);
         Scm_Printf(out, "              env = %p\n", cont->env);
-        Scm_Printf(out, "             argp = %p[%d]\n", cont->argp, cont->size);
-        if (cont->argp) {
+        Scm_Printf(out, "             size = %d\n", cont->size);
+        if (!C_CONTINUATION_P(cont)) {
             Scm_Printf(out, "               pc = %p ", cont->pc);
             Scm_Printf(out, "(%08x)\n", *cont->pc);
         } else {
@@ -2637,20 +2818,32 @@ void Scm__InitVM(void)
                                 0, 0);
 #endif /*USE_CUSTOM_STACK_MARKER*/
 
+    Scm_HashCoreInitSimple(&vm_table, SCM_HASH_EQ, 8, NULL);
+    SCM_INTERNAL_MUTEX_INIT(vm_table_mutex);
+
     /* Create root VM */
+    rootVM = Scm_NewVM(NULL, SCM_MAKE_STR_IMMUTABLE("root"));
+    rootVM->state = SCM_VM_RUNNABLE;
 #ifdef GAUCHE_USE_PTHREADS
     if (pthread_key_create(&vm_key, NULL) != 0) {
         Scm_Panic("pthread_key_create failed.");
     }
-    rootVM = Scm_NewVM(NULL, SCM_MAKE_STR_IMMUTABLE("root"));
     if (pthread_setspecific(vm_key, rootVM) != 0) {
         Scm_Panic("pthread_setspecific failed.");
     }
     rootVM->thread = pthread_self();
-#else   /* !GAUCHE_USE_PTHREADS */
-    rootVM = theVM = Scm_NewVM(NULL, SCM_MAKE_STR_IMMUTABLE("root"));
-#endif  /* !GAUCHE_USE_PTHREADS */
-    rootVM->state = SCM_VM_RUNNABLE;
+#elif  GAUCHE_USE_WTHREADS
+    vm_key = TlsAlloc();
+    if (vm_key == TLS_OUT_OF_INDEXES) {
+        Scm_Panic("TlsAlloc failed");
+    }
+    if (!TlsSetValue(vm_key, (LPVOID)rootVM)) {
+        Scm_Panic("TlsSetValue failed");
+    }
+    rootVM->thread = GetCurrentThread();
+#else   /* no threads */
+    theVM = rootVM;
+#endif  /* no threads */
 
 #ifdef COUNT_INSN_FREQUENCY
     Scm_AddCleanupHandler(dump_insn_frequency, NULL);
