@@ -1,12 +1,12 @@
 ;;;
 ;;; vminsn.scm - Virtual machine instruction definition
 ;;;
-;;;   Copyright (c) 2000-2010  Shiro Kawai  <shiro@acm.org>
-;;; 
+;;;   Copyright (c) 2000-2012  Shiro Kawai  <shiro@acm.org>
+;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
 ;;;   are met:
-;;; 
+;;;
 ;;;   1. Redistributions of source code must retain the above copyright
 ;;;      notice, this list of conditions and the following disclaimer.
 ;;;
@@ -35,7 +35,7 @@
 ;;; gauche/vminsn.h and vminsn.c, which are then included in vm.c.
 ;;; This file is also used by the compiler.
 ;;;
-;;; 
+;;;
 ;;; (define-insn <name> <num-params> <operand-type> [<combination>] [<body>])
 ;;;
 ;;;   <name> - instruction name.  In C, an enum SCM_VM_<name> is defined.
@@ -82,9 +82,9 @@
 ;                                      (RETURN-OP))
 ;                                    NEXT1)]
                        ))])
-  
+
 ;; variations of $result with type coercion
-(define-cise-stmt $result:b 
+(define-cise-stmt $result:b
   [(_ expr) `($result (SCM_MAKE_BOOL ,expr))])
 (define-cise-stmt $result:i
   [(_ expr) (let1 r (gensym "cise__")
@@ -107,6 +107,15 @@
               `(let* ([,r :: double ,expr])
                  ($result (Scm_VMReturnFlonum ,r))))])
 
+;; Extract local value.  If it is boxed, we need dereference.
+(define-cise-stmt $lref!
+  [(_ var env off)
+   (let1 v (gensym)
+     `(let* ([,v (ENV-DATA ,env ,off)])
+        (if (SCM_BOXP ,v)
+          (set! ,var (SCM_BOX_VALUE ,v))
+          (set! ,var ,v))))])
+
 ;;
 ;; ($w/argr <val> <expr> ...)
 ;; ($w/argp <val> <expr> ...)
@@ -131,15 +140,16 @@
                        [(3) (set! ,e (-> ,e up))]
                        [(2) (set! ,e (-> ,e up))]
                        [(1) (set! ,e (-> ,e up))]
-                       [(0) (set! ,val (ENV-DATA ,e ,off)) (break)]
+                       [(0) ($lref! ,val ,e ,off) (break)]
                        [else (while (> (post-- ,dep) 0) (set! ,e (-> ,e up)))
-                             (set! ,val (ENV-DATA ,e ,off)) (break)])
+                             ($lref! ,val ,e ,off) (break)])
                      ,@body))]
      [('lref d o) `(let* ((,val (ENV-DATA ,(let loop ((d d))
                                              (if (zero? d)
                                                'ENV
                                                `(-> ,(loop (- d 1)) up)))
                                           ,o)))
+                     (if (SCM_BOXP ,val) (set! ,val (SCM_BOX_VALUE ,val)))
                      ,@body)])])
 
 (define-cise-stmt $w/argr               ; use VAL0 default
@@ -272,11 +282,16 @@
 ;;
 (define-cise-stmt $lrefNN
   [(_ depth offset)
-   `($result (ENV_DATA ,(let loop ((d depth))
-                          (case d
-                            [(0) 'ENV]
-                            [else `(-> ,(loop (- d 1)) up)]))
-                       ,offset))])
+   ;; TODO 0.9.2: Extra check for backward compatibility
+   (let1 v (gensym)
+     `(let* ([,v :: ScmObj (ENV_DATA ,(let loop ((d depth))
+                                        (case d
+                                          [(0) 'ENV]
+                                          [else `(-> ,(loop (- d 1)) up)]))
+                                     ,offset)])
+        (if (SCM_BOXP ,v)
+          ($result (SCM_BOX_VALUE ,v))
+          ($result ,v))))])
 
 ;;
 ;; ($values)
@@ -348,7 +363,7 @@
     (PUSH-CONT next)
     INCR-PC
     NEXT))
-         
+
 ;; combined insn
 (define-insn PUSH-PRE-CALL 1 addr (PUSH PRE-CALL))
 
@@ -374,14 +389,14 @@
     ($include "./vmcall.c")))
 
 ;; TAIL-CALL(NARGS)
-;;  Call procedure in val0.  Same as CALL except this discards the 
+;;  Call procedure in val0.  Same as CALL except this discards the
 ;;  caller's arugment frame and shift the callee's argument frame.
 ;;
 (define-insn TAIL-CALL 1 none #f
   (begin (DISCARD-ENV) ($goto-insn CALL)))
 
 ;; JUMP <addr>
-;;  Jump to <addr>.  
+;;  Jump to <addr>.
 ;;
 (define-insn JUMP      0 addr #f
   (begin (FETCH-LOCATION PC) NEXT))
@@ -440,14 +455,16 @@
          NEXT))
 
 ;; LOCAL-ENV-CLOSURES(nlocals) <codelist>
-;;  Used for letrec.
+;;  A special instruction for efficient handling of letrec.
 ;;  Similar to LOCAL-ENV, but this doesn't use the current arg frame.
-;;  The operand contains a literal list of compiled codes.  This instruction
-;;  creates a frame of NLOCALS size, then creates closures
-;;  with the new environment and the given compiled codes, and fills the
-;;  new frame with the created closures.   CODELIST can have 'holes', i.e.
-;;  if it has #<undef> instead of a compiled code, the corresponding frame entry
-;;  is left undefined.
+;;  The operand contains a literal list of <compiled-code>s or <closure>s.
+;;  This instruction creates a frame of NLOCALS size and initializes each
+;;  slot with each element of <codelist>.   If the element is a
+;;  <compiled-code>, a new closure is created with the environment
+;;  including the frame just created.   If the element is a <closure>,
+;;  it is just used (such <closure> does not refer to the outside environment).
+;;  CODELIST can have 'holes', i.e. if it has #<undef>,
+;;  the corresponding frame entry is left undefined.
 ;;  This instruction also leaves the last closure in VAL0.
 (define-insn LOCAL-ENV-CLOSURES 1 codes #f
   (let* ([nlocals::int (SCM_VM_INSN_ARG code)]
@@ -460,9 +477,10 @@
     (set! e (get_env vm))
     (set! z (- (cast ScmObj* e) nlocals))
     (dolist [c cp]
-      (if (SCM_COMPILED_CODE_P c)
-        (set! (* (post++ z)) (set! clo (Scm_MakeClosure c e)))
-        (set! (* (post++ z)) c)))
+      (cond [(SCM_COMPILED_CODE_P c)
+             (set! (* (post++ z)) (set! clo (Scm_MakeClosure c e)))]
+            [(SCM_PROCEDUREP c) (set! (* (post++ z)) c) (set! clo c)]
+            [else (set! (* (post++ z)) c)]))
     ($result clo)))
 
 ;; POP-LOCAL-ENV
@@ -541,7 +559,7 @@
     (CHECK-STACK (-> vm base maxstack))
     (SCM_PROF_COUNT_CALL vm (SCM_OBJ (-> vm base)))
     NEXT))
-         
+
 (define-insn LOCAL-ENV-TAIL-CALL 1 none #f
   (let* ([nargs::int (cast int (- SP ARGP))] [to::ScmObj*])
     (VM-ASSERT (SCM_CLOSUREP VAL0))
@@ -585,8 +603,8 @@
 ;; Compare LREF(n,m) and VAL0 and branch.  This is not a simple combination
 ;; of LREF + BNLT etc. (which would compare stack top and LREF).  These insns
 ;; save one stack operation.  The compiler recognizes the pattern and
-;; emits these.  See pass3/if-numcmp and pass3/if-numeq.
-(define-insn LREF-VAL0-BNUMNE 2 addr #f ($arg-source lref ($insn-body BNUMNE))) 
+;; emits these.  See pass5/if-numcmp and pass5/if-numeq.
+(define-insn LREF-VAL0-BNUMNE 2 addr #f ($arg-source lref ($insn-body BNUMNE)))
 (define-insn LREF-VAL0-BNLT 2 addr #f ($arg-source lref ($insn-body BNLT)))
 (define-insn LREF-VAL0-BNLE 2 addr #f ($arg-source lref ($insn-body BNLE)))
 (define-insn LREF-VAL0-BNGT 2 addr #f ($arg-source lref ($insn-body BNGT)))
@@ -678,7 +696,7 @@
 ;;  into the stack and makes them into an environment.
 ;;  It is used primarily for the occasions that compiler knows it needs
 ;;  to save the values temporarily (e.g. for evaluate 'after' thunk of
-;;  dynamic-wind, the results of its body needs to be saved).  
+;;  dynamic-wind, the results of its body needs to be saved).
 ;;  This must be twined with VALUES-N, which reverses the effects, i.e.
 ;;  turn the values in the env frame into values.
 (define-insn RECEIVE-ALL 0 addr #f
@@ -689,8 +707,8 @@
     (PUSH-CONT nextpc)
     ($goto-insn TAIL-RECEIVE-ALL)))
 
-;; TAIL-RECEIVE-ALL 
-;;  Tail version of RECEIVE-ALL.  
+;; TAIL-RECEIVE-ALL
+;;  Tail version of RECEIVE-ALL.
 (define-insn TAIL-RECEIVE-ALL 0 none #f
   (begin (CHECK-STACK-PARANOIA (ENV-SIZE (+ (-> vm numVals) 1)))
          (PUSH-ARG VAL0)
@@ -726,7 +744,11 @@
     (VM-ASSERT (!= e NULL))
     (VM-ASSERT (> (-> e size) off))
     (SCM_FLONUM_ENSURE_MEM VAL0)
-    (set! (ENV-DATA e off) VAL0)
+    ;; TODO 0.9.2 : Extra check to maintain compatibility
+    (let* ([box::ScmObj (ENV-DATA e off)])
+      (if (SCM_BOXP box)
+        (SCM_BOX_SET box VAL0)
+        (set! (ENV-DATA e off) VAL0)))
     NEXT1))
 
 ;; GSET <location>
@@ -777,7 +799,11 @@
          (set! e (-> e up)))
     (VM-ASSERT (!= e NULL))
     (VM-ASSERT (> (-> e size) off))
-    ($result (ENV-DATA e off))))
+    ;; TODO 0.9.2: Extra test to keep compatiblity
+    (let* ((v::ScmObj (ENV-DATA e off)))
+      (if (SCM_BOXP v)
+        ($result (SCM_BOX_VALUE v))
+        ($result v)))))
 
 ;; Shortcut for the frequent depth/offset.
 ;; From statistics, we found that the following depth/offset combinations
@@ -918,12 +944,12 @@
         (POP-ARG arg)
         (set! cp (Scm_Cons arg cp))))
     ($result cp)))
-          
+
 (define-insn LENGTH      0 none #f      ; length
   (let* ([len::int (Scm_Length VAL0)])
     (when (< len 0) ($vm-err "proper list required, but got %S" VAL0))
     ($result:i len)))
-    
+
 (define-insn MEMQ 0 none #f ($w/argp v ($result (Scm_Memq v VAL0))))
 (define-insn MEMV 0 none #f ($w/argp v ($result (Scm_Memv v VAL0))))
 (define-insn ASSQ 0 none #f ($w/argp v ($result (Scm_Assq v VAL0))))
@@ -932,14 +958,19 @@
 (define-insn EQV  0 none #f ($w/argp v ($result:b (Scm_EqvP v VAL0))))
 
 (define-insn APPEND      1 none #f
-  (let* ([nargs::int (SCM_VM_INSN_ARG code)] [cp SCM_NIL] [arg])
+  (let* ([nargs::int (SCM_VM_INSN_ARG code)] [cp SCM_NIL] [args SCM_NIL] [a])
     (when (> nargs 0)
       (SCM_FLONUM_ENSURE_MEM VAL0)
       (set! cp VAL0)
+      ;; We want to pop all args before doing works, for Scm_Length may cause
+      ;; forcing lazy-pair.
       (while (> (pre-- nargs) 0)
-        (POP-ARG arg)
-        (when (< (Scm_Length arg) 0) ($vm-err "list required, but got %S" arg))
-        (set! cp (Scm_Append2 arg cp))))
+        (POP-ARG a)
+        (set! args (Scm_Cons a args)))
+      ;; Now it' safe to work on args (note that we work from tail to head).
+      (dolist [a (Scm_ReverseX args)]
+        (when (< (Scm_Length a) 0) ($vm-err "list required, but got %S" a))
+        (set! cp (Scm_Append2 a cp))))
     ($result cp)))
 
 (define-insn NOT      0 none #f ($w/argr v ($result:b (SCM_FALSEP v))))
@@ -962,7 +993,7 @@
 (define-insn TAIL-APPLY  1 none #f
   ;; Inlined apply.  Assumes the call is at the tail position.
   ;; NB: As of 0.9, all 'apply' call is expanded into this instruction.
-  ;; If the code is not at the tail position, compiler pass 3 inserts
+  ;; If the code is not at the tail position, compiler pass5 inserts
   ;; PRE-CALL instruction so that the call of apply becomes a tail call.
   ;;
   ;; Here, the stack should have the following layout.
@@ -979,17 +1010,18 @@
   ;; TAIL-CALL processing.  (The unfolding of rest
   ;; argument will be done in ADJUST_ARGUMENT_FRAME later,
   ;; if necessary.)
-  ;; 
+  ;;
   ;;   SP  >|      |
   ;;        | rest |
   ;;        | argN |
   ;;        |   :  |
   ;;   ARGP>| arg0 |        VAL0=proc
   ;;
-  (let* ([rargc::int (Scm_Length VAL0)]
+  (let* ([rest VAL0]
+         [rargc::int (check_arglist_tail_for_apply vm rest)]
          [nargc::int (- (SCM_VM_INSN_ARG code) 2)]
          [proc (* (- SP nargc 1))])
-    (when (< rargc 0) ($vm-err "improper list not allowed: %S" VAL0))
+    (when (< rargc 0) ($vm-err "improper list not allowed: %S" rest))
     (while (> nargc 0)
       (set! (* (- SP nargc 1)) (* (- SP nargc)))
       (post-- nargc))
@@ -1000,7 +1032,7 @@
       (set! VAL0 proc)
       ($goto-insn TAIL-CALL))
     ;; normal path
-    (set! (* (- SP 1)) VAL0)
+    (set! (* (- SP 1)) rest)
     (set! VAL0 proc)
     (DISCARD-ENV)
     (goto tail_apply_entry)))
@@ -1011,9 +1043,7 @@
     (let* ([c::ScmClass* (SCM_CLASS VAL0)])
       ;; be careful to handle class redifinition case
       (cond [(not (SCM_FALSEP (-> (Scm_ClassOf obj) redefined)))
-             (CHECK-STACK CONT_FRAME_SIZE)
-             (PUSH_CONT PC)
-             (set! PC PC_TO_RETURN)
+             (Scm__VMProtectStack vm)
              ($result (Scm_VMIsA obj c))]
             [else ($result:b (SCM_ISA obj c))]))))
 
@@ -1046,21 +1076,22 @@
         (SCM_FLONUM_ENSURE_MEM arg)
         (set! (SCM_VECTOR_ELEMENT vec 0) arg)))
     ($result vec)))
-    
+
 (define-insn LIST2VEC    0 none #f      ; list->vector
   ($w/argr v ($result (Scm_ListToVector v 0 -1))))
 
 (define-insn APP-VEC     1 none #f      ; (compose list->vector append)
-  (let* ([nargs::int (SCM_VM_INSN_ARG code)] [cp SCM_NIL] [arg])
+  (let* ([nargs::int (SCM_VM_INSN_ARG code)] [cp SCM_NIL] [args SCM_NIL] [a])
     (when (> nargs 0)
       (SCM_FLONUM_ENSURE_MEM VAL0)
       (set! cp VAL0)
       (while (> (pre-- nargs) 0)
-        (POP-ARG arg)
-        (when (< (Scm_Length arg) 0)
-          ($vm-err "list required, but got %S" arg))
-        (set! cp (Scm_Append2 arg cp)))
-      ($result (Scm_ListToVector cp 0 -1)))))
+        (POP-ARG a)
+        (set! args (Scm_Cons a args)))
+      (dolist [a (Scm_ReverseX args)]
+        (when (< (Scm_Length a) 0) ($vm-err "list required, but got %S" a))
+        (set! cp (Scm_Append2 a cp))))
+    ($result (Scm_ListToVector cp 0 -1))))
 
 (define-insn VEC-LEN     0 none #f      ; vector-length
   ($w/argr v
@@ -1089,7 +1120,7 @@
       (SCM_FLONUM_ENSURE_MEM v)
       (set! (SCM_VECTOR_ELEMENT vec k) v)
       ($result SCM_UNDEFINED))))
-  
+
 ;; VEC-REF and VEC-SET with immediate index.  VAL0 must be a vector.
 (define-insn VEC-REFI    1 none #f
   ($w/argr vec
@@ -1098,7 +1129,7 @@
       (when (or (< k 0) (>= k (SCM_VECTOR_SIZE vec)))
         ($vm-err "vector-ref index out of range: %d" k))
       ($result (SCM_VECTOR_ELEMENT vec k)))))
-  
+
 (define-insn VEC-SETI    1 none #f
   ($w/argp vec
     ($type-check vec SCM_VECTORP "vector")
@@ -1141,7 +1172,7 @@
      [(and (SCM_FLONUMP VAL0) (SCM_FLONUMP arg))
       ($result:b (== (SCM_FLONUM_VALUE VAL0) (SCM_FLONUM_VALUE arg)))]
      [else ($result:b (Scm_NumEq arg VAL0))])))
-  
+
 (define-insn NUMLT2  0 none #f ($w/numcmp r <  ($result:b r)))
 (define-insn NUMLE2  0 none #f ($w/numcmp r <= ($result:b r)))
 (define-insn NUMGT2  0 none #f ($w/numcmp r >  ($result:b r)))
@@ -1174,7 +1205,7 @@
             (and (SCM_FLONUMP VAL0) (SCM_REALP arg)))
       ($result:f (* (Scm_GetDouble arg) (Scm_GetDouble VAL0)))
       ($result (Scm_Mul arg VAL0)))))
-  
+
 (define-insn NUMDIV2 0 none #f          ; / (binary)
   ($w/argp arg
     (if (or (and (SCM_FLONUMP arg) (SCM_REALP VAL0))
@@ -1183,7 +1214,7 @@
       ($result (Scm_Div arg VAL0)))))
 
 (define-insn LREF-VAL0-NUMADD2 2 none #f ($arg-source lref ($insn-body NUMADD2)))
-  
+
 (define-insn NEGATE  0 none #f          ; -  (unary)
   ($w/argr v
     (cond
@@ -1197,24 +1228,24 @@
       ($result:f (+ (Scm_GetDouble arg) (Scm_GetDouble VAL0)))
       ($result (Scm_Add (Scm_Inexact arg)
                         (Scm_Inexact VAL0))))))
-      
+
 (define-insn NUMISUB2    0 none #f      ; -. (binary)
   ($w/argp arg
     (if (and (SCM_REALP arg) (SCM_REALP VAL0))
       ($result:f (- (Scm_GetDouble arg) (Scm_GetDouble VAL0)))
       ($result (Scm_Sub (Scm_Inexact arg) (Scm_Inexact VAL0))))))
-  
+
 (define-insn NUMIMUL2    0 none #f      ; *.
   ($w/argp arg
     (if (and (SCM_REALP arg) (SCM_REALP VAL0))
       ($result:f (* (Scm_GetDouble arg) (Scm_GetDouble VAL0)))
       ($result (Scm_Mul (Scm_Inexact arg) (Scm_Inexact VAL0))))))
-  
+
 (define-insn NUMIDIV2    0 none #f      ; /. (binary)
   ($w/argp arg
-    (if (and (SCM_REALP arg) (SCM_REALP VAL0))
+    (if (and (SCM_FLONUMP arg) (SCM_FLONUMP VAL0))
       ($result:f (/ (Scm_GetDouble arg) (Scm_GetDouble VAL0)))
-      ($result (Scm_Div (Scm_Inexact arg) (Scm_Inexact VAL0))))))
+      ($result (Scm_VMDivInexact arg VAL0)))))
 
 (define-insn NUMADDI     1 none #f      ; +, if one of op is small int
   (let* ([imm::long (SCM_VM_INSN_ARG code)])
@@ -1234,7 +1265,7 @@
             [(SCM_FLONUMP arg)
              ($result:f (- (cast double imm) (SCM_FLONUM_VALUE arg)))]
             [else           ($result (Scm_Sub (SCM_MAKE_INT imm) arg))]))))
-  
+
 
 ;(define-insn NUMQUOT     0 none)        ; quotient
 ;(define-insn NUMMOD      0 none)        ; modulo
@@ -1257,7 +1288,7 @@
            (set! port SCM_CURIN)])
     (set! ch (Scm_Getc port))
     ($result (?: (< ch 0) SCM_EOF (SCM_MAKE_CHAR ch)))))
-  
+
 (define-insn PEEK-CHAR   1 none #f      ; peek-char
   (let* ([nargs::int (SCM_VM_INSN_ARG code)] [ch::int 0] [port::ScmPort*])
     (cond [(== nargs 1)
@@ -1301,7 +1332,7 @@
       (SCM_FLONUM_ENSURE_MEM slot)
       (SCM_FLONUM_ENSURE_MEM VAL0)
       ($result (Scm_VMSlotSet obj slot VAL0)))))
-  
+
 (define-insn SLOT-REFC   0 obj #f       ; slot-ref with constant slot name
   (let* ((slot))
     (FETCH-OPERAND slot)
@@ -1309,7 +1340,7 @@
     (TAIL-CALL-INSTRUCTION)
     (SCM_FLONUM_ENSURE_MEM VAL0)
     ($result (Scm_VMSlotRef VAL0 slot FALSE))))
-    
+
 (define-insn SLOT-SETC   0 obj #f       ; slot-set! with constant slot name
   (let* ((slot))
     (FETCH-OPERAND slot)
@@ -1335,7 +1366,7 @@
     (SCM_FLONUM_ENSURE_MEM after)
     (set! (-> vm handlers) (Scm_Acons before after (-> vm handlers)))
     NEXT))
-  
+
 (define-insn POP-HANDLERS 0 none #f     ; pop dynamic handlers
   (begin
     (VM-ASSERT (SCM_PAIRP (-> vm handlers)))
@@ -1348,3 +1379,12 @@
 ;;
 
 (define-insn-lref* LREF-RET 0 none (LREF RET))
+
+(define-insn BOX 0 none #f              ; replace VAL0 with box(VAL0)
+  (begin
+    (SCM_FLONUM_ENSURE_MEM VAL0)
+    (let* ((b::ScmBox* (Scm_MakeBox VAL0)))
+      (set! VAL0 (SCM_OBJ b)))
+    NEXT))
+
+
