@@ -2,7 +2,7 @@
 
 (use gauche.test)
 
-(test-start "EUC-JP")
+(test-start "multibyte (euc-jp)")
 (use srfi-1)
 
 ;;-------------------------------------------------------------------
@@ -54,20 +54,40 @@
 (test* "string-join" "դФФ"
        (string-join '("դ" "Ф" "Ф") "" 'strict-infix))
 
-(test* "string-scan" 7
-       (string-scan "" ""))
-(test* "string-scan" ""
-       (string-scan "" "" 'before))
-(test* "string-scan" ""
-       (string-scan "" "" 'after))
-(test* "string-scan" '("" "")
-       (receive r (string-scan "" "" 'before*) r))
-(test* "string-scan" '("" "")
-       (receive r (string-scan "" "" 'after*) r))
-(test* "string-scan" '("" "")
-       (receive r (string-scan "" "" 'both) r))
-(test* "string-scan" #f
-       (string-scan "" ""))
+(let ()
+  (define (test-string-scan out s1 s2 . opt)
+    (if (pair? out)
+      (begin
+        (test* "string-scan" (car out) (apply string-scan s1 s2 opt))
+        (test* "string-scan-right" (cadr out)
+               (apply string-scan-right s1 s2 opt)))
+      (apply test-string-scan (list out out) s1 s2 opt)))
+  (define (test-string-scan2 out1 out2 s1 s2 . opt)
+    (if (pair? out1)
+      (begin
+        (test* "string-scan" (list (car out1) (car out2))
+               (receive r (apply string-scan s1 s2 opt) r))
+        (test* "string-scan-right" (list (cadr out1) (cadr out2))
+               (receive r (apply string-scan-right s1 s2 opt) r)))
+      (apply test-string-scan2 (list out1 out1) (list out2 out2) s1 s2 opt)))
+
+  (test-string-scan '(7 20)
+                    "" "")
+  (test-string-scan '("" "")
+                      "" "" 'before)
+  (test-string-scan '("" "")
+                    "" "" 'after)
+  (test-string-scan2 '("" "")
+                     '("" "")
+                     "" "" 'before*)
+  (test-string-scan2 '("" "")
+                     '("" "")
+                     "" "" 'after*)
+  (test-string-scan2 '("" "")
+                     '("" "")
+                     "" "" 'both)
+  (test-string-scan #f "" "")
+  )
 
 ;;-------------------------------------------------------------------
 (test-section "string-pointer")
