@@ -3,13 +3,67 @@
 ;; this test is splitted from io.scm, since this one uses util.isomorph,
 ;; and has to be done after the test of util.* module.
 
-;; $Id: io2.scm,v 1.4 2006/10/08 03:59:42 shirok Exp $
-
 (use gauche.test)
 (use srfi-1)
 (use util.isomorph)
 
-(test-start "write/ss and read/ss")
+(test-start "advanced read/write features")
+
+;;===============================================================
+;; Hash-bang handling (#!)
+;;
+
+(test-section "hash-bang")
+
+(test* "script hash-bang" 3
+       (read-from-string "#!/usr/bin/gosh -i\n3"))
+(test* "script hash-bang" 5
+       (read-from-string "#! /usr/bin/gosh -i\n5"))
+(test* "script hash-bang" (eof-object)
+       (read-from-string "#! /usr/bin/gosh -i"))
+
+(test* "#!fold-case" '(hello world)
+       (read-from-string "#!fold-case (Hello World)"))
+(test* "#!fold-case" '(Hello world)
+       (read-from-string "(Hello #!fold-case World)"))
+(test* "#!no-fold-case" '(hello World)
+       (read-from-string "(#!fold-case Hello #!no-fold-case World)"))
+
+(test* "customized hash-bang" -i
+       (let ()
+         (define-reader-directive 'usr/bin/gosh (lambda _ (values)))
+         (read-from-string "#!usr/bin/gosh -i\n8")))
+
+(test* "customized hash-bang" '(#t #f)
+       (let ()
+         (define-reader-directive 'true  (lambda _ #t))
+         (define-reader-directive 'false (lambda _ #f))
+         (read-from-string "#!/usr/bin/gosh -i\n(#!true #!false)")))
+
+;;===============================================================
+;; SRFI-10 Reader constructor (#,)
+;;
+
+(test-section "srfi-10 reader constructor")
+
+(test "read ctor 1a" '(1 2 #f "4 5")
+      (lambda ()
+        (define-reader-ctor 'list list)
+        (with-input-from-string "#,(list 1 2 #f \"4 5\")" read)))
+(test "read ctor 1b" 3
+      (lambda ()
+        (define-reader-ctor '+ +)
+        (with-input-from-string "#,(+ 1 2)" read)))
+(define-reader-ctor 'my-vector
+  (lambda x (apply vector (cons 'my-vector x))))
+(test* "read ctor 2a" '#(my-vector (my-vector 1 2))
+       (with-input-from-string "#,(my-vector (my-vector 1 2))" read))
+(test* "read ctor 2b" '#(my-vector #(my-vector 1 2))
+       (with-input-from-string "#,(my-vector #,(my-vector 1 2))" read))
+
+;;===============================================================
+;; Shared structures (#n= and #n#)
+;;
 
 ;;---------------------------------------------------------------
 (test-section "write/ss basic")
@@ -77,6 +131,9 @@
                 k j i h g f e d c b a)
           write/ss)))
 
+(test* "circular list involving abbrev syntax" "#0=((quote . #0#))"
+       (write-to-string (cdr #0='#0#) write/ss))
+
 (define-class <foo> ()
   ((a :init-keyword :a)
    (b :init-keyword :b)))
@@ -159,13 +216,13 @@
 (test* "scalar (harmless)" "aaa"
        (read-from-string "#1=#0=\"aaa\""))
 
-(test* "bad syntax" *test-error*
+(test* "bad syntax" (test-error)
        (read-from-string "#1"))
-(test* "bad syntax" *test-error*
+(test* "bad syntax" (test-error)
        (read-from-string "#3#"))
-(test* "bad syntax" *test-error*
+(test* "bad syntax" (test-error)
        (read-from-string "#99999999999999999999999999999999999=3"))
-(test* "bad syntax" *test-error*
+(test* "bad syntax" (test-error)
        (read-from-string "#99999999999999999999999999999999999#"))
 
 (test* "pair 1" (circular-list 1 2)
@@ -194,6 +251,25 @@
        (read-from-string "(#0=\"aa\" #1=\"aa\" #0# #1#)")
        isomorphic?)
 
+;;===============================================================
+;; These test is here since srfi-0 must have been tested before.
+;;
+(test-section "whitespaces")
+
+(test* "skipws" 'a
+       (read-from-string
+        (cond-expand
+         [gauche.ces.utf8 "\u00a0\u1680\u180e\u2000\u200a\u2028\u2029\
+                           \u202f\u205f\u3000a"]
+         [(or gauche.ces.eucjp gauche.ces.sjis) "\u3000a"]
+         [else "a"])))
+
+;;===============================================================
+;; Interference between srfi-10 and shared structure
+;;
+
+(test-section "combine srfi-10 and srfi-38")
+
 ;; NB: this is an experimental feature.  Do not count on this API!
 (define-reader-ctor 'foo
   (lambda x `(quote ,x))
