@@ -175,6 +175,15 @@
 (test "unwrap1" '(a 4) (lambda () (let ((a 4)) (unwrap1 a))) 
       (lambda (x y) (and (eq? (car x) (car y)) (eq? (cadr x) (cadr y)))))
 
+;; regression check for quasiquote hygienty handling code
+(define-syntax qq1 (syntax-rules ()
+                     ((_ a) `(,@a))))
+(define-syntax qq2 (syntax-rules ()
+                     ((_ a) `#(,@a))))
+
+(test "qq1" '()  (lambda () (qq1 '())))
+(test "qq2" '#() (lambda () (qq2 '())))
+
 ;;----------------------------------------------------------------------
 ;; cond, taken from R5RS section 7.3
 
@@ -890,6 +899,13 @@
             (values-ref (values 'a 'b 'c 'd 'e) 4 3 2 1 0)
           r)))
 
+(test "values->list" '(3 1)
+      (lambda () (values->list (quotient&remainder 10 3))))
+(test "values->list" '(1)
+      (lambda () (values->list 1)))
+(test "values->list" '()
+      (lambda () (values->list (values))))
+
 (test "let1" '(2 2 2)
       (lambda () (let1 x (+ 1 1) (list x x x))))
 (test "let1" '(2 4)
@@ -902,6 +918,47 @@
 (test "if-let1" 'bar
       (lambda () (if-let1 it (memq 'a '(b c d)) 'boo 'bar)))
 
+(test "ecase" 'b
+      (lambda () (ecase 3 ((1) 'a) ((2 3) 'b) ((4) 'c))))
+(test "ecase" (test-error)
+      (lambda () (ecase 5 ((1) 'a) ((2 3) 'b) ((4) 'c))))
+(test "ecase" 'd
+      (lambda () (ecase 5 ((1) 'a) ((2 3) 'b) ((4) 'c) (else 'd))))
+
+(test "$" '(0 1)
+      (lambda () ($ list 0 1)))
+(test "$" '(0 1 (2 3 (4 5 (6 7))))
+      (lambda () ($ list 0 1 $ list 2 3 $ list 4 5 $ list 6 7)))
+(test "$ - $*" '(0 1 (2 3 4 5 6 7))
+      (lambda () ($ list 0 1 $ list 2 3 $* list 4 5 $* list 6 7)))
+(test "$ - $*" '(0 1 2 3 (4 5 6 7))
+      (lambda () ($ list 0 1 $* list 2 3 $ list 4 5 $* list 6 7)))
+(test "$ - $*" '(0 1 2 3 4 5 (6 7))
+      (lambda () ($ list 0 1 $* list 2 3 $* list 4 5 $ list 6 7)))
+(test "$ - partial" '(0 1 (2 3 (4 5 a)))
+      (lambda () (($ list 0 1 $ list 2 3 $ list 4 5 $) 'a)))
+(test "$ - $* - partial" '(0 1 2 3 4 5 a)
+      (lambda () (($ list 0 1 $* list 2 3 $* list 4 5 $) 'a)))
+(test "$ - $* - partial" '(0 1 (2 3 (4 5 a b)))
+      (lambda () (($ list 0 1 $ list 2 3 $ list 4 5 $*) 'a 'b)))
+
+(test "$ - hygienty" `(0 1 a ,list 2 3 b ,list 4 5)
+      (lambda ()
+        (let-syntax ([$$ (syntax-rules ()
+                           [($$ . xs) ($ . xs)])])
+          (let ([$ 'a] [$* 'b])
+            ($$ list 0 1 $ list 2 3 $* list 4 5)))))
+
+(test* "cond-list" '() (cond-list))
+(test* "cond-list" '(a) (cond-list ('a)))
+(test* "cond-list" '(a) (cond-list (#t 'a) (#f 'b)))
+(test* "cond-list" '(b) (cond-list (#f 'a) (#t 'b)))
+(test* "cond-list" '(a b d) (cond-list (#t 'a) (#t 'b) (#f 'c) (#t 'd)))
+(test* "cond-list" '((b)) (cond-list (#f 'a) ('b => list)))
+(test* "cond-list" '(a b c d x)
+       (cond-list (#t @ '(a b)) (#t @ '(c d)) (#f @ '(e f))
+                  ('x => @ list)))
+
 ;;----------------------------------------------------------------------
 ;; macro-expand
 
