@@ -307,6 +307,18 @@
               (j (begin (set! (s-get-i s) "j") (s-get-i s))))
          (list i j)))
 
+;;----------------------------------------------------------------
+(test-section "method optional arguments")
+
+(define-method optarg ((a <x>)) 'x)
+(define-method optarg ((a <y>) :optional (x 0) (y 1) :key (z 2) :rest r)
+  (list x y z r))
+
+(test* "method optarg (no optarg)" 'x (optarg (make <x>)))
+(test* "method optarg (optarg)" '(0 1 2 ()) (optarg (make <y>)))
+(test* "method optarg (optarg)" '(a b 2 ()) (optarg (make <y>) 'a 'b))
+(test* "method optarg (optarg)" '(a b 4 (:z 4)) (optarg (make <y>) 'a 'b :z 4))
+
 ;;----------------------------------------------------------------
 (test-section "module and accessor")
 
@@ -841,12 +853,20 @@
 ;;----------------------------------------------------------------
 (test-section "metaclass redefintiion")
 
-;; NB: semantics of metaclass redefinition isn't really fixed yet.
-;; Should we allow it at all?  The problem is that the classes created
-;; by the original metaclass is semantically redefined, but what happens
-;; is that they are just updated (via change-class), so that instances
-;; of those classes won't be aware that their classes are changed.
-;; It works for trival cases, so we just allow it and see what happens.
+;; Create and save the instances with the original classes
+(define *docu*
+  (rlet1 obj (make <xxx>)
+    (slot-set! obj 'a 'a)
+    (slot-set! obj 'b 'B)
+    (slot-set! obj 'c 'c)))
+(define *docu-sub*
+  (rlet1 obj (make <xxx-sub>)
+    (slot-set! obj 'a 'A)
+    (slot-set! obj 'b 'b)
+    (slot-set! obj 'c 'C)
+    (slot-set! obj 'x 'x)
+    (slot-set! obj 'y 'Y)
+    (slot-set! obj 'z 'z)))
 
 (define-class <docu-meta> (<class>)
   ((doc  :init-keyword :doc :initform #t)
@@ -865,6 +885,13 @@
              (slot-ref <xxx-sub> 'sub)
              (slot-ref <xxx-sub> 'xtra)))
 
+(test* "redefinition of metaclass and existing instance"
+       '(a B c)
+       (map (cut slot-ref *docu* <>) '(a b c)))
+(test* "redefinition of metaclass and existing instance"
+       '(A b C x Y z)
+       (map (cut slot-ref *docu-sub* <>) '(a b c x y z)))
+
 ;;----------------------------------------------------------------
 (test-section "metaclass/singleton")
 
@@ -1368,7 +1395,33 @@
 (test* "apply special path (apply->normal)" '(z c b a)
        (app-sp-path-test2 (make <app-sp-path2>) 'a 'b 'c))
 
+;;----------------------------------------------------------------
+(test-section "applicable?")
 
+;; The procedure applicable? is in lib/gauche/procedure.scm and
+;; it's supposed to be tested in ./procedure.scm, but internally
+;; it depends on libobj's method-applicable-for-clases, so we test
+;; it here.
+
+(define-syntax atest*
+  (syntax-rules (=>)
+    [(_ fn (args => expected) ...)
+     (begin (test* (format "applicable? ~a~s" 'fn 'args)
+                   expected (applicable? fn . args))
+            ...)]))
+
+(atest* cons [() => #f] [(<top>) => #f] [(<top> <top>) => #t]
+        [(<top> <top> <top>) => #f])
+(atest* list [() => #t] [(<top>) => #t] [(<top> <top>) => #t])
+(atest* apply [() => #f] [(<top>) => #f] [(<top> <top>) => #t])
+
+(atest* ref [() => #f] [(<boolean>) => #f] [(<boolean> <top>) => #f]
+        [(<top>) => #f] [(<top> <symbol>) => #t] [(<top> <symbol> <top>) => #t]
+        [(<vector>) => #f] [(<vector> <integer>) => #t]
+        [(<vector> <integer> <integer>) => #f]
+        [(<vector> <integer> <top>) => #f])
+
+(atest* #/a/ [() => #f] [(<string>) => #t] [(<integer>) => #f])
 
 (test-end)
 
