@@ -191,17 +191,36 @@
 (prim-test "qq"  101            (lambda ()  `,quasi1))
 (prim-test "qq," '((1 . 2))     (lambda ()  `(,(cons 1 2))))
 (prim-test "qq," '((1 . 2) 3)   (lambda ()  `(,(cons 1 2) 3)))
+(prim-test "qq," '(0 (1 . 2))   (lambda ()  `(0 ,(cons 1 2))))
+(prim-test "qq," '(0 (1 . 2) 3) (lambda ()  `(0 ,(cons 1 2) 3)))
+(prim-test "qq," '(((1 . 2)))   (lambda ()  `((,(cons 1 2)))))
+(prim-test "qq," '(((1 . 2)) 3) (lambda ()  `((,(cons 1 2)) 3)))
 (prim-test "qq," '(99 3)        (lambda ()  `(,quasi0 3)))
 (prim-test "qq," '(3 99)        (lambda ()  `(3 ,quasi0)))
+(prim-test "qq," '(3 99 3)      (lambda ()  `(3 ,quasi0 3)))
 (prim-test "qq," '(100 3)       (lambda ()  `(,(+ quasi0 1) 3)))
 (prim-test "qq," '(3 100)       (lambda ()  `(3 ,(+ quasi0 1))))
 (prim-test "qq," '(101 3)       (lambda ()  `(,quasi1 3)))
 (prim-test "qq," '(3 101)       (lambda ()  `(3 ,quasi1)))
 (prim-test "qq," '(102 3)       (lambda ()  `(,(+ quasi1 1) 3)))
 (prim-test "qq," '(3 102)       (lambda ()  `(3 ,(+ quasi1 1))))
+(prim-test "qq,(r6rs)" '(98 99 (a b) 100)
+           (lambda () `(98 (unquote quasi0 quasi2) 100)))
+(prim-test "qq,(r6rs)" '(98 99 101 100)
+           (lambda () `(98 (unquote quasi0 quasi1) 100)))
+(prim-test "qq,(r6rs)" '(98 99 (a b) 100)
+           (lambda () `(98 (unquote quasi0 quasi2) 100)))
+(prim-test "qq,(r6rs)" '(98 99 (a b) (1 2) (3 4))
+           (lambda () `(98 (unquote quasi0 quasi2) (unquote (list 1 2) (list 3 4)))))
 (prim-test "qq@" '(1 2 3 4)     (lambda ()  `(1 ,@(list 2 3) 4)))
 (prim-test "qq@" '(1 2 3 4)     (lambda ()  `(1 2 ,@(list 3 4))))
 (prim-test "qq@" '(a b c d)     (lambda ()  `(,@quasi2 ,@quasi3)))
+(prim-test "qq@(r6rs)" '(1 a b a b 2)
+           (lambda () `(1 (unquote-splicing quasi2 quasi2) 2)))
+(prim-test "qq@(r6rs)" '(1 a b c d 2)
+           (lambda () `(1 (unquote-splicing quasi2 quasi3) 2)))
+(prim-test "qq@(r6rs)" '(1 a b c d 2)
+           (lambda () `(1 (unquote-splicing (list 'a 'b) '(c d)) ,@(list 2))))
 (prim-test "qq." '(1 2 3 4)     (lambda ()  `(1 2 . ,(list 3 4))))
 (prim-test "qq." '(a b c d)     (lambda ()  `(,@quasi2 . ,quasi3)))
 (prim-test "qq#," '#((1 . 2) 3) (lambda ()  `#(,(cons 1 2) 3)))
@@ -245,6 +264,10 @@
            (lambda ()  `(1 `(1 ,,quasi0 ,,quasi1) 1)))
 (prim-test "qqq"   '(1 `(1 ,@2 ,@(1 2)))
            (lambda () `(1 `(1 ,@2 ,@,(list 1 2)))))
+(prim-test "qqq"   '(1 `(1 ,@2 (unquote 1 2)))
+           (lambda () `(1 `(1 ,@2 ,,@(list 1 2)))))
+(prim-test "qqq"   '(1 `(1 ,@2 (unquote-splicing 1 2)))
+           (lambda () `(1 `(1 ,@2 ,@,@(list 1 2)))))
 (prim-test "qqq"   '(1 `(1 ,@(a b) ,@(c d)))
            (lambda () `(1 `(1 ,@,quasi2 ,@,quasi3))))
 (prim-test "qqq"   '(1 `(1 ,(a b x) ,(y c d)))
@@ -262,6 +285,13 @@
 (prim-test "qqq#"  '(1 `#(1 ,(a b x) ,(y c d)))
            (lambda () `(1 `#(1 ,(,@quasi2 x) ,(y ,@quasi3)))))
 
+(prim-test "qq-hygiene 0" '(2 1)
+           (lambda () (let ((quasiquote reverse)) `(list 1 2))))
+(prim-test "qq-hygiene 1" '(,(+ 1 2))
+           (lambda () (let ((unquote 3)) `(,(+ 1 2)))))
+(prim-test "qq-hygiene 2" '(,@(+ 1 2))
+           (lambda () (let ((unquote-splicing 3)) `(,@(+ 1 2)))))
+
 ;;----------------------------------------------------------------
 (test-section "multiple values")
 
@@ -350,134 +380,38 @@
            (lambda ()
              (let ((f (lambda (i) (set! i 0) i))) (f (f 1)))))
 
-;;----------------------------------------------------------------
-(test-section "lazy, delay & force")
-
-(prim-test "simple delay" 3
-      (lambda () (force (delay (+ 1 2)))))
-
-(prim-test "delay w/state" 3
-      (lambda ()
-        (let ((x 9))
-          (let ((d (delay (/ x 3))))
-            (force d)
-            (set! x 99)
-            (force d)))))
-
-(prim-test "delay recursive" 6  ;; R5RS 6.4
-      (lambda ()
-        (letrec ((count 0)
-                 (x 5)
-                 (p (delay (begin (set! count (+ count 1))
-                                  (if (> count x)
-                                      count
-                                      (force p))))))
-          (force p)
-          (set! x 10)
-          (force p))))
-
-;; check to see the compiler does the right thing about expanding
-;; (delay x) to (lazy (eager x)).
-(prim-test "delay compilation" 3
+;; this caused an internal compiler error in 0.9.1
+(define (zero) 0)
+(prim-test "pass3 inlining with pass3/$call optimization" #t
            (lambda ()
-             (force 
-              (let ((lazy list)
-                    (eager list))
-               (delay (force 3))))))
+             (eval '((letrec ((f (lambda (a b)
+                                   (do ((x a (+ x 1)))
+                                       ((>= x b))))))
+                       f)
+                     (zero) (zero))
+                   (interaction-environment))))
 
-;; srfi-45 test suite
-(prim-test "memoize 1" 1
+;; This caused internal error in 0.9.1, and infinite loop in dev version
+;; after it.
+(prim-test "pass3/$call inlining problem" #t
            (lambda ()
-             (let ((count 0))
-               (define s (delay (begin (set! count (+ count 1)) 1)))
-               (force s)
-               (force s)
-               count)))
-
-(prim-test "memoize 2" 1
-           (lambda ()
-             (let ((count 0))
-               (define s (delay (begin (set! count (+ count 1)) 1)))
-               (+ (force s) (force s))
-               count)))
-
-(prim-test "memoize 3" 1  ;; (Alejandro Forero Cuervo)
-           (lambda ()
-             (let ((count 0))
-               (let* ((r (delay (begin (set! count (+ count 1)) 1)))
-                      (s (lazy r))
-                      (t (lazy s)))
-                 (force t)
-                 (force r)
-                 count))))
-
-(prim-test "memoize 4" 5  ;; stream memoization
-           (lambda ()
-             (let ((count 0))
-               (define (stream-drop s index)
-                 (lazy
-                  (if (zero? index)
-                    s
-                    (stream-drop (cdr (force s)) (- index 1)))))
-               (define (ones)
-                 (delay (begin
-                          (set! count (+ count 1))
-                          (cons 1 (ones)))))
-               (let ((s (ones)))
-                 (car (force (stream-drop s 4)))
-                 (car (force (stream-drop s 4)))
-                 count))))
-
-(prim-test "reentrancy 1" 'second       ;; see srfi-40 post-discussion
-           (lambda ()
-             (define f
-               (let ((first? #t))
-                 (delay
-                   (if first?
-                     (begin
-                       (set! first? #f)
-                       (force f))
-                     'second))))
-             (force f)))
-
-(prim-test "reentrancy 2" '(5 0 10) ;; (John Shutt)
-           (lambda ()
-             (define q
-               (let ((count 5))
-                 (define (get-count) count)
-                 (define p (delay (if (<= count 0)
-                                    count
-                                    (begin (set! count (- count 1))
-                                           (force p)
-                                           (set! count (+ count 2))
-                                           count))))
-                 (list get-count p)))
-             (let* ((get-count (car q))
-                    (p (cadr q))
-                    (a (get-count))
-                    (b (force p))
-                    (c (get-count)))
-               (list a b c))))
-
-;; This leak test takes long time, so we exclude it by default.
-;; If you run this on pre-0.8.6 Gauche (with replacing lazy by delay+force),
-;; it'll bust the memory.
-;(prim-test "leak 1" 10000000
-;           (lambda ()
-;             (define (stream-filter p? s)
-;               (lazy
-;                (let ((lis (force s)))
-;                  (if (null? lis)
-;                    (delay '())
-;                    (let ((h (car lis))
-;                          (t (cdr lis)))
-;                      (if (p? h)
-;                        (delay (cons h (stream-filter p? t)))
-;                        (stream-filter p? t)))))))
-;             (define (from n)
-;               (delay (cons n (from (+ n 1)))))
-;             (car (force (stream-filter (lambda (n) (= n 10000000))
-;                                        (from 0))))))
+             (procedure?
+              (eval '(lambda (n p t)
+                       (define (y a r s f)
+                         (let loop ([e 0])
+                           (cond [(a n) (unwind-protect (s) (r n))]
+                                 [(< e 10) (loop (+ 1 e))]
+                                 [else (f)])))
+                       (define (l0 a r)
+                         (y a r (^() (r n)) (^() (error "oo"))))
+                       ;; Main locker
+                       (define (l1 a r)
+                         (y a r p (^() (if (and-let* ([ t ]
+                                                      [m (file-mtime n)])
+                                             (< (+ m t) 10))
+                                         (begin (l0 a r) (l1 a r))))))
+                       (error "zz"))
+                    (interaction-environment)))))
 
 ;;----------------------------------------------------------------
 (test-section "optimized frames")
