@@ -702,9 +702,9 @@ Content-Length: 4349
 (test* "decode" "a%ay" (uri-decode-string "a%ay"))
 (test* "decode" ""     (uri-decode-string ""))
 
-(test* "uri-scheme&specific" '("http" "//www.shiro.dreamhost.com/scheme/")
+(test* "uri-scheme&specific" '("http" "//practical-scheme.net/gauche/")
        (receive r
-           (uri-scheme&specific "http://www.shiro.dreamhost.com/scheme/")
+           (uri-scheme&specific "http://practical-scheme.net/gauche/")
          r))
 
 (test* "uri-scheme&specific" '(#f "/dev/tty")
@@ -775,6 +775,68 @@ Content-Length: 4349
 (test* "uri-parse" '("mailto" #f #f #f "shiro@example.com" #f #f)
        (receive r (uri-parse "mailto:shiro@example.com") r))
 
+(let ([base0 "http://a/b/c/d;p?q"])
+  (define (t base rel expect) 
+    (test* (format "merging ~s onto ~s" rel base)
+           expect (uri-merge base rel)))
+  (define t0 (pa$ t base0))
+  ;; examples given in RFC3986 section 5.4
+  ;; normal path
+  (t0 "g:h" "g:h")
+  (t0 "g" "http://a/b/c/g")
+  (t0 "./g" "http://a/b/c/g")
+  (t0 "g/" "http://a/b/c/g/")
+  (t0 "/g" "http://a/g")
+  (t0 "//g" "http://g")
+  (t0 "?y" "http://a/b/c/d;p?y")
+  (t0 "g?y" "http://a/b/c/g?y")
+  (t0 "#s" "http://a/b/c/d;p?q#s")
+  (t0 "g#s" "http://a/b/c/g#s")
+  (t0 "g?y#s" "http://a/b/c/g?y#s")
+  (t0 ";x" "http://a/b/c/;x")
+  (t0 "g;x" "http://a/b/c/g;x")
+  (t0 "g;x?y#s" "http://a/b/c/g;x?y#s")
+  (t0 "" "http://a/b/c/d;p?q")
+  (t0 "." "http://a/b/c/")
+  (t0 "./" "http://a/b/c/")
+  (t0 ".." "http://a/b/")
+  (t0 "../" "http://a/b/")
+  (t0 "../g" "http://a/b/g")
+  (t0 "../.." "http://a/")
+  (t0 "../../" "http://a/")
+  (t0 "../../g" "http://a/g")
+  ;; failure path
+  (t0 "../../../g" "http://a/g")
+  (t0 "../../../../g" "http://a/g")
+  (t0 "/./g" "http://a/g")
+  (t0 "/../g" "http://a/g")
+  (t0 "g." "http://a/b/c/g.")
+  (t0 ".g" "http://a/b/c/.g")
+  (t0 "g.." "http://a/b/c/g..")
+  (t0 "..g" "http://a/b/c/..g")
+  (t0 "./../g" "http://a/b/g")
+  (t0 "./g/." "http://a/b/c/g/")
+  (t0 "g/./h" "http://a/b/c/g/h")
+  (t0 "g/../h" "http://a/b/c/h")
+  (t0 "g;x=1/./y" "http://a/b/c/g;x=1/y")
+  (t0 "g;x=1/../y" "http://a/b/c/y")
+  (t0 "g?y/./x" "http://a/b/c/g?y/./x")
+  (t0 "g?y/../x" "http://a/b/c/g?y/../x")
+  (t0 "g#s/./x" "http://a/b/c/g#s/./x")
+  (t0 "g#s/../x" "http://a/b/c/g#s/../x")
+  (t0 "http:g" "http:g") ;; for strict parser
+
+  ;; some edge cases.  the first case works since we do pre-normalization
+  ;; of the base URI (RFC3986 5.2.1), which is optional.
+  (t "http://example.com/foo/.." "./" "http://example.com/")
+  (t "http://example.com/" "./foo/bar/.." "http://example.com/foo/")
+
+  ;; empty base-path case
+  (t "http://example.com"  "foo" "http://example.com/foo")
+  (t "http://example.com"  "./foo" "http://example.com/foo")
+  (t "http://example.com"  "../foo" "http://example.com/foo")
+  )
+
 ;;--------------------------------------------------------------------
 (test-section "rfc.http")
 (use rfc.http)
@@ -895,7 +957,7 @@ Content-Length: 4349
                    :receiver (lambda (code hdrs total retr)
                                (let loop ((result '()))
                                  (receive (port size) (retr)
-                                   (if (= size 0)
+                                   (if (and size (= size 0))
                                      result
                                      (loop (append result (read port)))))))
                    :my-header "foo")
@@ -932,11 +994,11 @@ Content-Length: 4349
                   (let loop ()
                     (receive (port size) (retr)
                       (cond
-                       [(= size 0) (close-output-port sink) 404]
+                       [(and size (= size 0)) (close-output-port sink) 404]
                        [else (copy-port port sink :size size) (loop)])))))]            ["200" (lambda (code hdrs total retr)
                 (let loop ((result '()))
                   (receive (port size) (retr)
-                    (if (= size 0)
+                    (if (and size (= size 0))
                       result
                       (loop (append result (read port)))))))]
        ))
@@ -964,6 +1026,27 @@ Content-Length: 4349
          (cond ((assoc-ref (read-from-string body) "request-uri")
                 => car))))
 
+(test* "http-get (no redirect)" "/redirect12"
+       (receive (code headers body)
+           (http-request 'GET #`"localhost:,*http-port*" "/redirect11"
+                         :redirect-handler #f)
+         (rfc822-header-ref headers "location")))
+
+(test* "http-get (custom redirect)" "/foofoo"
+       (receive (code headers body)
+           (http-request 'GET #`"localhost:,*http-port*" "/redirect11"
+                         :redirect-handler (^[meth code hdrs body]
+                                             `(GET . "/foofoo")))
+         (cond ((assoc-ref (read-from-string body) "request-uri")
+                => car))))
+
+(test* "http-get (custom redirect to HEAD)" #f
+       (receive (code headers body)
+           (http-request 'GET #`"localhost:,*http-port*" "/redirect11"
+                         :redirect-handler (^[meth code hdrs body]
+                                             `(HEAD . "/foofoo")))
+         body))
+
 (test* "http-get (loop)" (test-error <http-error>)
        (http-request 'GET #`"localhost:,*http-port*" "/loop1"))
 
