@@ -4,7 +4,7 @@
 
 (use gauche.test)
 
-(test-start "string")
+(test-start "strings (basic)")
 
 ;;-------------------------------------------------------------------
 (test-section "builtins")
@@ -73,33 +73,76 @@
 (test* "string-join" "foo::bar::baz"
        (string-join '("foo" "bar" "baz") "::" 'strict-infix))
 
-(test* "string-scan" 3 (string-scan "abcdefghi" "def"))
-(test* "string-scan" 3 (string-scan "abcdefghi" "def" 'index))
-(test* "string-scan" "abc" (string-scan "abcdefghi" "def" 'before))
-(test* "string-scan" "ghi" (string-scan "abcdefghi" "def" 'after))
-(test* "string-scan" '("abc" "defghi")
-       (receive r (string-scan "abcdefghi" "def" 'before*) r))
-(test* "string-scan" '("abcdef" "ghi")
-       (receive r (string-scan "abcdefghi" "def" 'after*) r))
-(test* "string-scan" '("abc" "ghi")
-       (receive r (string-scan "abcdefghi" "def" 'both) r))
+(let ()
+  (define (test-string-scan out s1 s2 . opt)
+    (if (pair? out)
+      (begin
+        (test* "string-scan" (car out) (apply string-scan s1 s2 opt))
+        (test* "string-scan-right" (cadr out)
+               (apply string-scan-right s1 s2 opt)))
+      (apply test-string-scan (list out out) s1 s2 opt)))
+  (define (test-string-scan2 out1 out2 s1 s2 . opt)
+    (if (pair? out1)
+      (begin
+        (test* "string-scan" (list (car out1) (car out2))
+               (receive r (apply string-scan s1 s2 opt) r))
+        (test* "string-scan-right" (list (cadr out1) (cadr out2))
+               (receive r (apply string-scan-right s1 s2 opt) r)))
+      (apply test-string-scan2 (list out1 out1) (list out2 out2) s1 s2 opt)))
 
-(test* "string-scan" 4 (string-scan "abcdefghi" #\e))
-(test* "string-scan" "abcd" (string-scan "abcdefghi" #\e 'before))
-(test* "string-scan" "fghi" (string-scan "abcdefghi" #\e 'after))
-(test* "string-scan" '("abcd" "efghi")
-       (receive r (string-scan "abcdefghi" #\e 'before*) r))
-(test* "string-scan" '("abcde" "fghi")
-       (receive r (string-scan "abcdefghi" #\e 'after*) r))
-(test* "string-scan" '("abcd" "fghi")
-       (receive r (string-scan "abcdefghi" #\e 'both) r))
+  (test-string-scan 3 "abcdefghi" "def")
+  (test-string-scan 3 "abcdefghi" "def" 'index)
+  (test-string-scan "abc" "abcdefghi" "def" 'before)
+  (test-string-scan "ghi" "abcdefghi" "def" 'after)
+  (test-string-scan2 "abc" "defghi" "abcdefghi" "def" 'before*)
+  (test-string-scan2 "abcdef" "ghi" "abcdefghi" "def" 'after*)
+  (test-string-scan2 "abc" "ghi" "abcdefghi" "def" 'both)
 
-(test* "string-scan (boyer-moore)" 216
-       (string-scan "abracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabr"
-                    "abracadabra"))
+  (test-string-scan 4 "abcdefghi" #\e)
+  (test-string-scan "abcd" "abcdefghi" #\e 'before)
+  (test-string-scan "fghi" "abcdefghi" #\e 'after)
+  (test-string-scan2 "abcd" "efghi" "abcdefghi" #\e 'before*)
+  (test-string-scan2 "abcde" "fghi" "abcdefghi" #\e 'after*)
+  (test-string-scan2 "abcd" "fghi" "abcdefghi" #\e 'both)
 
-(test* "string-scan (special case)" 0
-       (string-scan "abakjrgaker" ""))
+  ;; this tests boyer-moore 
+  (test-string-scan 216
+                    "abracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabrabracadababrabrabr"
+                    "abracadabra")
+
+  ;; results differ between leftmost and rightmost
+  (test-string-scan '(1 8) "abracadabra" "br")
+  (test-string-scan '("acadabra" "a") "abracadabra" "br" 'after)
+  (test-string-scan '("a" "abracada") "abracadabra" "br" 'before)
+  (test-string-scan2 '("abr" "abracadabr") '("acadabra" "a")
+                     "abracadabra" "br" 'after*)
+  (test-string-scan2 '("a" "abracada") '("bracadabra" "bra")
+                     "abracadabra" "br" 'before*)
+  (test-string-scan2 '("a" "abracada") '("acadabra" "a")
+                     "abracadabra" "br" 'both)
+
+  ;; edge case
+  (test-string-scan '(0 11) "abakjrgaker" "")
+  (test-string-scan '("abakjrgaker" "") "abakjrgaker" "" 'after)
+  (test-string-scan '("" "abakjrgaker") "abakjrgaker" "" 'before)
+  (test-string-scan2 '("" "abakjrgaker") '("abakjrgaker" "")
+                     "abakjrgaker" "" 'after*)
+  (test-string-scan2 '("" "abakjrgaker") '("abakjrgaker" "")
+                     "abakjrgaker" "" 'before*)
+  (test-string-scan2 '("" "abakjrgaker") '("abakjrgaker" "")
+                     "abakjrgaker" "" 'both)
+
+  ;; incomplete strings
+  (test-string-scan 3 #*"abcdefghi" "def")
+  (test-string-scan 3 "abcdefghi" #*"def")
+  (test-string-scan #*"ghi" #*"abcdefghi" "def" 'after)
+  (test-string-scan #*"abc" #*"abcdefghi" "def" 'before)
+  (test-string-scan2 #*"abcdef" #*"ghi" #*"abcdefghi" "def" 'after*)
+  (test-string-scan2 #*"abc" #*"defghi" #*"abcdefghi" "def" 'before*)
+  (test-string-scan2 #*"abc" #*"ghi" #*"abcdefghi" "def" 'both)
+  (test-string-scan2 #*"abc" #*"ghi" "abcdefghi" #*"def" 'both)
+  (test-string-scan2 #*"abcd" #*"fghi" #*"abcdefghi" #\e 'both)
+  )
 
 ;;-------------------------------------------------------------------
 (test-section "string-split")
@@ -141,7 +184,7 @@
        (string-split "--aa--bbb---c-c-" #/\w+/))
 (test* "string-split (regexp)" '("--aa--bbb---c-c-")
        (string-split "--aa--bbb---c-c-" #/z+/))
-(test* "string-split (regexp)" *test-error* ;; test detection of infinite loop
+(test* "string-split (regexp)" (test-error) ;; test detection of infinite loop
        (string-split "--aa--bbb---c-c-" #/-*/))
 
 (test* "string-split (charset)" '("aa" "bbb" "c" "d")
@@ -198,18 +241,6 @@
 (test* "string-join" #*"a:b:c"
        (string-join '("a" "b" "c") #*":"))
 
-(test* "string-scan" 3
-       (string-scan #*"abcdefghi" "def"))
-(test* "string-scan" 3
-       (string-scan "abcdefghi" #*"def"))
-(test* "string-scan" '(#*"abc" #*"ghi")
-       (receive r (string-scan #*"abcdefghi" "def" 'both) r))
-(test* "string-scan" '(#*"abc" #*"ghi")
-       (receive r (string-scan "abcdefghi" #*"def" 'both) r))
-(test* "string-scan" '(#*"abcd" #*"fghi")
-       (receive r (string-scan #*"abcdefghi" #\e 'both) r))
-
-
 ;; NB: should we allow this?
 (test* "string-set!" #*"abQde"
        (let ((s (string-copy #*"abcde")))
@@ -224,6 +255,14 @@
          (string-byte-set! s 2 (char->integer #\Q))
          s))
 
+;; In 0.9.1 this gets an error
+(unless (eq? (gauche-character-encoding) 'none)
+  (test* "string-byte-set!" (case (gauche-character-encoding)
+                              [(utf-8)  #*"\x01\x81\x82"]
+                              [(euc-jp) #*"\x01\xa2"]
+                              [(sjis)   #*"\x01\xa0"])
+         (string-byte-set! (string-copy "\u3042") 0 1)))
+
 (test* "substring" #*"ab"
        (substring #*"abcde" 0 2))
 
@@ -363,9 +402,23 @@
        (read (open-input-string "\"0123 \\   \n   4567\"")))
 (test* "line continuation" "0123-4567"
        (read (open-input-string "\"0123\\\n \\  \n -4567\"")))
-(test* "line continuation (invalid)" *test-error*
+(test* "line continuation (invalid)" (test-error)
        (read (open-input-string "\"1234\\ x\"")))
 
+(unless (eq? (gauche-character-encoding) 'none)
+  (test* "line continuation (extended chars)" "abc def1"
+         (read (open-input-string "\"abc \\\u3000\ndef1\"")))
+  (test* "line continuation (extended chars)" "abc def2"
+         (read (open-input-string "\"abc \\ \u3000\ndef2\"")))
+  (test* "line continuation (extended chars)" "abc def3"
+         (read (open-input-string "\"abc \\\u3000 \ndef3\"")))
+  (test* "line continuation (extended chars)" "ABC DEF1"
+         (read (open-input-string "\"ABC \\\n\u3000 DEF1\"")))
+  (test* "line continuation (extended chars)" "ABC DEF2"
+         (read (open-input-string "\"ABC \\\n \u3000DEF2\"")))
+  (test* "line continuation (extended chars)" "uvw xyz1"
+         (read (open-input-string "\"uvw \\\u3000\n\u3000\\\nxyz1\"")))
+  )
 
 (define (read-line-tester str)
   (let1 s (open-input-string str)
