@@ -88,6 +88,62 @@
 (test* "getcwd" (get-pwd-via-pwd)
        (sys-getcwd))
 
+;; putenv
+
+(cond-expand
+ [gauche.sys.setenv
+  (test* "sys-putenv" "foo"
+         (begin
+           (sys-putenv "ZZGGGBBB=foo")
+           (sys-getenv "ZZGGGBBB")))
+  (test* "sys-putenv" "foo"
+         (begin
+           (sys-putenv "ZZGGGBBB" "foo")  ;;old API
+           (sys-getenv "ZZGGGBBB")))]
+ [else])
+
+;; setenv
+
+(cond-expand
+ [gauche.sys.setenv
+  (test* "sys-setenv" "foo"
+         (begin
+           (sys-setenv "ZZGGGBBB" "foo" #t)
+           (sys-getenv "ZZGGGBBB")))
+  (test* "sys-setenv" "foo"
+         (begin
+           (sys-setenv "ZZGGGBBB" "bar" #f)
+           (sys-getenv "ZZGGGBBB")))]
+ [else])
+
+;; unsetenv
+
+(cond-expand
+ [gauche.sys.unsetenv
+  (test* "sys-unsetenv" #f
+         (begin
+           (sys-setenv "ZZGGGBBB" "foo" #t)
+           (sys-unsetenv "ZZGGGBBB")
+           (sys-getenv "ZZGGGBBB")))]
+ [else])
+
+;; environ
+(test* "sys-environ->alist" '(("A" . "B") ("A" . "") ("" . "B") ("A" . "B=C"))
+       (sys-environ->alist '("A=B" "A=" "=B" "A=B=C")))
+
+(let ([envs (sys-environ)])
+  (define (env-test var)
+    (test* #`"sys-environ (,var)" #t
+           (cond [(sys-getenv var)
+                  => (^[val] (boolean (member #`",|var|=,|val|" envs)))]
+                 [else #t])))
+  (env-test "HOME")
+  (env-test "USER")
+  (env-test "LANG")
+  (env-test "PWD")
+  (env-test "TERM")
+  (env-test "SHELL"))
+
 ;;-------------------------------------------------------------------
 (test-section "pathnames")
 
@@ -142,7 +198,7 @@
  (gauche.os.windows #t)
  (else
   (test* "normalize"
-         (n (string-append (get-command-output "echo $HOME") "/abc"))
+         (n (string-append (slot-ref (sys-getpwuid (sys-getuid)) 'dir) "/abc"))
          (sys-normalize-pathname "~/abc" :expand #t))))
 
 (test* "normalize" (n "/a/b/c/d/e")
@@ -263,6 +319,61 @@
          (sys-rmdir "test.dir")
          (sys-access "test.dir" F_OK)))
 
+;; sys-realpath
+
+(define (expected-path p)
+  (let1 pp (sys-normalize-pathname p :absolute #t :canonicalize #t)
+    (if (eqv? (string-ref pp (- (string-length pp) 1)) #\/)
+      (substring pp 0 (- (string-length pp) 1))
+      pp)))
+
+(test* "sys-realpath (/)" (sys-normalize-pathname "/") (sys-realpath "/"))
+(test* "sys-realpath (.)" (expected-path ".") (sys-realpath "."))
+
+(cond-expand
+ [gauche.sys.symlink
+  (sys-unlink "test1.o")
+  (sys-unlink "test2.o")
+  (with-output-to-file "test1.o" (cut print))
+
+  (sys-symlink "test1.o" "test2.o")
+  (test* "sys-realpath (symlink)"
+         (expected-path "./test1.o")
+         (sys-realpath "./test2.o"))
+  (sys-unlink "test2.o")
+
+  (sys-symlink "./test1.o" "test2.o")
+  (test* "sys-realpath (symlink)"
+         (expected-path "./test1.o")
+         (sys-realpath "./test2.o"))
+  (sys-unlink "test2.o")
+
+  (sys-symlink "../src/test1.o" "test2.o")
+  (test* "sys-realpath (symlink)"
+         (expected-path "./test1.o")
+         (sys-realpath "./test2.o"))
+
+  (sys-unlink "test1.o")
+  (test* "sys-realpath (dangling)"
+         (test-error)
+         (sys-realpath "./test2.o"))
+
+  (sys-mkdir "test1.o" #o777)
+  (with-output-to-file "test1.o/test.o" (cut print))
+  (test* "sys-realpath (symlink to dir)"
+         (expected-path "./test1.o/test.o")
+         (sys-realpath "./test2.o/test.o"))
+
+  (sys-unlink "test1.o/test.o")
+  (sys-rmdir "test1.o")
+  (sys-unlink "test2.o")
+
+  (test* "sys-realpath (NOENT)"
+         (test-error <system-error>)
+         (sys-realpath "./test2.o/test.o"))
+  ]
+ [else])
+
 ;;-------------------------------------------------------------------
 (test-section "time")
 
@@ -405,7 +516,7 @@
    (else (sys-sleep 1))))
 
 (cond-expand
- ((and (not gauche.os.windows)  ;; win32 doesn't support fork at all.
+ [(and (not gauche.os.windows)  ;; win32 doesn't support fork at all.
        (not gauche.os.cygwin))	;; cygwin's fork is not reliable.
   (test* "fork & wait" #t
          (let ((pid (sys-fork)))
@@ -494,14 +605,62 @@
                  (sys-sigmask SIG_SETMASK omask)
                  (sys-waitpid pid)
                  #t)))))
-  ) ; !gauche.os.windows
- (else))
+
+  ;; Testing fork&exec and detached process
+  ;; NB: these tests assume we're running the testing gosh in the
+  ;; current directory.
+  (cmd-rmrf "test.out")
+  (with-output-to-file "test.out"
+    (lambda ()
+      (write '(begin
+                (sys-nanosleep #e2e8)
+                (write (list (sys-getpid) (sys-getppid) (sys-getpgrp)))))))
+  (receive (in out) (sys-pipe :buffering :none)
+    (define (run-and-read detached)
+      (let1 pid (sys-fork-and-exec "./gosh"
+                                   `("./gosh" "-ftest" "./test.out")
+                                   :iomap `((1 . ,out))
+                                   :detached detached)
+        (begin0 (read in) (sys-waitpid pid))))
+    (test* "fork, exec and detached process (not detached)"
+           `(parent ,(sys-getpid) pgrp ,(sys-getpgrp))
+           (let1 r (run-and-read #f)
+             `(parent ,(cadr r) pgrp ,(caddr r))))
+    (test* "fork, exec and detached process (detached)"
+           `(parent 1 pgrp #t)
+           (let1 r (run-and-read #t)
+             `(parent ,(cadr r) pgrp ,(= (car r) (caddr r)))))
+    )
+  (cmd-rmrf "test.out")
+
+  ;; Testing GC in forked process---we don't explicitly spawn a thread here,
+  ;; but some architecture (OSX 10.7.3, at least) seems to create a thread
+  ;; implicitly.  In the child process all threads are gone except one,
+  ;; and GC needs to recognize that.  The following test fails on OSX 10.7.3
+  ;; without HANDLE_FORK defined in gc.
+  ;; NB: HANDLE_FORK doesn't seem to work on OSX 10.7.4.  Disabling for now.
+  ;; (test* "GC in forked process" "done"
+  ;;        (receive (in out) (sys-pipe)
+  ;;          (let ([pid (sys-fork)])
+  ;;            (if (= pid 0)
+  ;;              (begin
+  ;;                (dotimes (i 1000)
+  ;;                  (make-vector 10000))
+  ;;                (gc)
+  ;;                (display "done\n" out)
+  ;;                (sys-exit 0))
+  ;;              (let ((line (read-line in)))
+  ;;                (sys-waitpid pid)
+  ;;                line)))))
+  
+  ] ; !gauche.os.windows
+ [else])
 
 ;;-------------------------------------------------------------------
 (test-section "select")
 
 (cond-expand
- (gauche.sys.select
+ [gauche.sys.select
   (test* "fdset" '(3 #t #f #t #t #f)
          (let ((fdset (make <sys-fdset>)))
            (set! (sys-fdset-ref fdset (current-input-port)) #t)
@@ -545,31 +704,37 @@
   (test* "sys-fdset-clear!" '()
          (sys-fdset->list (sys-fdset-clear! (sys-fdset 1 2 3))))
 
-  (test* "select" '(0 #f #f #f #f 1 #t #f #f #t #\x)
-         (let*-values (((in out) (sys-pipe))
-                       ((pid) (sys-fork)))
-           (if (= pid 0)
-             (begin (sys-select #f #f #f 100000)
-                    (display "x" out)
-                    (close-output-port out)
-                    (sys-exit 0))
-             (let ((rfds (make <sys-fdset>)))
-               (sys-fdset-set! rfds in #t)
-               (receive (an ar aw ae)
-                   (sys-select rfds #f #f 0)
-                 (receive (bn br bw be)
-                     (sys-select! rfds #f #f #f)
-                   (begin0
-                    (list an (eq? ar rfds) aw ae
-                          (sys-fdset-ref ar in)
-                          bn (eq? br rfds) bw be
-                          (sys-fdset-ref rfds in)
-                          (read-char in))
-                    (sys-waitpid pid)))))
-             ))
-         )
-  )
- (else)) ; cond-expand gauche.sys.select
+  ;; NB: Windows' select() can't select on non-socket fds, so we skip
+  ;; this test.
+
+  (cond-expand
+   [(not gauche.os.windows)
+    (test* "select" '(0 #f #f #f #f 1 #t #f #f #t #\x)
+           (let*-values (((in out) (sys-pipe))
+                         ((pid) (sys-fork)))
+             (if (= pid 0)
+               (begin (sys-select #f #f #f 100000)
+                      (display "x" out)
+                      (close-output-port out)
+                      (sys-exit 0))
+               (let ((rfds (make <sys-fdset>)))
+                 (sys-fdset-set! rfds in #t)
+                 (receive (an ar aw ae)
+                     (sys-select rfds #f #f 0)
+                   (receive (bn br bw be)
+                       (sys-select! rfds #f #f #f)
+                     (begin0
+                         (list an (eq? ar rfds) aw ae
+                               (sys-fdset-ref ar in)
+                               bn (eq? br rfds) bw be
+                               (sys-fdset-ref rfds in)
+                               (read-char in))
+                       (sys-waitpid pid)))))
+               ))
+           )]
+   [else]) ; cond-expand (not gauche.os.windows)
+  ]
+ [else]) ; cond-expand gauche.sys.select
 
 ;;-------------------------------------------------------------------
 (test-section "signal handling")
