@@ -2,7 +2,7 @@
 
 (use gauche.test)
 
-(test-start "UTF8")
+(test-start "multibyte (utf-8)")
 (use srfi-1)
 
 ;;-------------------------------------------------------------------
@@ -54,23 +54,40 @@
 (test "string-join" "ふぅ！ばぁ！ばず"
       (lambda () (string-join '("ふぅ" "ばぁ" "ばず") "！" 'strict-infix)))
 
-(test "string-scan" 7
-      (lambda () (string-scan "あえいうえおあおあいうえお" "おあい")))
-(test "string-scan" "あえいうえおあ"
-      (lambda () (string-scan "あえいうえおあおあいうえお" "おあい" 'before)))
-(test "string-scan" "うえお"
-      (lambda () (string-scan "あえいうえおあおあいうえお" "おあい" 'after)))
-(test "string-scan" '("あえいうえおあ" "おあいうえお")
-      (lambda ()
-        (receive r (string-scan "あえいうえおあおあいうえお" "おあい" 'before*) r)))
-(test "string-scan" '("あえいうえおあおあい" "うえお")
-      (lambda ()
-        (receive r (string-scan "あえいうえおあおあいうえお" "おあい" 'after*) r)))
-(test "string-scan" '("あえいうえおあ" "うえお")
-      (lambda ()
-        (receive r (string-scan "あえいうえおあおあいうえお" "おあい" 'both) r)))
-(test "string-scan" #f
-      (lambda () (string-scan "あえいうえおあおあいうえお" "おい")))
+(let ()
+  (define (test-string-scan out s1 s2 . opt)
+    (if (pair? out)
+      (begin
+        (test* "string-scan" (car out) (apply string-scan s1 s2 opt))
+        (test* "string-scan-right" (cadr out)
+               (apply string-scan-right s1 s2 opt)))
+      (apply test-string-scan (list out out) s1 s2 opt)))
+  (define (test-string-scan2 out1 out2 s1 s2 . opt)
+    (if (pair? out1)
+      (begin
+        (test* "string-scan" (list (car out1) (car out2))
+               (receive r (apply string-scan s1 s2 opt) r))
+        (test* "string-scan-right" (list (cadr out1) (cadr out2))
+               (receive r (apply string-scan-right s1 s2 opt) r)))
+      (apply test-string-scan2 (list out1 out1) (list out2 out2) s1 s2 opt)))
+
+  (test-string-scan '(7 20)
+                    "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい")
+  (test-string-scan '("あえいうえおあ" "あえいうえおあおあいうえおあえいうえおあ")
+                      "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'before)
+  (test-string-scan '("うえおあえいうえおあおあいうえお" "うえお")
+                    "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'after)
+  (test-string-scan2 '("あえいうえおあ" "あえいうえおあおあいうえおあえいうえおあ")
+                     '("おあいうえおあえいうえおあおあいうえお" "おあいうえお")
+                     "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'before*)
+  (test-string-scan2 '("あえいうえおあおあい" "あえいうえおあおあいうえおあえいうえおあおあい")
+                     '("うえおあえいうえおあおあいうえお" "うえお")
+                     "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'after*)
+  (test-string-scan2 '("あえいうえおあ" "あえいうえおあおあいうえおあえいうえおあ")
+                     '("うえおあえいうえおあおあいうえお" "うえお")
+                     "あえいうえおあおあいうえおあえいうえおあおあいうえお" "おあい" 'both)
+  (test-string-scan #f "あえいうえおあおあいうえお" "おい")
+  )
 
 ;;-------------------------------------------------------------------
 (test-section "string-pointer")
@@ -619,4 +636,87 @@
          (rxmatch-after m) ;; memoizes start and length
          (list (rxmatch-before m) (rxmatch-substring m))))
 
+;;-------------------------------------------------------------------
+(test* "regexp/unicode-ci (aa)" "λ"
+       (cond ((rxmatch #/λ/i "λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (aA)" "Λ"
+       (cond ((rxmatch #/λ/i "Λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (Aa)" "λ"
+       (cond ((rxmatch #/Λ/i "λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (AA)" "Λ"
+       (cond ((rxmatch #/Λ/i "Λ") => rxmatch-substring)
+             (else #f)))
+
+(test* "regexp/unicode-ci (uncase + backref, aa)" "λλ"
+       (cond ((rxmatch #/(λ)(?i:\1)/ "λλ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (uncase + backref, aA)" "λΛ"
+       (cond ((rxmatch #/(λ)(?i:\1)/ "λΛ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (uncase + backref, Aa)" "Λλ"
+       (cond ((rxmatch #/(Λ)(?i:\1)/ "Λλ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (uncase + backref, AA)" "ΛΛ"
+       (cond ((rxmatch #/(Λ)(?i:\1)/ "ΛΛ") => rxmatch-substring)
+             (else #f)))
+
+(test* "regexp/unicode-ci (charset, aa)" "λ"
+       (cond ((rxmatch #/[λ]/i "λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (charset, aA)" "Λ"
+       (cond ((rxmatch #/[λ]/i "Λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (charset, Aa)" "λ"
+       (cond ((rxmatch #/[Λ]/i "λ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci (charset, AA)" "Λ"
+       (cond ((rxmatch #/[Λ]/i "Λ") => rxmatch-substring)
+             (else #f)))
+
+(test* "regexp/unicode-ci" "ΒΓ"
+       (cond ((rxmatch #/βγ/i "ΑΒΓΔ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "βΓ"
+       (cond ((rxmatch #/Βγ/i "ΑβΓΔ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "βγ"
+       (cond ((rxmatch #/ΒΓ/i "ΑβγΔ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/Βγ/ "ΑβΓΔ") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/ΒΓ/ "ΑΒγΔ") => rxmatch-substring)
+             (else #f)))
+
+(test* "regexp/unicode-ci" "ОНА"
+       (cond ((rxmatch #/о[а-я]а/i "ОНА") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "она"
+       (cond ((rxmatch #/О[А-Я]А/i "она") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "они"
+       (cond ((rxmatch #/[а-пР-Я][А-Пр-я]./i "они") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/о[а-я]а/ "ОНА") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/О[А-Я]А/ "она") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch #/[а-пР-Я][А-Пр-я]./ "они") => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" "она"
+       (cond ((rxmatch (string->regexp "о[А-Я]а" :case-fold #t) "она")
+              => rxmatch-substring)
+             (else #f)))
+(test* "regexp/unicode-ci" #f
+       (cond ((rxmatch (string->regexp "о[А-Я]а") "она")
+              => rxmatch-substring)
+             (else #f)))
+
 (test-end)
